MODULE CharData;
  (*/WIN:A*)
  
  FROM WMemory IMPORT ALLOCATE,DEALLOCATE;
  
  IMPORT WCLoop,WCreation,WCursor,WDebugging,WDisplay,Win31,Windows,WinFiles,WInformation,WinStart;
  IMPORT FileSystem,Strings,SYSTEM;
  IMPORT CDInit,MCStruct;
  
  
  CONST
        MainWindowClassName = "Char_Data_Window_Class";
        MainWindowTitle     = "Moria Character Data";
  
  VAR
      MainWindowClassData : Windows.WNDCLASS;
      MainWindowClassAtom : Windows.ATOM;
      MainWindowHandle    : Windows.HWND;
  
  
  PROCEDURE AllocateMainWindowClassData (instance : Windows.HINSTANCE);
  BEGIN (* AllocateMainWindowClassData *)
    WITH MainWindowClassData DO
      hbrBackground := Windows.COLOR_WINDOW + 1;
      lpszClassName := SYSTEM.ADR (MainWindowClassName);
      lpszMenuName  := SYSTEM.ADR ("Main_Menu");
      lpfnWndProc   := MainWindowTranslator;
      cbClsExtra    := 0;
      cbWndExtra    := 0;
      hInstance     := instance;
      hCursor       := WCursor.StandardCursorHandle (Windows.IDC_ARROW);
      hIcon         := Windows.NULL;
      style         := Windows.CS_HREDRAW + Windows.CS_VREDRAW;
    END; (* WITH *)
    
    WCreation.RegisterClass (MainWindowClassData,MainWindowClassAtom);
  END AllocateMainWindowClassData;
  
  PROCEDURE AllocateMainWindowHandle (VAR window : Windows.HWND);
    VAR
        shown : BOOLEAN;
    
  BEGIN (* AllocateMainWindowHandle *)
    WCreation.CreateDefaultWindow (MainWindowClassName,MainWindowTitle,Windows.WS_OVERLAPPEDWINDOW,Windows.NULL,WinStart.hInstance,window);
    WDisplay.ShowWindow (window,WinStart.cmdShow,shown);
  END AllocateMainWindowHandle;
  
  PROCEDURE MainWindowTranslator ['MAINWINDOWTRANSLATOR'] (window : Windows.HWND; message : CARDINAL; param1 : Windows.WPARAM; param2 : Windows.LPARAM) : Windows.LRESULT [WINDOWS,LONG];
    VAR
        result : Windows.LRESULT;
    
    PROCEDURE ExecuteCharDataCommand (window : Windows.HWND; command : CARDINAL);
      CONST
            CreateCommandControl = 100;
            ExitCommandControl   = 200;
      
      PROCEDURE ExecuteCreateDataCommand (window : Windows.HWND);
        VAR
            previous : Windows.HCURSOR;
            output   : ARRAY [0..127] OF CHAR;
            info     : MCStruct.CharacterResourceDataPointer;
            done     : BOOLEAN;
        
        PROCEDURE DetermineOutputFileName (window : Windows.HWND; VAR result : ARRAY OF CHAR; VAR done : BOOLEAN);
          VAR
              title : ARRAY [0..15] OF CHAR;
          
        BEGIN (* DetermineOutputFileName *)
          Strings.Assign ("CHARDATA.DAT",result);
          
          WinFiles.GetSaveFileName (window,"Moria Character Data Output File","","","",Win31.OFN_HIDEREADONLY + Win31.OFN_NOREADONLYRETURN + Win31.OFN_OVERWRITEPROMPT,result,title,done);
        END DetermineOutputFileName;
        
        PROCEDURE OutputCharacterInformation (name : ARRAY OF CHAR; VAR info : MCStruct.CharacterResourceData);
          VAR
              f : FileSystem.File;
          
        BEGIN (* OutputCharacterInformation *)
          WinFiles.CreateFile (f,name);
          
          IF f.res <> FileSystem.done THEN
            WDebugging.FatalAppExit ("Unable to create Moria character resource data file.");
          END; (* IF *)
          
          WinFiles.WriteDataStructure (f,info);
          
          IF f.res <> FileSystem.done THEN
            WDebugging.FatalAppExit ("Error while writing Moria character resource data file.");
          END; (* IF *)
          
          WinFiles.CloseFile (f);
          
          IF f.res <> FileSystem.done THEN
            WDebugging.FatalAppExit ("Unable to close Moria character resource data file.");
          END; (* IF *)
        END OutputCharacterInformation;
        
      BEGIN (* ExecuteCreateDataCommand *)
        DetermineOutputFileName (window,output,done);
        
        IF done THEN
          WCursor.SetCursor (WCursor.StandardCursorHandle (Windows.IDC_WAIT),previous);
          
          NEW (info);
          
          CDInit.DetermineCharacterInformation (WinStart.hInstance,info^);
          OutputCharacterInformation (output,info^);
          
          DISPOSE (info);
          
          WCursor.SetCursor (previous,previous);
        END; (* IF *)
      END ExecuteCreateDataCommand;
      
      PROCEDURE ExecuteExitCommand (window : Windows.HWND);
        VAR
            done : BOOLEAN;
        
      BEGIN (* ExecuteExitCommand *)
        WCreation.DestroyWindow (window,done);
      END ExecuteExitCommand;
      
    BEGIN (* ExecuteCharDataCommand *)
      CASE command OF
        CreateCommandControl : ExecuteCreateDataCommand (window); |
        ExitCommandControl   : ExecuteExitCommand (window);       |
      ELSE
        WDebugging.FatalAppExit ("Unrecognized main menu command control.");
      END; (* CASE *)
    END ExecuteCharDataCommand;
    
  BEGIN (* MainWindowTranslator *)
    result := Windows.FALSE_BOOL;
    
    CASE message OF
      Windows.WM_COMMAND : ExecuteCharDataCommand (window,param1); |
    ELSE
      result := WCreation.DefWindowProc (window,message,param1,param2);
    END; (* CASE *)
    
    RETURN (result);
  END MainWindowTranslator;
  
  
BEGIN (* CharData *)
  MainWindowHandle := WInformation.FindWindow (MainWindowClassName,MainWindowTitle);
  
  IF MainWindowHandle = Windows.NULL THEN
    AllocateMainWindowClassData (WinStart.hInstance);
    AllocateMainWindowHandle (MainWindowHandle);
    
    WCLoop.ExecuteCommandLoop (MainWindowHandle,Windows.NULL);
  ELSE
    WDisplay.RedisplayExistingInstance (MainWindowHandle);
  END; (* IF *)
END CharData.
