IMPLEMENTATION MODULE CSpells;
  (*/WIN:A*)
  
  IMPORT CSpells2,MCreatures,MMWindow,MRandom,MSight;
  IMPORT CStruct,DStruct,MStruct;
  IMPORT WDebugging,WString;
  IMPORT MSActs2;
  
  
  VAR
      ExecuteCreatureSpellAction : ARRAY CStruct.CreatureSpellMnemonic OF PROCEDURE (MStruct.PlayerInfoPointer,VAR DStruct.DungeonCreatureRecord);
  
  
  PROCEDURE AddCreatureSpellMessageToList (player : MStruct.PlayerInfoPointer; text : ARRAY OF CHAR; creature : DStruct.DungeonCreatureRecord);
    VAR
        message : ARRAY [0..127] OF CHAR;
    
  BEGIN (* AddCreatureSpellMessageToList *)
    IF creature.visible THEN
      WString.ReplaceStringParameter (text,"%CREATURE",player^.creatures^.table [creature.cursor].description,message);
    ELSE
      WString.ReplaceStringParameter (text,"The %CREATURE","It",message);
    END; (* IF *)
    
    MMWindow.AddMessageToList (player,message);
  END AddCreatureSpellMessageToList;
  
  PROCEDURE CastCreatureSpell (player : MStruct.PlayerInfoPointer; VAR creature : DStruct.DungeonCreatureRecord);
    VAR
        spell : CStruct.CreatureSpellMnemonic;
    
    PROCEDURE RandomSpellForCreature (spells : CStruct.CreatureSpellSet) : CStruct.CreatureSpellMnemonic;
      CONST
            MaxNumberSpells = ORD (MAX (CStruct.CreatureSpellMnemonic)) + 1;
      
      VAR
          number : CARDINAL;
          table  : ARRAY [0..MaxNumberSpells - 1] OF CStruct.CreatureSpellMnemonic;
      
      PROCEDURE DetermineCreatureSpellTable (spells : CStruct.CreatureSpellSet; VAR table : ARRAY OF CStruct.CreatureSpellMnemonic; VAR number : CARDINAL);
        VAR
            index : CStruct.CreatureSpellMnemonic;
        
      BEGIN (* DetermineCreatureSpellTable *)
        number := 0;
        
        FOR index := MIN (CStruct.CreatureSpellMnemonic) TO MAX (CStruct.CreatureSpellMnemonic) DO
          IF index IN spells THEN
            table [number] := index;
            number         := number + 1;
          END; (* IF *)
        END; (* FOR *)
      END DetermineCreatureSpellTable;
      
    BEGIN (* RandomSpellForCreature *)
      DetermineCreatureSpellTable (spells,table,number);
      
      RETURN (table [MRandom.CardinalRandomRangeValue (0,number - 1)]);
    END RandomSpellForCreature;
    
    PROCEDURE UpdateCreatureSpellMemory (player : MStruct.PlayerInfoPointer; VAR creature : CStruct.PassiveCreatureInfoRecord; spell : CStruct.CreatureSpellMnemonic);
    BEGIN (* UpdateCreatureSpellMemory *)
      IF ((player^.character^.variables.hpCUR <= 0.0) AND (creature.memory.deaths < CStruct.MaxRememberedDeaths)) THEN
        INC (creature.memory.deaths);
      END; (* IF *)
      
      INCL (creature.memory.spells,spell);
    END UpdateCreatureSpellMemory;
    
  BEGIN (* CastCreatureSpell *)
    spell := RandomSpellForCreature (player^.creatures^.table [creature.cursor].spells.known);
    
    MCreatures.UpdateCreatureDisplayInformation (player,creature,TRUE);
    
    IF ((spell > CStruct.TeleportLong) AND (spell <> CStruct.DrainMana)) THEN
      MSActs2.DisturbPlayer (player,TRUE);
    END; (* IF *)
    
    IF (((spell > CStruct.TeleportLong) AND (spell < CStruct.SummonMonster)) OR (spell = CStruct.SlowPlayer)) THEN
      AddCreatureSpellMessageToList (player,"The %CREATURE casts a spell.",creature);
    END; (* IF *)
    
    ExecuteCreatureSpellAction [spell] (player,creature);
    
    IF creature.visible THEN
      UpdateCreatureSpellMemory (player,player^.creatures^.table [creature.cursor],spell);
    END; (* IF *)
  END CastCreatureSpell;

  PROCEDURE CreatureShouldCastSpell (player : MStruct.PlayerInfoPointer; creature : DStruct.DungeonCreatureRecord) : BOOLEAN;
    CONST
          MaxCreatureSpellDistance = 20;
    
  BEGIN (* CreatureShouldCastSpell *)
    RETURN ((player^.creatures^.table [creature.cursor].spells.chance > 0) AND (creature.distance <= MaxCreatureSpellDistance) AND (MRandom.CardinalRandomRangeValue (1,player^.creatures^.table [creature.cursor].spells.chance) = 1) AND MSight.ClearLineOfSight (player,player^.character^.line,player^.character^.column,creature.line,creature.column));
  END CreatureShouldCastSpell;
  
  PROCEDURE InitializeExecuteCreatureSpellActionTable;
    VAR
        index : CStruct.CreatureSpellMnemonic;
    
  BEGIN (* InitializeExecuteCreatureSpellActionTable *)
    FOR index := MIN (CStruct.CreatureSpellMnemonic) TO MAX (CStruct.CreatureSpellMnemonic) DO
      ExecuteCreatureSpellAction [index] := InvalidCreatureSpellAction;
    END; (* FOR *)
    
    ExecuteCreatureSpellAction [CStruct.BreathAcid] := CSpells2.ExecuteBreathAcidSpell;
    ExecuteCreatureSpellAction [CStruct.BreathFire] := CSpells2.ExecuteBreathFireSpell;
    ExecuteCreatureSpellAction [CStruct.BreathFrost] := CSpells2.ExecuteBreathFrostSpell;
    ExecuteCreatureSpellAction [CStruct.BreathGas] := CSpells2.ExecuteBreathGasSpell;
    ExecuteCreatureSpellAction [CStruct.BreathLightning] := CSpells2.ExecuteBreathLightningSpell;
    ExecuteCreatureSpellAction [CStruct.CauseBlindness] := CSpells2.ExecuteCauseBlindnessSpell;
    ExecuteCreatureSpellAction [CStruct.CauseConfusion] := CSpells2.ExecuteCauseConfusionSpell;
    ExecuteCreatureSpellAction [CStruct.CauseFear] := CSpells2.ExecuteCauseFearSpell;
    ExecuteCreatureSpellAction [CStruct.CauseLightWounds] := CSpells2.ExecuteCauseLightWoundsSpell;
    ExecuteCreatureSpellAction [CStruct.CauseParalysis] := CSpells2.ExecuteCauseParalysisSpell;
    ExecuteCreatureSpellAction [CStruct.CauseSeriousWounds] := CSpells2.ExecuteCauseSeriousWoundsSpell;
    ExecuteCreatureSpellAction [CStruct.DrainMana] := CSpells2.ExecuteDrainManaSpell;
    ExecuteCreatureSpellAction [CStruct.SlowPlayer] := CSpells2.ExecuteSlowPlayerSpell;
    ExecuteCreatureSpellAction [CStruct.SummonMonster] := CSpells2.ExecuteSummonMonsterSpell;
    ExecuteCreatureSpellAction [CStruct.SummonPlayer] := CSpells2.ExecuteSummonPlayerSpell;
    ExecuteCreatureSpellAction [CStruct.SummonUndead] := CSpells2.ExecuteSummonUndeadSpell;
    ExecuteCreatureSpellAction [CStruct.TeleportLong] := CSpells2.ExecuteTeleportLongSpell;
    ExecuteCreatureSpellAction [CStruct.TeleportShort] := CSpells2.ExecuteTeleportShortSpell;
  END InitializeExecuteCreatureSpellActionTable;
  
  PROCEDURE InvalidCreatureSpellAction (player : MStruct.PlayerInfoPointer; VAR creature : DStruct.DungeonCreatureRecord);
  BEGIN (* InvalidCreatureSpellAction *)
    WDebugging.FatalAppExit ("Call to un-initialized creature spell.");
  END InvalidCreatureSpellAction;

BEGIN (* CSpells *)
  InitializeExecuteCreatureSpellActionTable;
END CSpells.
