IMPLEMENTATION MODULE DSActions;
  (*/WIN:A*)
  
  FROM WMemory IMPORT ALLOCATE,DEALLOCATE;
  
  IMPORT CStruct,DStruct,MRandom,MStruct,OStruct;
  IMPORT WDebugging,WMacro;
  
  
  PROCEDURE AdjacentCorridorCount (VAR dungeon : DStruct.DungeonInformation; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange) : CARDINAL;
    VAR
        count : CARDINAL;
        x     : DStruct.DungeonColumnRange;
        y     : DStruct.DungeonLineRange;
    
    PROCEDURE ValidCorridorLocation (VAR dungeon : DStruct.DungeonInformation; location : DStruct.PhysicalDungeonUnit) : BOOLEAN;
    BEGIN (* ValidCorridorLocation *)
      RETURN ((DStruct.CorridorFloor IN location.elements) AND ((location.artifact = DStruct.InvalidArtifactCursor) OR (dungeon.artifacts [location.artifact].object.class < OStruct.OpenDoor)));
    END ValidCorridorLocation;
    
  BEGIN (* AdjacentCorridorCount *)
    count := 0;
    
    FOR y := line - 1 TO line + 1 DO
      FOR x := column - 1 TO column + 1 DO
        IF ValidCorridorLocation (dungeon,dungeon.physical [y]^ [x]) THEN
          INC (count);
        END; (* IF *)
      END; (* FOR *)
    END; (* FOR *)
    
    RETURN (count);
  END AdjacentCorridorCount;
  
  PROCEDURE AdjacentWallCount (VAR dungeon : DStruct.DungeonInformation; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange) : CARDINAL;
    VAR
        count : CARDINAL;
    
  BEGIN (* AdjacentWallCount *)
    count := 0;
    
    IF dungeon.physical [line]^ [column - 1].elements * DStruct.DungeonWallElements <> DStruct.EmptyDungeonElementSet THEN
      INC (count);
    END; (* IF *)
    
    IF dungeon.physical [line]^ [column + 1].elements * DStruct.DungeonWallElements <> DStruct.EmptyDungeonElementSet THEN
      INC (count);
    END; (* IF *)
    
    IF dungeon.physical [line - 1]^ [column].elements * DStruct.DungeonWallElements <> DStruct.EmptyDungeonElementSet THEN
      INC (count);
    END; (* IF *)
    
    IF dungeon.physical [line + 1]^ [column].elements * DStruct.DungeonWallElements <> DStruct.EmptyDungeonElementSet THEN
      INC (count);
    END; (* IF *)
    
    RETURN (count);
  END AdjacentWallCount;
  
  PROCEDURE AllocateDungeonInformation (VAR ptr : DStruct.DungeonInfoPointer);
    VAR
        index : CARDINAL;
    
  BEGIN (* AllocateDungeonInformation *)
    NEW (ptr);
    
    FOR index := 0 TO DStruct.NumberDungeonLines - 1 DO
      NEW (ptr^.physical [index]);
    END; (* FOR *)
  END AllocateDungeonInformation;
  
  PROCEDURE DeallocateDungeonInformation (VAR ptr : DStruct.DungeonInfoPointer);
    VAR
        index : CARDINAL;
    
  BEGIN (* DeallocateDungeonInformation *)
    FOR index := 0 TO DStruct.NumberDungeonLines - 1 DO
      DISPOSE (ptr^.physical [index]);
    END; (* FOR *)
    
    DISPOSE (ptr);
  END DeallocateDungeonInformation;
  
  PROCEDURE DeleteArtifactInformation (VAR dungeon : DStruct.DungeonInformation; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange);
  BEGIN (* DeleteArtifactInformation *)
    WITH dungeon.physical [line]^ [column] DO
      IF DStruct.BlockedFloor IN elements THEN
        elements := elements / DStruct.DungeonElementSet {DStruct.CorridorFloor,DStruct.BlockedFloor};
      END; (* IF *)
      
      dungeon.artifacts [artifact].cursor := CStruct.InvalidCreatureCursor;
      artifact                            := DStruct.InvalidArtifactCursor;
      
      EXCL (elements,DStruct.fmFLAG);
    END; (* WITH *)
  END DeleteArtifactInformation;
  
  PROCEDURE DeleteCreatureAtLocation (VAR dungeon : DStruct.DungeonInformation; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange);
  BEGIN (* DeleteCreatureAtLocation *)
    dungeon.creatures [dungeon.physical [line]^ [column].creature].cursor := CStruct.InvalidCreatureCursor;
    dungeon.physical [line]^ [column].creature                            := DStruct.InvalidCreatureCursor;
  END DeleteCreatureAtLocation;
  
  PROCEDURE DetermineDungeonLocation (line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange; direction : CARDINAL; VAR resultLINE : DStruct.DungeonLineRange; VAR resultCOLUMN : DStruct.DungeonColumnRange);
  BEGIN (* DetermineDungeonLocation *)
    CASE direction OF
      1 : resultCOLUMN := column - 1; 
          resultLINE   := line + 1;   |
      2 : resultCOLUMN := column;     
          resultLINE   := line + 1;   |
      3 : resultCOLUMN := column + 1; 
          resultLINE   := line + 1;   |
      4 : resultCOLUMN := column - 1; 
          resultLINE   := line;       |
      5 : resultCOLUMN := column;     
          resultLINE   := line;       |
      6 : resultCOLUMN := column + 1; 
          resultLINE   := line;       |
      7 : resultCOLUMN := column - 1; 
          resultLINE   := line - 1;   |
      8 : resultCOLUMN := column;     
          resultLINE   := line - 1;   |
      9 : resultCOLUMN := column + 1; 
          resultLINE   := line - 1;   |
    ELSE
      WDebugging.FatalAppExit ("Invalid direction passed to DSActions.DetermineDungeonLocation.");
    END; (* CASE *)
  END DetermineDungeonLocation;
  
  PROCEDURE InitializeDungeonTown (VAR dungeon : DStruct.DungeonInformation);
  BEGIN (* InitializeDungeonTown *)
    dungeon.town := MRandom.RandomSeedValue ();
  END InitializeDungeonTown;
  
  PROCEDURE LocationHasLight (VAR dungeon : DStruct.DungeonInformation; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange) : BOOLEAN;
  BEGIN (* LocationHasLight *)
    RETURN (dungeon.physical [line]^ [column].elements * DStruct.DungeonElementSet {DStruct.plFLAG,DStruct.tlFLAG,DStruct.fmFLAG} <> DStruct.EmptyDungeonElementSet);
  END LocationHasLight;
  
  PROCEDURE LocationInBounds (VAR dungeon : DStruct.DungeonInformation; line : INTEGER; column : INTEGER) : BOOLEAN;
  BEGIN (* LocationInBounds *)
    RETURN ((line > 0) AND (line < dungeon.lines - 1) AND (column > 0) AND (column < dungeon.columns - 1));
  END LocationInBounds;

  PROCEDURE PlaceCreatureAtLocation (player : MStruct.PlayerInfoPointer; VAR dungeon : DStruct.DungeonInformation; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange; creature : DStruct.DungeonCreatureRecord; VAR cursor : CARDINAL; VAR done : BOOLEAN);
    PROCEDURE FirstAvailableCreatureLocation (VAR table : ARRAY OF DStruct.DungeonCreatureRecord) : CARDINAL;
      VAR
          index : CARDINAL;
      
    BEGIN (* FirstAvailableCreatureLocation *)
      FOR index := 0 TO HIGH (table) DO
        IF table [index].cursor = CStruct.InvalidCreatureCursor THEN
          RETURN (index);
        END; (* IF *)
      END; (* FOR *)
      
      RETURN (MAX (CARDINAL));
    END FirstAvailableCreatureLocation;
    
  BEGIN (* PlaceCreatureAtLocation *)
    cursor := FirstAvailableCreatureLocation (dungeon.creatures);
    done   := (cursor < DStruct.MaxDungeonCreatures);
    
    IF done THEN
      creature.distance := QuickDistance (player^.character^.line,player^.character^.column,line,column);
      creature.column   := column;
      creature.line     := line;
      
      dungeon.physical [line]^ [column].creature := cursor;
      dungeon.creatures [cursor]                 := creature;
    END; (* IF *)
  END PlaceCreatureAtLocation;
  
  PROCEDURE PlaceDungeonArtifact (VAR dungeon : DStruct.DungeonInformation; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange; artifact : OStruct.InventoryItemRecord);
    VAR
        location : CARDINAL;
    
    PROCEDURE FirstEmptyArtifactLocation (VAR table : ARRAY OF OStruct.InventoryItemRecord) : CARDINAL;
      VAR
          index : CARDINAL;
      
    BEGIN (* FirstEmptyArtifactLocation *)
      FOR index := 0 TO HIGH (table) DO
        IF table [index].object.class = OStruct.Nothing THEN
          RETURN (index);
        END; (* IF *)
      END; (* FOR *)
      
      WDebugging.FatalAppExit ("Dungeon artifact table overflow.");
    END FirstEmptyArtifactLocation;
    
  BEGIN (* PlaceDungeonArtifact *)
    location := FirstEmptyArtifactLocation (dungeon.artifacts);
    
    dungeon.physical [line]^ [column].artifact := location;
    dungeon.artifacts [location]               := artifact;
  END PlaceDungeonArtifact;
  
  PROCEDURE PlayerHasLight (player : MStruct.PlayerInfoPointer) : BOOLEAN;
  BEGIN (* PlayerHasLight *)
    RETURN (player^.dungeon^.physical [player^.character^.line]^ [player^.character^.column].elements * DStruct.DungeonElementSet {DStruct.tlFLAG,DStruct.plFLAG} <> DStruct.EmptyDungeonElementSet);
  END PlayerHasLight;
  
  PROCEDURE QuickDistance (line1 : DStruct.DungeonLineRange; column1 : DStruct.DungeonColumnRange; line2 : DStruct.DungeonLineRange; column2 : DStruct.DungeonColumnRange) : CARDINAL;
    VAR
        deltaX,deltaY : CARDINAL;
    
  BEGIN (* QuickDistance *)
    deltaX := ABS (column1 - column2);
    deltaY := ABS (line1 - line2);
    
    RETURN ((2 * (deltaX + deltaY) - WMacro.MinCardinal (deltaX,deltaY)) DIV 2);
  END QuickDistance;

  PROCEDURE SetDungeonLocationFVal (VAR location : DStruct.PhysicalDungeonUnit; value : DStruct.DungeonElementMnemonic);
  BEGIN (* SetDungeonLocationFVal *)
    location.elements := location.elements - DStruct.DungeonFValElements;
    INCL (location.elements,value);
  END SetDungeonLocationFVal;
END DSActions.
