IMPLEMENTATION MODULE MBolts;
  (*/WIN:A*)
  
  IMPORT DLineDDA,MPAttk2,MCreatures,MDWindow,MMWindow,MPAttacks;
  IMPORT CStruct,DStruct,MStruct;
  IMPORT WString;
  
  
  VAR
      ExecuteMissileDamage : MissileDamageProcedure;
      MissileTerminated    : BOOLEAN;
      MissileDistance      : CARDINAL;
  
  
  PROCEDURE DamageCreatureWithMagicBolt (player : MStruct.PlayerInfoPointer; VAR creature : DStruct.DungeonCreatureRecord; VAR info : CStruct.PassiveCreatureInfoRecord; damage : CARDINAL; defense : CStruct.CreatureDefenseInformation; spells : CStruct.CreatureSpellSet; spell : ARRAY OF CHAR);
    PROCEDURE DisplayStrikeMessage (player : MStruct.PlayerInfoPointer; creature : ARRAY OF CHAR; visible : BOOLEAN; spell : ARRAY OF CHAR);
      VAR
          message : ARRAY [0..127] OF CHAR;
      
    BEGIN (* DisplayStrikeMessage *)
      WString.ReplaceStringParameter ("The %SPELL strikes the %CREATURE.","%SPELL",spell,message);
      
      IF visible THEN
        WString.ReplaceStringParameter (message,"%CREATURE",creature,message);
      ELSE
        WString.ReplaceStringParameter (message,"the %CREATURE","it",message);
      END; (* IF *)
      
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,message);
    END DisplayStrikeMessage;
    
    PROCEDURE UpdateCreatureVisibility (player : MStruct.PlayerInfoPointer; VAR creature : DStruct.DungeonCreatureRecord; VAR location : DStruct.PhysicalDungeonUnit);
      VAR
          elements : DStruct.DungeonElementSet;
      
    BEGIN (* UpdateCreatureVisibility *)
      elements          := location.elements;
      location.elements := location.elements + DStruct.DungeonElementSet {DStruct.plFLAG};
      
      MCreatures.UpdateCreatureDisplayInformation (player,creature,FALSE);
      
      location.elements := elements;
    END UpdateCreatureVisibility;
    
  BEGIN (* DamageCreatureWithMagicBolt *)
    UpdateCreatureVisibility (player,creature,player^.dungeon^.physical [creature.line]^ [creature.column]);
    DisplayStrikeMessage (player,info.description,creature.visible,spell);
    
    IF defense * info.defense <> CStruct.CreatureDefenseInformation {} THEN
      IF creature.visible THEN
        info.defense := info.defense + defense;
      END; (* IF *)
      
      damage := damage * 2;
    ELSIF spells * info.spells.known <> CStruct.CreatureSpellSet {} THEN
      IF creature.visible THEN
        info.memory.spells := info.memory.spells + spells;
      END; (* IF *)
      
      damage := damage DIV 4;
    END; (* IF *)
    
    IF damage > 0 THEN
      MPAttk2.DamageCreature (creature,damage);
      
      IF creature.hitpoints = 0 THEN
        MPAttk2.KillCreatureAtLocation (player,creature.line,creature.column,info,creature.visible,TRUE);
        MMWindow.DisplayCreatureMessage (player,"The %CREATURE dies in a fit of agony.",info.description,creature.visible,TRUE);
      ELSE
        MMWindow.DisplayCreatureMessage (player,"The %CREATURE screams in agony.",info.description,creature.visible,TRUE);
      END; (* IF *)
    END; (* IF *)
  END DamageCreatureWithMagicBolt;
  
  PROCEDURE PlotMagicMissileLocation (player : MStruct.PlayerInfoPointer; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange; VAR result : BOOLEAN);
    CONST
          MaximumMissileRange = 18;
    
    PROCEDURE LocationStopsMissile (location : DStruct.PhysicalDungeonUnit) : BOOLEAN;
    BEGIN (* LocationStopsMissile *)
      RETURN ((location.creature <> DStruct.InvalidCreatureCursor) OR (location.elements * (DStruct.DungeonWallElements + DStruct.DungeonElementSet {DStruct.BlockedFloor}) <> DStruct.EmptyDungeonElementSet));
    END LocationStopsMissile;
    
  BEGIN (* PlotMagicMissileLocation *)
    IF NOT (MissileTerminated) THEN
      MissileDistance   := MissileDistance + 1;
      MissileTerminated := ((MissileDistance > MaximumMissileRange) OR LocationStopsMissile (player^.dungeon^.physical [line]^ [column]));
      
      IF MissileTerminated THEN
        ExecuteMissileDamage (player,line,column,result);
      ELSE
        MDWindow.DisplayPassingMissile (player,line,column,'*',TRUE);
      END; (* IF *)
      
      PreviousBoltColumn := column;
      PreviousBoltLine   := line;
    END; (* IF *)
  END PlotMagicMissileLocation;
  
  PROCEDURE TraceMagicBoltPath (player : MStruct.PlayerInfoPointer; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange; StrikeProcedure : MissileDamageProcedure; VAR result : BOOLEAN);
  BEGIN (* TraceMagicBoltPath *)
    ExecuteMissileDamage := StrikeProcedure;
    MissileTerminated    := FALSE;
    MissileDistance      := 0;
    
    PreviousBoltColumn := column;
    PreviousBoltLine   := line;
    
    DLineDDA.ExecuteDungeonTraceProcedure (player,player^.character^.line,player^.character^.column,line,column,PlotMagicMissileLocation,result);
    
    IF NOT (MissileTerminated) THEN
      PlotMagicMissileLocation (player,line,column,result);
    END; (* IF *)
  END TraceMagicBoltPath;
END MBolts.
