MODULE MCData;
  (*/WIN:A*)
  
  FROM WDebugging IMPORT CallingError;
  FROM WMemory    IMPORT ALLOCATE,DEALLOCATE;
  
  IMPORT WCLoop,WCreation,WCursor,WDebugging,WDisplay,WError,Win31,Windows,WinFiles,WInformation,WinStart,WResource;
  IMPORT FileSystem,SYSTEM;
  IMPORT CStruct,MParser;
  
  
  CONST
        MainWindowClassName = "MCDATA_Main_Window_Class";
        MainWindowTitle     = "MCDATA";
        ModuleName          = "MCData";
  
  VAR
      MainWindowClassData : Windows.WNDCLASS;
      MainWindowClassAtom : Windows.ATOM;
      MainWindowHandle    : Windows.HWND;
  
  
  PROCEDURE AllocateMainWindow (VAR window : Windows.HWND);
    CONST
          MainWindowStyle = Windows.WS_OVERLAPPEDWINDOW;
    
    VAR
        shown : BOOLEAN;
    
  BEGIN (* AllocateMainWindow *)
    WCreation.CreateDefaultWindow (MainWindowClassName,MainWindowTitle,MainWindowStyle,Windows.NULL,WinStart.hInstance,window);
    WDisplay.ShowWindow (window,WinStart.cmdShow,shown);
  END AllocateMainWindow;
  
  PROCEDURE AllocateMainWindowClass (instance : Windows.HINSTANCE);
  BEGIN (* AllocateMainWindowClass *)
    WITH MainWindowClassData DO
      hbrBackground := Windows.COLOR_WINDOW + 1;
      lpszClassName := SYSTEM.ADR (MainWindowClassName);
      lpszMenuName  := SYSTEM.ADR ("Main_Menu");
      lpfnWndProc   := MainWindowTranslator;
      cbClsExtra    := 0;
      cbWndExtra    := 0;
      hInstance     := instance;
      hCursor       := WCursor.StandardCursorHandle (Windows.IDC_ARROW);
      hIcon         := Windows.NULL;
      style         := Windows.CS_HREDRAW + Windows.CS_VREDRAW;
    END; (* WITH *)
    
    WCreation.RegisterClass (MainWindowClassData,MainWindowClassAtom);
  END AllocateMainWindowClass;
  
  PROCEDURE MainWindowTranslator ['MAINWINDOWTRANSLATOR'] (window : Windows.HWND; message : CARDINAL; param1 : Windows.WPARAM; param2 : Windows.LPARAM) : Windows.LRESULT [WINDOWS,LONG];
    VAR
        result : Windows.LRESULT;
    
    PROCEDURE ExecuteMainWindowCommand (window : Windows.HWND; command : CARDINAL);
      CONST
            CreateCommandControl  = 100;
            ExamineCommandControl = 200;
            ExitCommandControl    = 300;
      
      PROCEDURE ExecuteCreateDataCommand (window : Windows.HWND);
        VAR
            creatures : CStruct.CreatureInfoPointer;
            previous  : Windows.HCURSOR;
            output    : ARRAY [0..127] OF CHAR;
            done      : BOOLEAN;
        
        PROCEDURE DetermineOutputFileName (window : Windows.HWND; VAR name : ARRAY OF CHAR; VAR done : BOOLEAN);
          VAR
              title : ARRAY [0..15] OF CHAR;
          
        BEGIN (* DetermineOutputFileName *)
          name := "CREATURE.DAT";
          
          WinFiles.GetSaveFileName (window,"Creature Output File Name","","","DAT",Win31.OFN_HIDEREADONLY + Win31.OFN_NOREADONLYRETURN + Win31.OFN_OVERWRITEPROMPT,name,title,done);
        END DetermineOutputFileName;
        
        PROCEDURE InitializeCreatureInformation (VAR creatures : CStruct.CreatureInformation);
          PROCEDURE ComputeCreatureDepthTable (VAR table : ARRAY OF CStruct.PassiveCreatureInfoRecord; VAR result : ARRAY OF CARDINAL);
            PROCEDURE ClearCreatureDepthTable (VAR table : ARRAY OF CARDINAL);
              VAR
                  index : CARDINAL;
              
            BEGIN (* ClearCreatureDepthTable *)
              FOR index := 0 TO HIGH (table) DO
                table [index] := 0;
              END; (* FOR *)
            END ClearCreatureDepthTable;
            
            PROCEDURE ComputeDepthIndicies (VAR table : ARRAY OF CARDINAL);
              VAR
                  index : CARDINAL;
              
            BEGIN (* ComputeDepthIndicies *)
              FOR index := 1 TO HIGH (table) DO
                table [index] := table [index] + table [index - 1];
              END; (* FOR *)
            END ComputeDepthIndicies;
            
            PROCEDURE SumCreaturesPerDepth (VAR table : ARRAY OF CStruct.PassiveCreatureInfoRecord; VAR result : ARRAY OF CARDINAL);
              VAR
                  index : CARDINAL;
              
            BEGIN (* SumCreaturesPerDepth *)
              FOR index := 0 TO HIGH (table) DO
                IF table [index].depth <= HIGH (result) THEN
                  INC (result [table [index].depth]);
                END; (* IF *)
              END; (* FOR *)
            END SumCreaturesPerDepth;
            
          BEGIN (* ComputeCreatureDepthTable *)
            ClearCreatureDepthTable (result);
            
            SumCreaturesPerDepth (table,result);
            ComputeDepthIndicies (result);
          END ComputeCreatureDepthTable;
          
          PROCEDURE ParseCreatureAttackRecords (VAR table : ARRAY OF CStruct.CreatureAttackRecord);
            CONST
                  CreatureAttackStringBase = 500;
            
            VAR
                index : CARDINAL;
            
            PROCEDURE ParseCreatureAttackRecord (instance : Windows.HINSTANCE; cursor : CARDINAL; VAR result : CStruct.CreatureAttackRecord);
              VAR
                  length : CARDINAL;
                  str    : ARRAY [0..127] OF CHAR;
              
              PROCEDURE ParseAttackClassMnemonic (VAR str : ARRAY OF CHAR; term : CHAR; VAR result : CStruct.CreatureAttackClass);
                VAR
                    value : CARDINAL;
                
              BEGIN (* ParseAttackClassMnemonic *)
                MParser.ParseCardinalValue (str,term,value);
                
                IF value > ORD (MAX (CStruct.CreatureAttackClass)) THEN
                  WDebugging.FatalAppExit ("Invalid creature attack class while parsing Moria data.");
                END; (* IF *)
                
                result := VAL (CStruct.CreatureAttackClass,value);
              END ParseAttackClassMnemonic;
              
              PROCEDURE ParseAttackDescriptionMnemonic (VAR str : ARRAY OF CHAR; term : CHAR; VAR result : CStruct.CreatureAttackDescription);
                VAR
                    value : CARDINAL;
                
              BEGIN (* ParseAttackDescriptionMnemonic *)
                MParser.ParseCardinalValue (str,term,value);
                
                IF value > ORD (MAX (CStruct.CreatureAttackDescription)) THEN
                  WDebugging.FatalAppExit ("Invalid creature attack description while parsing Moria data.");
                END; (* IF *)
                
                result := VAL (CStruct.CreatureAttackDescription,value);
              END ParseAttackDescriptionMnemonic;
              
            BEGIN (* ParseCreatureAttackRecord *)
              WResource.LoadString (instance,cursor,str,length);
              
              WITH result DO
                ParseAttackClassMnemonic (str,MParser.StandardItemTerminator,class);
                ParseAttackDescriptionMnemonic (str,MParser.StandardItemTerminator,description);
                MParser.ParseRandomRangeSpecifier (str,MParser.FinalItemTerminator,FALSE,damage);
              END; (* WITH *)
            END ParseCreatureAttackRecord;
            
          BEGIN (* ParseCreatureAttackRecords *)
            FOR index := 0 TO HIGH (table) DO
              ParseCreatureAttackRecord (WinStart.hInstance,CreatureAttackStringBase + index,table [index]);
            END; (* FOR *)
          END ParseCreatureAttackRecords;
          
          PROCEDURE ParsePassiveCreatureInfoRecords (VAR table : ARRAY OF CStruct.PassiveCreatureInfoRecord);
            CONST
                  CreatureStringBase = 100;
            
            VAR
                index : CARDINAL;
            
            PROCEDURE ParsePassiveCreatureInfoRecord (instance : Windows.HINSTANCE; cursor : CARDINAL; VAR result : CStruct.PassiveCreatureInfoRecord);
              VAR
                  length : CARDINAL;
                  str    : ARRAY [0..255] OF CHAR;
              
              PROCEDURE ParseCreatureAttackList (VAR str : ARRAY OF CHAR; term : CHAR; VAR result : ARRAY OF CStruct.CreatureAttackCursor);
                VAR
                    index : CARDINAL;
                
              BEGIN (* ParseCreatureAttackList *)
                MParser.MatchCharacter (str,MParser.OpenListBracket);
                
                FOR index := 0 TO CStruct.MaxAttacksPerCreature - 1 DO
                  IF index < CStruct.MaxAttacksPerCreature - 1 THEN
                    MParser.ParseCardinalValue (str,MParser.StandardItemTerminator,result [index]);
                  ELSE
                    MParser.ParseCardinalValue (str,MParser.CloseListBracket,result [index]);
                  END; (* IF *)
                END; (* FOR *)
                
                MParser.MatchCharacter (str,term);
              END ParseCreatureAttackList;
              
              PROCEDURE ParseCreatureCMoveField (VAR str : ARRAY OF CHAR; term : CHAR; VAR movement : CStruct.CreatureMovementInformation; VAR treasure : CStruct.CreatureTreasureInformation; VAR balrog : BOOLEAN);
                VAR
                    value : LONGCARD;
                
                PROCEDURE DetermineMovementInformation (value : LONGCARD; VAR result : CStruct.CreatureMovementInformation);
                  PROCEDURE AddCreatureMovementMnemonics (value : LONGCARD; first,final : CStruct.CreatureMovementMnemonic; VAR result : CStruct.CreatureMovementInformation);
                    VAR
                        index : CStruct.CreatureMovementMnemonic;
                    
                  BEGIN (* AddCreatureMovementMnemonics *)
                    FOR index := first TO final DO
                      IF ODD (value) THEN
                        INCL (result,index);
                      END; (* IF *)
                      
                      value := value DIV 2;
                    END; (* FOR *)
                  END AddCreatureMovementMnemonics;
                  
                BEGIN (* DetermineMovementInformation *)
                  result := CStruct.CreatureMovementInformation {};
                  
                  AddCreatureMovementMnemonics (value DIV 10000H,CStruct.MoveInvisible,CStruct.MultiplyMonster,result);
                  AddCreatureMovementMnemonics (value DIV 8,CStruct.MoveRandom20,CStruct.MoveRandom75,result);
                  AddCreatureMovementMnemonics (value,CStruct.AttackOnly,CStruct.MagicOnly,result);
                END DetermineMovementInformation;
                
                PROCEDURE DetermineTreasureInformation (value : LONGCARD; VAR result : CStruct.CreatureTreasureInformation);
                  VAR
                      index : CStruct.CreatureTreasureMnemonic;
                  
                BEGIN (* DetermineTreasureInformation *)
                  result := CStruct.CreatureTreasureInformation {};
                  
                  FOR index := MIN (CStruct.CreatureTreasureMnemonic) TO MAX (CStruct.CreatureTreasureMnemonic) DO
                    IF ODD (value) THEN
                      INCL (result,index);
                    END; (* IF *)
                    
                    value := value DIV 2;
                  END; (* FOR *)
                END DetermineTreasureInformation;
                
              BEGIN (* ParseCreatureCMoveField *)
                MParser.ParseHexLongCardValue (str,term,value);
                
                balrog := ODD (value DIV 80000000H);
                
                DetermineTreasureInformation ((value MOD 80000000H) DIV 1000000H,treasure);
                DetermineMovementInformation (value MOD 1000000H,movement);
              END ParseCreatureCMoveField;
              
              PROCEDURE ParseCreatureDefenseField (VAR str : ARRAY OF CHAR; term : CHAR; VAR result : CStruct.CreatureDefenseInformation);
                VAR
                    value : LONGCARD;
                
                PROCEDURE AddCreatureDefenseMnemonics (value : LONGCARD; first,final : CStruct.CreatureDefenseMnemonic; VAR result : CStruct.CreatureDefenseInformation);
                  VAR
                      index : CStruct.CreatureDefenseMnemonic;
                  
                BEGIN (* AddCreatureDefenseMnemonics *)
                  FOR index := first TO final DO
                    IF ODD (value) THEN
                      INCL (result,index);
                    END; (* IF *)
                    
                    value := value DIV 2;
                  END; (* FOR *)
                END AddCreatureDefenseMnemonics;
                
              BEGIN (* ParseCreatureDefenseField *)
                MParser.ParseHexLongCardValue (str,term,value);
                
                result := CStruct.CreatureDefenseInformation {};
                
                AddCreatureDefenseMnemonics (value DIV 1000H,CStruct.ResistCharms,CStruct.MaxHitPoints,result);
                AddCreatureDefenseMnemonics (value,CStruct.HurtBySlayDragon,CStruct.HurtByStoneToMud,result);
              END ParseCreatureDefenseField;
              
              PROCEDURE ParseCreatureSpellsField (VAR str : ARRAY OF CHAR; term : CHAR; VAR result : CStruct.CreatureSpellInformation);
                VAR
                    value : LONGCARD;
                
                PROCEDURE AddCreatureSpellMnemonics (value : LONGCARD; first,final : CStruct.CreatureSpellMnemonic; VAR result : CStruct.CreatureSpellSet);
                  VAR
                      index : CStruct.CreatureSpellMnemonic;
                  
                BEGIN (* AddCreatureSpellMnemonics *)
                  FOR index := first TO final DO
                    IF ODD (value) THEN
                      INCL (result,index);
                    END; (* IF *)
                    
                    value := value DIV 2;
                  END; (* FOR *)
                END AddCreatureSpellMnemonics;
                
              BEGIN (* ParseCreatureSpellsField *)
                MParser.ParseHexLongCardValue (str,term,value);
                
                result.chance := value MOD 10H;
                result.known  := CStruct.CreatureSpellSet {};
                
                AddCreatureSpellMnemonics (value DIV 80000H,CStruct.BreathLightning,CStruct.BreathFire,result.known);
                AddCreatureSpellMnemonics (value DIV 10H,CStruct.TeleportShort,CStruct.DrainMana,result.known);
              END ParseCreatureSpellsField;
              
            BEGIN (* ParsePassiveCreatureInfoRecord *)
              WResource.LoadString (instance,cursor,str,length);
              
              WITH result DO
                MParser.ParseLiteralString (str,MParser.StandardItemTerminator,description);
                ParseCreatureCMoveField (str,MParser.StandardItemTerminator,movement,treasure,balrog);
                ParseCreatureSpellsField (str,MParser.StandardItemTerminator,spells);
                ParseCreatureDefenseField (str,MParser.StandardItemTerminator,defense);
                MParser.ParseCardinalValue (str,MParser.StandardItemTerminator,experience);
                MParser.ParseCardinalValue (str,MParser.StandardItemTerminator,sleep);
                MParser.ParseCardinalValue (str,MParser.StandardItemTerminator,perception);
                MParser.ParseCardinalValue (str,MParser.StandardItemTerminator,armor);
                MParser.ParseIntegerValue (str,MParser.StandardItemTerminator,speed);
                MParser.ParseLiteralChar (str,MParser.StandardItemTerminator,symbol);
                MParser.ParseRandomRangeSpecifier (str,MParser.StandardItemTerminator,TRUE,hitdie);
                ParseCreatureAttackList (str,MParser.StandardItemTerminator,attacks);
                MParser.ParseCardinalValue (str,MParser.FinalItemTerminator,depth);
                
                memory.movement := CStruct.CreatureMovementInformation {};
                memory.treasure := CStruct.CreatureTreasureInformation {};
                memory.ignored  := 0;
                memory.awoken   := 0;
                memory.spells   := CStruct.CreatureSpellSet {};
                memory.kills    := 0;
              END; (* WITH *)
            END ParsePassiveCreatureInfoRecord;
            
          BEGIN (* ParsePassiveCreatureInfoRecords *)
            FOR index := 0 TO HIGH (table) DO
              ParsePassiveCreatureInfoRecord (WinStart.hInstance,CreatureStringBase + index,table [index]);
            END; (* FOR *)
          END ParsePassiveCreatureInfoRecords;
          
        BEGIN (* InitializeCreatureInformation *)
          ParsePassiveCreatureInfoRecords (creatures.table);
          ParseCreatureAttackRecords (creatures.attacks);
          
          ComputeCreatureDepthTable (creatures.table,creatures.depth);
        END InitializeCreatureInformation;
        
        PROCEDURE OutputCreatureData (name : ARRAY OF CHAR; VAR creatures : CStruct.CreatureInformation);
          VAR
              f : FileSystem.File;
          
        BEGIN (* OutputCreatureData *)
          WinFiles.CreateFile (f,name);
          
          IF f.res <> FileSystem.done THEN
            WDebugging.FatalAppExit ("Error while creating Moria creature data file.");
          END; (* IF *)
          
          WinFiles.WriteDataStructure (f,creatures);
          
          IF f.res <> FileSystem.done THEN
            WDebugging.FatalAppExit ("Error while writing Moria creature data file.");
          END; (* IF *)
          
          WinFiles.CloseFile (f);
          
          IF f.res <> FileSystem.done THEN
            WDebugging.FatalAppExit ("Error while closing Moria creature data file.");
          END; (* IF *)
        END OutputCreatureData;
        
      BEGIN (* ExecuteCreateDataCommand *)
        DetermineOutputFileName (window,output,done);
        
        IF done THEN
          WCursor.SetCursor (WCursor.StandardCursorHandle (Windows.IDC_WAIT),previous);
          
          NEW (creatures);
          
          InitializeCreatureInformation (creatures^);
          OutputCreatureData (output,creatures^);
          
          DISPOSE (creatures);
          
          WCursor.SetCursor (previous,previous);
        END; (* IF *)
      END ExecuteCreateDataCommand;
      
      PROCEDURE ExecuteExamineDataCommand (window : Windows.HWND);
      BEGIN (* ExecuteExamineDataCommand *)
        CallingError (ModuleName,'ExecuteExamineDataCommand');
      END ExecuteExamineDataCommand;
      
      PROCEDURE ExecuteExitCommand (window : Windows.HWND);
        VAR
            done : BOOLEAN;
        
      BEGIN (* ExecuteExitCommand *)
        WCreation.DestroyWindow (window,done);
      END ExecuteExitCommand;
      
    BEGIN (* ExecuteMainWindowCommand *)
      CASE command OF
        ExamineCommandControl : ExecuteExamineDataCommand (window); |
        CreateCommandControl  : ExecuteCreateDataCommand (window);  |
        ExitCommandControl    : ExecuteExitCommand (window);        |
      ELSE
        WError.DisplayInformationBox (window,MainWindowTitle,"Unrecognized main menu command control.");
      END; (* CASE *)
    END ExecuteMainWindowCommand;
    
  BEGIN (* MainWindowTranslator *)
    result := Windows.FALSE_BOOL;
    
    CASE message OF
      Windows.WM_COMMAND : ExecuteMainWindowCommand (window,param1); |
    ELSE
      result := WCreation.DefWindowProc (window,message,param1,param2);
    END; (* CASE *)
    
    RETURN (result);
  END MainWindowTranslator;
  
  
BEGIN (* MCData *)
  MainWindowHandle := WInformation.FindWindow (MainWindowClassName,MainWindowTitle);
  
  IF MainWindowHandle = Windows.NULL THEN
    AllocateMainWindowClass (WinStart.hInstance);
    AllocateMainWindow (MainWindowHandle);
    
    WCLoop.ExecuteCommandLoop (MainWindowHandle,Windows.NULL);
  ELSE
    WError.DisplayInformationBox (MainWindowHandle,MainWindowTitle,"Switching to existing instance of MCDATA.");
    WDisplay.RedisplayExistingInstance (MainWindowHandle);
  END; (* IF *)
END MCData.
