IMPLEMENTATION MODULE MCDialog;
  (*/WIN:A*)
  
  IMPORT MCDDisplay,MDialogs,MCRoll,MResources,MSWindow;
  IMPORT WComboBox,WDebugging,WDialog,Windows;
  IMPORT MSActions,MCSActs2;
  IMPORT MCStruct,MStruct;
  
  
  CONST
        CharacterDialogTemplate = "Character_Dialog";
  
  CONST
        NameEditTextControl  = 105;
        RaceComboBoxControl  = 106;
        ClassComboBoxControl = 107;
        SexComboBoxControl   = 108;
  
  CONST
        RollAgainPushButtonControl = 182;
        AcceptPushButtonControl    = 183;
  
  CONST
        CloseDialogPushButtonControl = RollAgainPushButtonControl;
  
  VAR
      CurrentCharacterResources : MCStruct.CharacterResourceDataPointer;
  
  
  PROCEDURE CloseCharacterDisplayDialog (window : Windows.HWND; player : MStruct.PlayerInfoPointer);
  BEGIN (* CloseCharacterDisplayDialog *)
    ExecuteCharacterDisplayCommand (window,player,CloseDialogPushButtonControl,Windows.BN_CLICKED);
  END CloseCharacterDisplayDialog;
  
  PROCEDURE CloseCharGenDialog (window : Windows.HWND; player : MStruct.PlayerInfoPointer);
  BEGIN (* CloseCharGenDialog *)
    WDebugging.FatalAppExit ("Invalid attempt to close character generation dialog.");
  END CloseCharGenDialog;
  
  PROCEDURE DisplayCharacterInformation (window : Windows.HWND; player : MStruct.PlayerInfoPointer);
    VAR
        result : INTEGER;
    
  BEGIN (* DisplayCharacterInformation *)
    MDialogs.ExecuteMoriaDialogBox (window,CharacterDialogTemplate,player,InitializeCharacterDisplayDialog,ExecuteCharacterDisplayCommand,CloseCharacterDisplayDialog,result);
  END DisplayCharacterInformation;
  
  PROCEDURE ExecuteCharacterDisplayCommand (window : Windows.HWND; player : MStruct.PlayerInfoPointer; control : CARDINAL; command : CARDINAL);
    VAR
        length : CARDINAL;
    
  BEGIN (* ExecuteCharacterDisplayCommand *)
    IF ((control = CloseDialogPushButtonControl) AND (command = Windows.BN_CLICKED)) THEN
      IF player^.character^.variables.hpCUR > 0.0 THEN
        WDialog.GetDlgItemText (window,NameEditTextControl,player^.character^.constants.name,length);
        MSWindow.SetMainWindowTitle (player^.windows.frame,player);
      END; (* IF *)
      
      WDialog.EndDialog (window,Windows.TRUE_BOOL);
    ELSIF ((control = Windows.IDCANCEL) AND (command = Windows.BN_CLICKED)) THEN
      WDialog.EndDialog (window,Windows.FALSE_BOOL);
    END; (* IF *)
  END ExecuteCharacterDisplayCommand;
  
  PROCEDURE ExecuteCharGenCommand (window : Windows.HWND; player : MStruct.PlayerInfoPointer; control : CARDINAL; command : CARDINAL);
    VAR
        controlWND : Windows.HWND;
    
    PROCEDURE ExecuteAcceptPushButtonAction (window : Windows.HWND; player : MStruct.PlayerInfoPointer; command : CARDINAL);
      VAR
          length : CARDINAL;
      
      PROCEDURE TerminateCharGenDialog (window : Windows.HWND);
      BEGIN (* TerminateCharGenDialog *)
        CurrentCharacterResources := NIL;
        
        MResources.FreeCharacterResources;
        WDialog.EndDialog (window,0);
      END TerminateCharGenDialog;
      
    BEGIN (* ExecuteAcceptPushButtonAction *)
      WDialog.GetDlgItemText (window,NameEditTextControl,player^.character^.constants.name,length);
      
      MSWindow.SetMainWindowTitle (player^.windows.frame,player);
      TerminateCharGenDialog (window);
    END ExecuteAcceptPushButtonAction;
    
    PROCEDURE ExecuteClassComboBoxAction (window : Windows.HWND; player : MStruct.PlayerInfoPointer; control : Windows.HWND; command : CARDINAL);
      VAR
          selection : MCStruct.PlayerClassMnemonic;
      
      PROCEDURE CurrentlySelectedClassMnemonic (control : Windows.HWND) : MCStruct.PlayerClassMnemonic;
      BEGIN (* CurrentlySelectedClassMnemonic *)
        RETURN (VAL (MCStruct.PlayerClassMnemonic,WComboBox.GetItemData (control,WComboBox.GetCurSel (control))));
      END CurrentlySelectedClassMnemonic;
      
    BEGIN (* ExecuteClassComboBoxAction *)
      IF command = Windows.CBN_SELCHANGE THEN
        selection := CurrentlySelectedClassMnemonic (control);
        
        IF selection <> player^.character^.constants.class THEN
          WDialog.EnableDialogControl (window,AcceptPushButtonControl,FALSE);
        END; (* IF *)
        
        player^.character^.constants.class := selection;
      END; (* IF *)
    END ExecuteClassComboBoxAction;
    
    PROCEDURE ExecuteRaceComboBoxAction (window : Windows.HWND; player : MStruct.PlayerInfoPointer; control : Windows.HWND; command : CARDINAL);
      VAR
          selection : MCStruct.PlayerRaceMnemonic;
      
      PROCEDURE CurrentlySelectRaceMnemonic (control : Windows.HWND) : MCStruct.PlayerRaceMnemonic;
      BEGIN (* CurrentlySelectRaceMnemonic *)
        RETURN (VAL (MCStruct.PlayerRaceMnemonic,WComboBox.GetCurSel (control)));
      END CurrentlySelectRaceMnemonic;
      
    BEGIN (* ExecuteRaceComboBoxAction *)
      IF command = Windows.CBN_SELCHANGE THEN
        selection := CurrentlySelectRaceMnemonic (control);
        
        IF selection <> player^.character^.constants.race THEN
          InitializeClassComboBox (WDialog.GetDlgItem (window,ClassComboBoxControl),selection,player^.character^.constants.class);
          WDialog.EnableDialogControl (window,AcceptPushButtonControl,FALSE);
        END; (* IF *)
        
        player^.character^.constants.race := selection;
      END; (* IF *)
    END ExecuteRaceComboBoxAction;
    
    PROCEDURE ExecuteRollAgainPushButtonAction (window : Windows.HWND; player : MStruct.PlayerInfoPointer; command : CARDINAL);
    BEGIN (* ExecuteRollAgainPushButtonAction *)
      MCRoll.ExecuteRollAgainButton (player,CurrentCharacterResources);
      WDialog.EnableDialogControl (window,AcceptPushButtonControl,TRUE);
      
      MCDDisplay.DisplayCharacterInformation (window,player);
    END ExecuteRollAgainPushButtonAction;
    
    PROCEDURE ExecuteSexComboBoxAction (window : Windows.HWND; player : MStruct.PlayerInfoPointer; control : Windows.HWND; command : CARDINAL);
      VAR
          selection : MCStruct.PlayerSexMnemonic;
      
      PROCEDURE CurrentlySelectedSexMnemonic (control : Windows.HWND) : MCStruct.PlayerSexMnemonic;
      BEGIN (* CurrentlySelectedSexMnemonic *)
        RETURN (VAL (MCStruct.PlayerSexMnemonic,WComboBox.GetCurSel (control)));
      END CurrentlySelectedSexMnemonic;
      
    BEGIN (* ExecuteSexComboBoxAction *)
      IF command = Windows.CBN_SELCHANGE THEN
        selection := CurrentlySelectedSexMnemonic (control);
        
        IF selection <> player^.character^.constants.sex THEN
          WDialog.EnableDialogControl (window,AcceptPushButtonControl,FALSE);
        END; (* IF *)
        
        player^.character^.constants.sex := selection;
      END; (* IF *)
    END ExecuteSexComboBoxAction;
    
  BEGIN (* ExecuteCharGenCommand *)
    controlWND := WDialog.GetDlgItem (window,control);
    
    CASE control OF
      ClassComboBoxControl : ExecuteClassComboBoxAction (window,player,controlWND,command); |
      RaceComboBoxControl  : ExecuteRaceComboBoxAction (window,player,controlWND,command);  |
      SexComboBoxControl   : ExecuteSexComboBoxAction (window,player,controlWND,command);   |
      
      RollAgainPushButtonControl : ExecuteRollAgainPushButtonAction (window,player,command); |
      AcceptPushButtonControl    : ExecuteAcceptPushButtonAction (window,player,command);    |
    ELSE
      (* no action *)
    END; (* CASE *)
  END ExecuteCharGenCommand;
  
  PROCEDURE ExecuteCharGenDialog (window : Windows.HWND);
    VAR
        result : INTEGER;
    
  BEGIN (* ExecuteCharGenDialog *)
    MDialogs.ExecuteMoriaDialogBox (window,CharacterDialogTemplate,MStruct.CurrentPlayerInformation,InitializeCharacterGeneratorDialog,ExecuteCharGenCommand,CloseCharGenDialog,result);
  END ExecuteCharGenDialog;

  PROCEDURE InitializeCharacterDisplayDialog (window : Windows.HWND; player : MStruct.PlayerInfoPointer);
  BEGIN (* InitializeCharacterDisplayDialog *)
    MCDDisplay.DisplayCharacterInformation (window,player);
    
    WDialog.EnableDialogControl (window,ClassComboBoxControl,FALSE);
    WDialog.EnableDialogControl (window,RaceComboBoxControl,FALSE);
    WDialog.EnableDialogControl (window,SexComboBoxControl,FALSE);
    
    WDialog.ShowDialogControl (window,AcceptPushButtonControl,FALSE);
    
    IF player^.character^.variables.hpCUR = 0.0 THEN
      WDialog.EnableDialogControl (window,NameEditTextControl,FALSE);
    END; (* IF *)
    
    WDialog.SetDlgItemText (window,NameEditTextControl,player^.character^.constants.name);
    WDialog.SetDlgItemText (window,CloseDialogPushButtonControl,"&Close");
    
    InitializeClassComboBox (WDialog.GetDlgItem (window,ClassComboBoxControl),player^.character^.constants.race,player^.character^.constants.class);
    InitializeRaceComboBox (WDialog.GetDlgItem (window,RaceComboBoxControl),player^.character^.constants.race);
    InitializeSexComboBox (WDialog.GetDlgItem (window,SexComboBoxControl),player^.character^.constants.sex);
  END InitializeCharacterDisplayDialog;
  
  PROCEDURE InitializeCharacterGeneratorDialog (window : Windows.HWND; player : MStruct.PlayerInfoPointer);
  BEGIN (* InitializeCharacterGeneratorDialog *)
    MSActions.InitializeConstantPlayerInformation (player);
    
    CurrentCharacterResources := MResources.LoadCharacterResources ();
    
    MCRoll.ExecuteRollAgainButton (player,CurrentCharacterResources);
    
    InitializeClassComboBox (WDialog.GetDlgItem (window,ClassComboBoxControl),player^.character^.constants.race,player^.character^.constants.class);
    InitializeRaceComboBox (WDialog.GetDlgItem (window,RaceComboBoxControl),player^.character^.constants.race);
    InitializeSexComboBox (WDialog.GetDlgItem (window,SexComboBoxControl),player^.character^.constants.sex);
    
    WDialog.SetDlgItemText (window,NameEditTextControl,player^.character^.constants.name);
    
    MCDDisplay.DisplayCharacterInformation (window,player);
  END InitializeCharacterGeneratorDialog;

  PROCEDURE InitializeClassComboBox (control : Windows.HWND; race : MCStruct.PlayerRaceMnemonic; VAR class : MCStruct.PlayerClassMnemonic);
    VAR
        available : MCStruct.PlayerClassSet;
    
    PROCEDURE AddClassesToComboBox (control : Windows.HWND; classes : MCStruct.PlayerClassSet);
      VAR
          index : MCStruct.PlayerClassMnemonic;
      
      PROCEDURE AddClassToComboBox (control : Windows.HWND; class : MCStruct.PlayerClassMnemonic);
        VAR
            position : Windows.LRESULT;
            text     : ARRAY [0..15] OF CHAR;
        
      BEGIN (* AddClassToComboBox *)
        MCSActs2.ReturnClassMnemonicString (class,text);
        WComboBox.AddStringAndData (control,text,ORD (class),position);
      END AddClassToComboBox;
      
    BEGIN (* AddClassesToComboBox *)
      FOR index := MIN (MCStruct.PlayerClassMnemonic) TO MAX (MCStruct.PlayerClassMnemonic) DO
        IF index IN classes THEN
          AddClassToComboBox (control,index);
        END; (* IF *)
      END; (* FOR *)
    END AddClassesToComboBox;
    
    PROCEDURE SelectComboBoxClassMnemonic (control : Windows.HWND; class : MCStruct.PlayerClassMnemonic);
      VAR
          position : Windows.LRESULT;
          text     : ARRAY [0..15] OF CHAR;
      
    BEGIN (* SelectComboBoxClassMnemonic *)
      MCSActs2.ReturnClassMnemonicString (class,text);
      WComboBox.SelectString (control,-1,text,position);
    END SelectComboBoxClassMnemonic;
    
  BEGIN (* InitializeClassComboBox *)
    WComboBox.ResetContent (control);
    
    CASE race OF
      MCStruct.Human     : available := MCStruct.PlayerClassSet {MCStruct.Warrior..MCStruct.Paladin};              |
      MCStruct.HalfElf   : available := MCStruct.PlayerClassSet {MCStruct.Warrior..MCStruct.Paladin};              |
      MCStruct.Elf       : available := MCStruct.PlayerClassSet {MCStruct.Warrior..MCStruct.Ranger};               |
      MCStruct.Halfling  : available := MCStruct.PlayerClassSet {MCStruct.Warrior,MCStruct.Mage,MCStruct.Rogue};   |
      MCStruct.Gnome     : available := MCStruct.PlayerClassSet {MCStruct.Warrior..MCStruct.Rogue};                |
      MCStruct.Dwarf     : available := MCStruct.PlayerClassSet {MCStruct.Warrior,MCStruct.Priest};                |
      MCStruct.HalfOrc   : available := MCStruct.PlayerClassSet {MCStruct.Warrior,MCStruct.Priest,MCStruct.Rogue}; |
      MCStruct.HalfTroll : available := MCStruct.PlayerClassSet {MCStruct.Warrior,MCStruct.Priest};                |
    END; (* CASE *)
    
    AddClassesToComboBox (control,available);
    
    IF NOT (class IN available) THEN
      class := MCStruct.Warrior;
    END; (* IF *)
    
    SelectComboBoxClassMnemonic (control,class);
  END InitializeClassComboBox;
  
  PROCEDURE InitializeRaceComboBox (control : Windows.HWND; race : MCStruct.PlayerRaceMnemonic);
    VAR
        index : MCStruct.PlayerRaceMnemonic;
    
    PROCEDURE AddRaceMnemonicToComboBox (control : Windows.HWND; race : MCStruct.PlayerRaceMnemonic);
      VAR
          position : Windows.LRESULT;
          text     : ARRAY [0..15] OF CHAR;
      
    BEGIN (* AddRaceMnemonicToComboBox *)
      MCSActs2.ReturnRaceMnemonicString (race,text);
      WComboBox.AddStringAndData (control,text,ORD (race),position);
    END AddRaceMnemonicToComboBox;
    
    PROCEDURE SelectComboBoxRaceMnemonic (control : Windows.HWND; race : MCStruct.PlayerRaceMnemonic);
      VAR
          position : Windows.LRESULT;
          text     : ARRAY [0..15] OF CHAR;
      
    BEGIN (* SelectComboBoxRaceMnemonic *)
      MCSActs2.ReturnRaceMnemonicString (race,text);
      WComboBox.SelectString (control,-1,text,position);
    END SelectComboBoxRaceMnemonic;
    
  BEGIN (* InitializeRaceComboBox *)
    FOR index := MIN (MCStruct.PlayerRaceMnemonic) TO MAX (MCStruct.PlayerRaceMnemonic) DO
      AddRaceMnemonicToComboBox (control,index);
    END; (* FOR *)
    
    SelectComboBoxRaceMnemonic (control,race);
  END InitializeRaceComboBox;
  
  PROCEDURE InitializeSexComboBox (control : Windows.HWND; sex : MCStruct.PlayerSexMnemonic);
    VAR
        index : MCStruct.PlayerSexMnemonic;
    
    PROCEDURE AddSexMnemonicToComboBox (control : Windows.HWND; sex : MCStruct.PlayerSexMnemonic);
      VAR
          position : Windows.LRESULT;
          text     : ARRAY [0..15] OF CHAR;
      
    BEGIN (* AddSexMnemonicToComboBox *)
      MCSActs2.ReturnSexMnemonicString (sex,text);
      WComboBox.AddStringAndData (control,text,ORD (sex),position);
    END AddSexMnemonicToComboBox;
    
    PROCEDURE SelectComboBoxSex (control : Windows.HWND; sex : MCStruct.PlayerSexMnemonic);
      VAR
          position : Windows.LRESULT;
          text     : ARRAY [0..15] OF CHAR;
      
    BEGIN (* SelectComboBoxSex *)
      MCSActs2.ReturnSexMnemonicString (sex,text);
      WComboBox.SelectString (control,-1,text,position);
    END SelectComboBoxSex;
    
  BEGIN (* InitializeSexComboBox *)
    FOR index := MIN (MCStruct.PlayerSexMnemonic) TO MAX (MCStruct.PlayerSexMnemonic) DO
      AddSexMnemonicToComboBox (control,index);
    END; (* FOR *)
    
    SelectComboBoxSex (control,sex);
  END InitializeSexComboBox;

BEGIN (* MCDialog *)
  CurrentCharacterResources := NIL;
END MCDialog.
