IMPLEMENTATION MODULE MChests;
  (*/WIN:A*)
  
  IMPORT MCreatures,MDPainting,MDWindow,MMWindow,MPAttk2,MPDamage,MRandom;
  IMPORT CStruct,DStruct,MCStruct,MStruct,OStruct;
  IMPORT DSActions,MSActs2,OSActions;
  IMPORT WDebugging;
  
  
  TYPE
       ChestTrapRange = [OStruct.ChestLoseStrength..OStruct.ChestSummon];
  
  VAR
      ExecuteChestTrapAction : ARRAY ChestTrapRange OF PROCEDURE (MStruct.PlayerInfoPointer,DStruct.DungeonLineRange,DStruct.DungeonColumnRange);
  
  
  PROCEDURE AttemptToOpenChest (player : MStruct.PlayerInfoPointer; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange; ability : INTEGER; VAR chest : OStruct.InventoryItemRecord);
    PROCEDURE OpenChestAtLocation (player : MStruct.PlayerInfoPointer; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange; VAR chest : OStruct.InventoryItemRecord);
      VAR
          memory : CStruct.CreatureTreasureInformation;
      
      PROCEDURE IdentifyChestInformation (treasures : OStruct.TreasureInfoPointer; VAR chest : OStruct.InventoryItemRecord);
      BEGIN (* IdentifyChestInformation *)
        EXCL (chest.object.attributes,OStruct.ChestLocked);
        
        chest.object.value := 0;
        chest.special      := OStruct.NameEmpty;
        
        OSActions.IdentifyObjectAsKnown2 (treasures,chest);
      END IdentifyChestInformation;
      
      PROCEDURE TreasuresForChest (chest : OStruct.InventoryItemRecord) : CStruct.CreatureTreasureInformation;
        VAR
            result : CStruct.CreatureTreasureInformation;
            index  : CStruct.CreatureTreasureMnemonic;
        
        PROCEDURE ChestAttributeEquivalent (value : CStruct.CreatureTreasureMnemonic) : OStruct.ObjectAttributeMnemonic;
        BEGIN (* ChestAttributeEquivalent *)
          RETURN (VAL (OStruct.ObjectAttributeMnemonic,ORD (value) + 24));
        END ChestAttributeEquivalent;
        
      BEGIN (* TreasuresForChest *)
        result := CStruct.CreatureTreasureInformation {};
        
        FOR index := MIN (CStruct.CreatureTreasureMnemonic) TO MAX (CStruct.CreatureTreasureMnemonic) DO
          IF ChestAttributeEquivalent (index) IN chest.object.attributes THEN
            INCL (result,index);
          END; (* IF *)
        END; (* FOR *)
        
        RETURN (result);
      END TreasuresForChest;
      
    BEGIN (* OpenChestAtLocation *)
      ExecuteChestTraps (player,line,column,chest);
      
      IF NOT (OStruct.ChestExplode IN chest.object.attributes) THEN
        MPAttk2.GenerateCreatureTreasures (player,line,column,TreasuresForChest (chest),memory);
        IdentifyChestInformation (player^.treasures,chest);
      END; (* IF *)
    END OpenChestAtLocation;
    
    PROCEDURE PickChestLock (player : MStruct.PlayerInfoPointer; chest : OStruct.InventoryItemRecord);
    BEGIN (* PickChestLock *)
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You have picked the lock.");
      MSActs2.IncreasePlayerExperience (player,FLOAT (chest.object.depth));
    END PickChestLock;
    
  BEGIN (* AttemptToOpenChest *)
    IF NOT (OStruct.ChestLocked IN chest.object.attributes) THEN
      OpenChestAtLocation (player,line,column,chest);
    ELSIF player^.character^.variables.counters [MCStruct.ConfusedCOUNT] > 0 THEN
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You are too confused to pick the lock.");
    ELSIF ability - VAL (INTEGER,chest.object.depth) > MRandom.IntegerRandomRangeValue (1,100) THEN
      PickChestLock (player,chest);
      OpenChestAtLocation (player,line,column,chest);
    ELSE
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You failed to pick the lock.");
    END; (* IF *)
  END AttemptToOpenChest;
  
  PROCEDURE DisarmChestTrapAtLocation (player : MStruct.PlayerInfoPointer; ability : INTEGER; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange; VAR chest : OStruct.InventoryItemRecord);
    PROCEDURE ExcludeChestTraps (VAR chest : OStruct.InventoryItemRecord);
      VAR
          trap : ChestTrapRange;
      
    BEGIN (* ExcludeChestTraps *)
      FOR trap := MIN (ChestTrapRange) TO MAX (ChestTrapRange) DO
        EXCL (chest.object.attributes,trap);
      END; (* IF *)
      
      EXCL (chest.object.attributes,OStruct.ChestTrapped);
    END ExcludeChestTraps;
    
  BEGIN (* DisarmChestTrapAtLocation *)
    IF NOT (OSActions.InventoryItemIsKnown2 (chest)) THEN
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"I don't see a trap.");
      MStruct.CharacterTurnExecuted := FALSE;
    ELSIF NOT (OStruct.ChestTrapped IN chest.object.attributes) THEN
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"The chest was not trapped.");
      MStruct.CharacterTurnExecuted := FALSE;
    ELSIF ability - VAL (INTEGER,chest.object.depth) > MRandom.IntegerRandomRangeValue (1,100) THEN
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You have disarmed the chest.");
      
      IF OStruct.ChestLocked IN chest.object.attributes THEN
        chest.special := OStruct.NameLocked;
      ELSE
        chest.special := OStruct.NameDisarmed;
      END; (* IF *)
      
      MSActs2.IncreasePlayerExperience (player,FLOAT (chest.object.depth));
      ExcludeChestTraps (chest);
      
      OSActions.IdentifyObjectAsKnown2 (player^.treasures,chest);
    ELSIF ((ability > 5) AND (MRandom.CardinalRandomRangeValue (1,ability) > 5)) THEN
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You failed to disarm the chest.");
    ELSE
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You set a trap off!");
      
      OSActions.IdentifyObjectAsKnown2 (player^.treasures,chest);
      ExecuteChestTraps (player,line,column,chest);
    END; (* IF *)
  END DisarmChestTrapAtLocation;
  
  PROCEDURE ExecuteChestTraps (player : MStruct.PlayerInfoPointer; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange; chest : OStruct.InventoryItemRecord);
    VAR
        index : ChestTrapRange;
    
  BEGIN (* ExecuteChestTraps *)
    FOR index := MIN (ChestTrapRange) TO MAX (ChestTrapRange) DO
      IF index IN chest.object.attributes THEN
        ExecuteChestTrapAction [index] (player,line,column);
      END; (* IF *)
    END; (* FOR *)
  END ExecuteChestTraps;
  
  PROCEDURE ExecuteExplodeChestTrap (player : MStruct.PlayerInfoPointer; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange);
  BEGIN (* ExecuteExplodeChestTrap *)
    MPDamage.ReduceCharacterHitPoints (player,MRandom.CardinalRandomRangeValue (5,40),"an exploding chest");
    MMWindow.DisplayPopupMessage (player^.windows.frame,player,"There is a sudden explosion!");
    
    DSActions.DeleteArtifactInformation (player^.dungeon^,line,column);
    MDPainting.InvalidateDungeonLocation (player,line,column,FALSE,TRUE);
  END ExecuteExplodeChestTrap;
  
  PROCEDURE ExecuteLoseStrengthChestTrap (player : MStruct.PlayerInfoPointer; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange);
  BEGIN (* ExecuteLoseStrengthChestTrap *)
    MMWindow.DisplayPopupMessage (player^.windows.frame,player,"A small needle has pricked you!");
    
    IF MCStruct.SustainStrFLAG IN player^.character^.variables.status THEN
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You are unaffected.");
    ELSE
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You feel weakened.");
      MSActs2.DecrementStatistic (player,MCStruct.str);
      
      MPDamage.ReduceCharacterHitPoints (player,MRandom.CardinalRandomRangeValue (1,4),"a poison needle");
    END; (* IF *)
  END ExecuteLoseStrengthChestTrap;
  
  PROCEDURE ExecuteParalyseChestTrap (player : MStruct.PlayerInfoPointer; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange);
  BEGIN (* ExecuteParalyseChestTrap *)
    MMWindow.DisplayPopupMessage (player^.windows.frame,player,"A puff of yellow gas surrounds you!");
    
    IF MCStruct.FreeActionFLAG IN player^.character^.variables.status THEN
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You are unaffected.");
    ELSE
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You choke and pass out.");
      player^.character^.variables.counters [MCStruct.ParalysisCOUNT] := MRandom.CardinalRandomRangeValue (1,20) + 10;
    END; (* IF *)
  END ExecuteParalyseChestTrap;
  
  PROCEDURE ExecutePoisonChestTrap (player : MStruct.PlayerInfoPointer; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange);
  BEGIN (* ExecutePoisonChestTrap *)
    MMWindow.DisplayPopupMessage (player^.windows.frame,player,"A small needle has pricked you!");
    
    INC (player^.character^.variables.counters [MCStruct.PoisonedCOUNT],MRandom.CardinalRandomRangeValue (1,20) + 10);
    MPDamage.ReduceCharacterHitPoints (player,MRandom.CardinalRandomRangeValue (1,6),"a poison needle");
  END ExecutePoisonChestTrap;
  
  PROCEDURE ExecuteSummonChestTrap (player : MStruct.PlayerInfoPointer; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange);
    VAR
        count : CARDINAL;
        done  : BOOLEAN;
    
  BEGIN (* ExecuteSummonChestTrap *)
    FOR count := 1 TO 3 DO
      MCreatures.SummonRandomMonster (player,line,column,FALSE,done);
    END; (* FOR *)
  END ExecuteSummonChestTrap;
  
  PROCEDURE InitializeExecuteChestTrapActionTable;
    VAR
        index : ChestTrapRange;
    
  BEGIN (* InitializeExecuteChestTrapActionTable *)
    FOR index := MIN (ChestTrapRange) TO MAX (ChestTrapRange) DO
      ExecuteChestTrapAction [index] := InvalidChestTrapAction;
    END; (* FOR *)
    
    ExecuteChestTrapAction [OStruct.ChestLoseStrength] := ExecuteLoseStrengthChestTrap;
    ExecuteChestTrapAction [OStruct.ChestParalyse]     := ExecuteParalyseChestTrap;
    ExecuteChestTrapAction [OStruct.ChestExplode]      := ExecuteExplodeChestTrap;
    ExecuteChestTrapAction [OStruct.ChestPoison]       := ExecutePoisonChestTrap;
    ExecuteChestTrapAction [OStruct.ChestSummon]       := ExecuteSummonChestTrap;
  END InitializeExecuteChestTrapActionTable;
  
  PROCEDURE InvalidChestTrapAction (player : MStruct.PlayerInfoPointer; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange);
  BEGIN (* InvalidChestTrapAction *)
    WDebugging.FatalAppExit ("Call to un-initialized chest trap action.");
  END InvalidChestTrapAction;

BEGIN (* MChests *)
  InitializeExecuteChestTrapActionTable;
END MChests.
