IMPLEMENTATION MODULE MCreatures;
  (*/WIN:A*)
  
  IMPORT MCMovement,MDPainting,MDWindow,MMWindow,MRandom,MSight;
  IMPORT CStruct,DStruct,MCStruct,MStruct;
  IMPORT DSActions,MSActs2;
  IMPORT WMacro,WString;
  
  
  PROCEDURE ComputeCreatureVisibility (player : MStruct.PlayerInfoPointer; display : BOOLEAN);
    VAR
        index : CARDINAL;
    
  BEGIN (* ComputeCreatureVisibility *)
    index := 0;
    
    WHILE ((index < DStruct.MaxDungeonCreatures) AND (player^.dungeon^.creatures [index].cursor <> CStruct.InvalidCreatureCursor)) DO
      UpdateCreatureDisplayInformation (player,player^.dungeon^.creatures [index],display);
      INC (index);
    END; (* WHILE *)
  END ComputeCreatureVisibility;
  
  PROCEDURE DetermineCreatureInformation (creatures : CStruct.CreatureInfoPointer; cursor : CARDINAL; asleep : BOOLEAN; speed : INTEGER; VAR result : DStruct.DungeonCreatureRecord);
  BEGIN (* DetermineCreatureInformation *)
    result.cursor := cursor;
    
    IF CStruct.MaxHitPoints IN creatures^.table [result.cursor].defense THEN
      result.hitpoints := creatures^.table [result.cursor].hitdie.max;
    ELSE
      result.hitpoints := MRandom.CardinalRandomRangeValue (creatures^.table [result.cursor].hitdie.min,creatures^.table [result.cursor].hitdie.max);
    END; (* IF *)
    
    result.confused := FALSE;
    result.stunned  := 0;
    result.visible  := FALSE;
    result.symbol   := creatures^.table [result.cursor].symbol;
    result.speed    := creatures^.table [result.cursor].speed - 10 + speed;
    result.armor    := creatures^.table [result.cursor].armor;
    
    IF ((asleep OR (CAP (result.symbol) = 'D')) AND (creatures^.table [result.cursor].sleep > 0)) THEN
      result.sleep := 2 * creatures^.table [result.cursor].sleep + MRandom.CardinalRandomRangeValue (1,10 * creatures^.table [result.cursor].sleep);
    ELSE
      result.sleep := 0;
    END; (* IF *)
  END DetermineCreatureInformation;
  
  PROCEDURE DetermineRandomCreatureForDepth (creatures : CStruct.CreatureInfoPointer; depth : CARDINAL; asleep : BOOLEAN; speed : INTEGER; VAR result : DStruct.DungeonCreatureRecord);
    PROCEDURE RandomCreatureCursorForDepth (creatures : CStruct.CreatureInfoPointer; depth : INTEGER) : CARDINAL;
      CONST
            NastyMonsterChance = 50;
      
      VAR
          number : CARDINAL;
      
    BEGIN (* RandomCreatureCursorForDepth *)
      IF depth = 0 THEN
        RETURN (MRandom.CardinalRandomRangeValue (0,creatures^.depth [0] - 1));
      END; (* IF *)
      
      IF MRandom.CardinalRandomRangeValue (1,NastyMonsterChance) = 1 THEN
        depth := WMacro.MinCardinal (depth + ABS (MRandom.RandomNormalValue (0,4)) + 1,CStruct.MaxCreatureDepth);
      ELSE
        number := creatures^.depth [WMacro.MinCardinal (depth,CStruct.MaxCreatureDepth)] - creatures^.depth [0];
        depth  := creatures^.table [creatures^.depth [0] + WMacro.MaxCardinal (MRandom.CardinalRandomRangeValue (0,number - 1),MRandom.CardinalRandomRangeValue (0,number - 1))].depth;
      END; (* IF *)
      
      RETURN (MRandom.CardinalRandomRangeValue (creatures^.depth [depth - 1],creatures^.depth [depth] - 1));
    END RandomCreatureCursorForDepth;
    
  BEGIN (* DetermineRandomCreatureForDepth *)
    DetermineCreatureInformation (creatures,RandomCreatureCursorForDepth (creatures,depth),asleep,speed,result);
  END DetermineRandomCreatureForDepth;
  
  PROCEDURE DetermineSummonedCreatureLocation (player : MStruct.PlayerInfoPointer; VAR dungeon : DStruct.DungeonInformation; centerLINE : DStruct.DungeonLineRange; centerCOLUMN : DStruct.DungeonColumnRange; VAR resultLINE : INTEGER; VAR resultCOLUMN : INTEGER; VAR done : BOOLEAN);
    VAR
        count : CARDINAL;
    
  BEGIN (* DetermineSummonedCreatureLocation *)
    count := 0;
    
    REPEAT
      resultCOLUMN := centerCOLUMN - 2 + MRandom.IntegerRandomRangeValue (1,3);
      resultLINE   := centerLINE - 2 + MRandom.IntegerRandomRangeValue (1,3);
      
      count := count + 1;
      done  := (DSActions.LocationInBounds (dungeon,resultLINE,resultCOLUMN) AND ValidCreaturePlacementLocation (player,resultLINE,resultCOLUMN,dungeon.physical [resultLINE]^ [resultCOLUMN]));
    UNTIL (done OR (count = 10));
  END DetermineSummonedCreatureLocation;
  
  PROCEDURE ExecuteCreatureActions (player : MStruct.PlayerInfoPointer);
    VAR
        cursor : CARDINAL;
    
    PROCEDURE ActivateCreature (player : MStruct.PlayerInfoPointer; VAR creature : DStruct.DungeonCreatureRecord);
      VAR
          awoken,ignored : BOOLEAN;
          movement       : CStruct.CreatureMovementInformation;
          actions        : INTEGER;
      
      PROCEDURE NumberCreatureActions (player : MStruct.PlayerInfoPointer; creature : DStruct.DungeonCreatureRecord) : INTEGER;
      BEGIN (* NumberCreatureActions *)
        IF creature.speed > 0 THEN
          IF player^.character^.variables.counters [MCStruct.RestCOUNT] = 0 THEN
            RETURN (creature.speed);
          ELSE
            RETURN (1);
          END; (* IF *)
        ELSIF player^.character^.turn MOD VAL (LONGCARD,2 - creature.speed) = 0 THEN
          RETURN (1);
        ELSE
          RETURN (0);
        END; (* IF *)
      END NumberCreatureActions;
      
      PROCEDURE ShouldExecuteCreatureMovement (player : MStruct.PlayerInfoPointer; creature : DStruct.DungeonCreatureRecord) : BOOLEAN;
      BEGIN (* ShouldExecuteCreatureMovement *)
        IF creature.visible THEN
          RETURN (TRUE);
        ELSIF creature.distance <= player^.creatures^.table [creature.cursor].perception THEN
          RETURN (TRUE);
        ELSE
          RETURN (NOT (CStruct.MoveThroughWall IN player^.creatures^.table [creature.cursor].movement) AND (player^.dungeon^.physical [creature.line]^ [creature.column].elements * DStruct.DungeonWallElements <> DStruct.EmptyDungeonElementSet));
        END; (* IF *)
      END ShouldExecuteCreatureMovement;
      
      PROCEDURE UpdateCreatureRecallInformation (player : MStruct.PlayerInfoPointer; creature : DStruct.DungeonCreatureRecord; awoken,ignored : BOOLEAN; movement : CStruct.CreatureMovementInformation);
      BEGIN (* UpdateCreatureRecallInformation *)
        IF (creature.visible AND (creature.cursor <> DStruct.InvalidCreatureCursor)) THEN
          IF (awoken AND (player^.creatures^.table [creature.cursor].memory.awoken < CStruct.MaxCreatureAwakenings)) THEN
            INC (player^.creatures^.table [creature.cursor].memory.awoken);
          ELSIF (ignored AND (player^.creatures^.table [creature.cursor].memory.ignored < CStruct.MaxCreatureIgnored)) THEN
            INC (player^.creatures^.table [creature.cursor].memory.ignored);
          END; (* IF *)
          
          player^.creatures^.table [creature.cursor].memory.movement := player^.creatures^.table [creature.cursor].memory.movement + movement;
        END; (* IF *)
      END UpdateCreatureRecallInformation;
      
      PROCEDURE UpdateCreatureStatusInformation (player : MStruct.PlayerInfoPointer; VAR creature : DStruct.DungeonCreatureRecord; VAR awoken,ignored : BOOLEAN);
        CONST
              StunnedRecoveryMessage = "The %s recovers and glares at you.";
        
        VAR
            message : ARRAY [0..127] OF CHAR;
        
        PROCEDURE CreatureNoticesPlayer (stealth : INTEGER) : BOOLEAN;
          VAR
              value : LONGCARD;
          
          PROCEDURE Exponential (x,y : LONGCARD) : LONGCARD;
            VAR
                result : LONGCARD;
            
          BEGIN (* Exponential *)
            result := 1;
            
            WHILE y > 0 DO
              result := result * x;
              y      := y - 1;
            END; (* WHILE *)
            
            RETURN (result);
          END Exponential;
          
        BEGIN (* CreatureNoticesPlayer *)
          value := MRandom.CardinalRandomRangeValue (1,1024);
          value := value * value * value;
          
          RETURN (value <= Exponential (2,29 - stealth));
        END CreatureNoticesPlayer;
        
      BEGIN (* UpdateCreatureStatusInformation *)
        IF creature.sleep > 0 THEN
          IF MCStruct.AggravateFLAG IN player^.character^.variables.status THEN
            creature.sleep := 0;
          ELSIF (((player^.character^.variables.counters [MCStruct.RestCOUNT] + player^.character^.variables.counters [MCStruct.ParalysisCOUNT] = 0) OR (MRandom.CardinalRandomRangeValue (1,50) = 1)) AND CreatureNoticesPlayer (player^.character^.abilities.stealth)) THEN
            DEC (creature.sleep,WMacro.MinCardinal (100 DIV creature.distance,creature.sleep));
            
            ignored := (creature.sleep > 0);
            awoken  := (creature.sleep = 0);
          END; (* IF *)
        END; (* IF *)
        
        IF creature.stunned > 0 THEN
          IF MRandom.CardinalRandomRangeValue (1,5000) < player^.creatures^.table [creature.cursor].depth * player^.creatures^.table [creature.cursor].depth THEN
            creature.stunned := 0;
          ELSE
            DEC (creature.stunned);
          END; (* IF *)
          
          IF creature.stunned = 0 THEN
            IF creature.visible THEN
              WString.ReplaceStringParameter (StunnedRecoveryMessage,"%s",player^.creatures^.table [creature.cursor].description,message);
            ELSE
              WString.ReplaceStringParameter (StunnedRecoveryMessage,"The %s","It",message);
            END; (* IF *)
            
            MMWindow.DisplayPopupMessage (player^.windows.frame,player,message);
          END; (* IF *)
        END; (* IF *)
      END UpdateCreatureStatusInformation;
      
    BEGIN (* ActivateCreature *)
      creature.distance := DSActions.QuickDistance (player^.character^.line,player^.character^.column,creature.line,creature.column);
      actions           := NumberCreatureActions (player,creature);
      
      IF actions > 0 THEN
        LOOP
          movement := CStruct.CreatureMovementInformation {};
          ignored  := FALSE;
          awoken   := FALSE;
          
          IF ShouldExecuteCreatureMovement (player,creature) THEN
            UpdateCreatureStatusInformation (player,creature,awoken,ignored);
            
            IF creature.sleep + creature.stunned = 0 THEN
              MCMovement.ExecuteCreatureMovement (player,creature,movement);
            ELSE
              UpdateCreatureDisplayInformation (player,creature,TRUE);
            END; (* IF *)
          END; (* IF *)
          
          IF ((actions = 1) OR (creature.hitpoints = 0) OR (player^.character^.variables.hpCUR = 0.0)) THEN
            EXIT;
          END; (* IF *)
          
          UpdateCreatureDisplayInformation (player,creature,TRUE);
          UpdateCreatureRecallInformation (player,creature,awoken,ignored,movement);
          
          DEC (actions);
        END; (* LOOP *)
      END; (* IF *)
      
      IF creature.hitpoints > 0 THEN
        UpdateCreatureDisplayInformation (player,creature,TRUE);
      END; (* IF *)
    END ActivateCreature;
    
  BEGIN (* ExecuteCreatureActions *)
    cursor := 0;
    
    WHILE ((player^.character^.variables.hpCUR > 0.0) AND (cursor < DStruct.MaxDungeonCreatures)) DO
      IF player^.dungeon^.creatures [cursor].cursor <> CStruct.InvalidCreatureCursor THEN
        ActivateCreature (player,player^.dungeon^.creatures [cursor]);
      END; (* IF *)
      
      INC (cursor);
    END; (* WHILE *)
  END ExecuteCreatureActions;
  
  PROCEDURE ReplaceCreatureParameter (text : ARRAY OF CHAR; creature : ARRAY OF CHAR; visible : BOOLEAN; VAR result : ARRAY OF CHAR);
  BEGIN (* ReplaceCreatureParameter *)
    IF visible THEN
      WString.ReplaceStringParameter (text,"%CREATURE",creature,result);
    ELSE
      WString.ReplaceStringParameter (text,"The %CREATURE","It",result);
    END; (* IF *)
  END ReplaceCreatureParameter;
  
  PROCEDURE SummonRandomMonster (player : MStruct.PlayerInfoPointer; centerLINE : DStruct.DungeonLineRange; centerCOLUMN : DStruct.DungeonColumnRange; asleep : BOOLEAN; VAR done : BOOLEAN);
    VAR
        creature : DStruct.DungeonCreatureRecord;
        position : CARDINAL;
        column   : DStruct.DungeonColumnRange;
        line     : DStruct.DungeonLineRange;
    
  BEGIN (* SummonRandomMonster *)
    DetermineSummonedCreatureLocation (player,player^.dungeon^,centerLINE,centerCOLUMN,line,column,done);
    
    IF done THEN
      DetermineRandomCreatureForDepth (player^.creatures,player^.character^.variables.depthCUR + 2,asleep,player^.character^.variables.speed,creature);
      DSActions.PlaceCreatureAtLocation (player,player^.dungeon^,line,column,creature,position,done);
    END; (* IF *)
  END SummonRandomMonster;
  
  PROCEDURE SummonRandomUndead (player : MStruct.PlayerInfoPointer; centerLINE : DStruct.DungeonLineRange; centerCOLUMN : DStruct.DungeonColumnRange; asleep : BOOLEAN; VAR done : BOOLEAN);
    VAR
        creature : DStruct.DungeonCreatureRecord;
        position : CARDINAL;
        column   : DStruct.DungeonColumnRange;
        line     : DStruct.DungeonLineRange;
    
    PROCEDURE DetermineRandomUndeadCreature (creatures : CStruct.CreatureInfoPointer; asleep : BOOLEAN; speed : INTEGER; VAR result : DStruct.DungeonCreatureRecord);
      PROCEDURE RandomUndeadCreatureCursor (creatures : CStruct.CreatureInfoPointer) : CARDINAL;
        VAR
            counter : CARDINAL;
            cursor  : CARDINAL;
        
      BEGIN (* RandomUndeadCreatureCursor *)
        counter := 0;
        
        LOOP
          IF counter MOD 20 = 0 THEN
            cursor := MRandom.CardinalRandomRangeValue (0,creatures^.depth [CStruct.MaxCreatureDepth] - 1);
          ELSE
            cursor := cursor + 1;
          END; (* IF *)
          
          IF CStruct.HurtBySlayUndead IN creatures^.table [cursor].defense THEN
            RETURN (cursor);
          END; (* IF *)
          
          INC (counter);
        END; (* LOOP *)
      END RandomUndeadCreatureCursor;
      
    BEGIN (* DetermineRandomUndeadCreature *)
      DetermineCreatureInformation (creatures,RandomUndeadCreatureCursor (creatures),asleep,speed,result);
    END DetermineRandomUndeadCreature;
    
  BEGIN (* SummonRandomUndead *)
    DetermineSummonedCreatureLocation (player,player^.dungeon^,centerLINE,centerCOLUMN,line,column,done);
    
    IF done THEN
      DetermineRandomUndeadCreature (player^.creatures,asleep,player^.character^.variables.speed,creature);
      DSActions.PlaceCreatureAtLocation (player,player^.dungeon^,line,column,creature,position,done);
    END; (* IF *)
  END SummonRandomUndead;
  
  PROCEDURE UpdateCreatureDisplayInformation (player : MStruct.PlayerInfoPointer; VAR creature : DStruct.DungeonCreatureRecord; display : BOOLEAN);
    PROCEDURE DisplayDungeonCreature (player : MStruct.PlayerInfoPointer; VAR creature : DStruct.DungeonCreatureRecord; display : BOOLEAN);
    BEGIN (* DisplayDungeonCreature *)
      MSActs2.DisturbPlayer (player,TRUE);
      creature.visible := TRUE;
      
      IF display THEN
        MDPainting.InvalidateDungeonLocation (player,creature.line,creature.column,FALSE,TRUE);
      END; (* IF *)
    END DisplayDungeonCreature;
    
    PROCEDURE HideDungeonCreature (player : MStruct.PlayerInfoPointer; VAR creature : DStruct.DungeonCreatureRecord; display : BOOLEAN);
    BEGIN (* HideDungeonCreature *)
      creature.visible := FALSE;
      
      IF display THEN
        MDPainting.InvalidateDungeonLocation (player,creature.line,creature.column,FALSE,TRUE);
      END; (* IF *)
    END HideDungeonCreature;
    
    PROCEDURE PlayerCanSeeCreature (player : MStruct.PlayerInfoPointer; creatureACTIVE : DStruct.DungeonCreatureRecord) : BOOLEAN;
      VAR
          creaturePASSIVE : CStruct.PassiveCreatureInfoRecord;
          location        : DStruct.PhysicalDungeonUnit;
      
      PROCEDURE CompleteIntersection (value1,value2 : MCStruct.PlayerStatusFlags) : BOOLEAN;
        VAR
            index : MCStruct.PlayerStatusMnemonic;
        
      BEGIN (* CompleteIntersection *)
        FOR index := MIN (MCStruct.PlayerStatusMnemonic) TO MAX (MCStruct.PlayerStatusMnemonic) DO
          IF ((index IN value2) AND NOT (index IN value1)) THEN
            RETURN (FALSE);
          END; (* IF *)
        END; (* FOR *)
        
        RETURN (TRUE);
      END CompleteIntersection;
      
    BEGIN (* PlayerCanSeeCreature *)
      IF ((creatureACTIVE.distance <= MCStruct.MaxCharacterSightDistance) AND NOT (MCStruct.BlindFLAG IN player^.character^.variables.status) AND MDWindow.LocationIsInWindow (player,creatureACTIVE.line,creatureACTIVE.column)) THEN
        IF MCStruct.WizardFLAG IN player^.character^.variables.status THEN
          RETURN (TRUE);
        ELSIF MSight.ClearLineOfSight (player,player^.character^.line,player^.character^.column,creatureACTIVE.line,creatureACTIVE.column) THEN
          creaturePASSIVE := player^.creatures^.table [creatureACTIVE.cursor];
          location        := player^.dungeon^.physical [creatureACTIVE.line]^ [creatureACTIVE.column];
          
          IF ((location.elements * DStruct.DungeonElementSet {DStruct.plFLAG,DStruct.tlFLAG} <> DStruct.DungeonElementSet {}) OR ((creatureACTIVE.distance < 2) AND CompleteIntersection (player^.character^.variables.status,MCStruct.PlayerStatusFlags {MCStruct.SearchFLAG,MCStruct.PlayerLightFLAG}))) THEN
            IF NOT (CStruct.MoveInvisible IN creaturePASSIVE.movement) THEN
              RETURN (TRUE);
            ELSIF MCStruct.SeeInvisibleFLAG IN player^.character^.variables.status THEN
              INCL (player^.creatures^.table [creature.cursor].memory.movement,CStruct.MoveInvisible);
              RETURN (TRUE);
            END; (* IF *)
          ELSIF ((creatureACTIVE.distance <= player^.character^.abilities.infra) AND (CStruct.Endothermic IN creaturePASSIVE.defense)) THEN
            INCL (player^.creatures^.table [creature.cursor].memory.defense,CStruct.Endothermic);
            RETURN (TRUE);
          END; (* IF *)
        END; (* IF *)
      END; (* IF *)
      
      RETURN (FALSE);
    END PlayerCanSeeCreature;
    
  BEGIN (* UpdateCreatureDisplayInformation *)
    creature.distance := DSActions.QuickDistance (player^.character^.line,player^.character^.column,creature.line,creature.column);
    
    IF PlayerCanSeeCreature (player,creature) THEN
      IF NOT (creature.visible) THEN
        DisplayDungeonCreature (player,creature,display);
      END; (* IF *)
    ELSIF creature.visible THEN
      HideDungeonCreature (player,creature,display);
    END; (* IF *)
  END UpdateCreatureDisplayInformation;

  PROCEDURE ValidCreaturePlacementLocation (player : MStruct.PlayerInfoPointer; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange; location : DStruct.PhysicalDungeonUnit) : BOOLEAN;
  BEGIN (* ValidCreaturePlacementLocation *)
    RETURN (((line <> player^.character^.line) OR (column <> player^.character^.column)) AND (location.elements * DStruct.DungeonElementSet {DStruct.BlockedFloor,DStruct.TemporaryWall1,DStruct.TemporaryWall2,DStruct.GraniteWall,DStruct.MagmaWall,DStruct.QuartzWall,DStruct.BoundaryWall} = DStruct.EmptyDungeonElementSet) AND (location.creature = DStruct.InvalidCreatureCursor));
  END ValidCreaturePlacementLocation;
END MCreatures.
