IMPLEMENTATION MODULE MCRoll;
  (*/WIN:A*)
  
  IMPORT MCSActions,MSActions;
  IMPORT MCStruct,MStruct;
  IMPORT MRandom;
  IMPORT Strings;
  IMPORT WMacro;
  
  
  PROCEDURE ExecuteRollAgainButton (player : MStruct.PlayerInfoPointer; resources : MCStruct.CharacterResourceDataPointer);
    PROCEDURE DetermineBasicStatistics (player : MStruct.PlayerInfoPointer);
      CONST
            MinimumStatisticsTotal = 72;
            MaximumStatisticsTotal = 84;
      
      VAR
          total : INTEGER;
      
      PROCEDURE RollStatisticValues (VAR table : ARRAY OF MCStruct.PlayerStatisticRange; VAR total : INTEGER);
        VAR
            index : CARDINAL;
        
      BEGIN (* RollStatisticValues *)
        total := 0;
        
        FOR index := 0 TO HIGH (table) DO
          table [index] := MRandom.CardinalRandomRangeValue (8,17);
          total         := total + table [index];
        END; (* FOR *)
      END RollStatisticValues;
      
    BEGIN (* DetermineBasicStatistics *)
      REPEAT
        RollStatisticValues (player^.character^.statistics.maximum,total);
      UNTIL ((total >= MinimumStatisticsTotal) AND (total <= MaximumStatisticsTotal));
      
      player^.character^.statistics.current := player^.character^.statistics.maximum;
    END DetermineBasicStatistics;
    
    PROCEDURE DetermineCharacterHistory (player : MStruct.PlayerInfoPointer; VAR background : ARRAY OF MCStruct.BackgroundInfoRecord);
      CONST
            MaximumCharacterStanding = 100;
            MinimumCharacterStanding = 1;
      
      CONST
            TerminalBackgroundCursor = 0;
      
      VAR
          cursor : CARDINAL;
      
      PROCEDURE AddCharacterHistoryString (VAR background : ARRAY OF MCStruct.BackgroundInfoRecord; VAR cursor : CARDINAL; VAR history : ARRAY OF CHAR; VAR standing : INTEGER);
        VAR
            item : MCStruct.BackgroundInfoRecord;
        
        PROCEDURE IndexForBackgroundCursor (VAR background : ARRAY OF MCStruct.BackgroundInfoRecord; cursor : CARDINAL) : CARDINAL;
          VAR
              index : CARDINAL;
          
        BEGIN (* IndexForBackgroundCursor *)
          FOR index := 0 TO HIGH (background) DO
            IF background [index].cursor = cursor THEN
              RETURN (index);
            END; (* IF *)
          END; (* FOR *)
        END IndexForBackgroundCursor;
        
        PROCEDURE RandomBackgroundIndex (VAR background : ARRAY OF MCStruct.BackgroundInfoRecord; index : CARDINAL) : CARDINAL;
          VAR
              roll : CARDINAL;
          
        BEGIN (* RandomBackgroundIndex *)
          roll := MRandom.CardinalRandomRangeValue (1,100);
          
          WHILE roll > background [index].roll DO
            INC (index);
          END; (* WHILE *)
          
          RETURN (index);
        END RandomBackgroundIndex;
        
      BEGIN (* AddCharacterHistoryString *)
        item := background [RandomBackgroundIndex (background,IndexForBackgroundCursor (background,cursor))];
        
        Strings.Insert (item.text,history,HIGH (history));
        
        standing := standing + item.bonus - 50;
        cursor   := item.next;
      END AddCharacterHistoryString;
      
    BEGIN (* DetermineCharacterHistory *)
      player^.character^.constants.history [0] := 0C;
      player^.character^.constants.standing    := MRandom.CardinalRandomRangeValue (1,4);
      
      cursor := 3 * ORD (player^.character^.constants.race) + 1;
      
      REPEAT
        AddCharacterHistoryString (background,cursor,player^.character^.constants.history,player^.character^.constants.standing);
      UNTIL cursor = TerminalBackgroundCursor;
      
      IF player^.character^.constants.standing > MaximumCharacterStanding THEN
        player^.character^.constants.standing := MaximumCharacterStanding;
      ELSIF player^.character^.constants.standing < MinimumCharacterStanding THEN
        player^.character^.constants.standing := MinimumCharacterStanding;
      END; (* IF *)
    END DetermineCharacterHistory;
    
    PROCEDURE DetermineCharacterMoney (player : MStruct.PlayerInfoPointer);
      CONST
            MinimumStartingGold = 80;
      
      PROCEDURE NonCharismaAttributeMonetaryValue (statistics : MCStruct.PlayerStatisticsTable) : INTEGER;
        VAR
            index : MCStruct.PlayerStatisticMnemonic;
            total : INTEGER;
        
        PROCEDURE StatisticMonetaryValue (value : MCStruct.PlayerStatisticRange) : INTEGER;
        BEGIN (* StatisticMonetaryValue *)
          RETURN (5 * (value - 10));
        END StatisticMonetaryValue;
        
      BEGIN (* NonCharismaAttributeMonetaryValue *)
        total := 0;
        
        FOR index := MCStruct.str TO MCStruct.con DO
          total := total + StatisticMonetaryValue (statistics [index]);
        END; (* FOR *)
        
        RETURN (total);
      END NonCharismaAttributeMonetaryValue;
      
    BEGIN (* DetermineCharacterMoney *)
      player^.character^.variables.currentGOLD := 6 * player^.character^.constants.standing + MRandom.IntegerRandomRangeValue (1,25) + 325 - NonCharismaAttributeMonetaryValue (player^.character^.statistics.maximum);
      
      IF player^.character^.constants.sex = MCStruct.Female THEN
        INC (player^.character^.variables.currentGOLD,50);
      END; (* IF *)
      
      IF player^.character^.variables.currentGOLD < MinimumStartingGold THEN
        player^.character^.variables.currentGOLD := MinimumStartingGold;
      END; (* IF *)
    END DetermineCharacterMoney;
    
    PROCEDURE DetermineClassInformation (player : MStruct.PlayerInfoPointer; attribs : MCStruct.ClassAttributeRecord);
      PROCEDURE CalculateHitPointInformation (character : MCStruct.CharacterInfoPointer);
        VAR
            minimum,maximum : REAL;
        
        PROCEDURE ComputeHitPointTable (VAR table : ARRAY OF REAL; hitdie : CARDINAL);
          VAR
              index : CARDINAL;
          
        BEGIN (* ComputeHitPointTable *)
          FOR index := 1 TO HIGH (table) DO
            table [index] := table [index - 1] + FLOAT (MRandom.CardinalRandomRangeValue (1,hitdie));
          END; (* FOR *)
        END ComputeHitPointTable;
        
      BEGIN (* CalculateHitPointInformation *)
        character^.variables.hpMAX := FLOAT (VAL (INTEGER,character^.constants.hitdie) + MCSActions.CharacterConstitutionAdjustment (character));
        character^.variables.hpCUR := character^.variables.hpMAX;
        
        minimum := FLOAT (MCStruct.MaxPlayerLevel * 3 DIV 8 * (character^.constants.hitdie) + MCStruct.MaxPlayerLevel);
        maximum := FLOAT (MCStruct.MaxPlayerLevel * 5 DIV 8 * (character^.constants.hitdie) + MCStruct.MaxPlayerLevel);
        
        character^.constants.hpTABLE [1] := FLOAT (character^.constants.hitdie);
        
        REPEAT
          ComputeHitPointTable (character^.constants.hpTABLE,character^.constants.hitdie);
        UNTIL ((character^.constants.hpTABLE [MCStruct.MaxPlayerLevel] >= minimum) AND (character^.constants.hpTABLE [MCStruct.MaxPlayerLevel] <= maximum));
      END CalculateHitPointInformation;
      
    BEGIN (* DetermineClassInformation *)
      ModifyPlayerStatistics (player,attribs.statMODIFIERS);
      
      WITH player^.character^.abilities DO
        perception := perception + attribs.perception;
        martial    := martial + attribs.martial;
        missile    := missile + attribs.missile;
        stealth    := stealth + attribs.stealth;
        search     := search + attribs.search;
        disarm     := disarm + attribs.disarm;
        save       := save + attribs.save;
      END; (* WITH *)
      
      MCSActions.CalculatePlayerBonuses (player^.character);
      
      player^.character^.spells.firstLEVEL := attribs.spell;
      player^.character^.constants.factor  := player^.character^.constants.factor + attribs.experience;
      player^.character^.constants.hitdie  := player^.character^.constants.hitdie + attribs.hitdie;
      
      CalculateHitPointInformation (player^.character);
    END DetermineClassInformation;
    
    PROCEDURE DetermineRacialInformation (player : MStruct.PlayerInfoPointer; attribs : MCStruct.RacialAttributeRecord);
      PROCEDURE RandomPhysiqueValue (base,deviation : CARDINAL) : CARDINAL;
      BEGIN (* RandomPhysiqueValue *)
        RETURN (MRandom.RandomNormalValue (base,deviation));
      END RandomPhysiqueValue;
      
    BEGIN (* DetermineRacialInformation *)
      ModifyPlayerStatistics (player,attribs.statMODIFIERS);
      
      WITH player^.character^ DO
        constants.factor := attribs.experience;
        constants.hitdie := attribs.hitdie;
        constants.height := RandomPhysiqueValue (attribs.heightBASE [constants.sex],attribs.heightRANGE [constants.sex]);
        constants.weight := RandomPhysiqueValue (attribs.weightBASE [constants.sex],attribs.weightRANGE [constants.sex]);
        constants.age    := attribs.ageBASE + MRandom.CardinalRandomRangeValue (1,attribs.ageRANGE);
        
        abilities.perception := attribs.perception;
        abilities.martial    := attribs.martial;
        abilities.missile    := attribs.missile;
        abilities.stealth    := attribs.stealth;
        abilities.disarm     := attribs.disarm + MCSActions.CharacterDisarmingAdjustment (player^.character);
        abilities.search     := attribs.search;
        abilities.infra      := attribs.infra;
        abilities.save       := attribs.save;
      END; (* WITH *)
    END DetermineRacialInformation;
    
    PROCEDURE ModifyPlayerStatistics (player : MStruct.PlayerInfoPointer; table : MCStruct.StatisticModifierTable);
      VAR
          index : MCStruct.PlayerStatisticMnemonic;
      
      PROCEDURE ModifyPlayerStatistic (VAR value : MCStruct.PlayerStatisticRange; change : INTEGER);
        PROCEDURE DecreaseStatisticValue (VAR value : MCStruct.PlayerStatisticRange; change : CARDINAL);
          VAR
              count : CARDINAL;
          
          PROCEDURE DecrementStatisticValue (VAR value : MCStruct.PlayerStatisticRange);
          BEGIN (* DecrementStatisticValue *)
            CASE value OF
              109..118 : value := value - 1;                                                             |
              89..108  : value := value - MRandom.IntegerRandomRangeValue (3,8);                         |
              19..88   : value := WMacro.MaxInteger (value - MRandom.IntegerRandomRangeValue (6,20),18); |
              4..18    : value := value - 1;                                                             |
              3        : (* no action *)                                                                 |
            END; (* CASE *)
          END DecrementStatisticValue;
          
        BEGIN (* DecreaseStatisticValue *)
          FOR count := 1 TO change DO
            DecrementStatisticValue (value);
          END; (* FOR *)
        END DecreaseStatisticValue;
        
        PROCEDURE IncreaseStatisticValue (VAR value : MCStruct.PlayerStatisticRange; change : CARDINAL);
          VAR
              count : CARDINAL;
          
          PROCEDURE IncrementStatisticValue (VAR value : MCStruct.PlayerStatisticRange);
          BEGIN (* IncrementStatisticValue *)
            CASE value OF
              3..17    : value := value + 1;                                      |
              18..87   : value := value + MRandom.IntegerRandomRangeValue (6,20); |
              88..107  : value := value + MRandom.IntegerRandomRangeValue (3,8);  |
              108..117 : value := value + 1;                                      |
              118      : (* no action *)                                          |
            END; (* CASE *)
          END IncrementStatisticValue;
          
        BEGIN (* IncreaseStatisticValue *)
          FOR count := 1 TO change DO
            IncrementStatisticValue (value);
          END; (* FOR *)
        END IncreaseStatisticValue;
        
      BEGIN (* ModifyPlayerStatistic *)
        IF change < 0 THEN
          DecreaseStatisticValue (value,ABS (change));
        ELSE
          IncreaseStatisticValue (value,change);
        END; (* IF *)
      END ModifyPlayerStatistic;
      
    BEGIN (* ModifyPlayerStatistics *)
      FOR index := MIN (MCStruct.PlayerStatisticMnemonic) TO MAX (MCStruct.PlayerStatisticMnemonic) DO
        ModifyPlayerStatistic (player^.character^.statistics.maximum [index],table [index]);
        
        player^.character^.statistics.current [index] := player^.character^.statistics.maximum [index];
        MSActions.ComputePlayerUseStatistic (player,index);
      END; (* FOR *)
    END ModifyPlayerStatistics;
    
  BEGIN (* ExecuteRollAgainButton *)
    DetermineBasicStatistics (player);
    
    DetermineRacialInformation (player,resources^.raceATTRIBS [player^.character^.constants.race]);
    DetermineClassInformation (player,resources^.classATTRIBS [player^.character^.constants.class]);
    DetermineCharacterHistory (player,resources^.background);
    
    DetermineCharacterMoney (player);
  END ExecuteRollAgainButton;
END MCRoll.
