IMPLEMENTATION MODULE MCSActions;
  (*/WIN:A*)
  
  FROM WMemory IMPORT ALLOCATE,DEALLOCATE;
  
  IMPORT MRandom,MResources;
  IMPORT MCStruct,OStruct;
  IMPORT OSActions;
  IMPORT MathLib0;
  IMPORT WMacro;
  
  
  PROCEDURE AllocateCharacterInformation (VAR ptr : MCStruct.CharacterInfoPointer; VAR done : BOOLEAN);
  BEGIN (* AllocateCharacterInformation *)
    NEW (ptr);
    
    done := (ptr <> NIL);
  END AllocateCharacterInformation;
  
  PROCEDURE CalculatePlayerBonuses (character : MCStruct.CharacterInfoPointer);
    VAR
        armorDISP : INTEGER;
    
    PROCEDURE ClearPreviousBonusInformation (character : MCStruct.CharacterInfoPointer; VAR armorDISP : INTEGER);
    BEGIN (* ClearPreviousBonusInformation *)
      armorDISP := character^.variables.armorDISP;
      
      IF MCStruct.SlowDigestionFLAG IN character^.variables.status THEN
        INC (character^.variables.digestion);
      END; (* IF *)
      
      IF MCStruct.RegenerationFLAG IN character^.variables.status THEN
        DEC (character^.variables.digestion,3);
      END; (* IF *)
      
      EXCL (character^.variables.status,MCStruct.SeeInvisibleFLAG);
      EXCL (character^.variables.status,MCStruct.TeleportFLAG);
      EXCL (character^.variables.status,MCStruct.FreeActionFLAG);
      EXCL (character^.variables.status,MCStruct.SlowDigestionFLAG);
      EXCL (character^.variables.status,MCStruct.AggravateFLAG);
      EXCL (character^.variables.status,MCStruct.SustainStrFLAG);
      EXCL (character^.variables.status,MCStruct.SustainIntFLAG);
      EXCL (character^.variables.status,MCStruct.SustainWisFLAG);
      EXCL (character^.variables.status,MCStruct.SustainConFLAG);
      EXCL (character^.variables.status,MCStruct.SustainDexFLAG);
      EXCL (character^.variables.status,MCStruct.SustainChrFLAG);
      EXCL (character^.variables.status,MCStruct.ResistFireFLAG);
      EXCL (character^.variables.status,MCStruct.ResistAcidFLAG);
      EXCL (character^.variables.status,MCStruct.ResistColdFLAG);
      EXCL (character^.variables.status,MCStruct.RegenerationFLAG);
      EXCL (character^.variables.status,MCStruct.ResistLightningFLAG);
      EXCL (character^.variables.status,MCStruct.FeatherFallingFLAG);
    END ClearPreviousBonusInformation;
    
    PROCEDURE ComputeDigestionModifiers (character : MCStruct.CharacterInfoPointer);
    BEGIN (* ComputeDigestionModifiers *)
      IF MCStruct.SlowDigestionFLAG IN character^.variables.status THEN
        DEC (character^.variables.digestion);
      END; (* IF *)
      
      IF MCStruct.RegenerationFLAG IN character^.variables.status THEN
        INC (character^.variables.digestion,3);
      END; (* IF *)
    END ComputeDigestionModifiers;
    
    PROCEDURE ComputeEquipmentBonuses (character : MCStruct.CharacterInfoPointer);
      VAR
          index : MCStruct.PlayerEquipmentMnemonic;
      
      PROCEDURE AddEquipmentBonuses (character : MCStruct.CharacterInfoPointer; item : OStruct.InventoryItemRecord);
        PROCEDURE AddObjectToBonuses (object : OStruct.ObjectInfoRecord; VAR bonuses : MCStruct.PlayerBonusInformation; VAR armor : INTEGER);
        BEGIN (* AddObjectToBonuses *)
          bonuses.strike := bonuses.strike + object.strikeMOD;
          bonuses.armor  := bonuses.armor + object.armorMOD;
          armor          := armor + object.armor;
          
          IF object.class <> OStruct.MissileWeapon THEN
            bonuses.damage := bonuses.damage + object.damageMOD;
          END; (* IF *)
        END AddObjectToBonuses;
        
      BEGIN (* AddEquipmentBonuses *)
        AddObjectToBonuses (item.object,character^.variables.bonusREAL,character^.variables.armorREAL);
        
        IF OSActions.InventoryItemIsKnown2 (item) THEN
          AddObjectToBonuses (item.object,character^.variables.bonusDISP,character^.variables.armorDISP);
        ELSIF NOT (OStruct.Cursed IN item.object.attributes) THEN
          character^.variables.armorDISP := character^.variables.armorDISP + item.object.armor;
        END; (* IF *)
      END AddEquipmentBonuses;
      
    BEGIN (* ComputeEquipmentBonuses *)
      FOR index := MCStruct.Weapon1 TO MCStruct.Light DO
        IF character^.inventory.equipment [index].object.class <> OStruct.Nothing THEN
          AddEquipmentBonuses (character,character^.inventory.equipment [index]);
        END; (* IF *)
      END; (* FOR *)
      
      character^.variables.armorDISP := character^.variables.armorDISP + character^.variables.bonusDISP.armor;
      
      IF MCStruct.WeaponHeavyFLAG IN character^.variables.status THEN
        character^.variables.bonusDISP.strike := character^.variables.bonusDISP.strike + (15 * character^.statistics.used [MCStruct.str] - character^.inventory.equipment [MCStruct.Weapon1].object.weight);
      END; (* IF *)
    END ComputeEquipmentBonuses;
    
    PROCEDURE ComputeSpellBonuses (character : MCStruct.CharacterInfoPointer);
      CONST
            InvulnerableArmorBonus = 100;
            BlessedArmorBonus      = 2;
      
    BEGIN (* ComputeSpellBonuses *)
      IF character^.variables.counters [MCStruct.InvulnCOUNT] > 0 THEN
        INC (character^.variables.armorDISP,InvulnerableArmorBonus);
        INC (character^.variables.armorREAL,InvulnerableArmorBonus);
      END; (* IF *)
      
      IF character^.variables.counters [MCStruct.BlessedCOUNT] > 0 THEN
        INC (character^.variables.armorDISP,BlessedArmorBonus);
        INC (character^.variables.armorREAL,BlessedArmorBonus);
      END; (* IF *)
      
      IF character^.variables.counters [MCStruct.DetectInvisCOUNT] > 0 THEN
        INCL (character^.variables.status,MCStruct.DetectInvisibleFLAG);
      END; (* IF *)
    END ComputeSpellBonuses;
    
    PROCEDURE ComputeStatisticBonuses (character : MCStruct.CharacterInfoPointer);
      PROCEDURE CharacterArmorAdjustment (character : MCStruct.CharacterInfoPointer) : INTEGER;
      BEGIN (* CharacterArmorAdjustment *)
        CASE character^.statistics.used [MCStruct.dex] OF
          3        : RETURN (-4); |
          4        : RETURN (-3); |
          5        : RETURN (-2); |
          6        : RETURN (-1); |
          7..14    : RETURN ( 0); |
          15..17   : RETURN (+1); |
          18..58   : RETURN (+2); |
          59..93   : RETURN (+3); |
          94..116  : RETURN (+4); |
          117..118 : RETURN (+5); |
        END; (* CASE *)
      END CharacterArmorAdjustment;
      
      PROCEDURE CharacterDamageAdjustment (character : MCStruct.CharacterInfoPointer) : INTEGER;
      BEGIN (* CharacterDamageAdjustment *)
        CASE character^.statistics.used [MCStruct.str] OF
          3        : RETURN (-2); |
          4        : RETURN (-1); |
          5..15    : RETURN ( 0); |
          16       : RETURN (+1); |
          17       : RETURN (+2); |
          18..93   : RETURN (+3); |
          94..108  : RETURN (+4); |
          109..116 : RETURN (+5); |
          117..118 : RETURN (+6); |
        END; (* CASE *)
      END CharacterDamageAdjustment;
      
      PROCEDURE CharacterStrikeAdjustment (character : MCStruct.CharacterInfoPointer) : INTEGER;
        PROCEDURE DexterityStrikeAdjustment (character : MCStruct.CharacterInfoPointer) : INTEGER;
        BEGIN (* DexterityStrikeAdjustment *)
          CASE character^.statistics.used [MCStruct.dex] OF
            3       : RETURN (-3); |
            4..5    : RETURN (-2); |
            6..7    : RETURN (-1); |
            8..15   : RETURN ( 0); |
            16      : RETURN (+1); |
            17      : RETURN (+2); |
            18..68  : RETURN (+3); |
            69..117 : RETURN (+4); |
            118     : RETURN (+5); |
          END; (* CASE *)
        END DexterityStrikeAdjustment;
        
        PROCEDURE StrengthStrikeAdjustment (character : MCStruct.CharacterInfoPointer) : INTEGER;
        BEGIN (* StrengthStrikeAdjustment *)
          CASE character^.statistics.used [MCStruct.str] OF
            3        : RETURN (-3); |
            4        : RETURN (-2); |
            5..6     : RETURN (-1); |
            7..17    : RETURN ( 0); |
            18..93   : RETURN (+1); |
            94..108  : RETURN (+2); |
            109..116 : RETURN (+3); |
            117..118 : RETURN (+4); |
          END; (* CASE *)
        END StrengthStrikeAdjustment;
        
      BEGIN (* CharacterStrikeAdjustment *)
        RETURN (DexterityStrikeAdjustment (character) + StrengthStrikeAdjustment (character));
      END CharacterStrikeAdjustment;
      
    BEGIN (* ComputeStatisticBonuses *)
      character^.variables.bonusREAL.damage := CharacterDamageAdjustment (character);
      character^.variables.bonusREAL.strike := CharacterStrikeAdjustment (character);
      character^.variables.bonusREAL.armor  := CharacterArmorAdjustment (character);
      
      character^.variables.bonusDISP := character^.variables.bonusREAL;
      character^.variables.armorREAL := 0;
      character^.variables.armorDISP := 0;
    END ComputeStatisticBonuses;
    
    PROCEDURE SetEquipmentFlags (character : MCStruct.CharacterInfoPointer);
      VAR
          index : MCStruct.PlayerEquipmentMnemonic;
      
      PROCEDURE AddObjectFlagsToCharacter (character : MCStruct.CharacterInfoPointer; item : OStruct.InventoryItemRecord);
        PROCEDURE StatisticSustainedByObject (object : OStruct.ObjectInfoRecord) : MCStruct.PlayerStatusMnemonic;
        BEGIN (* StatisticSustainedByObject *)
          CASE object.miscellaneous OF
            1 : RETURN (MCStruct.SustainStrFLAG); |
            2 : RETURN (MCStruct.SustainIntFLAG); |
            3 : RETURN (MCStruct.SustainWisFLAG); |
            4 : RETURN (MCStruct.SustainConFLAG); |
            5 : RETURN (MCStruct.SustainDexFLAG); |
            6 : RETURN (MCStruct.SustainChrFLAG); |
          END; (* CASE *)
        END StatisticSustainedByObject;
        
      BEGIN (* AddObjectFlagsToCharacter *)
        IF OStruct.SlowDigestion IN item.object.attributes THEN
          INCL (character^.variables.status,MCStruct.SlowDigestionFLAG);
        END; (* IF *)
        
        IF OStruct.Aggravation IN item.object.attributes THEN
          INCL (character^.variables.status,MCStruct.AggravateFLAG);
        END; (* IF *)
        
        IF OStruct.Teleportation IN item.object.attributes THEN
          INCL (character^.variables.status,MCStruct.TeleportFLAG);
        END; (* IF *)
        
        IF OStruct.Regeneration IN item.object.attributes THEN
          INCL (character^.variables.status,MCStruct.RegenerationFLAG);
        END; (* IF *)
        
        IF OStruct.ResistFire IN item.object.attributes THEN
          INCL (character^.variables.status,MCStruct.ResistFireFLAG);
        END; (* IF *)
        
        IF OStruct.ResistAcid IN item.object.attributes THEN
          INCL (character^.variables.status,MCStruct.ResistAcidFLAG);
        END; (* IF *)
        
        IF OStruct.ResistCold IN item.object.attributes THEN
          INCL (character^.variables.status,MCStruct.ResistColdFLAG);
        END; (* IF *)
        
        IF OStruct.FreeAction IN item.object.attributes THEN
          INCL (character^.variables.status,MCStruct.FreeActionFLAG);
        END; (* IF *)
        
        IF OStruct.SeeInvisible IN item.object.attributes THEN
          INCL (character^.variables.status,MCStruct.SeeInvisibleFLAG);
        END; (* IF *)
        
        IF OStruct.ResistLightning IN item.object.attributes THEN
          INCL (character^.variables.status,MCStruct.ResistLightningFLAG);
        END; (* IF *)
        
        IF OStruct.FeatherFalling IN item.object.attributes THEN
          INCL (character^.variables.status,MCStruct.FeatherFallingFLAG);
        END; (* IF *)
        
        IF OStruct.SustainStat IN item.object.attributes THEN
          INCL (character^.variables.status,StatisticSustainedByObject (item.object));
        END; (* IF *)
      END AddObjectFlagsToCharacter;
      
    BEGIN (* SetEquipmentFlags *)
      FOR index := MCStruct.Weapon1 TO MCStruct.Light DO
        IF character^.inventory.equipment [index].object.class <> OStruct.Nothing THEN
          AddObjectFlagsToCharacter (character,character^.inventory.equipment [index]);
        END; (* IF *)
      END; (* FOR *)
    END SetEquipmentFlags;
    
  BEGIN (* CalculatePlayerBonuses *)
    ClearPreviousBonusInformation (character,armorDISP);
    
    ComputeStatisticBonuses (character);
    ComputeEquipmentBonuses (character);
    ComputeSpellBonuses (character);
    
    SetEquipmentFlags (character);
    
    IF character^.variables.armorDISP <> armorDISP THEN
      INCL (character^.variables.status,MCStruct.ArmorFLAG);
    END; (* IF *)
    
    ComputeDigestionModifiers (character);
  END CalculatePlayerBonuses;
  
  PROCEDURE CalculatePlayerHitPoints (character : MCStruct.CharacterInfoPointer);
    VAR
        hitpoints : REAL;
    
  BEGIN (* CalculatePlayerHitPoints *)
    hitpoints := WMacro.MaxReal (character^.constants.hpTABLE [character^.variables.level] + FLOAT (character^.variables.level) * MathLib0.real (CharacterConstitutionAdjustment (character)),FLOAT (character^.variables.level + 1));
    
    IF MCStruct.HeroismFLAG IN character^.variables.status THEN
      hitpoints := hitpoints + 10.0;
    END; (* IF *)
    
    IF MCStruct.SuperHeroismFLAG IN character^.variables.status THEN
      hitpoints := hitpoints + 20.0;
    END; (* IF *)
    
    IF ((hitpoints <> character^.variables.hpMAX) AND (character^.variables.hpMAX <> 0.0)) THEN
      character^.variables.hpCUR := character^.variables.hpCUR * (hitpoints / character^.variables.hpMAX);
      character^.variables.hpMAX := hitpoints;
      
      INCL (character^.variables.status,MCStruct.HpFLAG);
    END; (* IF *)
  END CalculatePlayerHitPoints;
  
  PROCEDURE CharacterCharismaAdjustment (character : MCStruct.CharacterInfoPointer) : LONGINT;
  BEGIN (* CharacterCharismaAdjustment *)
    CASE character^.statistics.used [MCStruct.chr] OF
      118      : RETURN ( 90); |
      108..117 : RETURN ( 92); |
      88..107  : RETURN ( 94); |
      68..87   : RETURN ( 96); |
      19..67   : RETURN ( 98); |
      18       : RETURN (100); |
      17       : RETURN (101); |
      16       : RETURN (102); |
      15       : RETURN (103); |
      14       : RETURN (104); |
      13       : RETURN (106); |
      12       : RETURN (108); |
      11       : RETURN (110); |
      10       : RETURN (112); |
      9        : RETURN (114); |
      8        : RETURN (116); |
      7        : RETURN (118); |
      6        : RETURN (120); |
      5        : RETURN (122); |
      4        : RETURN (125); |
      3        : RETURN (130); |
    END; (* CASE *)
  END CharacterCharismaAdjustment;
  
  PROCEDURE CharacterConstitutionAdjustment (character : MCStruct.CharacterInfoPointer) : INTEGER;
  BEGIN (* CharacterConstitutionAdjustment *)
    CASE character^.statistics.used [MCStruct.con] OF
      3..6     : RETURN (character^.statistics.used [MCStruct.con] - 7); |
      7..16    : RETURN ( 0);                                            |
      17       : RETURN (+1);                                            |
      18..93   : RETURN (+2);                                            |
      94..116  : RETURN (+3);                                            |
      117..118 : RETURN (+4);                                            |
    END; (* CASE *)
  END CharacterConstitutionAdjustment;
  
  PROCEDURE CharacterDisarmingAdjustment (character : MCStruct.CharacterInfoPointer) : INTEGER;
  BEGIN (* CharacterDisarmingAdjustment *)
    CASE character^.statistics.used [MCStruct.dex] OF
             3 : RETURN (-8); |
             4 : RETURN (-6); |
             5 : RETURN (-4); |
             6 : RETURN (-2); |
             7 : RETURN (-1); |
         8..12 : RETURN ( 0); |
        13..15 : RETURN (+1); |
        16..17 : RETURN (+2); |
        18..58 : RETURN (+4); |
        59..93 : RETURN (+5); |
       94..116 : RETURN (+6); |
      117..118 : RETURN (+8); |
    END; (* CASE *)
  END CharacterDisarmingAdjustment;
  
  PROCEDURE CharacterSpellFailureChance (character : MCStruct.CharacterInfoPointer; spell : MCStruct.PlayerSpellInfoRecord) : CARDINAL;
    CONST
          MaximumSpellFailureChance = 95;
          MinimumSpellFailureChance = 5;
    
    VAR
        chance : INTEGER;
    
  BEGIN (* CharacterSpellFailureChance *)
    chance := VAL (INTEGER,spell.difficulty) - 3 * (character^.variables.level - spell.level);
    
    IF character^.constants.class IN MCStruct.PriestClasses THEN
      chance := chance - 3 * CharacterStatisticAdjustment (character,MCStruct.wis);
    ELSE
      chance := chance - 3 * CharacterStatisticAdjustment (character,MCStruct.int);
    END; (* IF *)
    
    IF spell.mana > TRUNC (character^.variables.manaCUR) THEN
      chance := chance + 5 * VAL (INTEGER,spell.mana - TRUNC (character^.variables.manaCUR));
    END; (* IF *)
    
    IF chance > MaximumSpellFailureChance THEN
      RETURN (MaximumSpellFailureChance);
    ELSIF chance < MinimumSpellFailureChance THEN
      RETURN (MinimumSpellFailureChance);
    ELSE
      RETURN (chance);
    END; (* IF *)
  END CharacterSpellFailureChance;
  
  PROCEDURE CharacterStatisticAdjustment (character : MCStruct.CharacterInfoPointer; stat : MCStruct.PlayerStatisticMnemonic) : INTEGER;
  BEGIN (* CharacterStatisticAdjustment *)
    CASE character^.statistics.used [stat] OF
      118      : RETURN (7); |
      108..117 : RETURN (6); |
      88..107  : RETURN (5); |
      68..87   : RETURN (4); |
      18..67   : RETURN (3); |
      15..17   : RETURN (2); |
      8..14    : RETURN (1); |
      3..7     : RETURN (0); |
    END; (* CASE *)
  END CharacterStatisticAdjustment;
  
  PROCEDURE ClassDeviceLevelAdjustment (class : MCStruct.PlayerClassMnemonic) : INTEGER;
  BEGIN (* ClassDeviceLevelAdjustment *)
    CASE class OF
      MCStruct.Warrior : RETURN (2); |
      MCStruct.Mage    : RETURN (4); |
      MCStruct.Priest  : RETURN (4); |
      MCStruct.Rogue   : RETURN (3); |
      MCStruct.Ranger  : RETURN (3); |
      MCStruct.Paladin : RETURN (3); |
    END; (* CASE *)
  END ClassDeviceLevelAdjustment;
  
  PROCEDURE ClassDisarmingLevelAdjustment (class : MCStruct.PlayerClassMnemonic) : INTEGER;
  BEGIN (* ClassDisarmingLevelAdjustment *)
    CASE class OF
      MCStruct.Warrior : RETURN (2); |
      MCStruct.Mage    : RETURN (3); |
      MCStruct.Priest  : RETURN (3); |
      MCStruct.Rogue   : RETURN (4); |
      MCStruct.Ranger  : RETURN (3); |
      MCStruct.Paladin : RETURN (2); |
    END; (* CASE *)
  END ClassDisarmingLevelAdjustment;
  
  PROCEDURE ClassMartialLevelAdjustment (class : MCStruct.PlayerClassMnemonic) : INTEGER;
  BEGIN (* ClassMartialLevelAdjustment *)
    CASE class OF
      MCStruct.Warrior : RETURN (4); |
      MCStruct.Mage    : RETURN (2); |
      MCStruct.Priest  : RETURN (2); |
      MCStruct.Rogue   : RETURN (3); |
      MCStruct.Ranger  : RETURN (3); |
      MCStruct.Paladin : RETURN (3); |
    END; (* CASE *)
  END ClassMartialLevelAdjustment;
  
  PROCEDURE ClassMissileLevelAdjustment (class : MCStruct.PlayerClassMnemonic) : INTEGER;
  BEGIN (* ClassMissileLevelAdjustment *)
    CASE class OF
      MCStruct.Warrior : RETURN (4); |
      MCStruct.Mage    : RETURN (2); |
      MCStruct.Priest  : RETURN (2); |
      MCStruct.Rogue   : RETURN (4); |
      MCStruct.Ranger  : RETURN (4); |
      MCStruct.Paladin : RETURN (3); |
    END; (* CASE *)
  END ClassMissileLevelAdjustment;
  
  PROCEDURE ClassSaveLevelAdjustment (class : MCStruct.PlayerClassMnemonic) : INTEGER;
  BEGIN (* ClassSaveLevelAdjustment *)
    CASE class OF
      MCStruct.Warrior : RETURN (3); |
      MCStruct.Mage    : RETURN (3); |
      MCStruct.Priest  : RETURN (3); |
      MCStruct.Rogue   : RETURN (3); |
      MCStruct.Ranger  : RETURN (3); |
      MCStruct.Paladin : RETURN (3); |
    END; (* CASE *)
  END ClassSaveLevelAdjustment;
  
  PROCEDURE DeallocateCharacterInformation (VAR character : MCStruct.CharacterInfoPointer);
  BEGIN (* DeallocateCharacterInformation *)
    DISPOSE (character);
  END DeallocateCharacterInformation;
  
  PROCEDURE ExperienceNeededToAdvance (character : MCStruct.CharacterInfoPointer) : REAL;
  BEGIN (* ExperienceNeededToAdvance *)
    RETURN (LONGFLOAT (character^.constants.experience [character^.variables.level] * character^.constants.factor DIV 100));
  END ExperienceNeededToAdvance;
  
  PROCEDURE InitializeCharacterInformation (treasures : OStruct.TreasureInfoPointer; VAR character : MCStruct.CharacterInformation);
    PROCEDURE InitializeAbilities (VAR abilities : MCStruct.PlayerAbilityInformation);
    BEGIN (* InitializeAbilities *)
      WITH abilities DO
        perception := 0;
        stealth    := 0;
        martial    := 0;
        missile    := 0;
        disarm     := 0;
        search     := 0;
        infra      := 0;
        save       := 0;
      END; (* WITH *)
    END InitializeAbilities;
    
    PROCEDURE InitializeConstants (VAR constants : MCStruct.ConstantPlayerInformation);
      PROCEDURE LoadInitialExperienceTable (VAR result : MCStruct.CharacterExperienceTable);
        VAR
            ptr : MCStruct.CharacterResourceDataPointer;
        
      BEGIN (* LoadInitialExperienceTable *)
        ptr    := MResources.LoadCharacterResources ();
        result := ptr^.experience;
        
        MResources.FreeCharacterResources;
      END LoadInitialExperienceTable;
      
    BEGIN (* InitializeConstants *)
      WITH constants DO
        LoadInitialExperienceTable (experience);
        
        class := MCStruct.Warrior;
        race  := MCStruct.Human;
        name  := "Player";
        sex   := MCStruct.Male;
        
        history [0] := 0C;
        standing    := 1;
        
        height := 0;
        weight := 0;
        age    := 0;
        
        hitdie := 0;
        factor := 0;
      END; (* WITH *)
    END InitializeConstants;
    
    PROCEDURE InitializeInventory (treasures : OStruct.TreasureInfoPointer; VAR inventory : MCStruct.PlayerInventoryInformation);
    BEGIN (* InitializeInventory *)
      WITH inventory DO
        OSActions.ClearInventoryItemArray (treasures,equipment);
        OSActions.ClearInventoryItemArray (treasures,backpack);
        
        counter := 0;
        weight  := 0;
      END; (* FOR *)
    END InitializeInventory;
    
    PROCEDURE InitializeMovement (VAR movement : MCStruct.PlayerMovementInformation);
      PROCEDURE AssignMovementCHomeInformation (VAR table : ARRAY OF INTEGER; value0,value1,value2,value3,value4,value5,value6,value7,value8,value9 : INTEGER);
      BEGIN (* AssignMovementCHomeInformation *)
        table [0] := value0;
        table [1] := value1;
        table [2] := value2;
        table [3] := value3;
        table [4] := value4;
        table [5] := value5;
        table [6] := value6;
        table [7] := value7;
        table [8] := value8;
        table [9] := value9;
      END AssignMovementCHomeInformation;
      
      PROCEDURE AssignMovementCycleInformation (VAR table : ARRAY OF CARDINAL; value0,value1,value2,value3,value4,value5,value6,value7,value8,value9,value10,value11,value12,value13,value14,value15,value16 : INTEGER);
      BEGIN (* AssignMovementCycleInformation *)
        table [ 0] := value0;
        table [ 1] := value1;
        table [ 2] := value2;
        table [ 3] := value3;
        table [ 4] := value4;
        table [ 5] := value5;
        table [ 6] := value6;
        table [ 7] := value7;
        table [ 8] := value8;
        table [ 9] := value9;
        table [10] := value10;
        table [11] := value11;
        table [12] := value12;
        table [13] := value13;
        table [14] := value14;
        table [15] := value15;
        table [16] := value16;
      END AssignMovementCycleInformation;
      
    BEGIN (* InitializeMovement *)
      AssignMovementCycleInformation (movement.cycle, 1, 2, 3, 6, 9, 8, 7, 4, 1, 2, 3, 6, 9, 8, 7, 4, 1);
      AssignMovementCHomeInformation (movement.chome,-1, 8, 9, 10, 7, -1, 11, 6, 5, 4);
    END InitializeMovement;
    
    PROCEDURE InitializeSpells (VAR spells : MCStruct.PlayerSpellInformation);
      PROCEDURE ClearPlayerSpellOrder (VAR table : ARRAY OF CARDINAL);
        VAR
            index : CARDINAL;
        
      BEGIN (* ClearPlayerSpellOrder *)
        FOR index := 0 TO HIGH (table) DO
          table [index] := MCStruct.InvalidSpellCursor;
        END; (* FOR *)
      END ClearPlayerSpellOrder;
      
      PROCEDURE ClearPlayerSpellTable (VAR table : ARRAY OF MCStruct.PlayerSpellInfoRecord);
        VAR
            index : CARDINAL;
        
        PROCEDURE ReturnBlankPlayerSpellInfoRecord (VAR result : MCStruct.PlayerSpellInfoRecord);
        BEGIN (* ReturnBlankPlayerSpellInfoRecord *)
          WITH result DO
            difficulty := 0;
            experience := 0;
            level      := 0;
            mana       := 0;
          END; (* WITH *)
        END ReturnBlankPlayerSpellInfoRecord;
        
      BEGIN (* ClearPlayerSpellTable *)
        ReturnBlankPlayerSpellInfoRecord (table [0]);
        
        FOR index := 1 TO HIGH (table) DO
          table [index] := table [0];
        END; (* FOR *)
      END ClearPlayerSpellTable;
      
    BEGIN (* InitializeSpells *)
      WITH spells DO
        ClearPlayerSpellTable (table);
        
        forgotten := MCStruct.PlayerSpellSet {};
        learned   := MCStruct.PlayerSpellSet {};
        worked    := MCStruct.PlayerSpellSet {};
        
        ClearPlayerSpellOrder (order);
      END; (* WITH *)
    END InitializeSpells;
    
    PROCEDURE InitializeStatistics (VAR statistics : MCStruct.PlayerStatisticInformation);
      VAR
          index : MCStruct.PlayerStatisticMnemonic;
      
    BEGIN (* InitializeStatistics *)
      WITH statistics DO
        FOR index := MIN (MCStruct.PlayerStatisticMnemonic) TO MAX (MCStruct.PlayerStatisticMnemonic) DO
          modifiers [index] := 0;
          current [index]   := MIN (MCStruct.PlayerStatisticRange);
        END; (* FOR *)
        
        maximum := current;
        used    := current;
      END; (* WITH *)
    END InitializeStatistics;
    
    PROCEDURE InitializeStores (VAR stores : MCStruct.PlayerStoreInformation);
      VAR
          index : MCStruct.PlayerStoreMnemonic;
          data  : MCStruct.StoreResourceData;
      
      PROCEDURE InitializePlayerStore (VAR store : MCStruct.PlayerStoreRecord; owner : MCStruct.StoreOwnerRecord; stock : MCStruct.StoreStockTable);
      BEGIN (* InitializePlayerStore *)
        store.insultsCUR := 0;
        store.openTURN   := 0;
        store.counter    := 0;
        store.owner      := owner;
        store.stock      := stock;
      END InitializePlayerStore;
      
      PROCEDURE RandomOwnerForStore (store : MCStruct.PlayerStoreMnemonic) : CARDINAL;
        CONST
              OwnersPerStore = 3;
              NumberStores   = ORD (MAX (MCStruct.PlayerStoreMnemonic)) + 1;
        
      BEGIN (* RandomOwnerForStore *)
        RETURN (NumberStores * MRandom.CardinalRandomRangeValue (0,OwnersPerStore - 1) + ORD (store));
      END RandomOwnerForStore;
      
      PROCEDURE ReturnStoreResourceData (VAR result : MCStruct.StoreResourceData);
        VAR
            ptr : MCStruct.CharacterResourceDataPointer;
        
      BEGIN (* ReturnStoreResourceData *)
        ptr    := MResources.LoadCharacterResources ();
        result := ptr^.stores;
        
        MResources.FreeCharacterResources;
      END ReturnStoreResourceData;
      
    BEGIN (* InitializeStores *)
      ReturnStoreResourceData (data);
      
      FOR index := MIN (MCStruct.PlayerStoreMnemonic) TO MAX (MCStruct.PlayerStoreMnemonic) DO
        InitializePlayerStore (stores [index],data.owners [RandomOwnerForStore (index)],data.stock [index]);
      END; (* FOR *)
    END InitializeStores;
    
    PROCEDURE InitializeVariables (VAR variables : MCStruct.VariablePlayerInformation);
      PROCEDURE InitializePlayerBonusInformation (VAR bonuses : MCStruct.PlayerBonusInformation);
      BEGIN (* InitializePlayerBonusInformation *)
        WITH bonuses DO
          damage := 0;
          strike := 0;
          armor  := 0;
        END; (* WITH *)
      END InitializePlayerBonusInformation;
      
      PROCEDURE ClearPlayerCounters (VAR counters : ARRAY OF CARDINAL);
        VAR
            index : CARDINAL;
        
      BEGIN (* ClearPlayerCounters *)
        FOR index := 0 TO HIGH (counters) DO
          counters [index] := 0;
        END; (* FOR *)
      END ClearPlayerCounters;
      
    BEGIN (* InitializeVariables *)
      WITH variables DO
        experCUR := 0.0;
        experMAX := 0.0;
        depthCUR := 0;
        depthMAX := 0;
        manaCUR  := 0.0;
        manaMAX  := 0.0;
        hpCUR    := 0.0;
        hpMAX    := 0.0;
        
        InitializePlayerBonusInformation (bonusDISP);
        InitializePlayerBonusInformation (bonusREAL);
        ClearPlayerCounters (counters);
        
        currentGOLD := 0;
        digestion   := 2;
        armorDISP   := 0;
        armorREAL   := 0;
        status      := MCStruct.PlayerStatusFlags {};
        level       := 1;
        food        := 7500;
      END; (* WITH *)
    END InitializeVariables;
    
  BEGIN (* InitializeCharacterInformation *)
    WITH character DO
      InitializeInventory (treasures,inventory);
      
      InitializeStatistics (statistics);
      InitializeAbilities (abilities);
      InitializeConstants (constants);
      InitializeVariables (variables);
      InitializeMovement (movement);
      InitializeSpells (spells);
      InitializeStores (stores);
      
      turn := 0;
    END; (* WITH *)
  END InitializeCharacterInformation;
  
  PROCEDURE ModifyPlayerStatistic (value : MCStruct.PlayerStatisticRange; modifier : INTEGER; VAR result : MCStruct.PlayerStatisticRange);
    PROCEDURE DecreaseStatisticValue (VAR value : MCStruct.PlayerStatisticRange; change : CARDINAL);
      VAR
          count : CARDINAL;
      
      PROCEDURE DecrementStatisticValue (VAR value : MCStruct.PlayerStatisticRange);
      BEGIN (* DecrementStatisticValue *)
        CASE value OF
          28..118 : value := value - 10; |
          19..27  : value := 18;         |
          4..18   : value := value - 1;  |
          3       : (* no action *)      |
        END; (* CASE *)
      END DecrementStatisticValue;
      
    BEGIN (* DecreaseStatisticValue *)
      FOR count := 1 TO change DO
        DecrementStatisticValue (value);
      END; (* FOR *)
    END DecreaseStatisticValue;
    
    PROCEDURE IncreaseStatisticValue (VAR value : MCStruct.PlayerStatisticRange; change : CARDINAL);
      VAR
          count : CARDINAL;
      
      PROCEDURE IncrementStatisticValue (VAR value : MCStruct.PlayerStatisticRange);
      BEGIN (* IncrementStatisticValue *)
        CASE value OF
          3..17    : value := value + 1;  |
          18..107  : value := value + 10; |
          108..118 : value := 118;        |
        END; (* CASE *)
      END IncrementStatisticValue;
      
    BEGIN (* IncreaseStatisticValue *)
      FOR count := 1 TO change DO
        IncrementStatisticValue (value);
      END; (* FOR *)
    END IncreaseStatisticValue;
    
  BEGIN (* ModifyPlayerStatistic *)
    result := value;
    
    IF modifier < 0 THEN
      DecreaseStatisticValue (result,ABS (modifier));
    ELSE
      IncreaseStatisticValue (result,modifier);
    END; (* IF *)
  END ModifyPlayerStatistic;
END MCSActions.
