IMPLEMENTATION MODULE MCSpells;
  (*/WIN:A*)
  
  IMPORT MCSpell2,MDialogs,MInventory,MMWindow,MPBDialog,MRandom;
  IMPORT DSActions,MCSActions,MSActs2;
  IMPORT Windows,WMacro,WString;
  IMPORT MCStruct,MStruct,OStruct;
  IMPORT MathLib0;
  
  
  PROCEDURE CalculatePlayerMana (player : MStruct.PlayerInfoPointer);
    VAR
        mana : REAL;
    
    PROCEDURE CharacterMana (character : MCStruct.CharacterInfoPointer) : REAL;
      VAR
          levels,mana : REAL;
          statistic   : MCStruct.PlayerStatisticMnemonic;
      
    BEGIN (* CharacterMana *)
      levels := MathLib0.real (character^.variables.level - character^.spells.firstLEVEL + 1);
      
      IF character^.constants.class IN MCStruct.PriestClasses THEN
        statistic := MCStruct.wis;
      ELSE
        statistic := MCStruct.int;
      END; (* IF *)
      
      CASE MCSActions.CharacterStatisticAdjustment (character,statistic) OF
        0   : mana := 0.0;          |
        1,2 : mana := levels;       |
        3   : mana := 1.5 * levels; |
        4   : mana := 2.0 * levels; |
        5   : mana := 2.5 * levels; |
        6   : mana := 3.0 * levels; |
        7   : mana := 4.0 * levels; |
      END; (* CASE *)
      
      IF mana > 0.0 THEN
        mana := mana + 1.0;
      END; (* IF *)
      
      RETURN (mana);
    END CharacterMana;
    
  BEGIN (* CalculatePlayerMana *)
    IF player^.character^.spells.learned <> MCStruct.PlayerSpellSet {} THEN
      mana := CharacterMana (player^.character);
      
      IF player^.character^.variables.manaMAX <> mana THEN
        INCL (player^.character^.variables.status,MCStruct.ManaFLAG);
        
        IF player^.character^.variables.manaMAX <> 0.0 THEN
          player^.character^.variables.manaCUR := player^.character^.variables.manaCUR * (mana / player^.character^.variables.manaMAX);
        ELSE
          player^.character^.variables.manaCUR := mana;
        END; (* IF *)
        
        player^.character^.variables.manaMAX := mana;
      END; (* IF *)
    ELSIF player^.character^.variables.manaMAX <> 0.0 THEN
      player^.character^.variables.manaMAX := 0.0;
      player^.character^.variables.manaCUR := 0.0;
      
      INCL (player^.character^.variables.status,MCStruct.ManaFLAG);
    END; (* IF *)
  END CalculatePlayerMana;
  
  PROCEDURE CalculatePlayerSpells (player : MStruct.PlayerInfoPointer);
    VAR
        spells : INTEGER;
    
    PROCEDURE DetermineNumberNewSpells (character : MCStruct.CharacterInfoPointer; VAR result : INTEGER);
      PROCEDURE NumberSpellsAllowed (character : MCStruct.CharacterInfoPointer) : INTEGER;
        VAR
            statistic : MCStruct.PlayerStatisticMnemonic;
            levels    : INTEGER;
        
      BEGIN (* NumberSpellsAllowed *)
        levels := character^.variables.level - character^.spells.firstLEVEL + 1;
        
        IF character^.constants.class IN MCStruct.PriestClasses THEN
          statistic := MCStruct.wis;
        ELSE
          statistic := MCStruct.int;
        END; (* IF *)
        
        CASE MCSActions.CharacterStatisticAdjustment (character,statistic) OF
          0     : RETURN (0);                |
          1,2,3 : RETURN (levels);           |
          4,5   : RETURN (3 * levels DIV 2); |
          6     : RETURN (2 * levels);       |
          7     : RETURN (5 * levels DIV 2); |
        END; (* CASE *)
      END NumberSpellsAllowed;
      
      PROCEDURE NumberSpellsKnown (character : MCStruct.CharacterInfoPointer) : INTEGER;
        VAR
            number : CARDINAL;
            index  : CARDINAL;
        
      BEGIN (* NumberSpellsKnown *)
        number := 0;
        
        FOR index := 0 TO MCStruct.MaxNumberSpells - 1 DO
          IF index IN character^.spells.learned THEN
            INC (number);
          END; (* IF *)
        END; (* FOR *)
        
        RETURN (number);
      END NumberSpellsKnown;
      
    BEGIN (* DetermineNumberNewSpells *)
      result := NumberSpellsAllowed (character) - NumberSpellsKnown (character);
    END DetermineNumberNewSpells;
    
    PROCEDURE DisplayLearningMessage (player : MStruct.PlayerInfoPointer);
    BEGIN (* DisplayLearningMessage *)
      IF player^.windows.message.frame <> Windows.NULL THEN
        IF player^.character^.constants.class IN MCStruct.PriestClasses THEN
          MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You can learn some new prayers now.");
        ELSE
          MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You can learn some new spells now.");
        END; (* IF *)
      END; (* IF *)
    END DisplayLearningMessage;
    
    PROCEDURE ForgetCharacterSpell (player : MStruct.PlayerInfoPointer; spell : CARDINAL);
      VAR
          message : ARRAY [0..127] OF CHAR;
      
    BEGIN (* ForgetCharacterSpell *)
      INCL (player^.character^.spells.forgotten,spell);
      EXCL (player^.character^.spells.learned,spell);
      
      IF player^.character^.constants.class IN MCStruct.PriestClasses THEN
        WString.ReplaceStringParameter ("You have forgotten the prayer of %s.","%s",player^.character^.spells.names [spell],message);
      ELSE
        WString.ReplaceStringParameter ("You have forgotten the spell of %s.","%s",player^.character^.spells.names [spell],message);
      END; (* IF *)
      
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,message);
    END ForgetCharacterSpell;
    
    PROCEDURE ForgetMostRecentSpells (player : MStruct.PlayerInfoPointer; number : CARDINAL);
      VAR
          index : [0..MCStruct.MaxNumberSpells - 1];
      
    BEGIN (* ForgetMostRecentSpells *)
      index := MCStruct.MaxNumberSpells - 1;
      
      LOOP
        IF ((player^.character^.spells.order [index] <> MCStruct.InvalidSpellCursor) AND (player^.character^.spells.order [index] IN player^.character^.spells.learned)) THEN
          ForgetCharacterSpell (player,player^.character^.spells.order [index]);
          DEC (number);
        END; (* IF *)
        
        IF ((index = 0) OR (number = 0)) THEN
          EXIT;
        END; (* IF *)
        
        DEC (index);
      END; (* LOOP *)
    END ForgetMostRecentSpells;
    
    PROCEDURE ForgetSpellsAboveCurrentLevel (player : MStruct.PlayerInfoPointer);
      VAR
          index : CARDINAL;
      
    BEGIN (* ForgetSpellsAboveCurrentLevel *)
      FOR index := MCStruct.MaxNumberSpells - 1 TO 0 BY -1 DO
        IF ((index IN player^.character^.spells.learned) AND (player^.character^.spells.table [index].level > player^.character^.variables.level)) THEN
          ForgetCharacterSpell (player,index);
        END; (* IF *)
      END; (* FOR *)
    END ForgetSpellsAboveCurrentLevel;
    
    PROCEDURE NumberAvailableSpells (character : MCStruct.CharacterInfoPointer) : CARDINAL;
      VAR
          count : CARDINAL;
          index : CARDINAL;
      
    BEGIN (* NumberAvailableSpells *)
      count := 0;
      
      FOR index := 0 TO MCStruct.MaxNumberSpells - 1 DO
        IF ((character^.spells.table [index].level <= character^.variables.level) AND NOT (index IN character^.spells.learned)) THEN
          INC (count);
        END; (* IF *)
      END; (* FOR *)
      
      RETURN (count);
    END NumberAvailableSpells;
    
    PROCEDURE RememberForgottenSpells (player : MStruct.PlayerInfoPointer; VAR number : INTEGER);
      VAR
          index : CARDINAL;
      
      PROCEDURE RememberForgottenSpell (player : MStruct.PlayerInfoPointer; spell : CARDINAL);
        VAR
            message : ARRAY [0..127] OF CHAR;
        
      BEGIN (* RememberForgottenSpell *)
        EXCL (player^.character^.spells.forgotten,spell);
        INCL (player^.character^.spells.learned,spell);
        
        IF player^.character^.constants.class IN MCStruct.PriestClasses THEN
          WString.ReplaceStringParameter ("You have remembered the prayer of %s.","%s",player^.character^.spells.names [spell],message);
        ELSE
          WString.ReplaceStringParameter ("You have remembered the spell of %s.","%s",player^.character^.spells.names [spell],message);
        END; (* IF *)
        
        MMWindow.DisplayPopupMessage (player^.windows.frame,player,message);
      END RememberForgottenSpell;
      
    BEGIN (* RememberForgottenSpells *)
      index := 0;
      
      WHILE ((index < player^.character^.spells.known) AND (number > 0) AND (player^.character^.spells.forgotten <> MCStruct.PlayerSpellSet {})) DO
        IF ((player^.character^.spells.order [index] IN player^.character^.spells.forgotten) AND (player^.character^.spells.table [player^.character^.spells.order [index]].level <= player^.character^.variables.level)) THEN
          RememberForgottenSpell (player,player^.character^.spells.order [index]);
          DEC (number);
        END; (* IF *)
        
        INC (index);
      END; (* WHILE *)
    END RememberForgottenSpells;
    
  BEGIN (* CalculatePlayerSpells *)
    ForgetSpellsAboveCurrentLevel (player);
    DetermineNumberNewSpells (player^.character,spells);
    
    IF spells > 0 THEN
      RememberForgottenSpells (player,spells);
      spells := WMacro.MinCardinal (NumberAvailableSpells (player^.character),spells);
    ELSIF spells < 0 THEN
      ForgetMostRecentSpells (player,ABS (spells));
      spells := 0;
    END; (* IF *)
    
    IF spells <> VAL (INTEGER,player^.character^.spells.available) THEN
      INCL (player^.character^.variables.status,MCStruct.StudyFLAG);
      
      IF ((spells > 0) AND (player^.character^.spells.available = 0)) THEN
        DisplayLearningMessage (player);
      END; (* IF *)
      
      player^.character^.spells.available := spells;
    END; (* IF *)
  END CalculatePlayerSpells;
  
  PROCEDURE DecreasePlayerMana (player : MStruct.PlayerInfoPointer; cursor : MCStruct.PlayerSpellRange; worked : BOOLEAN);
    PROCEDURE SpellParalysisLength (required,available : CARDINAL) : CARDINAL;
    BEGIN (* SpellParalysisLength *)
      RETURN (MRandom.CardinalRandomRangeValue (1,5 * (required - available)));
    END SpellParalysisLength;
    
    PROCEDURE UpdateSpellWorkedInformation (player : MStruct.PlayerInfoPointer; VAR spells : MCStruct.PlayerSpellInformation; cursor : MCStruct.PlayerSpellRange);
      VAR
          worked : MCStruct.PlayerSpellSet;
      
    BEGIN (* UpdateSpellWorkedInformation *)
      worked := spells.worked;
      
      IF NOT (cursor IN worked) THEN
        MSActs2.IncreasePlayerExperience (player,FLOAT (4 * spells.table [cursor].experience));
        INCL (spells.worked,cursor);
      END; (* IF *)
    END UpdateSpellWorkedInformation;
    
  BEGIN (* DecreasePlayerMana *)
    IF worked THEN
      UpdateSpellWorkedInformation (player,player^.character^.spells,cursor);
    END; (* IF *)
    
    MStruct.CharacterTurnExecuted := TRUE;
    
    IF TRUNC (player^.character^.variables.manaCUR) >= player^.character^.spells.table [cursor].mana THEN
      player^.character^.variables.manaCUR := player^.character^.variables.manaCUR - FLOAT (player^.character^.spells.table [cursor].mana);
    ELSE
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You faint from the effort!");
      
      INC (player^.character^.variables.counters [MCStruct.ParalysisCOUNT],SpellParalysisLength (player^.character^.spells.table [cursor].mana,TRUNC (player^.character^.variables.manaCUR)));
      
      IF MRandom.CardinalRandomRangeValue (1,3) = 1 THEN
        MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You have damaged your health!");
        MSActs2.DecrementStatistic (player,MCStruct.con);
      END; (* IF *)
      
      player^.character^.variables.manaCUR := 0.0;
    END; (* IF *)
    
    INCL (player^.character^.variables.status,MCStruct.ManaFLAG);
  END DecreasePlayerMana;

  PROCEDURE ExecuteCastMagicSpellCommand (player : MStruct.PlayerInfoPointer);
    PROCEDURE SelectAndExecuteMagicSpell (player : MStruct.PlayerInfoPointer);
      VAR
          result : INTEGER;
      
    BEGIN (* SelectAndExecuteMagicSpell *)
      MDialogs.ExecuteMoriaDialogBox (player^.windows.frame,MPBDialog.CharacterSpellDialogTemplate,player,MPBDialog.InitializeSelectSpellDialog,MPBDialog.ExecuteSelectSpellDialogCommand,MPBDialog.CloseSelectSpellDialog,result);
      
      IF result >= 0 THEN
        IF MRandom.CardinalRandomRangeValue (1,100) >= MCSActions.CharacterSpellFailureChance (player^.character,player^.character^.spells.table [result]) THEN
          MCSpell2.ExecuteMagicSpell [result] (player);
        ELSE
          MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You failed to get the spell off!");
          DecreasePlayerMana (player,result,FALSE);
          
          MStruct.CharacterTurnExecuted := TRUE;
        END; (* IF *)
      END; (* IF *)
    END SelectAndExecuteMagicSpell;
    
  BEGIN (* ExecuteCastMagicSpellCommand *)
    MStruct.CharacterTurnExecuted := FALSE;
    
    IF player^.character^.variables.counters [MCStruct.BlindCOUNT] > 0 THEN
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You can't see to read your spell book!");
    ELSIF NOT (DSActions.PlayerHasLight (player)) THEN
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You have no light to read by.");
    ELSIF player^.character^.variables.counters [MCStruct.ConfusedCOUNT] > 0 THEN
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You are too confused.");
    ELSIF NOT (player^.character^.constants.class IN MCStruct.MageClasses) THEN
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You can't cast spells!");
    ELSIF NOT (MInventory.AnyObjectsWithClasses (player^.character^.inventory.backpack,OStruct.ObjectClassSet {OStruct.MagicBook})) THEN
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"But you are not carrying any spell-books!");
    ELSE
      SelectAndExecuteMagicSpell (player);
    END; (* IF *)
  END ExecuteCastMagicSpellCommand;
END MCSpells.
