IMPLEMENTATION MODULE MDColors;
  (*/WIN:A*)
  
  IMPORT WCreation,WDialog,WDTool,Windows,WMacro,WPalette;
  IMPORT SYSTEM;
  
  IMPORT MDialogs,MDPainting,MDSymbols,MDWindow,MInitFile;
  IMPORT MStruct;
  
  
  CONST
        FirstColorFrameControl = 1922;
        FinalColorFrameControl = 1930;
  

  TYPE
       DungeonColorTablePointer = POINTER TO MDSymbols.DungeonColorTable;
  
  VAR
      CurrentBackgroundBrush : Windows.HBRUSH;
      CurrentDialogBrush     : Windows.HBRUSH;
  
  
  PROCEDURE CloseDungeonColorsDialog (window : Windows.HWND; data : SYSTEM.ADDRESS);
  BEGIN (* CloseDungeonColorsDialog *)
    ExitDungeonColorDialog (window,Windows.FALSE_BOOL);
  END CloseDungeonColorsDialog;
  
  PROCEDURE DungeonControlColor (control : Windows.HWND; class : INTEGER; context : Windows.HDC; data : SYSTEM.ADDRESS) : Windows.HBRUSH;
    VAR
        color : Windows.COLORREF;
        done  : BOOLEAN;
    
    PROCEDURE DetermineColorForControl (table : DungeonColorTablePointer; control : CARDINAL; VAR result : Windows.COLORREF; VAR done : BOOLEAN);
    BEGIN (* DetermineColorForControl *)
      done := ((control >= FirstColorFrameControl) AND (control <= FinalColorFrameControl));
      
      IF done THEN
        color := table^ [VAL (MDSymbols.DungeonColorMnemonic,control - FirstColorFrameControl)];
      END; (* IF *)
    END DetermineColorForControl;
    
  BEGIN (* DungeonControlColor *)
    IF CurrentDialogBrush <> Windows.NULL THEN
      WDTool.DeleteObject (CurrentDialogBrush,done);
      CurrentDialogBrush := Windows.NULL;
    END; (*IF *)
    
    DetermineColorForControl (DungeonColorTablePointer (data),WDialog.GetDlgCtrlID (control),color,done);
    
    IF done THEN
      CurrentDialogBrush := WDTool.CreateSolidBrush (color);
    END; (* IF *)
    
    RETURN (CurrentDialogBrush);
  END DungeonControlColor;
  
  PROCEDURE ExecuteDungeonColorsCommand (player : MStruct.PlayerInfoPointer);
    VAR
        result : INTEGER;
        temp   : MDSymbols.DungeonColorTable;
    
  BEGIN (* ExecuteDungeonColorsCommand *)
    temp := MDSymbols.CurrentDungeonColorTable;
    
    MDialogs.ExecuteMoriaColorDialogBox (MStruct.CurrentPlayerFrameWindow,"Dungeon_Colors_Dialog",SYSTEM.ADR (temp),InitializeDungeonColorsDialog,ExecuteDungeonColorsDialogCommand,CloseDungeonColorsDialog,DungeonControlColor,result);
    
    IF result = Windows.TRUE_BOOL THEN
      MDSymbols.CurrentDungeonColorTable := temp;
      
      IF player <> NIL THEN
        InitializeBackgroundColor (player^.windows.dungeon);
        MDPainting.InvalidateDungeonWindow (player);
      END; (* IF *)
    END; (* IF *)
    
    MStruct.CharacterTurnExecuted := FALSE;
  END ExecuteDungeonColorsCommand;
  
  PROCEDURE ExecuteDungeonColorsDialogCommand (window : Windows.HWND; data : SYSTEM.ADDRESS; control : CARDINAL; command : CARDINAL);
    CONST
          FirstColorButton = 1911;
          FinalColorButton = 1919;
    
    CONST
          SaveColorsButton = 1921;
          CancelButton     = Windows.IDCANCEL;
          AcceptButton     = Windows.IDOK;
    
    PROCEDURE ExecuteAcceptButton (window : Windows.HWND; command : CARDINAL);
    BEGIN (* ExecuteAcceptButton *)
      IF command = Windows.BN_CLICKED THEN
        ExitDungeonColorDialog (window,Windows.TRUE_BOOL);
      END; (* IF *)
    END ExecuteAcceptButton;
    
    PROCEDURE ExecuteCancelButton (window : Windows.HWND; command : CARDINAL);
    BEGIN (* ExecuteCancelButton *)
      IF command = Windows.BN_CLICKED THEN
        ExitDungeonColorDialog (window,Windows.FALSE_BOOL);
      END; (* IF *)
    END ExecuteCancelButton;
    
    PROCEDURE ExecuteDungeonColorButton (window : Windows.HWND; table : DungeonColorTablePointer; element : MDSymbols.DungeonColorMnemonic; command : CARDINAL);
      VAR
          custom : WPalette.CustomColorTable;
          result : Windows.COLORREF;
          done   : BOOLEAN;
      
      PROCEDURE InitializeCustomColorTable (table : ARRAY OF Windows.COLORREF; VAR result : ARRAY OF Windows.COLORREF);
        VAR
            index : CARDINAL;
        
      BEGIN (* InitializeCustomColorTable *)
        FOR index := 0 TO HIGH (result) DO
          IF index <= HIGH (table) THEN
            result [index] := table [index];
          ELSE
            result [index] := WMacro.RGB (255,255,255);
          END; (* IF *)
        END; (* FOR *)
      END InitializeCustomColorTable;
      
    BEGIN (* ExecuteDungeonColorButton *)
      IF command = Windows.BN_CLICKED THEN
        InitializeCustomColorTable (table^,custom);
        
        WPalette.ChooseColor (window,custom,table^ [element],result,done);
        
        IF done THEN
          table^ [element] := result;
          WDialog.SetDlgItemText (window,FirstColorFrameControl + ORD (element),"");
        END; (* IF *)
      END; (* IF *)
    END ExecuteDungeonColorButton;
    
    PROCEDURE ExecuteSaveColorsButton (window : Windows.HWND; table : DungeonColorTablePointer; command : CARDINAL);
      VAR
          entry : ARRAY [0..31] OF CHAR;
          index : MDSymbols.DungeonColorMnemonic;
      
    BEGIN (* ExecuteSaveColorsButton *)
      IF command = Windows.BN_CLICKED THEN
        FOR index := MIN (MDSymbols.DungeonColorMnemonic) TO MAX (MDSymbols.DungeonColorMnemonic) DO
          MDSymbols.DungeonColorMnemonicToString (index,entry);
          MInitFile.SetDungeonColorEntry (entry,table^ [index]);
        END; (* FOR *)
        
        ExitDungeonColorDialog (window,Windows.TRUE_BOOL);
      END; (* IF *)
    END ExecuteSaveColorsButton;
    
  BEGIN (* ExecuteDungeonColorsDialogCommand *)
    CASE control OF
      FirstColorButton..FinalColorButton : ExecuteDungeonColorButton (window,DungeonColorTablePointer (data),VAL (MDSymbols.DungeonColorMnemonic,control - FirstColorButton),command); |
      SaveColorsButton                   : ExecuteSaveColorsButton (window,DungeonColorTablePointer (data),command);                                                         |
      AcceptButton                       : ExecuteAcceptButton (window,command);                                                                                             |
      CancelButton                       : ExecuteCancelButton (window,command);                                                                                             |
    ELSE
      (* no action *)
    END; (* CASE *)
  END ExecuteDungeonColorsDialogCommand;
  
  PROCEDURE ExitDungeonColorDialog (window : Windows.HWND; result : INTEGER);
    VAR
        done : BOOLEAN;
    
  BEGIN (* ExitDungeonColorDialog *)
    IF CurrentDialogBrush <> Windows.NULL THEN
      WDTool.DeleteObject (CurrentDialogBrush,done);
      CurrentDialogBrush := Windows.NULL;
    END; (* IF *)
    
    WDialog.EndDialog (window,result);
  END ExitDungeonColorDialog;
  
  PROCEDURE InitializeBackgroundColor (window : Windows.HWND);
    VAR
        previous : Windows.WORD;
        done     : BOOLEAN;
    
  BEGIN (* InitializeBackgroundColor *)
    IF CurrentBackgroundBrush <> Windows.NULL THEN
      WDTool.DeleteObject (CurrentBackgroundBrush,done);
    END; (* IF *)
    
    CurrentBackgroundBrush := WDTool.CreateSolidBrush (MDSymbols.CurrentBackgroundColor ());
    WCreation.SetClassWord (window,Windows.GCW_HBRBACKGROUND,CurrentBackgroundBrush,previous);
  END InitializeBackgroundColor;
  
  PROCEDURE InitializeDungeonColorsDialog (window : Windows.HWND; data : SYSTEM.ADDRESS);
  BEGIN (* InitializeDungeonColorsDialog *)
    (* no action *)
  END InitializeDungeonColorsDialog;
  
  PROCEDURE ReleaseBackgroundBrush;
    VAR
        done : BOOLEAN;
    
  BEGIN (* ReleaseBackgroundBrush *)
    IF CurrentBackgroundBrush <> Windows.NULL THEN
      WDTool.DeleteObject (CurrentBackgroundBrush,done);
    END; (* IF *)
  END ReleaseBackgroundBrush;
  
BEGIN (* MDColors *)
  CurrentBackgroundBrush := Windows.NULL;
  CurrentDialogBrush     := Windows.NULL;
END MDColors.
