IMPLEMENTATION MODULE MDGAlloc;
  (*/WIN:A*)
  
  
  IMPORT DSActions,MCreatures,MDPainting,MDWindow,MMWindow,MRandom,OMagic,OSActions;
  IMPORT CStruct,DStruct,MCStruct,MStruct,OStruct;
  IMPORT WDebugging,WMacro;
  
  
  CONST
        WinMonsterDepth = 50;
  
  
  PROCEDURE AllocateRandomCreatures (player : MStruct.PlayerInfoPointer; VAR dungeon : DStruct.DungeonInformation; number : CARDINAL; distance : CARDINAL; asleep : BOOLEAN);
    PROCEDURE AllocateRandomCreature (player : MStruct.PlayerInfoPointer; VAR dungeon : DStruct.DungeonInformation; distance : CARDINAL; asleep : BOOLEAN);
      VAR
          creature : DStruct.DungeonCreatureRecord;
          position : CARDINAL;
          column   : DStruct.DungeonColumnRange;
          line     : DStruct.DungeonLineRange;
          done     : BOOLEAN;
      
      PROCEDURE DetermineRandomCreatureLocation (player : MStruct.PlayerInfoPointer; VAR dungeon : DStruct.DungeonInformation; distance : CARDINAL; VAR line : DStruct.DungeonLineRange; VAR column : DStruct.DungeonColumnRange);
      BEGIN (* DetermineRandomCreatureLocation *)
        REPEAT
          column := MRandom.CardinalRandomRangeValue (1,dungeon.columns - 2);
          line   := MRandom.CardinalRandomRangeValue (1,dungeon.lines - 2);
        UNTIL (MCreatures.ValidCreaturePlacementLocation (player,line,column,dungeon.physical [line]^ [column]) AND (DSActions.QuickDistance (player^.character^.line,player^.character^.column,line,column) > distance));
      END DetermineRandomCreatureLocation;
      
    BEGIN (* AllocateRandomCreature *)
      MCreatures.DetermineRandomCreatureForDepth (player^.creatures,player^.character^.variables.depthCUR,asleep,player^.character^.variables.speed,creature);
      DetermineRandomCreatureLocation (player,dungeon,distance,line,column);
      
      DSActions.PlaceCreatureAtLocation (player,dungeon,line,column,creature,position,done);
    END AllocateRandomCreature;
    
  BEGIN (* AllocateRandomCreatures *)
    WHILE number > 0 DO
      AllocateRandomCreature (player,dungeon,distance,asleep);
      DEC (number);
    END; (* WHILE *)
  END AllocateRandomCreatures;
  
  PROCEDURE DetermineRandomCharacterLocation (VAR dungeon : DStruct.DungeonInformation; VAR line : DStruct.DungeonLineRange; VAR column : DStruct.DungeonColumnRange);
    PROCEDURE ValidCharacterLocation (location : DStruct.PhysicalDungeonUnit) : BOOLEAN;
    BEGIN (* ValidCharacterLocation *)
      RETURN ((location.elements * DStruct.DungeonElementSet {DStruct.BlockedFloor,DStruct.TemporaryWall1,DStruct.TemporaryWall2,DStruct.GraniteWall,DStruct.MagmaWall,DStruct.QuartzWall,DStruct.BoundaryWall} = DStruct.EmptyDungeonElementSet) AND (location.creature = DStruct.InvalidCreatureCursor) AND (location.artifact = DStruct.InvalidArtifactCursor));
    END ValidCharacterLocation;
    
  BEGIN (* DetermineRandomCharacterLocation *)
    REPEAT
      column := MRandom.CardinalRandomRangeValue (1,dungeon.columns - 2);
      line   := MRandom.CardinalRandomRangeValue (1,dungeon.lines - 2);
    UNTIL ValidCharacterLocation (dungeon.physical [line]^ [column]);
  END DetermineRandomCharacterLocation;
  
  PROCEDURE GenerateNonFixedDungeonElements (player : MStruct.PlayerInfoPointer; VAR dungeon : DStruct.DungeonInformation; allocation : CARDINAL);
    PROCEDURE AllocateRandomObjects (player : MStruct.PlayerInfoPointer; VAR dungeon : DStruct.DungeonInformation; depth : CARDINAL; valid : DStruct.DungeonElementSet; number : INTEGER);
      VAR
          column : DStruct.DungeonColumnRange;
          line   : DStruct.DungeonLineRange;
      
    BEGIN (* AllocateRandomObjects *)
      WHILE number > 0 DO
        DetermineRandomArtifactLocation (player,dungeon,valid,line,column);
        PlaceRandomObject (player,dungeon,line,column,depth,FALSE);
        
        DEC (number);
      END; (* WHILE *)
    END AllocateRandomObjects;
    
    PROCEDURE AllocateRandomRubble (player : MStruct.PlayerInfoPointer; VAR dungeon : DStruct.DungeonInformation; valid : DStruct.DungeonElementSet; number : CARDINAL);
      VAR
          column : DStruct.DungeonColumnRange;
          line   : DStruct.DungeonLineRange;
      
    BEGIN (* AllocateRandomRubble *)
      WHILE number > 0 DO
        DetermineRandomArtifactLocation (player,dungeon,valid,line,column);
        PlaceRubbleAtLocation (player^.treasures,dungeon,line,column);
        
        DEC (number);
      END; (* WHILE *)
    END AllocateRandomRubble;
    
    PROCEDURE AllocateRandomTraps (player : MStruct.PlayerInfoPointer; VAR dungeon : DStruct.DungeonInformation; valid : DStruct.DungeonElementSet; number : CARDINAL);
      VAR
          column : DStruct.DungeonColumnRange;
          line   : DStruct.DungeonLineRange;
      
    BEGIN (* AllocateRandomTraps *)
      WHILE number > 0 DO
        DetermineRandomArtifactLocation (player,dungeon,valid,line,column);
        PlaceTrapAtLocation (player^.treasures,dungeon,line,column);
        
        DEC (number);
      END; (* WHILE *)
    END AllocateRandomTraps;
    
    PROCEDURE AllocateRandomTreasures (player : MStruct.PlayerInfoPointer; VAR dungeon : DStruct.DungeonInformation; valid : DStruct.DungeonElementSet; number : CARDINAL);
      VAR
          column : DStruct.DungeonColumnRange;
          line   : DStruct.DungeonLineRange;
      
    BEGIN (* AllocateRandomTreasures *)
      WHILE number > 0 DO
        DetermineRandomArtifactLocation (player,dungeon,valid,line,column);
        PlaceRandomTreasure (player,player^.dungeon^,line,column,player^.character^.variables.depthCUR,FALSE);
        
        DEC (number);
      END; (* WHILE *)
    END AllocateRandomTreasures;
    
    PROCEDURE DetermineRandomArtifactLocation (player : MStruct.PlayerInfoPointer; VAR dungeon : DStruct.DungeonInformation; valid : DStruct.DungeonElementSet; VAR line : DStruct.DungeonLineRange; VAR column : DStruct.DungeonColumnRange);
      PROCEDURE ValidArtifactLocation (player : MStruct.PlayerInfoPointer; valid : DStruct.DungeonElementSet; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange; location : DStruct.PhysicalDungeonUnit) : BOOLEAN;
      BEGIN (* ValidArtifactLocation *)
        RETURN ((location.artifact = DStruct.InvalidArtifactCursor) AND (location.elements * valid <> DStruct.EmptyDungeonElementSet) AND (line <> player^.character^.line) AND (column <> player^.character^.column));
      END ValidArtifactLocation;
      
    BEGIN (* DetermineRandomArtifactLocation *)
      REPEAT
        column := MRandom.CardinalRandomRangeValue (1,dungeon.columns - 2);
        line   := MRandom.CardinalRandomRangeValue (1,dungeon.lines - 2);
      UNTIL ValidArtifactLocation (player,valid,line,column,dungeon.physical [line]^ [column]);
    END DetermineRandomArtifactLocation;
    
    PROCEDURE GenerateWinningMonster (player : MStruct.PlayerInfoPointer);
      VAR
          creature : DStruct.DungeonCreatureRecord;
          position : CARDINAL;
          column   : DStruct.DungeonColumnRange;
          line     : DStruct.DungeonLineRange;
          done     : BOOLEAN;
      
      PROCEDURE DetermineWinningMonsterPosition (player : MStruct.PlayerInfoPointer; VAR line : DStruct.DungeonLineRange; VAR column : DStruct.DungeonColumnRange);
        PROCEDURE ValidWinningMonsterPosition (player : MStruct.PlayerInfoPointer; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange; location : DStruct.PhysicalDungeonUnit) : BOOLEAN;
        BEGIN (* ValidWinningMonsterPosition *)
          RETURN ((location.elements * DStruct.DungeonFloorElements <> DStruct.EmptyDungeonElementSet) AND (location.creature = DStruct.InvalidCreatureCursor) AND (location.artifact = DStruct.InvalidArtifactCursor) AND (DSActions.QuickDistance (player^.character^.line,player^.character^.column,line,column) > MCStruct.MaxCharacterSightDistance));
        END ValidWinningMonsterPosition;
        
      BEGIN (* DetermineWinningMonsterPosition *)
        REPEAT
          column := MRandom.CardinalRandomRangeValue (1,player^.dungeon^.columns - 1);
          line   := MRandom.CardinalRandomRangeValue (1,player^.dungeon^.lines - 1);
        UNTIL ValidWinningMonsterPosition (player,line,column,player^.dungeon^.physical [line]^ [column]);
      END DetermineWinningMonsterPosition;
      
      PROCEDURE RandomWinningCreature () : CARDINAL;
        CONST
              EvilIggyCursor = CStruct.MaxNumberCreatures - 2;
              BalrogCursor   = CStruct.MaxNumberCreatures - 1;
        
      BEGIN (* RandomWinningCreature *)
        RETURN (MRandom.CardinalRandomRangeValue (EvilIggyCursor,BalrogCursor));
      END RandomWinningCreature;
      
    BEGIN (* GenerateWinningMonster *)
      MCreatures.DetermineCreatureInformation (player^.creatures,RandomWinningCreature (),FALSE,player^.character^.variables.speed,creature);
      DetermineWinningMonsterPosition (player,line,column);
      
      DSActions.PlaceCreatureAtLocation (player,player^.dungeon^,line,column,creature,position,done);
    END GenerateWinningMonster;
    
  BEGIN (* GenerateNonFixedDungeonElements *)
    DetermineRandomCharacterLocation (dungeon,player^.character^.line,player^.character^.column);
    
    AllocateRandomCreatures (player,dungeon,14 + MRandom.CardinalRandomRangeValue (1,8) + allocation,0,TRUE);
    
    AllocateRandomRubble (player,dungeon,DStruct.DungeonElementSet {DStruct.CorridorFloor,DStruct.BlockedFloor},MRandom.CardinalRandomRangeValue (1,allocation));
    AllocateRandomObjects (player,dungeon,player^.character^.variables.depthCUR,DStruct.DungeonElementSet {DStruct.DarkFloor,DStruct.LightFloor},MRandom.RandomNormalValue (7,3));
    AllocateRandomObjects (player,dungeon,player^.character^.variables.depthCUR,DStruct.DungeonElementSet {DStruct.CorridorFloor,DStruct.BlockedFloor,DStruct.DarkFloor,DStruct.LightFloor},WMacro.MaxInteger (MRandom.RandomNormalValue (2,3),0));
    AllocateRandomTreasures (player,dungeon,DStruct.DungeonElementSet {DStruct.CorridorFloor,DStruct.BlockedFloor,DStruct.DarkFloor,DStruct.LightFloor},WMacro.MaxInteger (MRandom.RandomNormalValue (2,3),0));
    AllocateRandomTraps (player,dungeon,DStruct.DungeonElementSet {DStruct.CorridorFloor,DStruct.BlockedFloor,DStruct.DarkFloor,DStruct.LightFloor},MRandom.CardinalRandomRangeValue (1,allocation));
    
    IF player^.character^.variables.depthCUR >= WinMonsterDepth THEN
      GenerateWinningMonster (player);
    END; (* IF *)
  END GenerateNonFixedDungeonElements;
  
  PROCEDURE GenerateRandomTreasures (player : MStruct.PlayerInfoPointer; VAR dungeon : DStruct.DungeonInformation; centerLINE : DStruct.DungeonLineRange; centerCOLUMN : DStruct.DungeonColumnRange; depth : CARDINAL; number : CARDINAL);
    PROCEDURE GenerateRandomTreasure (player : MStruct.PlayerInfoPointer; VAR dungeon : DStruct.DungeonInformation; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange; depth : CARDINAL);
      VAR
          done : BOOLEAN;
      
      PROCEDURE DetermineRandomTreasurePosition (VAR dungeon : DStruct.DungeonInformation; centerLINE : DStruct.DungeonLineRange; centerCOLUMN : DStruct.DungeonColumnRange; VAR resultLINE : DStruct.DungeonLineRange; VAR resultCOLUMN : DStruct.DungeonColumnRange; VAR done : BOOLEAN);
        VAR
            tempCOLUMN : INTEGER;
            tempLINE   : INTEGER;
            attempts   : CARDINAL;
        
        PROCEDURE ValidTreasureLocation (location : DStruct.PhysicalDungeonUnit) : BOOLEAN;
        BEGIN (* ValidTreasureLocation *)
          RETURN ((location.elements * DStruct.DungeonFloorElements <> DStruct.EmptyDungeonElementSet) AND (location.artifact = DStruct.InvalidArtifactCursor));
        END ValidTreasureLocation;
        
      BEGIN (* DetermineRandomTreasurePosition *)
        attempts := 0;
        
        REPEAT
          tempCOLUMN := centerCOLUMN - 4 + MRandom.IntegerRandomRangeValue (1,7);
          tempLINE   := centerLINE - 3 + MRandom.IntegerRandomRangeValue (1,5);
          attempts   := attempts + 1;
          done       := (DSActions.LocationInBounds (dungeon,tempLINE,tempCOLUMN) AND ValidTreasureLocation (dungeon.physical [tempLINE]^ [tempCOLUMN]));
        UNTIL (done OR (attempts = 10));
        
        IF done THEN
          resultCOLUMN := tempCOLUMN;
          resultLINE   := tempLINE;
        END; (* IF *)
      END DetermineRandomTreasurePosition;
      
    BEGIN (* GenerateRandomTreasure *)
      DetermineRandomTreasurePosition (dungeon,line,column,line,column,done);
      
      IF done THEN
        IF MRandom.CardinalRandomRangeValue (1,100) < 75 THEN
          PlaceRandomObject (player,dungeon,line,column,depth,TRUE);
        ELSE
          PlaceRandomTreasure (player,dungeon,line,column,depth,TRUE);
        END; (* IF *)
      END; (* IF *)
    END GenerateRandomTreasure;
    
  BEGIN (* GenerateRandomTreasures *)
    WHILE number > 0 DO
      GenerateRandomTreasure (player,dungeon,centerLINE,centerCOLUMN,depth);
      DEC (number);
    END; (* WHILE *)
  END GenerateRandomTreasures;
  
  PROCEDURE PlaceRandomObject (player : MStruct.PlayerInfoPointer; VAR dungeon : DStruct.DungeonInformation; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange; depth : CARDINAL; message : BOOLEAN);
    VAR
        item : OStruct.InventoryItemRecord;
    
    PROCEDURE RandomObjectForDepth (treasures : OStruct.TreasureInfoPointer; depth : CARDINAL) : CARDINAL;
      PROCEDURE UnsortedObjectCursor (treasures : OStruct.TreasureInfoPointer; depth : CARDINAL) : CARDINAL;
        PROCEDURE TripletMaximum (treasures : OStruct.TreasureInfoPointer; depth : CARDINAL) : CARDINAL;
        BEGIN (* TripletMaximum *)
          RETURN (WMacro.MaxCardinal (MRandom.CardinalRandomRangeValue (0,treasures^.depth [depth] - 1),WMacro.MaxCardinal (MRandom.CardinalRandomRangeValue (0,treasures^.depth [depth] - 1),MRandom.CardinalRandomRangeValue (0,treasures^.depth [depth] - 1))));
        END TripletMaximum;
        
      BEGIN (* UnsortedObjectCursor *)
        IF depth = 0 THEN
          RETURN (MRandom.CardinalRandomRangeValue (0,treasures^.depth [0] - 1));
        END; (* IF *)
        
        IF MRandom.CardinalRandomRangeValue (1,OStruct.GreatObjectChance) = 1 THEN
          depth := depth * OStruct.MaxObjectDepth DIV MRandom.CardinalRandomRangeValue (1,OStruct.MaxObjectDepth) + 1;
        END; (* IF *)
        
        depth := WMacro.MinCardinal (depth,OStruct.MaxObjectDepth);
        
        IF MRandom.CardinalRandomRangeValue (1,2) = 1 THEN
          RETURN (MRandom.CardinalRandomRangeValue (0,treasures^.depth [depth] - 1));
        END; (* IF *)
        
        depth := treasures^.table [TripletMaximum (treasures,depth)].depth;
        
        IF depth > 0 THEN
          RETURN (MRandom.CardinalRandomRangeValue (treasures^.depth [depth - 1],treasures^.depth [depth] - 1));
        ELSE
          RETURN (MRandom.CardinalRandomRangeValue (0,treasures^.depth [0] - 1));
        END; (* IF *)
      END UnsortedObjectCursor;
      
    BEGIN (* RandomObjectForDepth *)
      RETURN (treasures^.index [UnsortedObjectCursor (treasures,depth)]);
    END RandomObjectForDepth;
    
  BEGIN (* PlaceRandomObject *)
    OSActions.ReturnInventoryItemRecord (player^.treasures,RandomObjectForDepth (player^.treasures,depth),item);
    OMagic.DetermineObjectMagic (item,depth);
    
    DSActions.PlaceDungeonArtifact (dungeon,line,column,item);
    
    IF (message AND (line = player^.character^.line) AND (column = player^.character^.column)) THEN
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You feel something roll beneath your feet.");
    ELSE
      MDPainting.InvalidateDungeonLocation (player,line,column,FALSE,TRUE);
    END; (* IF *)
  END PlaceRandomObject;
  
  PROCEDURE PlaceRandomTreasure (player : MStruct.PlayerInfoPointer; VAR dungeon : DStruct.DungeonInformation; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange; depth : CARDINAL; message : BOOLEAN);
    VAR
        item : OStruct.InventoryItemRecord;
    
    PROCEDURE RandomTreasureCursor (depth : CARDINAL) : CARDINAL;
      CONST
            MaximumTreasureOffset = 17;
            BaseTreasureCursor    = 399;
      
      VAR
          offset : CARDINAL;
      
    BEGIN (* RandomTreasureCursor *)
      offset := (MRandom.CardinalRandomRangeValue (1,depth + 2) + 2) DIV 2 - 1;
      
      IF MRandom.CardinalRandomRangeValue (1,OStruct.GreatObjectChance) = 1 THEN
        INC (offset,MRandom.CardinalRandomRangeValue (1,depth + 1));
      END; (* IF *)
      
      RETURN (BaseTreasureCursor + WMacro.MinCardinal (offset,MaximumTreasureOffset));
    END RandomTreasureCursor;
    
  BEGIN (* PlaceRandomTreasure *)
    OSActions.ReturnInventoryItemRecord (player^.treasures,RandomTreasureCursor (depth),item);
    item.object.value := VAL (LONGINT,8 * MRandom.CardinalRandomRangeValue (1,item.object.value) + MRandom.CardinalRandomRangeValue (1,8));
    
    DSActions.PlaceDungeonArtifact (dungeon,line,column,item);
    
    IF (message AND (line = player^.character^.line) AND (column = player^.character^.column)) THEN
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You feel something roll beneath your feet.");
    ELSE
      MDPainting.InvalidateDungeonLocation (player,line,column,FALSE,TRUE);
    END; (* IF *)
  END PlaceRandomTreasure;

  PROCEDURE PlaceRubbleAtLocation (treasures : OStruct.TreasureInfoPointer; VAR dungeon : DStruct.DungeonInformation; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange);
    VAR
        item : OStruct.InventoryItemRecord;
    
  BEGIN (* PlaceRubbleAtLocation *)
    OSActions.ReturnInventoryItemRecord (treasures,OStruct.RubbleCursor,item);
    
    DSActions.SetDungeonLocationFVal (dungeon.physical [line]^ [column],DStruct.BlockedFloor);
    DSActions.PlaceDungeonArtifact (dungeon,line,column,item);
  END PlaceRubbleAtLocation;
  
  PROCEDURE PlaceTrapAtLocation (treasures : OStruct.TreasureInfoPointer; VAR dungeon : DStruct.DungeonInformation; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange);
    VAR
        item : OStruct.InventoryItemRecord;
    
    PROCEDURE RandomTrapCursor () : CARDINAL;
    BEGIN (* RandomTrapCursor *)
      RETURN (378 + MRandom.CardinalRandomRangeValue (0,17));
    END RandomTrapCursor;
    
  BEGIN (* PlaceTrapAtLocation *)
    OSActions.ReturnInventoryItemRecord (treasures,RandomTrapCursor (),item);
    DSActions.PlaceDungeonArtifact (dungeon,line,column,item);
  END PlaceTrapAtLocation;
END MDGAlloc.
