IMPLEMENTATION MODULE MDGDoors;
  (*/WIN:A*)

  IMPORT DSActions,DStruct,MDGenerator,OSActions,OStruct;
  IMPORT MRandom;
  
  
  PROCEDURE GenerateDungeonDoors (treasures : OStruct.TreasureInfoPointer; VAR dungeon : DStruct.DungeonInformation; VAR doors : MDGenerator.DungeonLocationTable);
    VAR
        index : CARDINAL;
    
    PROCEDURE AttemptJunctionDoorPlacement (treasures : OStruct.TreasureInfoPointer; VAR dungeon : DStruct.DungeonInformation; location : MDGenerator.DungeonLocation);
      PROCEDURE AttemptCorridorDoorPlacement (treasures : OStruct.TreasureInfoPointer; VAR dungeon : DStruct.DungeonInformation; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange);
        PROCEDURE LocationBetweenTwoWalls (VAR dungeon : DStruct.DungeonInformation; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange) : BOOLEAN;
          PROCEDURE HorizontalBracingWalls (VAR dungeon : DStruct.DungeonInformation; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange) : BOOLEAN;
          BEGIN (* HorizontalBracingWalls *)
            RETURN ((dungeon.physical [line]^ [column - 1].elements * DStruct.DungeonWallElements <> DStruct.EmptyDungeonElementSet) AND (dungeon.physical [line]^ [column + 1].elements * DStruct.DungeonWallElements <> DStruct.EmptyDungeonElementSet));
          END HorizontalBracingWalls;
          
          PROCEDURE VerticalBracingWalls (VAR dungeon : DStruct.DungeonInformation; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange) : BOOLEAN;
          BEGIN (* VerticalBracingWalls *)
            RETURN ((dungeon.physical [line - 1]^ [column].elements * DStruct.DungeonWallElements <> DStruct.EmptyDungeonElementSet) AND (dungeon.physical [line + 1]^ [column].elements * DStruct.DungeonWallElements <> DStruct.EmptyDungeonElementSet));
          END VerticalBracingWalls;
          
        BEGIN (* LocationBetweenTwoWalls *)
          RETURN ((DSActions.AdjacentCorridorCount (dungeon,line,column) > 2) AND (HorizontalBracingWalls (dungeon,line,column) OR VerticalBracingWalls (dungeon,line,column)));
        END LocationBetweenTwoWalls;
        
      BEGIN (* AttemptCorridorDoorPlacement *)
        IF ((DStruct.CorridorFloor IN dungeon.physical [line]^ [column].elements) AND (MRandom.CardinalRandomRangeValue (1,100) > 15) AND LocationBetweenTwoWalls (dungeon,line,column)) THEN
          GenerateRandomDoor (treasures,dungeon,line,column);
        END; (* IF *)
      END AttemptCorridorDoorPlacement;
      
    BEGIN (* AttemptJunctionDoorPlacement *)
      AttemptCorridorDoorPlacement (treasures,dungeon,location.line - 1,location.column);
      AttemptCorridorDoorPlacement (treasures,dungeon,location.line + 1,location.column);
      AttemptCorridorDoorPlacement (treasures,dungeon,location.line,location.column - 1);
      AttemptCorridorDoorPlacement (treasures,dungeon,location.line,location.column + 1);
    END AttemptJunctionDoorPlacement;
    
  BEGIN (* GenerateDungeonDoors *)
    IF doors.counter > 0 THEN
      FOR index := 0 TO doors.counter - 1 DO
        AttemptJunctionDoorPlacement (treasures,dungeon,doors.locations [index]);
      END; (* FOR *)
    END; (* IF *)
  END GenerateDungeonDoors;
  
  PROCEDURE GenerateLockedDoor (treasures : OStruct.TreasureInfoPointer; VAR dungeon : DStruct.DungeonInformation; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange);
    VAR
        item : OStruct.InventoryItemRecord;
    
  BEGIN (* GenerateLockedDoor *)
    OSActions.ReturnInventoryItemRecord (treasures,OStruct.ClosedDoorObjectCursor,item);
    item.object.miscellaneous := MRandom.CardinalRandomRangeValue (20,30);
    
    DSActions.SetDungeonLocationFVal (dungeon.physical [line]^ [column],DStruct.BlockedFloor);
    DSActions.PlaceDungeonArtifact (dungeon,line,column,item);
  END GenerateLockedDoor;
  
  PROCEDURE GenerateRandomDoor (treasures : OStruct.TreasureInfoPointer; VAR dungeon : DStruct.DungeonInformation; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange);
    PROCEDURE GenerateClosedDoor (treasures : OStruct.TreasureInfoPointer; VAR dungeon : DStruct.DungeonInformation; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange);
      PROCEDURE PlaceStuckDoor (treasures : OStruct.TreasureInfoPointer; VAR dungeon : DStruct.DungeonInformation; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange);
        VAR
            item : OStruct.InventoryItemRecord;
        
      BEGIN (* PlaceStuckDoor *)
        OSActions.ReturnInventoryItemRecord (treasures,OStruct.ClosedDoorObjectCursor,item);
        item.object.miscellaneous := -MRandom.IntegerRandomRangeValue (20,30);
        
        DSActions.SetDungeonLocationFVal (dungeon.physical [line]^ [column],DStruct.BlockedFloor);
        DSActions.PlaceDungeonArtifact (dungeon,line,column,item);
      END PlaceStuckDoor;
      
    BEGIN (* GenerateClosedDoor *)
      CASE MRandom.CardinalRandomRangeValue (1,12) OF
        1..2  : GenerateLockedDoor (treasures,dungeon,line,column); |
        3     : PlaceStuckDoor (treasures,dungeon,line,column);  |
        4..12 : PlaceClosedDoor (treasures,dungeon,line,column); |
      END; (* CASE *)
    END GenerateClosedDoor;
    
    PROCEDURE GenerateOpenDoor (treasures : OStruct.TreasureInfoPointer; VAR dungeon : DStruct.DungeonInformation; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange);
      PROCEDURE PlaceBrokenDoor (treasures : OStruct.TreasureInfoPointer; VAR dungeon : DStruct.DungeonInformation; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange);
        VAR
            item : OStruct.InventoryItemRecord;
        
      BEGIN (* PlaceBrokenDoor *)
        OSActions.ReturnInventoryItemRecord (treasures,OStruct.OpenDoorObjectCursor,item);
        item.object.miscellaneous := 1;
        
        INCL (dungeon.physical [line]^ [column].elements,DStruct.CorridorFloor);
        DSActions.PlaceDungeonArtifact (dungeon,line,column,item);
      END PlaceBrokenDoor;
      
      PROCEDURE PlaceOpenDoor (treasures : OStruct.TreasureInfoPointer; VAR dungeon : DStruct.DungeonInformation; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange);
        VAR
            item : OStruct.InventoryItemRecord;
        
      BEGIN (* PlaceOpenDoor *)
        OSActions.ReturnInventoryItemRecord (treasures,OStruct.OpenDoorObjectCursor,item);
        
        INCL (dungeon.physical [line]^ [column].elements,DStruct.CorridorFloor);
        DSActions.PlaceDungeonArtifact (dungeon,line,column,item);
      END PlaceOpenDoor;
      
    BEGIN (* GenerateOpenDoor *)
      IF MRandom.CardinalRandomRangeValue (1,4) = 1 THEN
        PlaceBrokenDoor (treasures,dungeon,line,column);
      ELSE
        PlaceOpenDoor (treasures,dungeon,line,column);
      END; (* IF *)
    END GenerateOpenDoor;
    
  BEGIN (* GenerateRandomDoor *)
    CASE MRandom.CardinalRandomRangeValue (1,4) OF
      1,2 : GenerateSecretDoor (treasures,dungeon,line,column); |
      3   : GenerateClosedDoor (treasures,dungeon,line,column); |
      4   : GenerateOpenDoor (treasures,dungeon,line,column);   |
    END; (* CASE *)
  END GenerateRandomDoor;

  PROCEDURE GenerateSecretDoor (treasures : OStruct.TreasureInfoPointer; VAR dungeon : DStruct.DungeonInformation; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange);
    VAR
        item : OStruct.InventoryItemRecord;
    
  BEGIN (* GenerateSecretDoor *)
    OSActions.ReturnInventoryItemRecord (treasures,369,item);
    DSActions.PlaceDungeonArtifact (dungeon,line,column,item);
    
    DSActions.SetDungeonLocationFVal (dungeon.physical [line]^ [column],DStruct.BlockedFloor);
  END GenerateSecretDoor;

  PROCEDURE PlaceClosedDoor (treasures : OStruct.TreasureInfoPointer; VAR dungeon : DStruct.DungeonInformation; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange);
    VAR
        item : OStruct.InventoryItemRecord;
    
  BEGIN (* PlaceClosedDoor *)
    OSActions.ReturnInventoryItemRecord (treasures,OStruct.ClosedDoorObjectCursor,item);
    
    DSActions.SetDungeonLocationFVal (dungeon.physical [line]^ [column],DStruct.BlockedFloor);
    DSActions.PlaceDungeonArtifact (dungeon,line,column,item);
  END PlaceClosedDoor;
END MDGDoors.
