IMPLEMENTATION MODULE MDGenerator;
  (*/WIN:A*)
  
  FROM WMemory IMPORT ALLOCATE,DEALLOCATE;
  
  IMPORT Windows,WMacro;
  
  IMPORT MCreatures,MDPainting,MDWindow,MPMovement,MRandom,MSUpdate;
  IMPORT MDGAlloc,MDGDoors,MDGRooms,MDGTown,MDGTunnel;
  IMPORT CStruct,DStruct,MCStruct,MStruct,OStruct;
  IMPORT DSActions,OSActions;
  
  
  PROCEDURE DetermineRandomStairLocation (VAR dungeon : DStruct.DungeonInformation; walls : CARDINAL; VAR line : DStruct.DungeonLineRange; VAR column : DStruct.DungeonColumnRange);
    VAR
        done : BOOLEAN;
    
    PROCEDURE StairSearchDungeonLocation (VAR dungeon : DStruct.DungeonInformation; walls : CARDINAL; VAR line : DStruct.DungeonLineRange; VAR column : DStruct.DungeonColumnRange; VAR done : BOOLEAN);
      VAR
          minX,minY : CARDINAL;
          tries     : CARDINAL;
      
      PROCEDURE StairSearchDungeonArea (VAR dungeon : DStruct.DungeonInformation; minX,maxX : DStruct.DungeonColumnRange; minY,maxY : DStruct.DungeonLineRange; walls : CARDINAL; VAR line : DStruct.DungeonLineRange; VAR column : DStruct.DungeonColumnRange; VAR done : BOOLEAN);
        PROCEDURE ValidStairLocation (VAR dungeon : DStruct.DungeonInformation; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange; walls : CARDINAL) : BOOLEAN;
        BEGIN (* ValidStairLocation *)
          RETURN ((dungeon.physical [line]^ [column].artifact = DStruct.InvalidArtifactCursor) AND (dungeon.physical [line]^ [column].elements * DStruct.DungeonElementSet {DStruct.BlockedFloor,DStruct.TemporaryWall1,DStruct.TemporaryWall2,DStruct.GraniteWall,DStruct.MagmaWall,DStruct.QuartzWall,DStruct.BoundaryWall} = DStruct.EmptyDungeonElementSet) AND (DSActions.AdjacentWallCount (dungeon,line,column) >= walls));
        END ValidStairLocation;
        
      BEGIN (* StairSearchDungeonArea *)
        column := minX;
        line   := minY;
        
        LOOP
          done := ValidStairLocation (dungeon,line,column,walls);
          
          IF (done OR ((line = maxY) AND (column = maxX))) THEN
            EXIT;
          ELSIF column = maxX THEN
            column := minX;
            line   := line + 1;
          ELSE
            column := column + 1;
          END; (* IF *)
        END; (* LOOP *)
      END StairSearchDungeonArea;
      
    BEGIN (* StairSearchDungeonLocation *)
      tries := 0;

      REPEAT
        minX := MRandom.CardinalRandomRangeValue (1,dungeon.columns - 14);
        minY := MRandom.CardinalRandomRangeValue (1,dungeon.lines - 14);
        
        StairSearchDungeonArea (dungeon,minX,minX + 12,minY,minY + 12,walls,line,column,done);
        
        INC (tries);
      UNTIL (done OR (tries = 30));
    END StairSearchDungeonLocation;
    
  BEGIN (* DetermineRandomStairLocation *)
    LOOP
      StairSearchDungeonLocation (dungeon,walls,line,column,done);
      
      IF done THEN
        EXIT;
      END; (* IF *)
      
      DEC (walls);
    END; (* LOOP *)
  END DetermineRandomStairLocation;
  
  PROCEDURE GeneratePlayerDungeon (player : MStruct.PlayerInfoPointer);
    PROCEDURE ClearDungeonInformation (treasure : OStruct.TreasureInfoPointer; VAR dungeon : DStruct.DungeonInformation);
      VAR
          index : CARDINAL;
      
      PROCEDURE ClearDungeonCreatures (VAR table : ARRAY OF DStruct.DungeonCreatureRecord);
        VAR
            index : CARDINAL;
        
      BEGIN (* ClearDungeonCreatures *)
        FOR index := 0 TO HIGH (table) DO
          table [index].cursor := CStruct.InvalidCreatureCursor;
        END; (* FOR *)
      END ClearDungeonCreatures;
      
      PROCEDURE ClearPhysicalDungeonLine (VAR line : DStruct.PhysicalDungeonLine);
        VAR
            column : CARDINAL;
        
      BEGIN (* ClearPhysicalDungeonLine *)
        WITH line [0] DO
          artifact := DStruct.InvalidArtifactCursor;
          creature := DStruct.InvalidCreatureCursor;
          elements := DStruct.DungeonElementSet {DStruct.NullWall};
        END; (* WITH *)
        
        FOR column := 1 TO DStruct.NumberDungeonColumns - 1 DO
          line [column] := line [0];
        END; (* FOR *)
      END ClearPhysicalDungeonLine;
      
    BEGIN (* ClearDungeonInformation *)
      ClearPhysicalDungeonLine (dungeon.physical [0]^);
      
      FOR index := 1 TO DStruct.NumberDungeonLines - 1 DO
        dungeon.physical [index]^ := dungeon.physical [0]^;
      END; (* FOR *)
      
      OSActions.ClearInventoryItemArray (treasure,dungeon.artifacts);
      ClearDungeonCreatures (dungeon.creatures);
      
      dungeon.multipliedCreatures := 0;
      dungeon.columns             := 0;
      dungeon.lines               := 0;
    END ClearDungeonInformation;
    
    PROCEDURE GenerateRandomDungeonLevel (player : MStruct.PlayerInfoPointer);
      VAR
          info : DungeonGeneratorInfoPointer;
      
      PROCEDURE AllocationLevel (depth : CARDINAL) : CARDINAL;
      BEGIN (* AllocationLevel *)
        RETURN (WMacro.MinCardinal (WMacro.MaxCardinal (depth DIV 3,2),10));
      END AllocationLevel;
      
      PROCEDURE GenerateDungeonStairs (treasures : OStruct.TreasureInfoPointer; VAR dungeon : DStruct.DungeonInformation);
        PROCEDURE GenerateRandomDungeonStairs (treasures : OStruct.TreasureInfoPointer; VAR dungeon : DStruct.DungeonInformation; number : CARDINAL; walls : CARDINAL; down : BOOLEAN);
          VAR
              column : DStruct.DungeonColumnRange;
              line   : DStruct.DungeonLineRange;
          
        BEGIN (* GenerateRandomDungeonStairs *)
          WHILE number > 0 DO
            DetermineRandomStairLocation (dungeon,walls,line,column);
            
            IF down THEN
              PlaceDownStair (treasures,dungeon,line,column);
            ELSE
              PlaceUpStair (treasures,dungeon,line,column);
            END; (* IF *)
            
            DEC (number);
          END; (* WHILE *)
        END GenerateRandomDungeonStairs;
        
      BEGIN (* GenerateDungeonStairs *)
        GenerateRandomDungeonStairs (treasures,dungeon,MRandom.CardinalRandomRangeValue (1,2),3,FALSE);
        GenerateRandomDungeonStairs (treasures,dungeon,MRandom.CardinalRandomRangeValue (3,4),3,TRUE);
      END GenerateDungeonStairs;
      
      PROCEDURE GenerateDungeonWalls (player : MStruct.PlayerInfoPointer; VAR dungeon : DStruct.DungeonInformation);
        CONST
              NumberQuartzSeams = 2;
              NumberMagaSeams   = 3;
        
        CONST
              QuartzTreasureChance = 40;
              MagmaTreasureChance  = 90;
        
        PROCEDURE GenerateDungeonBoundary (VAR dungeon : DStruct.DungeonInformation);
          VAR
              column : DStruct.DungeonColumnRange;
              line   : DStruct.DungeonLineRange;
          
        BEGIN (* GenerateDungeonBoundary *)
          FOR column := 0 TO dungeon.columns - 1 DO
            DSActions.SetDungeonLocationFVal (dungeon.physical [dungeon.lines - 1]^ [column],DStruct.BoundaryWall);
            DSActions.SetDungeonLocationFVal (dungeon.physical [0]^ [column],DStruct.BoundaryWall);
          END; (* FOR *)
          
          FOR line := 0 TO dungeon.lines - 1 DO
            DSActions.SetDungeonLocationFVal (dungeon.physical [line]^ [dungeon.columns - 1],DStruct.BoundaryWall);
            DSActions.SetDungeonLocationFVal (dungeon.physical [line]^ [0],DStruct.BoundaryWall);
          END; (* FOR *)
        END GenerateDungeonBoundary;
        
        PROCEDURE GenerateGraniteWalls (VAR dungeon : DStruct.DungeonInformation);
          VAR
              line : DStruct.DungeonLineRange;
          
          PROCEDURE GenerateGraniteDungeonLine (VAR line : ARRAY OF DStruct.PhysicalDungeonUnit);
            VAR
                column : CARDINAL;
            
          BEGIN (* GenerateGraniteDungeonLine *)
            FOR column := 0 TO HIGH (line) DO
              IF line [column].elements * DStruct.DungeonElementSet {DStruct.NullWall,DStruct.TemporaryWall1,DStruct.TemporaryWall2} <> DStruct.EmptyDungeonElementSet THEN
                line [column].elements := (line [column].elements - DStruct.DungeonFValElements) + DStruct.DungeonElementSet {DStruct.GraniteWall};
              END; (* IF *)
            END; (* FOR *)
          END GenerateGraniteDungeonLine;
          
        BEGIN (* GenerateGraniteWalls *)
          FOR line := 0 TO dungeon.lines - 1 DO
            GenerateGraniteDungeonLine (dungeon.physical [line]^);
          END; (* FOR *)
        END GenerateGraniteWalls;
        
        PROCEDURE GenerateRockSeams (player : MStruct.PlayerInfoPointer; VAR dungeon : DStruct.DungeonInformation; element : DStruct.DungeonElementMnemonic; number : CARDINAL; chance : CARDINAL);
          PROCEDURE GenerateRockSeam (player : MStruct.PlayerInfoPointer; VAR dungeon : DStruct.DungeonInformation; element : DStruct.DungeonElementMnemonic; chance : CARDINAL);
            VAR
                direction : CARDINAL;
                column    : DStruct.DungeonColumnRange;
                line      : DStruct.DungeonLineRange;
            
            PROCEDURE AttemptRockSeamLocation (player : MStruct.PlayerInfoPointer; VAR dungeon : DStruct.DungeonInformation; lineCENTER : DStruct.DungeonLineRange; columnCENTER : DStruct.DungeonColumnRange; element : DStruct.DungeonElementMnemonic; chance : CARDINAL);
              CONST
                    StreamerDensity = 5;
                    StreamerWidth   = 2;
              
              VAR
                  column : INTEGER;
                  count  : CARDINAL;
                  line   : INTEGER;
              
            BEGIN (* AttemptRockSeamLocation *)
              FOR count := 1 TO StreamerDensity DO
                column := columnCENTER + MRandom.IntegerRandomRangeValue (1,2 * StreamerWidth + 1) - (StreamerWidth + 1);
                line   := lineCENTER + MRandom.IntegerRandomRangeValue (1,2 * StreamerWidth + 1) - (StreamerWidth + 1);
                
                IF (DSActions.LocationInBounds (dungeon,line,column) AND (DStruct.GraniteWall IN dungeon.physical [line]^ [column].elements)) THEN
                  EXCL (dungeon.physical [line]^ [column].elements,DStruct.GraniteWall);
                  INCL (dungeon.physical [line]^ [column].elements,element);
                  
                  IF MRandom.CardinalRandomRangeValue (1,chance) = 1 THEN
                    MDGAlloc.PlaceRandomTreasure (player,player^.dungeon^,line,column,player^.character^.variables.depthCUR,FALSE);
                  END; (* IF *)
                END; (* IF *)
              END; (* FOR *)
            END AttemptRockSeamLocation;
            
            PROCEDURE RandomDirection () : CARDINAL;
              VAR
                  value : CARDINAL;
              
            BEGIN (* RandomDirection *)
              value := MRandom.CardinalRandomRangeValue (1,8);
              
              IF value > 4 THEN
                INC (value);
              END; (* IF *)
              
              RETURN (value);
            END RandomDirection;
            
          BEGIN (* GenerateRockSeam *)
            direction := RandomDirection ();
            column    := dungeon.columns DIV 2 + 16 - MRandom.IntegerRandomRangeValue (1,33);
            line      := dungeon.lines DIV 2 + 11 - MRandom.IntegerRandomRangeValue (1,23);
            
            REPEAT
              AttemptRockSeamLocation (player,dungeon,line,column,element,chance);
              DSActions.DetermineDungeonLocation (line,column,direction,line,column);
            UNTIL NOT (DSActions.LocationInBounds (dungeon,line,column));
          END GenerateRockSeam;
          
        BEGIN (* GenerateRockSeams *)
          WHILE number > 0 DO
            GenerateRockSeam (player,dungeon,element,chance);
            DEC (number);
          END; (* WHILE *)
        END GenerateRockSeams;
        
      BEGIN (* GenerateDungeonWalls *)
        GenerateGraniteWalls (dungeon);
        
        GenerateRockSeams (player,dungeon,DStruct.QuartzWall,NumberQuartzSeams,QuartzTreasureChance);
        GenerateRockSeams (player,dungeon,DStruct.MagmaWall,NumberMagaSeams,MagmaTreasureChance);
        
        GenerateDungeonBoundary (dungeon);
      END GenerateDungeonWalls;
      
    BEGIN (* GenerateRandomDungeonLevel *)
      NEW (info);
      
      player^.character^.column := DStruct.NumberDungeonColumns;
      player^.character^.line   := DStruct.NumberDungeonLines;
      
      player^.dungeon^.columns := DStruct.NumberDungeonColumns;
      player^.dungeon^.lines   := DStruct.NumberDungeonLines;
      
      info^.treasures := player^.treasures;
      
      MDGRooms.GenerateDungeonRooms (player,player^.dungeon^,player^.character^.variables.depthCUR,info^);
      MDGTunnel.GenerateDungeonTunnels (player^.dungeon^,info^);
      GenerateDungeonWalls (player,player^.dungeon^);
      MDGDoors.GenerateDungeonDoors (player^.treasures,player^.dungeon^,info^.doorTABLE);
      GenerateDungeonStairs (player^.treasures,player^.dungeon^);
      
      MDGAlloc.GenerateNonFixedDungeonElements (player,player^.dungeon^,AllocationLevel (player^.character^.variables.depthCUR));
      
      DISPOSE (info);
    END GenerateRandomDungeonLevel;
    
    PROCEDURE PlayerHasLight (item : OStruct.InventoryItemRecord) : BOOLEAN;
    BEGIN (* PlayerHasLight *)
      RETURN ((item.object.class <> OStruct.Nothing) AND (item.object.miscellaneous > 0));
    END PlayerHasLight;
    
  BEGIN (* GeneratePlayerDungeon *)
    ClearDungeonInformation (player^.treasures,player^.dungeon^);
    
    IF player^.character^.variables.depthCUR > 0 THEN
      GenerateRandomDungeonLevel (player);
    ELSE
      MDGTown.GenerateDungeonTown (player,player^.dungeon^);
    END; (* IF *)
    
    MDWindow.CenterDungeonOnLocation (player,player^.character^.line,player^.character^.column,TRUE);
    MCreatures.ComputeCreatureVisibility (player,FALSE);
    
    IF player^.character^.variables.depthCUR > player^.character^.variables.depthMAX THEN
      player^.character^.variables.depthMAX := player^.character^.variables.depthCUR;
    END; (* IF *)
    
    IF PlayerHasLight (player^.character^.inventory.equipment [MCStruct.Light]) THEN
      MPMovement.UpdatePlayerLightDisplay (player,player^.character^.line,player^.character^.column);
    END; (* IF *)
    
    IF player^.windows.dungeon <> Windows.NULL THEN
      MSUpdate.UpdateCurrentDepthWindow (player);
      MDPainting.InvalidateDungeonWindow (player);
    END; (* IF *)
  END GeneratePlayerDungeon;

  PROCEDURE PlaceDownStair (treasures : OStruct.TreasureInfoPointer; VAR dungeon : DStruct.DungeonInformation; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange);
    CONST
          DownStairTreasureCursor = 371;
    
    VAR
        artifact : OStruct.InventoryItemRecord;
    
  BEGIN (* PlaceDownStair *)
    OSActions.ReturnInventoryItemRecord (treasures,DownStairTreasureCursor,artifact);
    DSActions.PlaceDungeonArtifact (dungeon,line,column,artifact);
  END PlaceDownStair;
  
  PROCEDURE PlaceUpStair (treasures : OStruct.TreasureInfoPointer; VAR dungeon : DStruct.DungeonInformation; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange);
    CONST
          UpStairTreasureCursor = 370;
    
    VAR
        artifact : OStruct.InventoryItemRecord;
    
  BEGIN (* PlaceUpStair *)
    OSActions.ReturnInventoryItemRecord (treasures,UpStairTreasureCursor,artifact);
    DSActions.PlaceDungeonArtifact (dungeon,line,column,artifact);
  END PlaceUpStair;
END MDGenerator.
