IMPLEMENTATION MODULE MDGTunnel;
  (*/WIN:A*)
  
  FROM WMemory IMPORT ALLOCATE,DEALLOCATE;
  
  IMPORT DSActions,DStruct,MDGDoors,MDGenerator,MRandom,OStruct;
  
  
  PROCEDURE GenerateDungeonTunnels (VAR dungeon : DStruct.DungeonInformation; VAR info : MDGenerator.DungeonGeneratorInformation);
    VAR
        index : CARDINAL;
    
    PROCEDURE GenerateDungeonTunnel (VAR dungeon : DStruct.DungeonInformation; location1,location2 : MDGenerator.DungeonLocation; VAR info : MDGenerator.DungeonGeneratorInformation);
      VAR
          tunnel : POINTER TO MDGenerator.DungeonLocationTable;
          walls  : POINTER TO MDGenerator.DungeonLocationTable;
      
      PROCEDURE DetermineTunnelPath (VAR dungeon : DStruct.DungeonInformation; location1,location2 : MDGenerator.DungeonLocation; VAR tunnel : MDGenerator.DungeonLocationTable; VAR doors : MDGenerator.DungeonLocationTable; VAR walls : MDGenerator.DungeonLocationTable);
        CONST
              MaxTunnelCounter = 2000;
        
        VAR
            tunnelStopped : BOOLEAN;
            deltaX,deltaY : INTEGER;
            elements      : DStruct.DungeonElementSet;
            location      : MDGenerator.DungeonLocation;
            doorFlag      : BOOLEAN;
            counter       : CARDINAL;
            start         : MDGenerator.DungeonLocation;
        
        PROCEDURE DetermineCorrectTunnelDirections (location1,location2 : MDGenerator.DungeonLocation; VAR deltaX,deltaY : INTEGER);
          PROCEDURE Sigma (value : INTEGER) : INTEGER;
          BEGIN (* Sigma *)
            IF value < 0 THEN
              RETURN (-1);
            ELSIF value > 0 THEN
              RETURN (+1);
            ELSE
              RETURN (0);
            END; (* IF *)
          END Sigma;
          
        BEGIN (* DetermineCorrectTunnelDirections *)
          deltaX := Sigma (location2.column - location1.column);
          deltaY := Sigma (location2.line - location1.line);
          
          IF ((deltaX <> 0) AND (deltaY <> 0)) THEN
            IF MRandom.CardinalRandomRangeValue (1,2) = 1 THEN
              deltaY := 0;
            ELSE
              deltaX := 0;
            END; (* IF *)
          END; (* IF *)
        END DetermineCorrectTunnelDirections;
        
        PROCEDURE DetermineNewTunnelLocation (VAR dungeon : DStruct.DungeonInformation; location1,location2 : MDGenerator.DungeonLocation; VAR deltaX,deltaY : INTEGER; VAR result : MDGenerator.DungeonLocation; VAR elements : DStruct.DungeonElementSet);
          CONST
                RandomDirectionChange = 9;
          
          PROCEDURE DetermineRandomTunnelDirections (VAR deltaX,deltaY : INTEGER);
            VAR
                value : INTEGER;
            
          BEGIN (* DetermineRandomTunnelDirections *)
            value := MRandom.IntegerRandomRangeValue (1,4);
            
            IF value < 3 THEN
              deltaY := -3 + value * 2;
              deltaX := 0;
            ELSE
              deltaX := -7 + value * 2;
              deltaY := 0;
            END; (* IF *)
          END DetermineRandomTunnelDirections;
          
        BEGIN (* DetermineNewTunnelLocation *)
          IF MRandom.CardinalRandomRangeValue (1,100) > 77 THEN
            IF MRandom.CardinalRandomRangeValue (1,RandomDirectionChange) <> 1 THEN
              DetermineCorrectTunnelDirections (location1,location2,deltaX,deltaY);
            ELSE
              DetermineRandomTunnelDirections (deltaX,deltaY);
            END; (* IF *)
          END; (* IF *)
          
          LOOP
            result.column := location1.column + deltaX;
            result.line   := location1.line + deltaY;
            
            IF ((result.column > 0) AND (result.column < dungeon.columns - 1) AND (result.line > 0) AND (result.line < dungeon.lines - 1)) THEN
              EXIT;
            END; (* IF *)
            
            IF MRandom.CardinalRandomRangeValue (1,RandomDirectionChange) <> 1 THEN
              DetermineCorrectTunnelDirections (location1,location2,deltaX,deltaY);
            ELSE
              DetermineRandomTunnelDirections (deltaX,deltaY);
            END; (* IF *)
          END; (* LOOP *)
          
          elements := dungeon.physical [result.line]^ [result.column].elements;
        END DetermineNewTunnelLocation;
        
        PROCEDURE ExecuteCorridorIntersectionAction (start,location : MDGenerator.DungeonLocation; VAR doors : MDGenerator.DungeonLocationTable; VAR doorFLAG,stopped : BOOLEAN);
        BEGIN (* ExecuteCorridorIntersectionAction *)
          IF NOT (doorFlag) THEN
            IF doors.counter < MDGenerator.MaxLocationTableSize THEN
              doors.locations [doors.counter] := location;
            END; (* IF *)
            
            INC (doors.counter);
          END; (* IF *)
          
          stopped := ((MRandom.CardinalRandomRangeValue (1,100) > 15) AND ((ABS (start.column - location.column) > 10) OR (ABS (start.line - location.line) > 10)));
        END ExecuteCorridorIntersectionAction;
        
        PROCEDURE ExecuteEmptyLocationAction (VAR tunnel : MDGenerator.DungeonLocationTable; location : MDGenerator.DungeonLocation);
        BEGIN (* ExecuteEmptyLocationAction *)
          IF tunnel.counter < MDGenerator.MaxLocationTableSize THEN
            tunnel.locations [tunnel.counter] := location;
            tunnel.counter                    := tunnel.counter + 1;
          END; (* IF *)
        END ExecuteEmptyLocationAction;
        
        PROCEDURE ExecuteWallIntersectionAction (VAR dungeon : DStruct.DungeonInformation; location : MDGenerator.DungeonLocation; VAR walls : MDGenerator.DungeonLocationTable);
          CONST
                DoorFrameMask = DStruct.DungeonElementSet {DStruct.GraniteWall,DStruct.TemporaryWall2};
          
          VAR
              column : DStruct.DungeonColumnRange;
              line   : DStruct.DungeonLineRange;
          
        BEGIN (* ExecuteWallIntersectionAction *)
          IF walls.counter < MDGenerator.MaxLocationTableSize THEN
            walls.locations [walls.counter] := location;
            walls.counter                   := walls.counter + 1;
          END; (* FOR *)
          
          FOR line := location.line - 1 TO location.line + 1 DO
            FOR column := location.column - 1 TO location.column + 1 DO
              IF (DSActions.LocationInBounds (dungeon,line,column) AND (DStruct.GraniteWall IN dungeon.physical [line]^ [column].elements)) THEN
                dungeon.physical [line]^ [column].elements := dungeon.physical [line]^ [column].elements / DoorFrameMask;
              END; (* IF *)
            END; (* FOR *)
          END; (* FOR *)
        END ExecuteWallIntersectionAction;
        
      BEGIN (* DetermineTunnelPath *)
        tunnel.counter := 0;
        doors.counter  := 0;
        walls.counter  := 0;
        
        DetermineCorrectTunnelDirections (location1,location2,deltaX,deltaY);
        
        tunnelStopped := FALSE;
        doorFlag      := FALSE;
        counter       := 0;
        start         := location1;
        
        REPEAT
          DetermineNewTunnelLocation (dungeon,location1,location2,deltaX,deltaY,location,elements);
          
          IF elements * DStruct.DungeonElementSet {DStruct.CorridorFloor,DStruct.BlockedFloor} <> DStruct.EmptyDungeonElementSet THEN
            ExecuteCorridorIntersectionAction (start,location,doors,doorFlag,tunnelStopped);
            
            location1 := location;
          ELSIF DStruct.GraniteWall IN elements THEN
            ExecuteWallIntersectionAction (dungeon,location,walls);
            
            location1 := location;
          ELSIF DStruct.NullWall IN elements THEN
            ExecuteEmptyLocationAction (tunnel,location);
            
            location1 := location;
            doorFlag  := FALSE;
          ELSIF NOT (DStruct.TemporaryWall2 IN elements) THEN
            location1 := location;
          END; (* IF *)
          
          INC (counter);
        UNTIL (((location1.line = location2.line) AND (location1.column = location2.column)) OR (counter > MaxTunnelCounter) OR tunnelStopped);
      END DetermineTunnelPath;
      
      PROCEDURE GenerateTunnelDoors (treasures : OStruct.TreasureInfoPointer; VAR dungeon : DStruct.DungeonInformation; VAR walls : MDGenerator.DungeonLocationTable);
        VAR
            index : CARDINAL;
        
        PROCEDURE FixDungeonWallIntersection (treasures : OStruct.TreasureInfoPointer; VAR dungeon : DStruct.DungeonInformation; location : MDGenerator.DungeonLocation);
        BEGIN (* FixDungeonWallIntersection *)
          IF DStruct.TemporaryWall2 IN dungeon.physical [location.line]^ [location.column].elements THEN
            EXCL (dungeon.physical [location.line]^ [location.column].elements,DStruct.TemporaryWall2);
            
            IF MRandom.CardinalRandomRangeValue (1,100) < 25 THEN
              MDGDoors.GenerateRandomDoor (treasures,dungeon,location.line,location.column);
            ELSE
              INCL (dungeon.physical [location.line]^ [location.column].elements,DStruct.CorridorFloor);
            END; (* IF *)
          END; (* IF *)
        END FixDungeonWallIntersection;
        
      BEGIN (* GenerateTunnelDoors *)
        IF walls.counter > 0 THEN
          FOR index := 0 TO walls.counter - 1 DO
            FixDungeonWallIntersection (treasures,dungeon,walls.locations [index]);
          END; (* FOR *)
        END; (* IF *)
      END GenerateTunnelDoors;
      
      PROCEDURE GenerateTunnelFloor (VAR dungeon : DStruct.DungeonInformation; VAR tunnel : MDGenerator.DungeonLocationTable);
        VAR
            index : CARDINAL;
        
      BEGIN (* GenerateTunnelFloor *)
        IF tunnel.counter > 0 THEN
          FOR index := 0 TO tunnel.counter - 1 DO
            dungeon.physical [tunnel.locations [index].line]^ [tunnel.locations [index].column].elements := DStruct.DungeonElementSet {DStruct.CorridorFloor};
          END; (* FOR *)
        END; (* IF *)
      END GenerateTunnelFloor;
      
    BEGIN (* GenerateDungeonTunnel *)
      NEW (tunnel);
      NEW (walls);
      
      DetermineTunnelPath (dungeon,location1,location2,tunnel^,info.doorTABLE,walls^);
      
      GenerateTunnelFloor (dungeon,tunnel^);
      GenerateTunnelDoors (info.treasures,dungeon,walls^);
      
      DISPOSE (tunnel);
      DISPOSE (walls);
    END GenerateDungeonTunnel;
    
    PROCEDURE RandomizeRoomConnections (VAR table : ARRAY OF MDGenerator.DungeonLocation; counter : CARDINAL);
      VAR
          index : CARDINAL;
      
      PROCEDURE SwapDungeonLocations (VAR value1,value2 : MDGenerator.DungeonLocation);
        VAR
            temp : MDGenerator.DungeonLocation;
        
      BEGIN (* SwapDungeonLocations *)
        temp   := value1;
        value1 := value2;
        value2 := temp;
      END SwapDungeonLocations;
      
    BEGIN (* RandomizeRoomConnections *)
      FOR index := 0 TO counter - 1 DO
        SwapDungeonLocations (table [MRandom.CardinalRandomRangeValue (0,counter - 1)],table [MRandom.CardinalRandomRangeValue (0,counter - 1)]);
      END; (* FOR *)
      
      table [counter] := table [0];
    END RandomizeRoomConnections;
    
  BEGIN (* GenerateDungeonTunnels *)
    RandomizeRoomConnections (info.roomTABLE,info.roomCOUNTER);
    
    info.doorTABLE.counter := 0;
    
    FOR index := 0 TO info.roomCOUNTER - 1 DO
      GenerateDungeonTunnel (dungeon,info.roomTABLE [index],info.roomTABLE [index + 1],info);
    END; (* FOR *)
  END GenerateDungeonTunnels;
END MDGTunnel.
