IMPLEMENTATION MODULE MDialogs;
  (*/WIN:A*)
  
  IMPORT Windows,WCreation,WDebugging,WDialog,Windows,WInput,WinStart,WMacro,WModule;
  IMPORT MStruct;
  IMPORT SYSTEM;
  IMPORT MCLoop;
  
  
  CONST
        MaxStackedDialogs = 8;
  
  TYPE
       DialogClassMnemonic = (ColorDialog,ParameterDialog,StandardDialog);
       DialogInformation   = RECORD
                               player : MStruct.PlayerInfoPointer;
                               window : Windows.HWND;
                               data   : SYSTEM.ADDRESS;
                               
                               CASE class : DialogClassMnemonic OF
                                 ColorDialog : InitializeColorDialog : MoriaColorDialogInitializationProcedure; 
                                               ExecuteColorCommand   : MoriaColorDialogCommandProcedure;        
                                               CloseColorDialog      : MoriaColorDialogCloseProcedure;          
                                               ControlColor          : MoriaColorControlProcedure;              |
                                 
                                 ParameterDialog : InitializeParameterDialog : MoriaParamDialogInitializationProcedure; 
                                                   ExecuteParameterCommand   : MoriaParamDialogCommandProcedure;        
                                                   CloseParameterDialog      : MoriaParamDialogCloseProcedure;          |
                                 
                                 StandardDialog : InitializeStandardDialog : MoriaDialogInitializationProcedure; 
                                                  ExecuteStandardCommand   : MoriaDialogCommandProcedure;        
                                                  CloseStandardDialog      : MoriaDialogCloseProcedure;          |
                               END; (* CASE *)
                             END; (* DialogInformation *)
  
  VAR
      CurrentNumberDialogs : [0..MaxStackedDialogs];
      DialogStack          : ARRAY [0..MaxStackedDialogs - 1] OF DialogInformation;
  
  VAR
      CurrentModelessDialog : DialogInformation;
      ModelessDialogHandler : Windows.DLGPROC;
  
  
  PROCEDURE AssignStandardDialogInformation (player : MStruct.PlayerInfoPointer; InitializeDialog : MoriaDialogInitializationProcedure; ExecuteCommand : MoriaDialogCommandProcedure; CloseDialog : MoriaDialogCloseProcedure; VAR result : DialogInformation);
  BEGIN (* AssignStandardDialogInformation *)
    result.player := player;
    result.window := Windows.NULL;
    result.class  := StandardDialog;
    result.data   := NIL;
    
    result.InitializeStandardDialog := InitializeDialog;
    result.ExecuteStandardCommand   := ExecuteCommand;
    result.CloseStandardDialog      := CloseDialog;
  END AssignStandardDialogInformation;
  
  PROCEDURE DestroyCurrentModelessDialog (player : MStruct.PlayerInfoPointer);
    VAR
        previous : BOOLEAN;
        done     : BOOLEAN;
    
  BEGIN (* DestroyCurrentModelessDialog *)
    WInput.EnableWindow (player^.windows.frame,TRUE,previous);
    
    WCreation.DestroyWindow (player^.windows.dialog,done);
    
    IF NOT (done) THEN
      WDebugging.FatalAppExit ("Unable to destroy current modeless dialog.");
    END; (* IF *)
    
    player^.windows.dialog := Windows.NULL;
    
    WModule.FreeProcInstance (Windows.FARPROC (ModelessDialogHandler));
  END DestroyCurrentModelessDialog;
  
  PROCEDURE ExecuteModelessMoriaDialogBox (window : Windows.HWND; template : ARRAY OF CHAR; player : MStruct.PlayerInfoPointer; InitializeDialog : MoriaDialogInitializationProcedure; ExecuteCommand : MoriaDialogCommandProcedure; CloseDialog : MoriaDialogCloseProcedure);
    VAR
        dialog : Windows.HWND;
    
  BEGIN (* ExecuteModelessMoriaDialogBox *)
    ModelessDialogHandler := Windows.DLGPROC (WModule.MakeProcInstance (WinStart.hInstance,SYSTEM.ADR (ModelessDialogTranslator)));
    
    AssignStandardDialogInformation (player,InitializeDialog,ExecuteCommand,CloseDialog,CurrentModelessDialog);
    WDialog.CreateDialog (WinStart.hInstance,template,window,ModelessDialogHandler,dialog);
    
    IF dialog = Windows.NULL THEN
      WDebugging.FatalAppExit ("Unable to create modless dialog.");
    END; (* IF *)
    
    MCLoop.SetCurrentModelessDialog (player,dialog);
  END ExecuteModelessMoriaDialogBox;
  
  PROCEDURE DialogProcedureResult (VAR dialog : DialogInformation; window : Windows.HWND; message : CARDINAL; param1 : Windows.WPARAM; param2 : Windows.LPARAM) : Windows.BOOL;
    CONST
          UnrecognizedDialogClass = "Unrecognized dialog class in MDialogs.";
    
    PROCEDURE CloseMoriaDialog (info : DialogInformation);
    BEGIN (* CloseMoriaDialog *)
      CASE info.class OF
        ParameterDialog : info.CloseParameterDialog (info.window,info.player,info.data); |
        StandardDialog  : info.CloseStandardDialog (info.window,info.player);            |
        ColorDialog     : info.CloseColorDialog (info.window,info.data);                 |
      ELSE
        WDebugging.FatalAppExit (UnrecognizedDialogClass);
      END; (* CASE *)
    END CloseMoriaDialog;
    
    PROCEDURE ColorForControl (info : DialogInformation; control : Windows.HWND; class : INTEGER; context : Windows.HDC) : Windows.HBRUSH;
    BEGIN (* ColorForControl *)
      IF info.class = ColorDialog THEN
        RETURN (info.ControlColor (control,class,context,info.data));
      ELSE
        RETURN (Windows.NULL);
      END; (* IF *)
    END ColorForControl;
    
    PROCEDURE ExecuteMoriaDialogCommand (info : DialogInformation; control : CARDINAL; command : CARDINAL);
    BEGIN (* ExecuteMoriaDialogCommand *)
      CASE info.class OF
        ParameterDialog : info.ExecuteParameterCommand (info.window,info.player,info.data,control,command); |
        StandardDialog  : info.ExecuteStandardCommand (info.window,info.player,control,command);            |
        ColorDialog     : info.ExecuteColorCommand (info.window,info.data,control,command);                 |
      ELSE
        WDebugging.FatalAppExit (UnrecognizedDialogClass);
      END; (* CASE *)
    END ExecuteMoriaDialogCommand;
    
    PROCEDURE InitializeMoriaDialog (window : Windows.HWND; VAR info : DialogInformation);
    BEGIN (* InitializeMoriaDialog *)
      info.window := window;
      
      CASE info.class OF
        ParameterDialog : info.InitializeParameterDialog (window,info.player,info.data); |
        StandardDialog  : info.InitializeStandardDialog (window,info.player);            |
        ColorDialog     : info.InitializeColorDialog (window,info.data);                 |
      ELSE
        WDebugging.FatalAppExit (UnrecognizedDialogClass);
      END; (* CASE *)
    END InitializeMoriaDialog;
    
  BEGIN (* DialogProcedureResult *)
    CASE message OF
      Windows.WM_INITDIALOG : InitializeMoriaDialog (window,dialog); 
                              RETURN (Windows.TRUE_BOOL);                                                             |
      Windows.WM_CTLCOLOR   : RETURN (ColorForControl (dialog,WMacro.LOWORD (param2),WMacro.HIWORD (param2),param1)); |
      
      Windows.WM_COMMAND : ExecuteMoriaDialogCommand (dialog,param1,WMacro.HIWORD (param2)); |
      Windows.WM_CLOSE   : CloseMoriaDialog (dialog);                                        |
    ELSE
      (* no action *)
    END; (* CASE *)
    
    RETURN (Windows.FALSE_BOOL);
  END DialogProcedureResult;
  
  PROCEDURE ExecuteMoriaColorDialogBox (window : Windows.HWND; template : ARRAY OF CHAR; data : SYSTEM.ADDRESS; InitializeDialog : MoriaColorDialogInitializationProcedure; ExecuteCommand : MoriaColorDialogCommandProcedure; CloseDialog : MoriaColorDialogCloseProcedure; ColorControl : MoriaColorControlProcedure; VAR result : INTEGER);
    PROCEDURE PushColorDialogInformation (data : SYSTEM.ADDRESS; InitializeDialog : MoriaColorDialogInitializationProcedure; ExecuteCommand : MoriaColorDialogCommandProcedure; CloseDialog : MoriaColorDialogCloseProcedure; ColorControl : MoriaColorControlProcedure);
      VAR
          info : DialogInformation;
      
    BEGIN (* PushColorDialogInformation *)
      info.window := Windows.NULL;
      info.player := NIL;
      info.class  := ColorDialog;
      info.data   := data;
      
      info.InitializeColorDialog := InitializeDialog;
      info.ExecuteColorCommand   := ExecuteCommand;
      info.CloseColorDialog      := CloseDialog;
      info.ControlColor          := ColorControl;
      
      PushDialogInformation (info);
    END PushColorDialogInformation;
    
  BEGIN (* ExecuteMoriaColorDialogBox *)
    PushColorDialogInformation (data,InitializeDialog,ExecuteCommand,CloseDialog,ColorControl);
    WDialog.AppModalDialogBox (WinStart.hInstance,template,window,SYSTEM.ADR (MoriaDialogTranslator),result);
    
    DEC (CurrentNumberDialogs);
  END ExecuteMoriaColorDialogBox;
  
  PROCEDURE ExecuteMoriaDialogBox (window : Windows.HWND; template : ARRAY OF CHAR; player : MStruct.PlayerInfoPointer; InitializeDialog : MoriaDialogInitializationProcedure; ExecuteCommand : MoriaDialogCommandProcedure; CloseDialog : MoriaDialogCloseProcedure; VAR result : INTEGER);
    PROCEDURE PushStandardDialogInformation (player : MStruct.PlayerInfoPointer; InitializeDialog : MoriaDialogInitializationProcedure; ExecuteCommand : MoriaDialogCommandProcedure; CloseDialog : MoriaDialogCloseProcedure);
      VAR
          info : DialogInformation;
      
    BEGIN (* PushStandardDialogInformation *)
      AssignStandardDialogInformation (player,InitializeDialog,ExecuteCommand,CloseDialog,info);
      PushDialogInformation (info);
    END PushStandardDialogInformation;
    
  BEGIN (* ExecuteMoriaDialogBox *)
    PushStandardDialogInformation (player,InitializeDialog,ExecuteCommand,CloseDialog);
    WDialog.AppModalDialogBox (WinStart.hInstance,template,window,SYSTEM.ADR (MoriaDialogTranslator),result);
    
    DEC (CurrentNumberDialogs);
  END ExecuteMoriaDialogBox;
  
  PROCEDURE ExecuteMoriaDialogBoxParam (window : Windows.HWND; template : ARRAY OF CHAR; player : MStruct.PlayerInfoPointer; data : SYSTEM.ADDRESS; InitializeDialog : MoriaParamDialogInitializationProcedure; ExecuteCommand : MoriaParamDialogCommandProcedure; CloseDialog : MoriaParamDialogCloseProcedure; VAR result : INTEGER);
    PROCEDURE PushParameterDialogInformation (player : MStruct.PlayerInfoPointer; data : SYSTEM.ADDRESS; InitializeDialog : MoriaParamDialogInitializationProcedure; ExecuteCommand : MoriaParamDialogCommandProcedure; CloseDialog : MoriaParamDialogCloseProcedure);
      VAR
          info : DialogInformation;
      
    BEGIN (* PushParameterDialogInformation *)
      info.window := Windows.NULL;
      info.player := player;
      info.class  := ParameterDialog;
      info.data   := data;
      
      info.InitializeParameterDialog := InitializeDialog;
      info.ExecuteParameterCommand   := ExecuteCommand;
      info.CloseParameterDialog      := CloseDialog;
      
      PushDialogInformation (info);
    END PushParameterDialogInformation;
    
  BEGIN (* ExecuteMoriaDialogBoxParam *)
    PushParameterDialogInformation (player,data,InitializeDialog,ExecuteCommand,CloseDialog);
    WDialog.AppModalDialogBox (WinStart.hInstance,template,window,SYSTEM.ADR (MoriaDialogTranslator),result);
    
    DEC (CurrentNumberDialogs);
  END ExecuteMoriaDialogBoxParam;
  
  PROCEDURE ModelessDialogTranslator ['MODELESSDIALOGTRANSLATOR'] (window : Windows.HWND; message : CARDINAL; param1 : Windows.WPARAM; param2 : Windows.LPARAM) : Windows.BOOL [WINDOWS,LONG];
  BEGIN (* ModelessDialogTranslator *)
    RETURN (DialogProcedureResult (CurrentModelessDialog,window,message,param1,param2));
  END ModelessDialogTranslator;
  
  PROCEDURE MoriaDialogTranslator ['MORIADIALOGTRANSLATOR'] (window : Windows.HWND; message : CARDINAL; param1 : Windows.WPARAM; param2 : Windows.LPARAM) : Windows.BOOL [WINDOWS,LONG];
    PROCEDURE DialogForWindow (window : Windows.HWND) : CARDINAL;
      VAR
          index : CARDINAL;
      
    BEGIN (* DialogForWindow *)
      FOR index := 0 TO CurrentNumberDialogs - 1 DO
        IF DialogStack [index].window = window THEN
          RETURN (index);
        END; (* IF *)
      END; (* FOR *)
      
      IF window <> Windows.NULL THEN
        RETURN (DialogForWindow (Windows.NULL));
      END; (* IF *)
      
      WDebugging.FatalAppExit ("Unable to find dialog handler.");
    END DialogForWindow;
    
  BEGIN (* MoriaDialogTranslator *)
    RETURN (DialogProcedureResult (DialogStack [DialogForWindow (window)],window,message,param1,param2));
  END MoriaDialogTranslator;
  
  PROCEDURE PushDialogInformation (info : DialogInformation);
  BEGIN (* PushDialogInformation *)
    DialogStack [CurrentNumberDialogs] := info;
    INC (CurrentNumberDialogs);
  END PushDialogInformation;

BEGIN (* MDialogs *)
  CurrentNumberDialogs := 0;
END MDialogs.
