IMPLEMENTATION MODULE MDPainting;
  (*/WIN:A*)
  
  IMPORT WDAttrib,WDisplay,WDTool,Windows,WMacro,WPainting,WText;
  
  IMPORT MDSymbols,MDScrolling,MDWindow;
  IMPORT DStruct,MStruct;
  IMPORT MFonts;
  
  
  PROCEDURE ExecuteDungeonWindowPaintAction (window : Windows.HWND; player : MStruct.PlayerInfoPointer);
    TYPE
         DungeonPaintingInformation = RECORD
                                        struct : Windows.PAINTSTRUCT;
                                        color  : Windows.COLORREF;
                                        
                                        minX,maxX : INTEGER;
                                        minY,maxY : INTEGER;
                                        
                                        firstCOLUMN : INTEGER;
                                        firstLINE   : INTEGER;
                                        
                                        height,width : INTEGER;
                                      END; (* DungeonPaintingInformation *)
    
    VAR
        painting : DungeonPaintingInformation;
    
    PROCEDURE InitializeDungeonPaintingInformation (window : Windows.HWND; player : MStruct.PlayerInfoPointer; VAR result : DungeonPaintingInformation);
      VAR
          previousFONT : Windows.HGDIOBJ;
          context      : Windows.HDC;
      
      PROCEDURE DetermineDungeonUpdateArea (window : Windows.HWND; context : Windows.HDC; area : Windows.RECT; firstLINE,firstCOLUMN : INTEGER; VAR minX,maxX : DStruct.DungeonColumnRange; VAR minY,maxY : DStruct.DungeonLineRange; VAR height,width : INTEGER);
      BEGIN (* DetermineDungeonUpdateArea *)
        MFonts.DetermineContextCharacterMetrics (context,height,width);
        
        maxX := WMacro.MinInteger (firstCOLUMN + area.right DIV width,MAX (DStruct.DungeonColumnRange));
        minX := WMacro.MinInteger (WMacro.MaxInteger (firstCOLUMN + area.left DIV width,MIN (DStruct.DungeonColumnRange)),MAX (DStruct.DungeonColumnRange));
        maxY := WMacro.MinInteger (firstLINE + area.bottom DIV height,MAX (DStruct.DungeonLineRange));
        minY := WMacro.MinInteger (WMacro.MaxInteger (firstLINE + area.top DIV height,MIN (DStruct.DungeonLineRange)),MAX (DStruct.DungeonLineRange));
      END DetermineDungeonUpdateArea;
      
    BEGIN (* InitializeDungeonPaintingInformation *)
      WPainting.BeginPaint (window,result.struct,context);
      
      IF player <> NIL THEN
        WDTool.SelectObject (result.struct.hdc,WDTool.GetStockObject (DungeonWindowFont),previousFONT);
        
        result.firstCOLUMN := player^.dungeon^.firstCOLUMN;
        result.firstLINE   := player^.dungeon^.firstLINE;
        
        DetermineDungeonUpdateArea (window,context,result.struct.rcPaint,result.firstLINE,result.firstCOLUMN,result.minX,result.maxX,result.minY,result.maxY,result.height,result.width);
        
        result.maxX := WMacro.MinInteger (result.maxX,player^.dungeon^.columns - 1);
        result.minX := WMacro.MaxInteger (result.minX,0);
        result.maxY := WMacro.MinInteger (result.maxY,player^.dungeon^.lines - 1);
        result.minY := WMacro.MaxInteger (result.minY,0);
        
        result.color := WDAttrib.GetTextColor (result.struct.hdc);
      END; (* IF *)
    END InitializeDungeonPaintingInformation;
    
    PROCEDURE UpdateDungeonDisplayArea (player : MStruct.PlayerInfoPointer; VAR painting : DungeonPaintingInformation);
      VAR
          previous : Windows.COLORREF;
          column   : DStruct.DungeonColumnRange;
          line     : DStruct.DungeonLineRange;
      
      PROCEDURE UpdateDungeonDisplayLocation (player : MStruct.PlayerInfoPointer; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange; VAR painting : DungeonPaintingInformation);
        VAR
            color,previous : Windows.COLORREF;
            symbol         : CHAR;
            done           : BOOLEAN;
        
      BEGIN (* UpdateDungeonDisplayLocation *)
        MDSymbols.DetermineLocationDisplayInformation (player,line,column,symbol,color);
        
        IF color <> painting.color THEN
          WDAttrib.SetTextColor (painting.struct.hdc,color,previous);
          painting.color := color;
        END; (* IF *)
        
        (*/NOCHECK*)
        WText.TextOut (painting.struct.hdc,(column - painting.firstCOLUMN) * painting.width,(line - painting.firstLINE) * painting.height,symbol,done);
        (*/CHECK*)
      END UpdateDungeonDisplayLocation;
      
    BEGIN (* UpdateDungeonDisplayArea *)
      WDAttrib.SetBkColor (painting.struct.hdc,MDSymbols.CurrentBackgroundColor (),previous);
      
      (*/NOCHECK*)
      FOR line := painting.minY TO painting.maxY DO
        FOR column := painting.minX TO painting.maxX DO
          UpdateDungeonDisplayLocation (player,line,column,painting);
        END; (* FOR *)
      END; (* FOR *)
      (*/CHECK*)
    END UpdateDungeonDisplayArea;
    
  BEGIN (* ExecuteDungeonWindowPaintAction *)
    InitializeDungeonPaintingInformation (window,player,painting);
    
    IF player <> NIL THEN
      UpdateDungeonDisplayArea (player,painting);
    END; (* IF *)
    
    WPainting.EndPaint (window,painting.struct);
  END ExecuteDungeonWindowPaintAction;
  
  PROCEDURE InvalidateDungeonArea (player : MStruct.PlayerInfoPointer; minX,maxX : DStruct.DungeonColumnRange; minY,maxY : DStruct.DungeonLineRange);
    VAR
        fontHEIGHT,fontWIDTH : INTEGER;
        x1,y1,x2,y2          : INTEGER;
    
  BEGIN (* InvalidateDungeonArea *)
    MFonts.DetermineWindowCharacterMetrics (player^.windows.dungeon,DungeonWindowFont,fontHEIGHT,fontWIDTH);
    
    x2 := (maxX - player^.dungeon^.firstCOLUMN) * fontWIDTH + fontWIDTH;
    x1 := (minX - player^.dungeon^.firstCOLUMN) * fontWIDTH;
    y2 := (maxY - player^.dungeon^.firstLINE) * fontHEIGHT + fontHEIGHT;
    y1 := (minY - player^.dungeon^.firstLINE) * fontHEIGHT;
    
    WPainting.InvalidateWindowArea (player^.windows.dungeon,x1,x2,y1,y2,TRUE);
  END InvalidateDungeonArea;
  
  PROCEDURE InvalidateDungeonLocation (player : MStruct.PlayerInfoPointer; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange; center : BOOLEAN; update : BOOLEAN);
    VAR
        fontHEIGHT,fontWIDTH : INTEGER;
        x,y                  : INTEGER;
    
    PROCEDURE AreaFitsOnCurrentWindow (window : Windows.HWND; minX,maxX : INTEGER; minY,maxY : INTEGER) : BOOLEAN;
      VAR
          client : Windows.RECT;
      
    BEGIN (* AreaFitsOnCurrentWindow *)
      WDisplay.GetClientRect (window,client);
      RETURN ((minX >= client.left) AND (maxX <= client.right) AND (minY >= client.top) AND (maxY <= client.bottom));
    END AreaFitsOnCurrentWindow;
    
  BEGIN (* InvalidateDungeonLocation *)
    MFonts.DetermineWindowCharacterMetrics (player^.windows.dungeon,DungeonWindowFont,fontHEIGHT,fontWIDTH);
    
    x := (column - player^.dungeon^.firstCOLUMN) * fontWIDTH;
    y := (line - player^.dungeon^.firstLINE) * fontHEIGHT;
    
    IF AreaFitsOnCurrentWindow (player^.windows.dungeon,x,x + fontWIDTH,y,y + fontHEIGHT) THEN
      WPainting.InvalidateWindowArea (player^.windows.dungeon,x,x + fontWIDTH,y,y + fontHEIGHT,TRUE);
    ELSIF center THEN
      MDWindow.CenterDungeonOnLocation (player,line,column,TRUE);
    END; (* IF *)
    
    IF update THEN
      WPainting.UpdateWindow (player^.windows.dungeon);
    END; (* IF *)
  END InvalidateDungeonLocation;
  
  PROCEDURE InvalidateDungeonWindow (player : MStruct.PlayerInfoPointer);
  BEGIN (* InvalidateDungeonWindow *)
    MDScrolling.ResetDungeonScrollBars (player);
    WPainting.InvalidateEntireWindow (player^.windows.dungeon,TRUE);
  END InvalidateDungeonWindow;
END MDPainting.
