IMPLEMENTATION MODULE MDSymbols;
  (*/WIN:A*)
  
  IMPORT WDebugging,Windows,WSystem;
  IMPORT Strings;
  
  IMPORT DStruct,MCStruct,MStruct,OStruct;
  IMPORT MInitFile,MRandom;
  
  
  CONST
        InvalidOverrideSymbol = 0C;
  
  VAR
      OverrideSymbol : CHAR;
      
  
  PROCEDURE CurrentBackgroundColor () : Windows.COLORREF;
  BEGIN (* CurrentBackgroundColor *)
    RETURN (CurrentDungeonColorTable [BackgroundColor]);
  END CurrentBackgroundColor;
  
  PROCEDURE DetermineLocationDisplayInformation (player : MStruct.PlayerInfoPointer; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange; VAR symbol : CHAR; VAR color : Windows.COLORREF);
    VAR
        location : DStruct.PhysicalDungeonUnit;
    
    PROCEDURE  ColorElementForArtifact (artifact : OStruct.InventoryItemRecord) : DungeonColorMnemonic;
    BEGIN (* ColorElementForArtifact *)
      IF artifact.object.class <> OStruct.SecretDoor THEN
        RETURN (ArtifactColor);
      ELSE
        RETURN (GraniteColor);
      END; (* IF *)
    END ColorElementForArtifact;
    
    PROCEDURE ColorElementForWall (elements : DStruct.DungeonElementSet) : DungeonColorMnemonic;
    BEGIN (* ColorElementForWall *)
      IF ((DStruct.GraniteWall IN elements) OR (DStruct.BoundaryWall IN elements)) THEN
        RETURN (GraniteColor);
      ELSIF DStruct.QuartzWall IN elements THEN
        RETURN (QuartzColor);
      ELSE
        RETURN (MagmaColor);
      END; (* IF *)
    END ColorElementForWall;
    
  BEGIN (* DetermineLocationDisplayInformation *)
    IF OverrideSymbol = InvalidOverrideSymbol THEN
      IF player = NIL THEN
        WDebugging.FatalAppExit ("Invalid attempt to determine location display information.");
      END; (* IF *)
      
      symbol := 0C;
      
      (*/NOCHECK*)
      IF ((player^.character^.line = line) AND (player^.character^.column = column)) THEN
        symbol := '@';
        color  := CurrentDungeonColorTable [PlayerColor];
      ELSIF MCStruct.BlindFLAG IN player^.character^.variables.status THEN
        symbol := ' ';
        color  := CurrentDungeonColorTable [BackgroundColor];
      ELSIF ((player^.character^.variables.counters [MCStruct.ImageCOUNT] > 0) AND (MRandom.CardinalRandomRangeValue (1,12) = 1)) THEN
        symbol := CHR (MRandom.CardinalRandomRangeValue (32,96));
        color  := CurrentDungeonColorTable [HallucinationColor];
      END; (* IF *)
      
      IF symbol = 0C THEN
        location := player^.dungeon^.physical [line]^ [column];
        
        IF ((location.creature <> DStruct.InvalidCreatureCursor) AND player^.dungeon^.creatures [location.creature].visible) THEN
          symbol := player^.dungeon^.creatures [location.creature].symbol;
          color  := CurrentDungeonColorTable [CreatureColor];
        ELSIF location.elements * DStruct.DungeonElementSet {DStruct.plFLAG,DStruct.tlFLAG,DStruct.fmFLAG} = DStruct.DungeonElementSet {} THEN
          symbol := ' ';
          color  := CurrentDungeonColorTable [BackgroundColor];
        ELSIF ((location.artifact <> DStruct.InvalidArtifactCursor) AND (player^.dungeon^.artifacts [location.artifact].object.class <> OStruct.InvisibleTrap)) THEN
          symbol := player^.dungeon^.artifacts [location.artifact].object.symbol;
          color  := CurrentDungeonColorTable [ColorElementForArtifact (player^.dungeon^.artifacts [location.artifact])];
        ELSIF location.elements * DStruct.DungeonElementSet {DStruct.GraniteWall,DStruct.MagmaWall,DStruct.QuartzWall,DStruct.BoundaryWall} <> DStruct.DungeonElementSet {} THEN
          symbol := DStruct.DungeonWallSymbol;
          color  := CurrentDungeonColorTable [ColorElementForWall (location.elements)];
        ELSIF location.elements * DStruct.DungeonElementSet {DStruct.DarkFloor,DStruct.LightFloor,DStruct.CorridorFloor,DStruct.BlockedFloor} <> DStruct.DungeonElementSet {} THEN
          symbol := '.';
          color  := CurrentDungeonColorTable [FloorColor];
        ELSE
          WDebugging.FatalAppExit ("Unable to determine symbol for dungeon location.");
        END; (* IF *)
      END; (* IF *)
    ELSE
      symbol := OverrideSymbol;
      color  := CurrentDungeonColorTable [ArtifactColor];
      
      SetOverrideSymbol (InvalidOverrideSymbol);
    END; (* IF *)
    (*/CHECK*)
  END DetermineLocationDisplayInformation;
  
  PROCEDURE DungeonColorMnemonicToString (entry : DungeonColorMnemonic; VAR result : ARRAY OF CHAR);
  BEGIN (* DungeonColorMnemonicToString *)
    CASE entry OF
      ArtifactColor      : Strings.Assign ("Artifact",result);      |
      BackgroundColor    : Strings.Assign ("Background",result);    |
      CreatureColor      : Strings.Assign ("Creature",result);      |
      FloorColor         : Strings.Assign ("Floor",result);         |
      GraniteColor       : Strings.Assign ("Granite",result);       |
      HallucinationColor : Strings.Assign ("Hallucination",result); |
      MagmaColor         : Strings.Assign ("Magma",result);         |
      QuartzColor        : Strings.Assign ("Quartz",result);        |
      PlayerColor        : Strings.Assign ("Player",result);        |
    ELSE
      WDebugging.FatalAppExit ("Invalid dungeon color entry.");
    END; (* CASE *)
  END DungeonColorMnemonicToString;
  
  PROCEDURE DungeonSymbolForLocation (player : MStruct.PlayerInfoPointer; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange) : CHAR;
    VAR
        symbol : CHAR;
        color  : Windows.COLORREF;
    
  BEGIN (* DungeonSymbolForLocation *)
    DetermineLocationDisplayInformation (player,line,column,symbol,color);
    RETURN (symbol);
  END DungeonSymbolForLocation;
  
  PROCEDURE InitializeCurrentDungeonColorTable;
    VAR
        defaultBACKGROUND,defaultFOREGROUND : Windows.COLORREF;
        index                               : DungeonColorMnemonic;
    
    PROCEDURE InitializeDungeonColorEntry (color : DungeonColorMnemonic; default : Windows.COLORREF; VAR result : Windows.COLORREF);
      VAR
          entry : ARRAY [0..31] OF CHAR;
      
    BEGIN (* InitializeDungeonColorEntry *)
      DungeonColorMnemonicToString (color,entry);
      MInitFile.DetermineDungeonColorEntry (entry,default,result);
    END InitializeDungeonColorEntry;
    
  BEGIN (* InitializeCurrentDungeonColorTable *)
    defaultFOREGROUND := WSystem.GetSysColor (Windows.COLOR_WINDOWTEXT);
    defaultBACKGROUND := WSystem.GetSysColor (Windows.COLOR_WINDOW);
    
    FOR index := MIN (DungeonColorMnemonic) TO MAX (DungeonColorMnemonic) DO
      IF index <> BackgroundColor THEN
        InitializeDungeonColorEntry (index,defaultFOREGROUND,CurrentDungeonColorTable [index]);
      ELSE
        InitializeDungeonColorEntry (index,defaultBACKGROUND,CurrentDungeonColorTable [index]);
      END; (* IF *)
    END; (* FOR *)
  END InitializeCurrentDungeonColorTable;
  
  PROCEDURE SetOverrideSymbol (symbol : CHAR);
  BEGIN (* SetOverrideSymbol *)
    OverrideSymbol := symbol;
  END SetOverrideSymbol;
  
BEGIN (* MDSymbols *)
  InitializeCurrentDungeonColorTable;
  
  SetOverrideSymbol (InvalidOverrideSymbol);
END MDSymbols.
