IMPLEMENTATION MODULE MDWindow;
  (*/WIN:A*)
  
  IMPORT WCreation,WDisplay,Windows,WinStart,WMacro;
  
  IMPORT MDColors,MDPainting,MDScrolling,MDSymbols;
  IMPORT MFonts,MMouse,MTimer;
  IMPORT DStruct,MStruct;
  
  
  PROCEDURE CenterDungeonOnLocation (player : MStruct.PlayerInfoPointer; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange; redraw : BOOLEAN);
    VAR
        columns : INTEGER;
        lines   : INTEGER;
    
  BEGIN (* CenterDungeonOnLocation *)
    IF player^.windows.dungeon <> Windows.NULL THEN
      columns := CurrentDungeonWindowWidth (player);
      lines   := CurrentDungeonWindowHeight (player);
      
      IF column <= columns DIV 2 THEN
        player^.dungeon^.firstCOLUMN := 0;
      ELSIF column + columns DIV 2 >= player^.dungeon^.columns - 1 THEN
        player^.dungeon^.firstCOLUMN := WMacro.MaxInteger (player^.dungeon^.columns - columns,0);
      ELSE
        player^.dungeon^.firstCOLUMN := column - columns DIV 2;
      END; (* IF *)
      
      IF line <= lines DIV 2 THEN
        player^.dungeon^.firstLINE := 0;
      ELSIF line + lines DIV 2 >= player^.dungeon^.lines - 1 THEN
        player^.dungeon^.firstLINE := WMacro.MaxInteger (player^.dungeon^.lines - lines,0);
      ELSE
        player^.dungeon^.firstLINE := line - lines DIV 2;
      END; (* IF *)
      
      IF redraw THEN
        MDScrolling.ResetDungeonScrollBars (player);
        MDPainting.InvalidateDungeonWindow (player);
      END; (* IF *)
    END; (* IF *)
  END CenterDungeonOnLocation;
  
  PROCEDURE CreateDungeonWindow (parent : Windows.HWND; VAR result : Windows.HWND);
    CONST
          DungeonWindowStyle = Windows.WS_CAPTION + Windows.WS_CHILD + Windows.WS_CLIPSIBLINGS + Windows.WS_HSCROLL + Windows.WS_VSCROLL;
    
    VAR
        shown : BOOLEAN;
        rect  : Windows.RECT;
    
  BEGIN (* CreateDungeonWindow *)
    WDisplay.GetClientRect (parent,rect);
    
    WCreation.CreateWindow (MStruct.DungeonWindowClassName,"Dungeon",DungeonWindowStyle,rect.left + (rect.right - rect.left + 1) DIV 4,rect.top + (rect.bottom - rect.top + 1) DIV 4,(rect.right - rect.left + 1) * 3 DIV 4,(rect.bottom - rect.top + 1) * 3 DIV 4,parent,Windows.NULL,WinStart.hInstance,Windows.LP_NULL,result);
    WDisplay.ShowWindow (result,Windows.SW_SHOWNORMAL,shown);
  END CreateDungeonWindow;
  
  PROCEDURE CurrentDungeonWindowHeight (player : MStruct.PlayerInfoPointer) : INTEGER;
    VAR
        client : Windows.RECT;
    
  BEGIN (* CurrentDungeonWindowHeight *)
    WDisplay.GetClientRect (player^.windows.dungeon,client);
    RETURN (client.bottom DIV DungeonFontHeight (player^.windows.dungeon));
  END CurrentDungeonWindowHeight;
  
  PROCEDURE CurrentDungeonWindowWidth (player : MStruct.PlayerInfoPointer) : INTEGER;
    VAR
        client : Windows.RECT;
    
  BEGIN (* CurrentDungeonWindowWidth *)
    WDisplay.GetClientRect (player^.windows.dungeon,client);
    RETURN (client.right DIV DungeonFontWidth (player^.windows.dungeon));
  END CurrentDungeonWindowWidth;
  
  PROCEDURE DetermineDungeonWindowExtent (player : MStruct.PlayerInfoPointer; VAR minX,maxX : DStruct.DungeonColumnRange; VAR minY,maxY : DStruct.DungeonLineRange; center,redraw : BOOLEAN);
    VAR
        client : Windows.RECT;
    
  BEGIN (* DetermineDungeonWindowExtent *)
    IF center THEN
      CenterDungeonOnLocation (player,player^.character^.line,player^.character^.column,redraw);
    END; (* IF *)
    
    WDisplay.GetClientRect (player^.windows.dungeon,client);
    
    minX := player^.dungeon^.firstCOLUMN;
    minY := player^.dungeon^.firstLINE;
    maxY := WMacro.MinInteger (minY + (client.bottom - client.top + 1) DIV DungeonFontHeight (player^.windows.dungeon),player^.dungeon^.lines - 1);
    maxX := WMacro.MinInteger (minX + (client.right - client.left + 1) DIV DungeonFontWidth (player^.windows.dungeon),player^.dungeon^.columns - 1);
  END DetermineDungeonWindowExtent;
  
  PROCEDURE DisplayPassingMissile (player : MStruct.PlayerInfoPointer; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange; symbol : CHAR; erase : BOOLEAN);
  BEGIN (* DisplayPassingMissile *)
    MDSymbols.SetOverrideSymbol (symbol);
    
    MDPainting.InvalidateDungeonLocation (player,line,column,FALSE,TRUE);
    
    IF erase THEN
      MTimer.WaitForMilliseconds (5);
      MDPainting.InvalidateDungeonLocation (player,line,column,FALSE,TRUE);
    END; (* IF *)
  END DisplayPassingMissile;
  
  PROCEDURE DungeonFontHeight (window : Windows.HWND) : INTEGER;
    VAR
        height,width : INTEGER;
    
  BEGIN (* DungeonFontHeight *)
    MFonts.DetermineWindowCharacterMetrics (window,MDPainting.DungeonWindowFont,height,width);
    RETURN (height);
  END DungeonFontHeight;
  
  PROCEDURE DungeonFontWidth (window : Windows.HWND) : INTEGER;
    VAR
        height,width : INTEGER;
    
  BEGIN (* DungeonFontWidth *)
    MFonts.DetermineWindowCharacterMetrics (window,MDPainting.DungeonWindowFont,height,width);
    RETURN (width);
  END DungeonFontWidth;
  
  PROCEDURE DungeonWindowTranslator ['DUNGEONWINDOWTRANSLATOR'] (window : Windows.HWND; message : CARDINAL; param1 : Windows.WPARAM; param2 : Windows.LPARAM) : Windows.LRESULT [WINDOWS,LONG];
    VAR
        result : Windows.LRESULT;
    
    PROCEDURE DungeonColumnPosition (player : MStruct.PlayerInfoPointer; x : INTEGER) : DStruct.DungeonColumnRange;
      PROCEDURE FirstWindowColumn (window : Windows.HWND) : INTEGER;
        VAR
            client : Windows.RECT;
        
      BEGIN (* FirstWindowColumn *)
        WDisplay.GetClientRect (window,client);
        RETURN (client.left);
      END FirstWindowColumn;
      
    BEGIN (* DungeonColumnPosition *)
      RETURN (WMacro.MinInteger (((x - FirstWindowColumn (player^.windows.dungeon)) DIV DungeonFontWidth (player^.windows.dungeon) + player^.dungeon^.firstCOLUMN),player^.dungeon^.columns - 1));
    END DungeonColumnPosition;
    
    PROCEDURE DungeonLinePosition (player : MStruct.PlayerInfoPointer; y : INTEGER) : DStruct.DungeonLineRange;
      PROCEDURE FirstWindowLine (window : Windows.HWND) : INTEGER;
        VAR
            client : Windows.RECT;
        
      BEGIN (* FirstWindowLine *)
        WDisplay.GetClientRect (window,client);
        RETURN (client.top);
      END FirstWindowLine;
      
    BEGIN (* DungeonLinePosition *)
      RETURN (WMacro.MinInteger ((y - FirstWindowLine (player^.windows.dungeon)) DIV DungeonFontHeight (player^.windows.dungeon) + player^.dungeon^.firstLINE,player^.dungeon^.lines - 1));
    END DungeonLinePosition;
    
  BEGIN (* DungeonWindowTranslator *)
    result := Windows.FALSE_BOOL;
    
    CASE message OF
      Windows.WM_HSCROLL : MDScrolling.ExecuteHorizontalDungeonScroll (window,MStruct.CurrentPlayerInformation,param1,WMacro.LOWORD (param2)); |
      Windows.WM_VSCROLL : MDScrolling.ExecuteVerticalDungeonScroll (window,MStruct.CurrentPlayerInformation,param1,WMacro.LOWORD (param2));   |
      Windows.WM_CREATE  : MDColors.InitializeBackgroundColor (window);                                                                        |
      Windows.WM_PAINT   : MDPainting.ExecuteDungeonWindowPaintAction (window,MStruct.CurrentPlayerInformation);                               |
      Windows.WM_MOVE    : ResizeDungeonWindow (MStruct.CurrentPlayerInformation);                                                             |
      
      Windows.WM_LBUTTONDOWN : MMouse.ExecuteLeftMouseButtonAction (MStruct.CurrentPlayerInformation,param1,DungeonLinePosition (MStruct.CurrentPlayerInformation,WMacro.HIWORD (param2)),DungeonColumnPosition (MStruct.CurrentPlayerInformation,WMacro.LOWORD (param2)));   |
      Windows.WM_RBUTTONDOWN : MMouse.ExecuteRightMouseButtonAction (MStruct.CurrentPlayerInformation,param1,DungeonLinePosition (MStruct.CurrentPlayerInformation,WMacro.HIWORD (param2)),DungeonColumnPosition (MStruct.CurrentPlayerInformation,WMacro.LOWORD (param2)));  |
    ELSE
      result := WCreation.DefWindowProc (window,message,param1,param2);
    END; (* CASE *)
    
    RETURN (result);
  END DungeonWindowTranslator;
  
  PROCEDURE LocationIsInWindow (player : MStruct.PlayerInfoPointer; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange) : BOOLEAN;
    VAR
        client : Windows.RECT;
    
  BEGIN (* LocationIsInWindow *)
    IF ((column < player^.dungeon^.firstCOLUMN) OR (line < player^.dungeon^.firstLINE)) THEN
      RETURN (FALSE);
    END; (* IF *)
    
    WDisplay.GetClientRect (player^.windows.dungeon,client);
    
    RETURN (((column - player^.dungeon^.firstCOLUMN) * DungeonFontWidth (player^.windows.dungeon) <= client.right) AND ((line - player^.dungeon^.firstLINE) * DungeonFontHeight (player^.windows.dungeon) <= client.bottom));
  END LocationIsInWindow;
  
  PROCEDURE ResizeDungeonWindow (player : MStruct.PlayerInfoPointer);
    VAR
        client : Windows.RECT;
        done   : BOOLEAN;
    
  BEGIN (* ResizeDungeonWindow *)
    IF player <> NIL THEN
      WDisplay.GetClientRect (player^.windows.frame,client);
      WDisplay.MoveWindow (player^.windows.dungeon,client.left + (client.right - client.left + 1) DIV 4,client.top + (client.bottom - client.top + 1) DIV 4,(client.right - client.left + 1) * 3 DIV 4,(client.bottom - client.top + 1) * 3 DIV 4,TRUE,done);
    END; (* IF *)
  END ResizeDungeonWindow;
END MDWindow.
