IMPLEMENTATION MODULE MInventory;
  (*/WIN:A*)
  
  IMPORT MCStruct,MStruct,OStruct;
  IMPORT OSActions;
  
  
  PROCEDURE AddObjectToPlayerInventory (player : MStruct.PlayerInfoPointer; item : OStruct.InventoryItemRecord; VAR location : CARDINAL);
    VAR
        done : BOOLEAN;
    
    PROCEDURE InsertObjectIntoInventory (item : OStruct.InventoryItemRecord; VAR inventory : MCStruct.PlayerInventoryInformation; location : CARDINAL);
      VAR
          index : CARDINAL;
      
    BEGIN (* InsertObjectIntoInventory *)
      FOR index := inventory.counter TO location + 1 BY -1 DO
        inventory.backpack [index] := inventory.backpack [index - 1];
      END; (* FOR *)
      
      inventory.backpack [location] := item;
      inventory.counter             := inventory.counter + 1;
    END InsertObjectIntoInventory;
    
    PROCEDURE ItemsCanBeAdded (treasures : OStruct.TreasureInfoPointer; item1,item2 : OStruct.InventoryItemRecord) : BOOLEAN;
    BEGIN (* ItemsCanBeAdded *)
      IF ((item1.object.class <> item2.object.class) OR (item1.object.subclass <> item2.object.subclass) OR (item1.object.subclass < OStruct.ObjectMinimumSingleStackSubclass)) THEN
        RETURN (FALSE);
      ELSIF item1.object.quantity + item2.object.quantity >= 256 THEN
        RETURN (FALSE);
      ELSIF ((item1.object.subclass >= OStruct.ObjectMinimumItemGroupSubclass) AND (item1.object.miscellaneous <> item2.object.miscellaneous)) THEN
        RETURN (FALSE);
      ELSE
        RETURN (OSActions.InventoryItemIsKnown1 (treasures,item1) = OSActions.InventoryItemIsKnown1 (treasures,item2));
      END; (* IF *)
    END ItemsCanBeAdded;
    
    PROCEDURE ItemShouldBeInserted (item1,item2 : OStruct.InventoryItemRecord) : BOOLEAN;
    BEGIN (* ItemShouldBeInserted *)
      RETURN ((item1.object.class > item2.object.class) OR ((item1.object.class = item2.object.class) AND (item1.object.subclass < item2.object.subclass) AND OSActions.ObjectIsAlwaysKnown1 (item1.object)));
    END ItemShouldBeInserted;
    
  BEGIN (* AddObjectToPlayerInventory *)
    location := 0;
    done     := FALSE;
    
    REPEAT
      IF ItemsCanBeAdded (player^.treasures,item,player^.character^.inventory.backpack [location]) THEN
        INC (player^.character^.inventory.backpack [location].object.quantity,item.object.quantity);
        done := TRUE;
      ELSIF ItemShouldBeInserted (item,player^.character^.inventory.backpack [location]) THEN
        InsertObjectIntoInventory (item,player^.character^.inventory,location);
        done := TRUE;
      ELSE
        INC (location);
      END; (* IF *)
    UNTIL done;
    
    INC (player^.character^.inventory.weight,item.object.quantity * VAL (CARDINAL,item.object.weight));
    INCL (player^.character^.variables.status,MCStruct.StrWgtFLAG);
  END AddObjectToPlayerInventory;
  
  PROCEDURE AnyObjectsWithClasses (VAR inventory : ARRAY OF OStruct.InventoryItemRecord; classes : OStruct.ObjectClassSet) : BOOLEAN;
    VAR
        index : CARDINAL;
    
  BEGIN (* AnyObjectsWithClasses *)
    FOR index := 0 TO HIGH (inventory) DO
      IF inventory [index].object.class IN classes THEN
        RETURN (TRUE);
      END; (* IF *)
    END; (* FOR *)
    
    RETURN (FALSE);
  END AnyObjectsWithClasses;
  
  PROCEDURE DecrementInventoryItemQuantity (player : MStruct.PlayerInfoPointer; position : CARDINAL; VAR deleted : BOOLEAN);
  BEGIN (* DecrementInventoryItemQuantity *)
    deleted := NOT ((player^.character^.inventory.backpack [position].object.quantity > 1) AND (player^.character^.inventory.backpack [position].object.subclass <= OStruct.ObjectMaximumSingleStackSubclass));
    
    IF NOT (deleted) THEN
      DEC (player^.character^.inventory.weight,player^.character^.inventory.backpack [position].object.weight);
      DEC (player^.character^.inventory.backpack [position].object.quantity);
    ELSE
      DeleteInventoryItem (player,position);
    END; (* IF *)
    
    INCL (player^.character^.variables.status,MCStruct.StrWgtFLAG);
  END DecrementInventoryItemQuantity;
  
  PROCEDURE DeleteInventoryItem (player : MStruct.PlayerInfoPointer; position : CARDINAL);
    PROCEDURE ShiftInventoryItemsLeft (VAR inventory : ARRAY OF OStruct.InventoryItemRecord; position : CARDINAL);
      VAR
          index : CARDINAL;
      
    BEGIN (* ShiftInventoryItemsLeft *)
      FOR index := position + 1 TO HIGH (inventory) DO
        inventory [index - 1] := inventory [index];
      END; (* FOR *)
    END ShiftInventoryItemsLeft;
    
  BEGIN (* DeleteInventoryItem *)
    DEC (player^.character^.inventory.weight,VAL (CARDINAL,player^.character^.inventory.backpack [position].object.weight) * player^.character^.inventory.backpack [position].object.quantity);
    
    ShiftInventoryItemsLeft (player^.character^.inventory.backpack,position);
    DEC (player^.character^.inventory.counter);
    
    OSActions.ReturnInventoryItemRecord (player^.treasures,OStruct.EmptyObjectCursor,player^.character^.inventory.backpack [MCStruct.MaxInventoryItems - 1]);
    
    INCL (player^.character^.variables.status,MCStruct.StrWgtFLAG);
  END DeleteInventoryItem;
  
  PROCEDURE ObjectClassPosition (VAR inventory : ARRAY OF OStruct.InventoryItemRecord; class : OStruct.ObjectClassMnemonic) : CARDINAL;
    VAR
        index : CARDINAL;
    
  BEGIN (* ObjectClassPosition *)
    FOR index := 0 TO HIGH (inventory) DO
      IF inventory [index].object.class = class THEN
        RETURN (index);
      END; (* IF *)
    END; (* FOR *)
    
    RETURN (MAX (CARDINAL));
  END ObjectClassPosition;
  
  PROCEDURE ObjectFitsInInventory (treasures : OStruct.TreasureInfoPointer; item : OStruct.InventoryItemRecord; VAR inventory : ARRAY OF OStruct.InventoryItemRecord) : BOOLEAN;
    VAR
        index : CARDINAL;
    
    PROCEDURE ObjectsCanBeAdded (treasures : OStruct.TreasureInfoPointer; item1,item2 : OStruct.InventoryItemRecord) : BOOLEAN;
    BEGIN (* ObjectsCanBeAdded *)
      IF ((item1.object.class <> item2.object.class) OR (item1.object.subclass <> item2.object.subclass)) THEN
        RETURN (FALSE);
      ELSIF item1.object.quantity + item2.object.quantity >= 256 THEN
        RETURN (FALSE);
      ELSIF ((item1.object.subclass >= OStruct.ObjectMinimumItemGroupSubclass) AND (item1.object.miscellaneous <> item2.object.miscellaneous)) THEN
        RETURN (FALSE);
      END; (* IF *)
      
      RETURN (OSActions.InventoryItemIsKnown1 (treasures,item1) = OSActions.InventoryItemIsKnown1 (treasures,item2));
    END ObjectsCanBeAdded;
    
  BEGIN (* ObjectFitsInInventory *)
    IF inventory [HIGH (inventory)].object.class = OStruct.Nothing THEN
      RETURN (TRUE);
    ELSIF item.object.subclass < OStruct.ObjectMinimumSingleStackSubclass THEN
      RETURN (FALSE);
    END; (* IF *)
    
    FOR index := 0 TO HIGH (inventory) DO
      IF ObjectsCanBeAdded (treasures,item,inventory [index]) THEN
        RETURN (TRUE);
      END; (* IF *)
    END; (* FOR *)
    
    RETURN (FALSE);
  END ObjectFitsInInventory;
END MInventory.
