IMPLEMENTATION MODULE MLook;
  (*/WIN:A*)
  
  IMPORT DLineDDA,MMemory,MMTranslator,MMWindow,ODescriptions;
  IMPORT CStruct,DStruct,MCStruct,MStruct,OStruct;
  IMPORT DSActions,OSActions;
  IMPORT WDebugging,WString;
  
  
  VAR
      LineOfSightClear : BOOLEAN;
  
  
  PROCEDURE DescribeLocationContents (player : MStruct.PlayerInfoPointer; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange; VAR done : BOOLEAN);
    VAR
        artifact,creature,wall : BOOLEAN;
        location               : DStruct.PhysicalDungeonUnit;
    
    PROCEDURE ArtifactIsVisible (artifact : OStruct.InventoryItemRecord) : BOOLEAN;
    BEGIN (* ArtifactIsVisible *)
      RETURN (artifact.object.class <> OStruct.InvisibleTrap);
    END ArtifactIsVisible;
    
    PROCEDURE DescribeArtifact (player : MStruct.PlayerInfoPointer; VAR location : DStruct.PhysicalDungeonUnit; item : OStruct.InventoryItemRecord; creature : BOOLEAN; VAR artifact : BOOLEAN);
    BEGIN (* DescribeArtifact *)
      IF item.object.class = OStruct.SecretDoor THEN
        INCL (location.elements,DStruct.GraniteWall);
        artifact := FALSE;
      ELSIF creature THEN
        OSActions.DisplayObjectMessage (player^.windows.frame,player,"It is on %OBJECT","%OBJECT",item,TRUE,TRUE);
      ELSE
        OSActions.DisplayObjectMessage (player^.windows.frame,player,"You see %OBJECT","%OBJECT",item,TRUE,TRUE);
      END; (* IF *)
      
      LineOfSightClear := NOT (DStruct.BlockedFloor IN location.elements);
    END DescribeArtifact;
    
    PROCEDURE DescribeCreature (player : MStruct.PlayerInfoPointer; creature : CStruct.PassiveCreatureInfoRecord);
      VAR
          message : ARRAY [0..127] OF CHAR;
      
    BEGIN (* DescribeCreature *)
      WString.ReplaceStringParameter ("You see %ARTICLE %CREATURE.  Recall?","%CREATURE",creature.description,message);
      
      IF ODescriptions.IsVowel (creature.description [0]) THEN
        WString.ReplaceStringParameter (message,"%ARTICLE","an",message);
      ELSE
        WString.ReplaceStringParameter (message,"%ARTICLE","a",message);
      END; (* IF *)
      
      IF MMWindow.QueryYesNo (player^.windows.frame,player,message,FALSE) THEN
        MMemory.RecallCreatureInformation (player,creature);
      END; (* IF *)
    END DescribeCreature;
    
    PROCEDURE DescribeDungeonWall (player : MStruct.PlayerInfoPointer; elements : DStruct.DungeonElementSet; creature,artifact : BOOLEAN);
      VAR
          message : ARRAY [0..127] OF CHAR;
      
    BEGIN (* DescribeDungeonWall *)
      IF elements * DStruct.DungeonElementSet {DStruct.BoundaryWall,DStruct.GraniteWall} <> DStruct.EmptyDungeonElementSet THEN
        message := "%PREFACE a granite wall.";
      ELSIF DStruct.QuartzWall IN elements THEN
        message := "%PREFACE a quartz vein.";
      ELSIF DStruct.MagmaWall IN elements THEN
        message := "%PREFACE some dark rock.";
      ELSE
        WDebugging.FatalAppExit ("Unrecognized wall type while looking.");
      END; (* IF *)
      
      LineOfSightClear := FALSE;
      
      IF (artifact OR creature) THEN
        WString.ReplaceStringParameter (message,"%PREFACE","It is in",message);
      ELSE
        WString.ReplaceStringParameter (message,"%PREFACE","You see",message);
      END; (* IF *)
      
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,message);
    END DescribeDungeonWall;
    
  BEGIN (* DescribeLocationContents *)
    LineOfSightClear := (LineOfSightClear AND (DSActions.QuickDistance (player^.character^.line,player^.character^.column,line,column) <= MCStruct.MaxCharacterSightDistance));
    
    IF LineOfSightClear THEN
      location := player^.dungeon^.physical [line]^ [column];
      creature := ((location.creature <> DStruct.InvalidCreatureCursor) AND player^.dungeon^.creatures [location.creature].visible);
      
      IF creature THEN
        DescribeCreature (player,player^.creatures^.table [player^.dungeon^.creatures [location.creature].cursor]);
      END; (* IF *)
      
      IF location.elements * DStruct.DungeonElementSet {DStruct.tlFLAG,DStruct.plFLAG,DStruct.fmFLAG} <> DStruct.EmptyDungeonElementSet THEN
        artifact := ((location.artifact <> DStruct.InvalidArtifactCursor) AND ArtifactIsVisible (player^.dungeon^.artifacts [location.artifact]));
        
        IF artifact THEN
          DescribeArtifact (player,location,player^.dungeon^.artifacts [location.artifact],creature,artifact);
        END; (* IF *)
        
        wall := (location.elements * DStruct.DungeonWallElements <> DStruct.EmptyDungeonElementSet);
        
        IF wall THEN
          DescribeDungeonWall (player,location.elements,creature,artifact);
        END; (* IF *)
      ELSE
        artifact := FALSE;
        wall     := FALSE;
      END; (* IF *)
      
      done := (done OR creature OR artifact OR wall);
    END; (* IF *)
  END DescribeLocationContents;
  
  PROCEDURE ExecuteLookAction (player : MStruct.PlayerInfoPointer; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange);
    VAR
        done : BOOLEAN;
    
  BEGIN (* ExecuteLookAction *)
    LineOfSightClear := TRUE;
    done             := FALSE;
    
    DLineDDA.ExecuteDungeonTraceProcedure (player,player^.character^.line,player^.character^.column,line,column,DescribeLocationContents,done);
    
    IF LineOfSightClear THEN
      DescribeLocationContents (player,line,column,done);
    END; (* IF *)
    
    IF NOT (done) THEN
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You see nothing of interest in that direction.");
    END; (* IF *)
  END ExecuteLookAction;
  
  PROCEDURE ExecuteLookCommand (player : MStruct.PlayerInfoPointer);
  BEGIN (* ExecuteLookCommand *)
    IF player^.character^.variables.counters [MCStruct.BlindCOUNT] > 0 THEN
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You can't see a damn thing!");
    ELSIF player^.character^.variables.counters [MCStruct.ImageCOUNT] > 0 THEN
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You can't believe what you are seeing! It's like a dream!");
    ELSE
      MMTranslator.ExecuteTargetedCommand (player,"Look at what?",ExecuteLookAction);
    END; (* IF *)
    
    MStruct.CharacterTurnExecuted := FALSE;
  END ExecuteLookCommand;
END MLook.
