IMPLEMENTATION MODULE MMTranslator;
  (*/WIN:A*)
  
  IMPORT WCreation,Windows,WInput,WMacro,WMessage;
  
  IMPORT MCommands,MDColors,MDWindow,MFiles,MKeys,MMenu,MMouse,MMTrans2,MMWindow,MSColors,MSWindow,MTitle;
  IMPORT MStruct;
  
  
  PROCEDURE ExecuteDirectedCharAction (player : MStruct.PlayerInfoPointer; value : CARDINAL);
  BEGIN (* ExecuteDirectedCharAction *)
    IF MKeys.ValidDirectionCharacter (value) THEN
      ExecuteDirectionCommand (player,MKeys.DirectionForCharacter (value));
      
      MMouse.ExecuteLeftMouseButtonAction := MMouse.ExecuteStandardLeftMouseButtonAction;
      ExecuteKeyDownAction                := ExecuteStandardKeyDownAction;
      ExecuteCharAction                   := ExecuteStandardCharAction;
      
      MMenu.EnableMainWindowMenu (player^.windows.frame,TRUE);
    ELSIF value = Windows.VK_ESCAPE THEN
      MMWindow.AddMessageToList (player,"Directed command canceled.");
      
      MMouse.ExecuteLeftMouseButtonAction := MMouse.ExecuteStandardLeftMouseButtonAction;
      ExecuteKeyDownAction                := ExecuteStandardKeyDownAction;
      ExecuteCharAction                   := ExecuteStandardCharAction;
      
      MMenu.EnableMainWindowMenu (player^.windows.frame,TRUE);
    END; (* IF *)
  END ExecuteDirectedCharAction;
  
  PROCEDURE ExecuteDirectedCommand (player : MStruct.PlayerInfoPointer; prompt : ARRAY OF CHAR; command : DirectedCommandProcedure);
  BEGIN (* ExecuteDirectedCommand *)
    MMenu.EnableMainWindowMenu (player^.windows.frame,FALSE);
    
    MMouse.ExecuteLeftMouseButtonAction := MMTrans2.ExecuteDirectedMouseAction;
    MStruct.CharacterTurnExecuted       := FALSE;
    ExecuteDirectionCommand             := command;
    ExecuteKeyDownAction                := ExecuteDirectedCharAction;
    ExecuteCharAction                   := ExecuteDirectedCharAction;
    
    MMWindow.AddMessageToList (player,prompt);
    
    IF player^.target.defined THEN
      MMTrans2.ExecuteDirectedMouseAction (player,0,player^.target.line,player^.target.column);
      player^.target.defined := FALSE;
    END; (* IF *)
  END ExecuteDirectedCommand;
  
  PROCEDURE ExecuteStandardCharAction (player : MStruct.PlayerInfoPointer; value : CARDINAL);
    VAR
        command : MCommands.MoriaCommandMnemonic;
        done    : BOOLEAN;
    
  BEGIN (* ExecuteStandardCharAction *)
    IF player <> NIL THEN
      MCommands.ConvertToCommandMnemonic (player,value,command,done);
      
      IF done THEN
        MCommands.ExecuteCharacterCommand (player,command);
      END; (* IF *)
    END; (* CASE *)
  END ExecuteStandardCharAction;
  
  PROCEDURE ExecuteStandardKeyDownAction (player : MStruct.PlayerInfoPointer; value : CARDINAL);
  BEGIN (* ExecuteStandardKeyDownAction *)
    IF ((player <> NIL) AND MKeys.ValidKeyDownValue (value)) THEN
      ExecuteStandardCharAction (player,ORD (MKeys.CharForKeyDownValue (value)));
    END; (* IF *)
  END ExecuteStandardKeyDownAction;
  
  PROCEDURE ExecuteTargetedCommand (player : MStruct.PlayerInfoPointer; prompt : ARRAY OF CHAR; command : TargetedCommandProcedure);
  BEGIN (* ExecuteTargetedCommand *)
    MMenu.EnableMainWindowMenu (player^.windows.frame,FALSE);
    
    MMouse.ExecuteLeftMouseButtonAction := MMTrans2.ExecuteTargetedMouseAction;
    MStruct.CharacterTurnExecuted       := FALSE;
    ExecuteTargetCommand                := command;
    ExecuteKeyDownAction                := MMTrans2.ExecuteTargetedCharAction;
    ExecuteCharAction                   := MMTrans2.ExecuteTargetedCharAction;
    
    MMWindow.AddMessageToList (player,prompt);
    
    IF player^.target.defined THEN
      MMTrans2.ExecuteTargetedMouseAction (player,0,player^.target.line,player^.target.column);
      player^.target.defined := FALSE;
    END; (* IF *)
  END ExecuteTargetedCommand;
  
  PROCEDURE MainWindowTranslator ['MAINWINDOWTRANSLATOR'] (window : Windows.HWND; message : CARDINAL; param1 : Windows.WPARAM; param2 : Windows.LPARAM) : Windows.LRESULT [WINDOWS,LONG];
    VAR
        result : Windows.LRESULT;
    
    PROCEDURE ExecuteDestroyAction (window : Windows.HWND);
    BEGIN (* ExecuteDestroyAction *)
      MSColors.ReleaseBackgroundBrushes;
      MDColors.ReleaseBackgroundBrush;
      MTitle.DestroyTitleBitmap;
                                                                  
      WMessage.PostQuitMessage (0);
    END ExecuteDestroyAction;
    
    PROCEDURE ExecuteMainWindowCommand (window : Windows.HWND; VAR player : MStruct.PlayerInfoPointer; param1 : Windows.WPARAM; param2 : Windows.LPARAM; VAR result : Windows.LRESULT);
      VAR
          previous : Windows.HWND;
      
      PROCEDURE ExecuteMainMenuCommand (window : Windows.HWND; VAR player : MStruct.PlayerInfoPointer; command : CARDINAL);
        VAR
            result : MCommands.MoriaCommandMnemonic;
            done   : BOOLEAN;
        
      BEGIN (* ExecuteMainMenuCommand *)
        MCommands.ConvertMenuItemToCommandMnemonic (player,command,result,done);
        
        IF done THEN
          MCommands.ExecuteCharacterCommand (player,result);
        ELSE
          MMWindow.DisplayPopupMessage (window,player,"Unrecognized main menu item control.");
        END; (* IF *)
      END ExecuteMainMenuCommand;
      
    BEGIN (* ExecuteMainWindowCommand *)
      IF WMacro.HIWORD (param2) = 0 THEN
        ExecuteMainMenuCommand (window,player,param1);
        result := Windows.FALSE_BOOL;
      ELSIF WMacro.HIWORD (param2) = Windows.LBN_SETFOCUS THEN
        WInput.SetFocus (window,previous);
        result := Windows.FALSE_BOOL;
      ELSE
        result := Windows.TRUE_BOOL;
      END; (* IF *)
    END ExecuteMainWindowCommand;
    
    PROCEDURE ExecuteQueryEndSessionCommand (player : MStruct.PlayerInfoPointer; VAR result : Windows.LRESULT);
      VAR
          cancel : BOOLEAN;
      
    BEGIN (* ExecuteQueryEndSessionCommand *)
      IF player <> NIL THEN
        MFiles.OptionallySaveCharacter (player,cancel);
      ELSE
        cancel := FALSE;
      END; (* IF *)
      
      result := ORD (NOT (cancel));
    END ExecuteQueryEndSessionCommand;
    
    PROCEDURE ExecuteSizeAction (window : Windows.HWND; player : MStruct.PlayerInfoPointer);
    BEGIN (* ExecuteSizeAction *)
      MDWindow.ResizeDungeonWindow (player);
      MMWindow.ResizeMessageWindow (player);
      MSWindow.ResizeStatusWindow (player);
    END ExecuteSizeAction;
    
  BEGIN (* MainWindowTranslator *)
    result := Windows.FALSE_BOOL;
    
    CASE message OF
      Windows.WM_QUERYENDSESSION : ExecuteQueryEndSessionCommand (MStruct.CurrentPlayerInformation,result);                 |
      Windows.WM_COMMAND         : ExecuteMainWindowCommand (window,MStruct.CurrentPlayerInformation,param1,param2,result); |
      Windows.WM_DESTROY         : ExecuteDestroyAction (window);                                                           |
      Windows.WM_CREATE          : MTitle.InitializeTitleBitmap;                                                            |
      Windows.WM_CLOSE           : MFiles.ExecuteCloseAction (window,MStruct.CurrentPlayerInformation);                     |
      Windows.WM_PAINT           : MTitle.DisplayTitleBitmap (window);                                                      |
      Windows.WM_SIZE            : ExecuteSizeAction (window,MStruct.CurrentPlayerInformation);                             |
      
      Windows.WM_KEYDOWN : ExecuteKeyDownAction (MStruct.CurrentPlayerInformation,param1); |
      Windows.WM_CHAR    : ExecuteCharAction (MStruct.CurrentPlayerInformation,param1);    |
    ELSE
      result := WCreation.DefWindowProc (window,message,param1,param2);
    END; (* CASE *)
    
    RETURN (result);
  END MainWindowTranslator;
  
BEGIN (* MMTranslator *)
  MMouse.ExecuteLeftMouseButtonAction := MMouse.ExecuteStandardLeftMouseButtonAction;
  ExecuteKeyDownAction                := ExecuteStandardKeyDownAction;
  ExecuteCharAction                   := ExecuteStandardCharAction;
END MMTranslator.
