IMPLEMENTATION MODULE MMWindow;
  (*/WIN:A*)
  
  IMPORT WCreation,WDisplay,WError,Windows,WinStart,WListBox;
  IMPORT Strings;
  
  IMPORT MCreatures,MFonts;
  IMPORT MStruct;
  
  
  PROCEDURE AddMessageToList (player : MStruct.PlayerInfoPointer; message : ARRAY OF CHAR);
    VAR
        position : Windows.LRESULT;
        result   : Windows.LRESULT;
        count    : Windows.LRESULT;
        width    : CARDINAL;
    
  BEGIN (* AddMessageToList *)
    IF player^.windows.message.count = MStruct.MaxMoriaMessages THEN
      WListBox.DeleteString (player^.windows.message.listbox,0,count);
      player^.windows.message.count := count;
    END; (* IF *)
    
    WListBox.AddString (player^.windows.message.listbox,message,position);
    WListBox.SetCurSel (player^.windows.message.listbox,position,result);
    
    INC (player^.windows.message.count);
    
    width := MFonts.WindowTextStringWidth (player^.windows.message.listbox,message) + 10;
    
    IF width > player^.windows.message.width THEN
      WListBox.SetHorizontalExtent (player^.windows.message.listbox,width);
      player^.windows.message.width := width;
    END; (* IF *)
  END AddMessageToList;
  
  PROCEDURE ComputeMessageTitleWindowInformation (parent : Windows.HWND; VAR x,y : INTEGER; VAR width,height : INTEGER);
    VAR
        client : Windows.RECT;
    
  BEGIN (* ComputeMessageTitleWindowInformation *)
    WDisplay.GetClientRect (parent,client);
    
    x := client.left + (client.right - client.left + 1) DIV 4;
    y := client.top;
    
    height := client.top + (client.bottom - client.top + 1) DIV 4;
    width  := client.right - x;
  END ComputeMessageTitleWindowInformation;
  
  PROCEDURE CreateMessageWindow (parent : Windows.HWND; VAR result : MStruct.MessageWindowInformation);
    CONST
          MessageWindowStyle = Windows.WS_CAPTION + Windows.WS_CHILD + Windows.WS_CLIPSIBLINGS;
    
    CONST
          MaxMoriaMessageLength = 128;
    
    VAR
        height,width : INTEGER;
        shown        : BOOLEAN;
        x,y          : INTEGER;
    
    PROCEDURE CreateMessageListBox (parent : Windows.HWND; VAR result : Windows.HWND);
      VAR
          shown : BOOLEAN;
          rect  : Windows.RECT;
      
    BEGIN (* CreateMessageListBox *)
      WDisplay.GetClientRect (parent,rect);
      
      WCreation.CreateWindow ("LISTBOX","",Windows.LBS_NOINTEGRALHEIGHT + Windows.WS_CHILD + Windows.WS_CLIPSIBLINGS + Windows.WS_HSCROLL + Windows.WS_VSCROLL,rect.left,rect.top,rect.right - rect.left + 1,rect.bottom - rect.top + 1,parent,Windows.NULL,WinStart.hInstance,Windows.LP_NULL,result);
      WDisplay.ShowWindow (result,Windows.SW_NORMAL,shown);
    END CreateMessageListBox;
    
  BEGIN (* CreateMessageWindow *)
    ComputeMessageTitleWindowInformation (parent,x,y,width,height);
    
    WCreation.CreateWindow (MStruct.MessageWindowClassName,"Messages",MessageWindowStyle,x,y,width,height,parent,Windows.NULL,WinStart.hInstance,Windows.LP_NULL,result.frame);
    CreateMessageListBox (result.frame,result.listbox);
    
    WDisplay.ShowWindow (result.frame,Windows.SW_SHOWNORMAL,shown);
    
    result.width := 0;
    result.count := 0;
  END CreateMessageWindow;
  
  PROCEDURE DisplayCreatureMessage (player : MStruct.PlayerInfoPointer; text : ARRAY OF CHAR; creature : ARRAY OF CHAR; visible : BOOLEAN; popup : BOOLEAN);
    VAR
        message : ARRAY [0..127] OF CHAR;
    
  BEGIN (* DisplayCreatureMessage *)
    MCreatures.ReplaceCreatureParameter (text,creature,visible,message);
    
    IF popup THEN
      DisplayPopupMessage (player^.windows.frame,player,message);
    ELSE
      AddMessageToList (player,message);
    END; (* IF *)
  END DisplayCreatureMessage;
  
  PROCEDURE DisplayPopupMessage (window : Windows.HWND; player : MStruct.PlayerInfoPointer; message : ARRAY OF CHAR);
    CONST
          MoriaMessageTitle = "Moria";
    
  BEGIN (* DisplayPopupMessage *)
    IF ((player <> NIL) AND (player^.windows.dialog <> Windows.NULL)) THEN
      WError.DisplayInformationBox (player^.windows.dialog,MoriaMessageTitle,message);
    ELSE
      WError.DisplayInformationBox (window,MoriaMessageTitle,message);
    END; (* IF *)
    
    IF player <> NIL THEN
      AddMessageToList (player,message);
    END; (* IF *)
  END DisplayPopupMessage;
  
  PROCEDURE DisplayStoreMessage (window : Windows.HWND; player : MStruct.PlayerInfoPointer; owner : ARRAY OF CHAR; message : ARRAY OF CHAR);
  BEGIN (* DisplayStoreMessage *)
    ReturnStoreOwnerName (owner,owner);
    
    WError.DisplayInformationBox (window,owner,message);
    AddMessageToList (player,message);
  END DisplayStoreMessage;
  
  PROCEDURE MessageWindowTranslator ['MESSAGEWINDOWTRANSLATOR'] (window : Windows.HWND; message : CARDINAL; param1 : Windows.WPARAM; param2 : Windows.LPARAM) : Windows.LRESULT [WINDOWS,LONG];
    VAR
        result : Windows.LRESULT;
    
  BEGIN (* MessageWindowTranslator *)
    IF message = Windows.WM_MOVE THEN
      ResizeMessageWindow (MStruct.CurrentPlayerInformation);
      result := Windows.FALSE_BOOL;
    ELSE
      result := WCreation.DefWindowProc (window,message,param1,param2);
    END; (* IF *)
    
    RETURN (result);
  END MessageWindowTranslator;
  
  PROCEDURE QueryYesNo (window : Windows.HWND; player : MStruct.PlayerInfoPointer; query : ARRAY OF CHAR; defaultNO : BOOLEAN) : BOOLEAN;
    VAR
        message : ARRAY [0..127] OF CHAR;
        confirm : BOOLEAN;
    
  BEGIN (* QueryYesNo *)
    confirm := WError.Confirmed (window,MStruct.MainWindowTitle,query,defaultNO);
    
    IF confirm THEN
      Strings.Concat (query," YES",message);
    ELSE
      Strings.Concat (query," NO",message);
    END; (* IF *)
    
    AddMessageToList (player,message);
    
    RETURN (confirm);
  END QueryYesNo;
  
  PROCEDURE QueryYesNoCancel (window : Windows.HWND; player : MStruct.PlayerInfoPointer; query : ARRAY OF CHAR; VAR accept,cancel : BOOLEAN);
    VAR
        message : ARRAY [0..127] OF CHAR;
        result  : INTEGER;
    
  BEGIN (* QueryYesNoCancel *)
    WError.MessageBox (window,MStruct.MainWindowTitle,query,Windows.MB_APPLMODAL + Windows.MB_ICONQUESTION + Windows.MB_YESNOCANCEL,result);
    
    cancel := (result = Windows.IDCANCEL);
    accept := (result = Windows.IDYES);
    
    IF cancel THEN
      Strings.Concat (query," CANCEL",message);
    ELSIF accept THEN
      Strings.Concat (query," YES",message);
    ELSE
      Strings.Concat (query," NO",message);
    END; (* IF *)
    
    AddMessageToList (player,message);
  END QueryYesNoCancel;
  
  PROCEDURE ResizeMessageWindow (player : MStruct.PlayerInfoPointer);
    VAR
        height,width : INTEGER;
        done         : BOOLEAN;
        x,y          : INTEGER;
    
    PROCEDURE MoveListBox (parent,listbox : Windows.HWND);
      VAR
          rect : Windows.RECT;
          done : BOOLEAN;
      
    BEGIN (* MoveListBox *)
      WDisplay.GetClientRect (parent,rect);
      WDisplay.MoveWindow (listbox,rect.left,rect.top,rect.right - rect.left + 1,rect.bottom - rect.top + 1,TRUE,done);
    END MoveListBox;
    
  BEGIN (* ResizeMessageWindow *)
    IF ((player <> NIL) AND (player^.windows.message.frame <> Windows.NULL)) THEN
      ComputeMessageTitleWindowInformation (player^.windows.frame,x,y,width,height);
      
      WDisplay.MoveWindow (player^.windows.message.frame,x,y,width,height,TRUE,done);
      MoveListBox (player^.windows.message.frame,player^.windows.message.listbox);
    END; (* IF *)
  END ResizeMessageWindow;

  PROCEDURE ReturnStoreOwnerName (description : ARRAY OF CHAR; VAR result : ARRAY OF CHAR);
    PROCEDURE OwnerNameTerminator (name : ARRAY OF CHAR) : CARDINAL;
      VAR
          position : CARDINAL;
      
    BEGIN (* OwnerNameTerminator *)
      position := Strings.Pos ('(',name);
      
      WHILE name [position - 1] = ' ' DO
        DEC (position);
      END; (* WHILE *)
      
      RETURN (position);
    END OwnerNameTerminator;
    
  BEGIN (* ReturnStoreOwnerName *)
    Strings.Copy (description,0,OwnerNameTerminator (description),result);
  END ReturnStoreOwnerName;
END MMWindow.
