MODULE MOData;
  (*/WIN:A*)
  
  FROM WMemory IMPORT ALLOCATE,DEALLOCATE;
  
  IMPORT WCLoop,WCreation,WCursor,WDebugging,WDisplay,WError,Win31,Windows,WinFiles,WInformation,WinStart,WResource;
  IMPORT FileSystem,Strings,SYSTEM;
  IMPORT MOParser,OStruct;
  
  
  CONST
        MainWindowClassName = "MOData_Main_Window_Class";
        MainWindowTitle     = "MODATA";
  
  VAR
      MainWindowClassData : Windows.WNDCLASS;
      MainWindowClassAtom : Windows.ATOM;
      MainWindowHandle    : Windows.HWND;
  
  
  PROCEDURE AllocateMainWindowClassData (instance : Windows.HINSTANCE);
  BEGIN (* AllocateMainWindowClassData *)
    WITH MainWindowClassData DO
      hbrBackground := Windows.COLOR_WINDOW + 1;
      lpszClassName := SYSTEM.ADR (MainWindowClassName);
      lpszMenuName  := SYSTEM.ADR ("Main_Menu");
      lpfnWndProc   := MainWindowTranslator;
      cbClsExtra    := 0;
      cbWndExtra    := 0;
      hInstance     := instance;
      hCursor       := WCursor.StandardCursorHandle (Windows.IDC_ARROW);
      hIcon         := Windows.NULL;
      style         := Windows.CS_HREDRAW + Windows.CS_VREDRAW;
    END; (* WITH *)
    
    WCreation.RegisterClass (MainWindowClassData,MainWindowClassAtom);
  END AllocateMainWindowClassData;
  
  PROCEDURE AllocateMainWindowHandle (VAR window : Windows.HWND);
    VAR
        shown : BOOLEAN;
    
  BEGIN (* AllocateMainWindowHandle *)
    WCreation.CreateDefaultWindow (MainWindowClassName,MainWindowTitle,Windows.WS_OVERLAPPEDWINDOW,Windows.NULL,WinStart.hInstance,window);
    WDisplay.ShowWindow (window,WinStart.cmdShow,shown);
  END AllocateMainWindowHandle;
  
  PROCEDURE MainWindowTranslator ['MAINWINDOWTRANSLATOR'] (window : Windows.HWND; message : CARDINAL; param1 : Windows.WPARAM; param2 : Windows.LPARAM) : Windows.LRESULT [WINDOWS,LONG];
    VAR
        result : Windows.LRESULT;
    
    PROCEDURE ExecuteMainWindowCommand (window : Windows.HWND; command : CARDINAL);
      CONST
            CreateCommandControl = 100;
            ExitCommandControl   = 200;
      
      PROCEDURE ExecuteCreateCommand (window : Windows.HWND);
        VAR
            treasures : OStruct.TreasureInfoPointer;
            previous  : Windows.HCURSOR;
            output    : ARRAY [0..127] OF CHAR;
            done      : BOOLEAN;
        
        PROCEDURE DetermineOutputFileName (window : Windows.HWND; VAR name : ARRAY OF CHAR; VAR done : BOOLEAN);
          VAR
              title : ARRAY [0..15] OF CHAR;
          
        BEGIN (* DetermineOutputFileName *)
          Strings.Assign ("TREASURE.DAT",name);
          
          WinFiles.GetSaveFileName (window,"Treasure Output File Name","","","",Win31.OFN_HIDEREADONLY + Win31.OFN_NOREADONLYRETURN + Win31.OFN_OVERWRITEPROMPT,name,title,done);
        END DetermineOutputFileName;
        
        PROCEDURE DetermineTreasureInformation (instance : Windows.HINSTANCE; VAR treasures : OStruct.TreasureInformation);
          PROCEDURE ComputeDungeonDepthTable (VAR treasures : OStruct.TreasureInformation);
            PROCEDURE ClearDungeonDepthTable (VAR table : ARRAY OF CARDINAL);
              VAR
                  index : CARDINAL;
              
            BEGIN (* ClearDungeonDepthTable *)
              FOR index := 0 TO HIGH (table) DO
                table [index] := 0;
              END; (* FOR *)
            END ClearDungeonDepthTable;
            
            PROCEDURE CountObjectsPerDepth (VAR treasures : OStruct.TreasureInformation);
              VAR
                  index : CARDINAL;
              
            BEGIN (* CountObjectsPerDepth *)
              FOR index := 0 TO OStruct.NumberDungeonObjects - 1 DO
                INC (treasures.depth [treasures.table [index].depth]);
              END; (* FOR *)
            END CountObjectsPerDepth;
            
            PROCEDURE SumObjectsPerDepth (VAR table : ARRAY OF CARDINAL);
              VAR
                  index : CARDINAL;
              
            BEGIN (* SumObjectsPerDepth *)
              FOR index := 1 TO HIGH (table) DO
                table [index] := table [index] + table [index - 1];
              END; (* FOR *)
            END SumObjectsPerDepth;
            
          BEGIN (* ComputeDungeonDepthTable *)
            ClearDungeonDepthTable (treasures.depth);
            
            CountObjectsPerDepth (treasures);
            SumObjectsPerDepth (treasures.depth);
          END ComputeDungeonDepthTable;
          
          PROCEDURE ParseObjectDescriptions (instance : Windows.HINSTANCE; VAR treasures : OStruct.TreasureInformation);
            CONST
                  MushroomStringBase = 600;
                  AmuletStringBase   = 700;
                  PotionStringBase   = 800;
                  StaffStringBase    = 900;
                  RingStringBase     = 1000;
                  WandStringBase     = 1100;
            
            CONST
                  SyllableStringBase = 1200;
            
            PROCEDURE LoadObjectDescriptionStrings (instance : Windows.HINSTANCE; cursor : CARDINAL; VAR table : ARRAY OF OStruct.ObjectDescriptionString);
              VAR
                  length : CARDINAL;
                  index  : CARDINAL;
              
            BEGIN (* LoadObjectDescriptionStrings *)
              FOR index := 0 TO HIGH (table) DO
                WResource.LoadString (instance,cursor + index,table [index],length);
              END; (* FOR *)
            END LoadObjectDescriptionStrings;
            
            PROCEDURE LoadSyllableStrings (instance : Windows.HINSTANCE; cursor : CARDINAL; VAR table : ARRAY OF OStruct.TitleSyllableString);
              VAR
                  length : CARDINAL;
                  index  : CARDINAL;
              
            BEGIN (* LoadSyllableStrings *)
              FOR index := 0 TO HIGH (table) DO
                WResource.LoadString (instance,cursor + index,table [index],length);
              END; (* FOR *)
            END LoadSyllableStrings;
            
          BEGIN (* ParseObjectDescriptions *)
            WITH treasures DO
              LoadObjectDescriptionStrings (instance,MushroomStringBase,mushrooms);
              LoadObjectDescriptionStrings (instance,AmuletStringBase,amulets);
              LoadObjectDescriptionStrings (instance,PotionStringBase,potions);
              LoadObjectDescriptionStrings (instance,StaffStringBase,staves);
              LoadObjectDescriptionStrings (instance,RingStringBase,rings);
              LoadObjectDescriptionStrings (instance,WandStringBase,wands);
              
              LoadSyllableStrings (instance,SyllableStringBase,syllables);
            END; (* WITH *)
          END ParseObjectDescriptions;
          
          PROCEDURE ParseObjectInfoRecords (instance : Windows.HINSTANCE; VAR table : ARRAY OF OStruct.ObjectInfoRecord);
            CONST
                  ObjectStringBase = 100;
            
            VAR
                index : CARDINAL;
            
          BEGIN (* ParseObjectInfoRecords *)
            FOR index := 0 TO HIGH (table) DO
              MOParser.ParseObjectInfoRecord (instance,ObjectStringBase + index,table [index]);
            END; (* FOR *)
          END ParseObjectInfoRecords;
          
          PROCEDURE ParseObjectSpecialNames (instance : Windows.HINSTANCE; VAR result : ARRAY OF OStruct.SpecialNameText);
            CONST
                  SpecialNameStringBase = 1400;
            
            VAR
                length : CARDINAL;
                index  : CARDINAL;
            
          BEGIN (* ParseObjectSpecialNames *)
            FOR index := 0 TO HIGH (result) DO
              WResource.LoadString (instance,SpecialNameStringBase + index,result [index],length);
            END; (* FOR *)
          END ParseObjectSpecialNames;
          
          PROCEDURE SortDungeonObjects (VAR treasures : OStruct.TreasureInformation);
            VAR
                object : OStruct.ObjectInfoRecord;
                table  : ARRAY [0..OStruct.MaxObjectDepth] OF CARDINAL;
                index  : CARDINAL;
            
            PROCEDURE ClearSortingCountTable (VAR table : ARRAY OF CARDINAL);
              VAR
                  index : CARDINAL;
              
            BEGIN (* ClearSortingCountTable *)
              FOR index := 0 TO HIGH (table) DO
                table [index] := 1;
              END; (* FOR *)
            END ClearSortingCountTable;
            
          BEGIN (* SortDungeonObjects *)
            ClearSortingCountTable (table);
            
            FOR index := 0 TO OStruct.NumberDungeonObjects - 1 DO
              object                                                                  := treasures.table [index];
              treasures.index [treasures.depth [object.depth] - table [object.depth]] := index;
              
              INC (table [object.depth]);
            END; (* FOR *)
          END SortDungeonObjects;
          
        BEGIN (* DetermineTreasureInformation *)
          ParseObjectSpecialNames (instance,treasures.specials);
          ParseObjectDescriptions (instance,treasures);
          
          ParseObjectInfoRecords (instance,treasures.table);
          
          ComputeDungeonDepthTable (treasures);
          SortDungeonObjects (treasures);
        END DetermineTreasureInformation;
        
        PROCEDURE OutputTreasureInformation (name : ARRAY OF CHAR; VAR treasures : OStruct.TreasureInformation);
          VAR
              f : FileSystem.File;
          
        BEGIN (* OutputTreasureInformation *)
          WinFiles.CreateFile (f,name);
          
          IF f.res <> FileSystem.done THEN
            WDebugging.FatalAppExit ("Error while creating Moria treasure data.");
          END; (* IF *)
          
          WinFiles.WriteDataStructure (f,treasures);
          
          IF f.res <> FileSystem.done THEN
            WDebugging.FatalAppExit ("Error while writing Moria treasure data.");
          END; (* IF *)
          
          WinFiles.CloseFile (f);
          
          IF f.res <> FileSystem.done THEN
            WDebugging.FatalAppExit ("Error while closing Moria treasure data file.");
          END; (* IF *)
        END OutputTreasureInformation;
        
      BEGIN (* ExecuteCreateCommand *)
        DetermineOutputFileName (window,output,done);
        
        IF done THEN
          WCursor.SetCursor (WCursor.StandardCursorHandle (Windows.IDC_WAIT),previous);
          
          NEW (treasures);
          
          DetermineTreasureInformation (WinStart.hInstance,treasures^);
          OutputTreasureInformation (output,treasures^);
          
          DISPOSE (treasures);
          
          WCursor.SetCursor (previous,previous);
        END; (* IF *)
      END ExecuteCreateCommand;
      
      PROCEDURE ExecuteExitCommand (window : Windows.HWND);
        VAR
            done : BOOLEAN;
        
      BEGIN (* ExecuteExitCommand *)
        WCreation.DestroyWindow (window,done);
      END ExecuteExitCommand;
      
    BEGIN (* ExecuteMainWindowCommand *)
      CASE command OF
        CreateCommandControl : ExecuteCreateCommand (window); |
        ExitCommandControl   : ExecuteExitCommand (window);   |
      ELSE
        WError.DisplayInformationBox (window,MainWindowTitle,"Unrecognized main window command.");
      END; (* CASE *)
    END ExecuteMainWindowCommand;
    
  BEGIN (* MainWindowTranslator *)
    result := Windows.FALSE_BOOL;
    
    CASE message OF
      Windows.WM_COMMAND : ExecuteMainWindowCommand (window,param1); |
    ELSE
      result := WCreation.DefWindowProc (window,message,param1,param2);
    END; (* CASE *)
    
    RETURN (result);
  END MainWindowTranslator;
  
  
BEGIN (* MOData *)
  MainWindowHandle := WInformation.FindWindow (MainWindowClassName,MainWindowTitle);
  
  IF MainWindowHandle = Windows.NULL THEN
    AllocateMainWindowClassData (WinStart.hInstance);
    AllocateMainWindowHandle (MainWindowHandle);
    
    WCLoop.ExecuteCommandLoop (MainWindowHandle,Windows.NULL);
  ELSE
    WDisplay.RedisplayExistingInstance (MainWindowHandle);
  END; (* IF *)
END MOData.
