IMPLEMENTATION MODULE MParser;
  (*/WIN:A*)
  
  IMPORT NumberConversion,Strings;
  IMPORT WDebugging;
  IMPORT MRandom;
  
  
  CONST
        QuoteSymbol = "`";
  
  
  PROCEDURE MatchCharacter (VAR str : ARRAY OF CHAR; symbol : CHAR);
  BEGIN (* MatchCharacter *)
    RemoveInitialWhiteSpace (str);
    
    IF str [0] <> symbol THEN
      WDebugging.FatalAppExit ("Unable to match symbol while parsing Moria data.");
    END; (* IF *)
    
    Strings.Delete (str,0,1);
  END MatchCharacter;
  
  PROCEDURE ParseCardinalValue (VAR str : ARRAY OF CHAR; term : CHAR; VAR result : CARDINAL);
    VAR
        temp : ARRAY [0..15] OF CHAR;
        done : BOOLEAN;
    
  BEGIN (* ParseCardinalValue *)
    ParseNextSymbol (str,term,temp);
    
    NumberConversion.StringToCard (temp,result,done);
    
    IF NOT (done) THEN
      WDebugging.FatalAppExit ("Invalid CARDINAL value while parsing Moria data.");
    END; (* IF *)
  END ParseCardinalValue;
  
  PROCEDURE ParseHexLongCardValue (VAR str : ARRAY OF CHAR; term : CHAR; VAR result : LONGCARD);
    VAR
        temp : ARRAY [0..31] OF CHAR;
        done : BOOLEAN;
    
    PROCEDURE HexStringToLongCard (str : ARRAY OF CHAR; VAR result : LONGCARD; VAR done : BOOLEAN);
      CONST
            InvalidHexidecimalDigit = MAX (LONGCARD);
            HexidecimalPlaceValue   = LONGCARD (10H);
      
      VAR
          digit : LONGCARD;
          index : CARDINAL;
      
      PROCEDURE HexidecimalDigitValue (ch : CHAR) : LONGCARD;
      BEGIN (* HexidecimalDigitValue *)
        CASE CAP (ch) OF
          'A'..'F' : RETURN (ORD (ch) - ORD ('A') + 10); |
          '0'..'9' : RETURN (ORD (ch) - ORD ('0'));      |
        ELSE
          RETURN (InvalidHexidecimalDigit);
        END; (* CASE *)
      END HexidecimalDigitValue;
      
    BEGIN (* HexStringToLongCard *)
      result := 0;
      index  := 0;
      done   := (Strings.Pos ("0x",str) = 0);
      
      Strings.Delete (str,0,2);
      
      WHILE (done AND (index <= HIGH (str)) AND (str [index] <> 0C)) DO
        digit := HexidecimalDigitValue (str [index]);
        done  := ((digit <> InvalidHexidecimalDigit) AND (result <= MAX (LONGCARD) DIV HexidecimalPlaceValue) AND (result * HexidecimalPlaceValue <= MAX (LONGCARD) - digit));
        
        IF done THEN
          result := result * HexidecimalPlaceValue + digit;
        END; (* IF *)
        
        INC (index);
      END; (* WHILE *)
    END HexStringToLongCard;
    
  BEGIN (* ParseHexLongCardValue *)
    ParseNextSymbol (str,term,temp);
    
    HexStringToLongCard (temp,result,done);
    
    IF NOT (done) THEN
      WDebugging.FatalAppExit ("Invalid hexidecimal LONGCARD value while parsing Moria data.");
    END; (* IF *)
  END ParseHexLongCardValue;
  
  PROCEDURE ParseIntegerValue (VAR str : ARRAY OF CHAR; term : CHAR; VAR result : INTEGER);
    VAR
        temp : ARRAY [0..15] OF CHAR;
        done : BOOLEAN;
    
  BEGIN (* ParseIntegerValue *)
    ParseNextSymbol (str,term,temp);
    
    NumberConversion.StringToInt (temp,result,done);
    
    IF NOT (done) THEN
      WDebugging.FatalAppExit ("Invalid INTEGER value while parsing Moria data.");
    END; (* IF *)
  END ParseIntegerValue;
  
  PROCEDURE ParseLiteralChar (VAR str : ARRAY OF CHAR; term : CHAR; VAR result : CHAR);
    PROCEDURE ParseNumericCharacter (VAR str : ARRAY OF CHAR; term : CHAR; VAR result : CHAR);
      VAR
          temp : CARDINAL;
      
    BEGIN (* ParseNumericCharacter *)
      ParseCardinalValue (str,term,temp);
      
      IF temp > ORD (MAX (CHAR)) THEN
        WDebugging.FatalAppExit ("Invalid numeric CHAR value while parsing Moria data.");
      END; (* IF *)
      
      result := CHR (temp);
    END ParseNumericCharacter;
    
    PROCEDURE ParseStringCharacter (VAR str : ARRAY OF CHAR; term : CHAR; VAR result : CHAR);
      VAR
          temp : ARRAY [0..127] OF CHAR;
      
    BEGIN (* ParseStringCharacter *)
      ParseLiteralString (str,term,temp);
      
      IF Strings.Length (temp) <> 1 THEN
        WDebugging.FatalAppExit ("Invalid string CHAR value while parsing Moria data.");
      END; (* IF *)
      
      result := temp [0];
    END ParseStringCharacter;
    
  BEGIN (* ParseLiteralChar *)
    RemoveInitialWhiteSpace (str);
    
    IF str [0] <> QuoteSymbol THEN
      ParseNumericCharacter (str,term,result);
    ELSE
      ParseStringCharacter (str,term,result);
    END; (* IF *)
  END ParseLiteralChar;
  
  PROCEDURE ParseLiteralString (VAR str : ARRAY OF CHAR; term : CHAR; VAR result : ARRAY OF CHAR);
  BEGIN (* ParseLiteralString *)
    RemoveInitialWhiteSpace (str);
    
    MatchCharacter (str,QuoteSymbol);
    ParseNextSymbol (str,QuoteSymbol,result);
    MatchCharacter (str,term);
  END ParseLiteralString;
  
  PROCEDURE ParseLongCardValue (VAR str : ARRAY OF CHAR; term : CHAR; VAR result : LONGCARD);
    VAR
        temp : ARRAY [0..31] OF CHAR;
        done : BOOLEAN;
    
  BEGIN (* ParseLongCardValue *)
    RemoveInitialWhiteSpace (str);
    ParseNextSymbol (str,term,temp);
    
    StringToLongCard (temp,result,done);
    
    IF NOT (done) THEN
      WDebugging.FatalAppExit ("Invalid LONGCARD value while parsing Moria data.");
    END; (* IF *)
  END ParseLongCardValue;
  
  PROCEDURE ParseLongIntValue (VAR str : ARRAY OF CHAR; term : CHAR; VAR result : LONGINT);
    VAR
        temp : ARRAY [0..31] OF CHAR;
        done : BOOLEAN;
    
  BEGIN (* ParseLongIntValue *)
    ParseNextSymbol (str,term,temp);
    
    NumberConversion.StringToLongInt (temp,result,done);
    
    IF NOT (done) THEN
      WDebugging.FatalAppExit ("Invalid LONGINT value while parsing Moria data.");
    END; (* IF *)
  END ParseLongIntValue;
  
  PROCEDURE ParseNextSymbol (VAR str : ARRAY OF CHAR; term : CHAR; VAR result : ARRAY OF CHAR);
    VAR
        length : CARDINAL;
    
  BEGIN (* ParseNextSymbol *)
    IF term <> 0C THEN
      length := Strings.Pos (term,str);
      
      IF length > HIGH (str) THEN
        WDebugging.FatalAppExit ("Terminal symbol not found while parsing Moria data.");
      END; (* IF *)
    ELSE
      length := Strings.Length (str);
    END; (* IF *)
    
    Strings.Copy (str,0,length,result);
    Strings.Delete (str,0,length + 1);
  END ParseNextSymbol;
  
  PROCEDURE ParseRandomRangeSpecifier (VAR str : ARRAY OF CHAR; term : CHAR; brackets : BOOLEAN; VAR result : MRandom.RandomRangeSpecification);
    VAR
        number,size : CARDINAL;
    
  BEGIN (* ParseRandomRangeSpecifier *)
    IF brackets THEN
      MatchCharacter (str,OpenListBracket);
    END; (* IF *)
    
    ParseCardinalValue (str,StandardItemTerminator,number);
    
    IF brackets THEN
      ParseCardinalValue (str,CloseListBracket,size);
      MatchCharacter (str,term);
    ELSE
      ParseCardinalValue (str,term,size);
    END; (* IF *)
    
    result.max := number * size;
    result.min := number;
  END ParseRandomRangeSpecifier;
  
  PROCEDURE RemoveInitialWhiteSpace (VAR str : ARRAY OF CHAR);
    PROCEDURE InitialWhiteSpaceLength (str : ARRAY OF CHAR) : CARDINAL;
      VAR
          length : CARDINAL;
      
    BEGIN (* InitialWhiteSpaceLength *)
      length := 0;
      
      WHILE ((length <= HIGH (str)) AND (str [length] = ' ')) DO
        INC (length);
      END; (* WHILE *)
      
      RETURN (length);
    END InitialWhiteSpaceLength;
    
  BEGIN (* RemoveInitialWhiteSpace *)
    Strings.Delete (str,0,InitialWhiteSpaceLength (str));
  END RemoveInitialWhiteSpace;

  PROCEDURE StringToLongCard (str : ARRAY OF CHAR; VAR result : LONGCARD; VAR done : BOOLEAN);
    CONST
          InvalidDecimalDigit = MAX (CARDINAL);
          DecimalPlaceValue   = 10;
    
    VAR
        digit : LONGCARD;
        index : CARDINAL;
    
    PROCEDURE DecimalDigitValue (ch : CHAR) : CARDINAL;
    BEGIN (* DecimalDigitValue *)
      IF ((ch >= '0') AND (ch <= '9')) THEN
        RETURN (ORD (ch) - ORD ('0'));
      ELSE
        RETURN (InvalidDecimalDigit);
      END; (* IF *)
    END DecimalDigitValue;
    
  BEGIN (* StringToLongCard *)
    result := 0;
    index  := 0;
    done   := (str [0] <> 0C);
    
    WHILE (done AND (index <= HIGH (str)) AND (str [index] <> 0C)) DO
      digit := DecimalDigitValue (str [index]);
      done  := ((digit <> VAL (LONGCARD,InvalidDecimalDigit)) AND (result <= MAX (LONGCARD) DIV DecimalPlaceValue) AND (result * DecimalPlaceValue <= MAX (LONGCARD) - digit));
      
      IF done THEN
        result := result * DecimalPlaceValue + digit;
      END; (* IF *)
      
      index := index + 1;
    END; (* WHILE *)
  END StringToLongCard;
END MParser.
