IMPLEMENTATION MODULE MPCounters;
  (*/WIN:A*)
  
  IMPORT MCreatures,MDGenerator,MDistMove,MDPainting,MDWindow,MMWindow,MPDamage,MPRest,MSUpdate;
  IMPORT MCSActions,MSActions,MSActs2,MSActs3;
  IMPORT MCStruct,MStruct;
  IMPORT WDebugging;
  
  
  TYPE
       StatusUpdateProcedure = PROCEDURE (MStruct.PlayerInfoPointer);
  
  VAR
      UpdateCountedCondition : ARRAY MCStruct.PlayerCounterMnemonic OF StatusUpdateProcedure;
  
  
  PROCEDURE InitializeUpdateCountedConditionTable;
    PROCEDURE ClearStatusUpdateTable (VAR table : ARRAY OF StatusUpdateProcedure);
      VAR
          index : CARDINAL;
      
    BEGIN (* ClearStatusUpdateTable *)
      FOR index := 0 TO HIGH (table) DO
        table [index] := InvalidStatusUpdateProcedure;
      END; (* FOR *)
    END ClearStatusUpdateTable;
    
  BEGIN (* InitializeUpdateCountedConditionTable *)
    ClearStatusUpdateTable (UpdateCountedCondition);
    
    UpdateCountedCondition [MCStruct.AfraidCOUNT] := UpdateAfraidCOUNT;
    UpdateCountedCondition [MCStruct.BlessedCOUNT] := UpdateBlessedCOUNT;
    UpdateCountedCondition [MCStruct.BlindCOUNT] := UpdateBlindCOUNT;
    UpdateCountedCondition [MCStruct.ConfusedCOUNT] := UpdateConfusedCOUNT;
    UpdateCountedCondition [MCStruct.DetectInvisCOUNT] := UpdateDetectInvisCOUNT;
    UpdateCountedCondition [MCStruct.FastCOUNT] := UpdateFastCOUNT;
    UpdateCountedCondition [MCStruct.HeroCOUNT] := UpdateHeroCOUNT;
    UpdateCountedCondition [MCStruct.ImageCOUNT] := UpdateImageCOUNT;
    UpdateCountedCondition [MCStruct.InvulnCOUNT] := UpdateInvulnCOUNT;
    UpdateCountedCondition [MCStruct.ParalysisCOUNT] := UpdateParalysisCOUNT;
    UpdateCountedCondition [MCStruct.PoisonedCOUNT] := UpdatePoisonedCOUNT;
    UpdateCountedCondition [MCStruct.ProtectionCOUNT] := UpdateProtectionCOUNT;
    UpdateCountedCondition [MCStruct.ProtEvilCOUNT] := UpdateProtEvilCOUNT;
    UpdateCountedCondition [MCStruct.ResistColdCOUNT] := UpdateResistColdCOUNT;
    UpdateCountedCondition [MCStruct.ResistHeatCOUNT] := UpdateResistHeatCOUNT;
    UpdateCountedCondition [MCStruct.SlowCOUNT] := UpdateSlowCOUNT;
    UpdateCountedCondition [MCStruct.SuperHeroCOUNT] := UpdateSuperHeroCOUNT;
    UpdateCountedCondition [MCStruct.TimedInfraCOUNT] := UpdateTimedInfraCOUNT;
    UpdateCountedCondition [MCStruct.WordOfRecallCOUNT] := UpdateWordOfRecallCOUNT;
    
    UpdateCountedCondition [MCStruct.RestCOUNT] := MPRest.UpdateRestCOUNT;
  END InitializeUpdateCountedConditionTable;
  
  PROCEDURE InvalidStatusUpdateProcedure (player : MStruct.PlayerInfoPointer);
  BEGIN (* InvalidStatusUpdateProcedure *)
    WDebugging.FatalAppExit ("Un-initialized status update procedure in MPStatus.");
  END InvalidStatusUpdateProcedure;
  
  PROCEDURE UpdateCountedConditions (player : MStruct.PlayerInfoPointer);
    VAR
        index : MCStruct.PlayerCounterMnemonic;
    
  BEGIN (* UpdateCountedConditions *)
    FOR index := MIN (MCStruct.PlayerCounterMnemonic) TO MAX (MCStruct.PlayerCounterMnemonic) DO
      IF player^.character^.variables.counters [index] > 0 THEN
        UpdateCountedCondition [index] (player);
      END; (* IF *)
    END; (* FOR *)
  END UpdateCountedConditions;
  

  PROCEDURE UpdateAfraidCOUNT (player : MStruct.PlayerInfoPointer);
    PROCEDURE InitializeAfraidCondition (player : MStruct.PlayerInfoPointer);
    BEGIN (* InitializeAfraidCondition *)
      IF PlayerHasHeroism (player) THEN
        player^.character^.variables.counters [MCStruct.AfraidCOUNT] := 0;
      ELSE
        INCL (player^.character^.variables.status,MCStruct.FearFLAG);
        MSUpdate.UpdateAfraidStatusWindow (player);
      END; (* IF *)
    END InitializeAfraidCondition;
    
    PROCEDURE PlayerHasHeroism (player : MStruct.PlayerInfoPointer) : BOOLEAN;
    BEGIN (* PlayerHasHeroism *)
      RETURN (player^.character^.variables.counters [MCStruct.HeroCOUNT] + player^.character^.variables.counters [MCStruct.SuperHeroCOUNT] > 0);
    END PlayerHasHeroism;
    
    PROCEDURE RemoveAfraidCondition (player : MStruct.PlayerInfoPointer);
    BEGIN (* RemoveAfraidCondition *)
      EXCL (player^.character^.variables.status,MCStruct.FearFLAG);
      
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You feel bolder now.");
      MSUpdate.UpdateAfraidStatusWindow (player);
      
      MSActs2.DisturbPlayer (player,TRUE);
    END RemoveAfraidCondition;
    
  BEGIN (* UpdateAfraidCOUNT *)
    IF NOT (MCStruct.FearFLAG IN player^.character^.variables.status) THEN
      InitializeAfraidCondition (player);
    ELSIF PlayerHasHeroism (player) THEN
      player^.character^.variables.counters [MCStruct.AfraidCOUNT] := 1;
    END; (* IF *)
    
    IF player^.character^.variables.counters [MCStruct.AfraidCOUNT] > 0 THEN
      DEC (player^.character^.variables.counters [MCStruct.AfraidCOUNT]);
      
      IF player^.character^.variables.counters [MCStruct.AfraidCOUNT] = 0 THEN
        RemoveAfraidCondition (player);
      END; (* IF *)
    END; (* IF *)
  END UpdateAfraidCOUNT;
  
  PROCEDURE UpdateBlessedCOUNT (player : MStruct.PlayerInfoPointer);
    CONST
          BlessingMartialModifier = 5;
          BlessingMissileModifier = 5;
          BlessingArmorModifier   = 2;
    
    PROCEDURE InitializeBlessedCondition (player : MStruct.PlayerInfoPointer);
    BEGIN (* InitializeBlessedCondition *)
      INCL (player^.character^.variables.status,MCStruct.BlessedFLAG);
      
      INC (player^.character^.variables.armorREAL,BlessingArmorModifier);
      INC (player^.character^.variables.armorDISP,BlessingArmorModifier);
      INC (player^.character^.abilities.missile,BlessingMissileModifier);
      INC (player^.character^.abilities.martial,BlessingMartialModifier);
      
      MSUpdate.UpdateCurrentArmorClassWindow (player);
      
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You feel righteous!");
      MSActs2.DisturbPlayer (player,FALSE);
    END InitializeBlessedCondition;
    
    PROCEDURE RemoveBlessedCondition (player : MStruct.PlayerInfoPointer);
    BEGIN (* RemoveBlessedCondition *)
      EXCL (player^.character^.variables.status,MCStruct.BlessedFLAG);
      
      DEC (player^.character^.variables.armorREAL,BlessingArmorModifier);
      DEC (player^.character^.variables.armorDISP,BlessingArmorModifier);
      DEC (player^.character^.abilities.missile,BlessingMissileModifier);
      DEC (player^.character^.abilities.martial,BlessingMartialModifier);
      
      MSUpdate.UpdateCurrentArmorClassWindow (player);
      
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"The prayer has expired.");
      MSActs2.DisturbPlayer (player,FALSE);
    END RemoveBlessedCondition;
    
  BEGIN (* UpdateBlessedCOUNT *)
    IF NOT (MCStruct.BlessedFLAG IN player^.character^.variables.status) THEN
      InitializeBlessedCondition (player);
    END; (* IF *)
    
    DEC (player^.character^.variables.counters [MCStruct.BlessedCOUNT]);
    
    IF player^.character^.variables.counters [MCStruct.BlessedCOUNT] = 0 THEN
      RemoveBlessedCondition (player);
    END; (* IF *)
  END UpdateBlessedCOUNT;
  
  PROCEDURE UpdateBlindCOUNT (player : MStruct.PlayerInfoPointer);
    PROCEDURE InitializeBlindCondition (player : MStruct.PlayerInfoPointer);
    BEGIN (* InitializeBlindCondition *)
      INCL (player^.character^.variables.status,MCStruct.BlindFLAG);
      
      MCreatures.ComputeCreatureVisibility (player,TRUE);
      MSUpdate.UpdateBlindnessStatusWindow (player);
      MDPainting.InvalidateDungeonWindow (player);
      
      MSActs2.DisturbPlayer (player,TRUE);
    END InitializeBlindCondition;
    
    PROCEDURE RemoveBlindCondition (player : MStruct.PlayerInfoPointer);
    BEGIN (* RemoveBlindCondition *)
      EXCL (player^.character^.variables.status,MCStruct.BlindFLAG);
      
      MCreatures.ComputeCreatureVisibility (player,TRUE);
      MSUpdate.UpdateBlindnessStatusWindow (player);
      MDPainting.InvalidateDungeonWindow (player);
      
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"The veil of darkness lifts.");
      MSActs2.DisturbPlayer (player,TRUE);
    END RemoveBlindCondition;
    
  BEGIN (* UpdateBlindCOUNT *)
    IF NOT (MCStruct.BlindFLAG IN player^.character^.variables.status) THEN
      InitializeBlindCondition (player);
    END; (* IF *)
    
    DEC (player^.character^.variables.counters [MCStruct.BlindCOUNT]);
    
    IF player^.character^.variables.counters [MCStruct.BlindCOUNT] = 0 THEN
      RemoveBlindCondition (player);
    END; (* IF *)
  END UpdateBlindCOUNT;
  
  PROCEDURE UpdateConfusedCOUNT (player : MStruct.PlayerInfoPointer);
    PROCEDURE InitializeConfusedStatus (player : MStruct.PlayerInfoPointer);
    BEGIN (* InitializeConfusedStatus *)
      INCL (player^.character^.variables.status,MCStruct.ConfusedFLAG);
      MSUpdate.UpdateConfusedStatusWindow (player);
    END InitializeConfusedStatus;
    
    PROCEDURE RemoveConfusedStatus (player : MStruct.PlayerInfoPointer);
    BEGIN (* RemoveConfusedStatus *)
      EXCL (player^.character^.variables.status,MCStruct.ConfusedFLAG);
      
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You feel less confused now.");
      MSUpdate.UpdateConfusedStatusWindow (player);
      
      MSActs2.DisturbPlayer (player,TRUE);
    END RemoveConfusedStatus;
    
  BEGIN (* UpdateConfusedCOUNT *)
    IF NOT (MCStruct.ConfusedFLAG IN player^.character^.variables.status) THEN
      InitializeConfusedStatus (player);
    END; (* IF *)
    
    DEC (player^.character^.variables.counters [MCStruct.ConfusedCOUNT]);
    
    IF player^.character^.variables.counters [MCStruct.ConfusedCOUNT] = 0 THEN
      RemoveConfusedStatus (player);
    END; (* IF *)
  END UpdateConfusedCOUNT;
  
  PROCEDURE UpdateDetectInvisCOUNT (player : MStruct.PlayerInfoPointer);
    PROCEDURE InitializeDetectInvisibleStatus (player : MStruct.PlayerInfoPointer);
    BEGIN (* InitializeDetectInvisibleStatus *)
      INCL (player^.character^.variables.status,MCStruct.DetectInvisibleFLAG);
      INCL (player^.character^.variables.status,MCStruct.SeeInvisibleFLAG);
      
      MCreatures.ComputeCreatureVisibility (player,TRUE);
    END InitializeDetectInvisibleStatus;
    
    PROCEDURE RemoveDetectInvisibleStatus (player : MStruct.PlayerInfoPointer);
    BEGIN (* RemoveDetectInvisibleStatus *)
      EXCL (player^.character^.variables.status,MCStruct.DetectInvisibleFLAG);
      MCSActions.CalculatePlayerBonuses (player^.character);
      
      MCreatures.ComputeCreatureVisibility (player,TRUE);
    END RemoveDetectInvisibleStatus;
    
  BEGIN (* UpdateDetectInvisCOUNT *)
    IF NOT (MCStruct.DetectInvisibleFLAG IN player^.character^.variables.status) THEN
      InitializeDetectInvisibleStatus (player);
    END; (* IF *)
    
    DEC (player^.character^.variables.counters [MCStruct.DetectInvisCOUNT]);
    
    IF player^.character^.variables.counters [MCStruct.DetectInvisCOUNT] = 0 THEN
      RemoveDetectInvisibleStatus (player);
    END; (* IF *)
  END UpdateDetectInvisCOUNT;
  
  PROCEDURE UpdateFastCOUNT (player : MStruct.PlayerInfoPointer);
    PROCEDURE InitializeFastStatus (player : MStruct.PlayerInfoPointer);
    BEGIN (* InitializeFastStatus *)
      INCL (player^.character^.variables.status,MCStruct.FastFLAG);
      
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You feel yourself moving faster.");
      
      MSActs3.AlterPlayerSpeed (player,-1);
      MSActs2.DisturbPlayer (player,FALSE);
    END InitializeFastStatus;
    
    PROCEDURE RemoveFastStatus (player : MStruct.PlayerInfoPointer);
    BEGIN (* RemoveFastStatus *)
      EXCL (player^.character^.variables.status,MCStruct.FastFLAG);
      
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You feel yourself slow down.");
      
      MSActs3.AlterPlayerSpeed (player,+1);
      MSActs2.DisturbPlayer (player,FALSE);
    END RemoveFastStatus;
    
  BEGIN (* UpdateFastCOUNT *)
    IF NOT (MCStruct.FastFLAG IN player^.character^.variables.status) THEN
      InitializeFastStatus (player);
    END; (* IF *)
    
    DEC (player^.character^.variables.counters [MCStruct.FastCOUNT]);
    
    IF player^.character^.variables.counters [MCStruct.FastCOUNT] = 0 THEN
      RemoveFastStatus (player);
    END; (* IF *)
  END UpdateFastCOUNT;
  
  PROCEDURE UpdateHeroCOUNT (player : MStruct.PlayerInfoPointer);
    CONST
          HeroismHitPointModifier = 10.0;
          HeroismAbilityModifier  = 12;
    
    PROCEDURE InitializeHeroStatus (player : MStruct.PlayerInfoPointer);
    BEGIN (* InitializeHeroStatus *)
      INCL (player^.character^.variables.status,MCStruct.HeroismFLAG);
      
      player^.character^.variables.hpCUR := player^.character^.variables.hpCUR + HeroismHitPointModifier;
      player^.character^.variables.hpMAX := player^.character^.variables.hpMAX + HeroismHitPointModifier;
      
      MSUpdate.UpdateCurrentHitPointsWindow (player);
      MSUpdate.UpdateMaximumHitPointsWindow (player);
      
      INC (player^.character^.abilities.missile,HeroismAbilityModifier);
      INC (player^.character^.abilities.martial,HeroismAbilityModifier);
      
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You feel like a HERO!");
      MSActs2.DisturbPlayer (player,FALSE);
    END InitializeHeroStatus;
    
    PROCEDURE RemoveHeroStatus (player : MStruct.PlayerInfoPointer);
    BEGIN (* RemoveHeroStatus *)
      EXCL (player^.character^.variables.status,MCStruct.HeroismFLAG);
      
      player^.character^.variables.hpMAX := player^.character^.variables.hpMAX - HeroismHitPointModifier;
      MSUpdate.UpdateMaximumHitPointsWindow (player);
      
      IF player^.character^.variables.hpCUR > player^.character^.variables.hpMAX THEN
        player^.character^.variables.hpCUR := player^.character^.variables.hpMAX;
        MSUpdate.UpdateCurrentHitPointsWindow (player);
      END; (* IF *)
      
      DEC (player^.character^.abilities.missile,HeroismAbilityModifier);
      DEC (player^.character^.abilities.martial,HeroismAbilityModifier);
      
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"The heroism wears off.");
      MSActs2.DisturbPlayer (player,FALSE);
    END RemoveHeroStatus;
    
  BEGIN (* UpdateHeroCOUNT *)
    IF NOT (MCStruct.HeroismFLAG IN player^.character^.variables.status) THEN
      InitializeHeroStatus (player);
    END; (* IF *)
    
    DEC (player^.character^.variables.counters [MCStruct.HeroCOUNT]);
    
    IF player^.character^.variables.counters [MCStruct.HeroCOUNT] = 0 THEN
      RemoveHeroStatus (player);
    END; (* IF *)
  END UpdateHeroCOUNT;
  
  PROCEDURE UpdateImageCOUNT (player : MStruct.PlayerInfoPointer);
  BEGIN (* UpdateImageCOUNT *)
    MDistMove.EndDistanceMovement (player);
    
    DEC (player^.character^.variables.counters [MCStruct.ImageCOUNT]);
    
    IF player^.character^.variables.counters [MCStruct.ImageCOUNT] = 0 THEN
      MDPainting.InvalidateDungeonWindow (player);
    END; (* IF *)
  END UpdateImageCOUNT;
  
  PROCEDURE UpdateInvulnCOUNT (player : MStruct.PlayerInfoPointer);
    CONST
          InvulnerableArmorBonus = 100;
    
    PROCEDURE InitializeInvulnStatus (player : MStruct.PlayerInfoPointer);
    BEGIN (* InitializeInvulnStatus *)
      INCL (player^.character^.variables.status,MCStruct.InvulnerableFLAG);
      
      INC (player^.character^.variables.armorDISP,InvulnerableArmorBonus);
      INC (player^.character^.variables.armorREAL,InvulnerableArmorBonus);
      
      MSUpdate.UpdateCurrentArmorClassWindow (player);
      MSActs2.DisturbPlayer (player,FALSE);
      
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"Your skin turns into steel!.");
    END InitializeInvulnStatus;
    
    PROCEDURE RemoveInvulnStatus (player : MStruct.PlayerInfoPointer);
    BEGIN (* RemoveInvulnStatus *)
      EXCL (player^.character^.variables.status,MCStruct.InvulnerableFLAG);
      
      DEC (player^.character^.variables.armorDISP,InvulnerableArmorBonus);
      DEC (player^.character^.variables.armorREAL,InvulnerableArmorBonus);
      
      MSUpdate.UpdateCurrentArmorClassWindow (player);
      MSActs2.DisturbPlayer (player,FALSE);
      
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"Your skin returns to normal.");
    END RemoveInvulnStatus;
    
  BEGIN (* UpdateInvulnCOUNT *)
    IF NOT (MCStruct.InvulnerableFLAG IN player^.character^.variables.status) THEN
      InitializeInvulnStatus (player);
    END; (* IF *)
    
    DEC (player^.character^.variables.counters [MCStruct.InvulnCOUNT]);
    
    IF player^.character^.variables.counters [MCStruct.InvulnCOUNT] = 0 THEN
      RemoveInvulnStatus (player);
    END; (* IF *)
  END UpdateInvulnCOUNT;
  
  PROCEDURE UpdateParalysisCOUNT (player : MStruct.PlayerInfoPointer);
  BEGIN (* UpdateParalysisCOUNT *)
    DEC (player^.character^.variables.counters [MCStruct.ParalysisCOUNT]);
    MSActs2.DisturbPlayer (player,(player^.character^.variables.counters [MCStruct.ParalysisCOUNT] = 0));
  END UpdateParalysisCOUNT;
  
  PROCEDURE UpdatePoisonedCOUNT (player : MStruct.PlayerInfoPointer);
    PROCEDURE InitializePoisonedStatus (player : MStruct.PlayerInfoPointer);
    BEGIN (* InitializePoisonedStatus *)
      INCL (player^.character^.variables.status,MCStruct.PoisonedFLAG);
      MSUpdate.UpdatePoisonedStatusWindow (player);
    END InitializePoisonedStatus;
    
    PROCEDURE PoisonDamage (player : MStruct.PlayerInfoPointer) : CARDINAL;
    BEGIN (* PoisonDamage *)
      CASE MCSActions.CharacterConstitutionAdjustment (player^.character) OF
        -4    : RETURN (4); |
        -3,-2 : RETURN (3); |
        -1    : RETURN (2); |
        0     : RETURN (1); |
        
        1,2,3 : RETURN (ORD (player^.character^.turn MOD 2 = 0)); |
        4,5   : RETURN (ORD (player^.character^.turn MOD 3 = 0)); |
        6     : RETURN (ORD (player^.character^.turn MOD 4 = 0)); |
      ELSE
        WDebugging.FatalAppExit ("Unrecognized character constitution adjustment.");
      END; (* CASE *)
    END PoisonDamage;
    
    PROCEDURE RemovePoisonedStatus (player : MStruct.PlayerInfoPointer);
    BEGIN (* RemovePoisonedStatus *)
      EXCL (player^.character^.variables.status,MCStruct.PoisonedFLAG);
      
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You feel better.");
      MSUpdate.UpdatePoisonedStatusWindow (player);
      
      MSActs2.DisturbPlayer (player,FALSE);
    END RemovePoisonedStatus;
    
  BEGIN (* UpdatePoisonedCOUNT *)
    IF NOT (MCStruct.PoisonedFLAG IN player^.character^.variables.status) THEN
      InitializePoisonedStatus (player);
    END; (* IF *)
    
    DEC (player^.character^.variables.counters [MCStruct.PoisonedCOUNT]);
    
    IF player^.character^.variables.counters [MCStruct.PoisonedCOUNT] > 0 THEN
      MPDamage.ReduceCharacterHitPoints (player,PoisonDamage (player),"poison");
    ELSE
      RemovePoisonedStatus (player);
    END; (* IF *)
  END UpdatePoisonedCOUNT;
  
  PROCEDURE UpdateProtectionCOUNT (player : MStruct.PlayerInfoPointer);
  BEGIN (* UpdateProtectionCOUNT *)
    WDebugging.FatalAppExit ("Call to UpdateProtectionCOUNT");
  END UpdateProtectionCOUNT;
  
  PROCEDURE UpdateProtEvilCOUNT (player : MStruct.PlayerInfoPointer);
  BEGIN (* UpdateProtEvilCOUNT *)
    DEC (player^.character^.variables.counters [MCStruct.ProtEvilCOUNT]);
    
    IF player^.character^.variables.counters [MCStruct.ProtEvilCOUNT] = 0 THEN
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You no longer feel safe from evil.");
    END; (* IF *)
  END UpdateProtEvilCOUNT;
  
  PROCEDURE UpdateResistColdCOUNT (player : MStruct.PlayerInfoPointer);
  BEGIN (* UpdateResistColdCOUNT *)
    DEC (player^.character^.variables.counters [MCStruct.ResistColdCOUNT]);
    
    IF player^.character^.variables.counters [MCStruct.ResistColdCOUNT] = 0 THEN
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You no longer feel safe from cold.");
    END; (* IF *)
  END UpdateResistColdCOUNT;
  
  PROCEDURE UpdateResistHeatCOUNT (player : MStruct.PlayerInfoPointer);
  BEGIN (* UpdateResistHeatCOUNT *)
    DEC (player^.character^.variables.counters [MCStruct.ResistHeatCOUNT]);
    
    IF player^.character^.variables.counters [MCStruct.ResistHeatCOUNT] = 0 THEN
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You no longer feel safe from flame.");
    END; (* IF *)
  END UpdateResistHeatCOUNT;
  
  PROCEDURE UpdateSlowCOUNT (player : MStruct.PlayerInfoPointer);
    PROCEDURE InitializeSlowStatus (player : MStruct.PlayerInfoPointer);
    BEGIN (* InitializeSlowStatus *)
      INCL (player^.character^.variables.status,MCStruct.SlowFLAG);
      
      MSActs3.AlterPlayerSpeed (player,+1);
      MSActs2.DisturbPlayer (player,TRUE);
      
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You feel yourself moving slower.");
    END InitializeSlowStatus;
    
    PROCEDURE RemoveSlowStatus (player : MStruct.PlayerInfoPointer);
    BEGIN (* RemoveSlowStatus *)
      EXCL (player^.character^.variables.status,MCStruct.SlowFLAG);
      
      MSActs3.AlterPlayerSpeed (player,-1);
      MSActs2.DisturbPlayer (player,TRUE);
      
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You feel yourself speed up.");
    END RemoveSlowStatus;
    
  BEGIN (* UpdateSlowCOUNT *)
    IF NOT (MCStruct.SlowFLAG IN player^.character^.variables.status) THEN
      InitializeSlowStatus (player);
    END; (* IF *)
    
    DEC (player^.character^.variables.counters [MCStruct.SlowCOUNT]);
    
    IF player^.character^.variables.counters [MCStruct.SlowCOUNT] = 0 THEN
      RemoveSlowStatus (player);
    END; (* IF *)
  END UpdateSlowCOUNT;
  
  PROCEDURE UpdateSuperHeroCOUNT (player : MStruct.PlayerInfoPointer);
    CONST
          SuperHeroismHitPointModifier = 20.0;
          SuperHeroismAbilityModifier  = 24;
    
    PROCEDURE InitializeSuperHeroStatus (player : MStruct.PlayerInfoPointer);
    BEGIN (* InitializeSuperHeroStatus *)
      INCL (player^.character^.variables.status,MCStruct.SuperHeroismFLAG);
      
      player^.character^.variables.hpCUR := player^.character^.variables.hpCUR + SuperHeroismHitPointModifier;
      player^.character^.variables.hpMAX := player^.character^.variables.hpMAX + SuperHeroismHitPointModifier;
      
      MSUpdate.UpdateCurrentHitPointsWindow (player);
      MSUpdate.UpdateMaximumHitPointsWindow (player);
      
      INC (player^.character^.abilities.missile,SuperHeroismAbilityModifier);
      INC (player^.character^.abilities.martial,SuperHeroismAbilityModifier);
      
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You feel like a SUPER HERO!");
      MSActs2.DisturbPlayer (player,FALSE);
    END InitializeSuperHeroStatus;
    
    PROCEDURE RemoveSuperHeroStatus (player : MStruct.PlayerInfoPointer);
    BEGIN (* RemoveSuperHeroStatus *)
      EXCL (player^.character^.variables.status,MCStruct.SuperHeroismFLAG);
      
      player^.character^.variables.hpMAX := player^.character^.variables.hpMAX - SuperHeroismHitPointModifier;
      MSUpdate.UpdateMaximumHitPointsWindow (player);
      
      IF player^.character^.variables.hpCUR > player^.character^.variables.hpMAX THEN
        player^.character^.variables.hpCUR := player^.character^.variables.hpMAX;
        MSUpdate.UpdateCurrentHitPointsWindow (player);
      END; (* IF *)
      
      DEC (player^.character^.abilities.missile,SuperHeroismAbilityModifier);
      DEC (player^.character^.abilities.martial,SuperHeroismAbilityModifier);
      
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"The super heroism wears off.");
      MSActs2.DisturbPlayer (player,FALSE);
    END RemoveSuperHeroStatus;
    
  BEGIN (* UpdateSuperHeroCOUNT *)
    IF NOT (MCStruct.SuperHeroismFLAG IN player^.character^.variables.status) THEN
      InitializeSuperHeroStatus (player);
    END; (* IF *)
    
    DEC (player^.character^.variables.counters [MCStruct.SuperHeroCOUNT]);
    
    IF player^.character^.variables.counters [MCStruct.SuperHeroCOUNT] = 0 THEN
      RemoveSuperHeroStatus (player);
    END; (* IF *)
  END UpdateSuperHeroCOUNT;
  
  PROCEDURE UpdateTimedInfraCOUNT (player : MStruct.PlayerInfoPointer);
    PROCEDURE InitializeTimedInfraStatus (player : MStruct.PlayerInfoPointer);
    BEGIN (* InitializeTimedInfraStatus *)
      INCL (player^.character^.variables.status,MCStruct.TimedInfraFLAG);
      INC (player^.character^.abilities.infra);
      
      MCreatures.ComputeCreatureVisibility (player,TRUE);
    END InitializeTimedInfraStatus;
    
    PROCEDURE RemoveTimedInfraStatus (player : MStruct.PlayerInfoPointer);
    BEGIN (* RemoveTimedInfraStatus *)
      EXCL (player^.character^.variables.status,MCStruct.TimedInfraFLAG);
      DEC (player^.character^.abilities.infra);
      
      MCreatures.ComputeCreatureVisibility (player,TRUE);
    END RemoveTimedInfraStatus;
    
  BEGIN (* UpdateTimedInfraCOUNT *)
    IF NOT (MCStruct.TimedInfraFLAG IN player^.character^.variables.status) THEN
      InitializeTimedInfraStatus (player);
    END; (* IF *)
    
    DEC (player^.character^.variables.counters [MCStruct.TimedInfraCOUNT]);
    
    IF player^.character^.variables.counters [MCStruct.TimedInfraCOUNT] = 0 THEN
      RemoveTimedInfraStatus (player);
    END; (* IF *)
  END UpdateTimedInfraCOUNT;
  
  PROCEDURE UpdateWordOfRecallCOUNT (player : MStruct.PlayerInfoPointer);
  BEGIN (* UpdateWordOfRecallCOUNT *)
    DEC (player^.character^.variables.counters [MCStruct.WordOfRecallCOUNT]);
    
    IF ((player^.character^.variables.counters [MCStruct.WordOfRecallCOUNT] = 0) AND (player^.character^.variables.depthMAX > 0)) THEN
      INC (player^.character^.variables.counters [MCStruct.ParalysisCOUNT]);
      
      IF player^.character^.variables.depthCUR = 0 THEN
        MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You feel yourself yanked downwards!");
        player^.character^.variables.depthCUR := player^.character^.variables.depthMAX;
      ELSE
        MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You feel yourself yanked upwards!");
        player^.character^.variables.depthCUR := 0;
      END; (* IF *)
      
      MDGenerator.GeneratePlayerDungeon (player);
    END; (* IF *)
  END UpdateWordOfRecallCOUNT;
  
BEGIN (* MPCounters *)
  InitializeUpdateCountedConditionTable;
END MPCounters.
