IMPLEMENTATION MODULE MPSearch;
  (*/WIN:A*)
  
  IMPORT MDistMove,MDPainting,MDWindow,MMenu,MMWindow,MRandom,MSUpdate;
  IMPORT DStruct,MCStruct,MStruct,OStruct;
  IMPORT MSActions,MSActs3,OSActions;
  
  
  PROCEDURE ExecuteSearchToggleCommand (player : MStruct.PlayerInfoPointer);
    PROCEDURE PlayerSearchingOn (player : MStruct.PlayerInfoPointer);
    BEGIN (* PlayerSearchingOn *)
      INCL (player^.character^.variables.status,MCStruct.SearchFLAG);
      INC (player^.character^.variables.digestion);
      
      MSActs3.AlterPlayerSpeed (player,+1);
      
      MSUpdate.UpdateSearchRestStatusWindow (player);
      MSUpdate.UpdateCurrentSpeedWindow (player);
      
      MMenu.SetSearchMarker (player^.windows.frame,TRUE);
    END PlayerSearchingOn;
    
  BEGIN (* ExecuteSearchToggleCommand *)
    MStruct.CharacterTurnExecuted := FALSE;
    
    IF MCStruct.SearchFLAG IN player^.character^.variables.status THEN
      PlayerSearchingOff (player);
    ELSE
      PlayerSearchingOn (player);
    END; (* IF *)
  END ExecuteSearchToggleCommand;
  
  PROCEDURE PlayerSearchingOff (player : MStruct.PlayerInfoPointer);
  BEGIN (* PlayerSearchingOff *)
    EXCL (player^.character^.variables.status,MCStruct.SearchFLAG);
    DEC (player^.character^.variables.digestion);
    
    MSActs3.AlterPlayerSpeed (player,-1);
    
    MSUpdate.UpdateSearchRestStatusWindow (player);
    MSUpdate.UpdateCurrentSpeedWindow (player);
    
    MMenu.SetSearchMarker (player^.windows.frame,FALSE);
  END PlayerSearchingOff;
  
  PROCEDURE SearchCurrentLocation (player : MStruct.PlayerInfoPointer);
    VAR
        column : DStruct.DungeonColumnRange;
        chance : CARDINAL;
        line   : DStruct.DungeonLineRange;
    
    PROCEDURE DisplayHiddenObjects (player : MStruct.PlayerInfoPointer; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange; location : DStruct.PhysicalDungeonUnit);
      PROCEDURE DisplayChestInformation (player : MStruct.PlayerInfoPointer; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange; VAR chest : OStruct.InventoryItemRecord);
      BEGIN (* DisplayChestInformation *)
        IF OStruct.ChestTrapped IN chest.object.attributes THEN
          IF OSActions.InventoryItemIsKnown2 (chest) THEN
            MMWindow.AddMessageToList (player,"The chest is trapped!");
          ELSE
            MMWindow.AddMessageToList (player,"You have discovered a trap on the chest!");
            OSActions.IdentifyObjectAsKnown2 (player^.treasures,chest);
          END; (* IF *)
        END; (* IF *)
      END DisplayChestInformation;
      
      PROCEDURE DisplaySecretDoorInformation (player : MStruct.PlayerInfoPointer; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange; VAR door : OStruct.InventoryItemRecord);
      BEGIN (* DisplaySecretDoorInformation *)
        door.object.symbol := player^.treasures^.table [OStruct.ClosedDoorObjectCursor].symbol;
        door.object.class  := player^.treasures^.table [OStruct.ClosedDoorObjectCursor].class;
        door.cursor        := OStruct.ClosedDoorObjectCursor;
        
        MMWindow.AddMessageToList (player,"You have found a secret door.");
        MDPainting.InvalidateDungeonLocation (player,line,column,FALSE,TRUE);
        
        MDistMove.EndDistanceMovement (player);
      END DisplaySecretDoorInformation;
      
      PROCEDURE DisplayTrapInformation (player : MStruct.PlayerInfoPointer; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange; VAR trap : OStruct.InventoryItemRecord);
      BEGIN (* DisplayTrapInformation *)
        trap.object.class := OStruct.VisibleTrap;
        
        OSActions.DisplayObjectMessage (player^.windows.frame,player,"You have found %OBJECT","%OBJECT",trap,TRUE,TRUE);
        MDPainting.InvalidateDungeonLocation (player,line,column,FALSE,TRUE);
        
        MDistMove.EndDistanceMovement (player);
      END DisplayTrapInformation;
      
    BEGIN (* DisplayHiddenObjects *)
      IF location.artifact <> DStruct.InvalidArtifactCursor THEN
        IF player^.dungeon^.artifacts [location.artifact].object.class = OStruct.InvisibleTrap THEN
          DisplayTrapInformation (player,line,column,player^.dungeon^.artifacts [location.artifact]);
        ELSIF player^.dungeon^.artifacts [location.artifact].object.class = OStruct.SecretDoor THEN
          DisplaySecretDoorInformation (player,line,column,player^.dungeon^.artifacts [location.artifact]);
        ELSIF player^.dungeon^.artifacts [location.artifact].object.class = OStruct.Chest THEN
          DisplayChestInformation (player,line,column,player^.dungeon^.artifacts [location.artifact]);
        END; (* IF *)
      END; (* IF *)
    END DisplayHiddenObjects;
    
    PROCEDURE SuccessfulSearchProbability (player : MStruct.PlayerInfoPointer) : CARDINAL;
      VAR
          chance : CARDINAL;
      
      PROCEDURE PlayerHasSearchingLight (player : MStruct.PlayerInfoPointer) : BOOLEAN;
      BEGIN (* PlayerHasSearchingLight *)
        RETURN (player^.dungeon^.physical [player^.character^.line]^ [player^.character^.column].elements * DStruct.DungeonElementSet {DStruct.plFLAG,DStruct.tlFLAG} <> DStruct.EmptyDungeonElementSet);
      END PlayerHasSearchingLight;
      
    BEGIN (* SuccessfulSearchProbability *)
      chance := player^.character^.abilities.search;
      
      IF player^.character^.variables.counters [MCStruct.ConfusedCOUNT] > 0 THEN
        chance := chance DIV 10;
      END; (* IF *)
      
      IF ((player^.character^.variables.counters [MCStruct.BlindCOUNT] > 0) OR NOT (PlayerHasSearchingLight (player))) THEN
        chance := chance DIV 10;
      END; (*IF *)
      
      IF player^.character^.variables.counters [MCStruct.ImageCOUNT] > 0 THEN
        chance := chance DIV 10;
      END; (* FI *)
      
      RETURN (chance);
    END SuccessfulSearchProbability;
    
  BEGIN (* SearchCurrentLocation *)
    chance := SuccessfulSearchProbability (player);
    
    IF chance > 0 THEN
      FOR line := player^.character^.line - 1 TO player^.character^.line + 1 DO
        FOR column := player^.character^.column - 1 TO player^.character^.column + 1 DO
          IF (((line <> player^.character^.line) OR (column <> player^.character^.column)) AND (MRandom.CardinalRandomRangeValue (1,100) <= chance)) THEN
            DisplayHiddenObjects (player,line,column,player^.dungeon^.physical [line]^ [column]);
          END; (* IF *)
        END; (* FOR *)
      END; (* FOR *)
    END; (* IF *)
  END SearchCurrentLocation;

  PROCEDURE ShouldSearchCurrentLocation (player : MStruct.PlayerInfoPointer) : BOOLEAN;
  BEGIN (* ShouldSearchCurrentLocation *)
    RETURN ((MCStruct.SearchFLAG IN player^.character^.variables.status) OR (player^.character^.abilities.perception <= 1) OR (MRandom.CardinalRandomRangeValue (1,player^.character^.abilities.perception) = 1));
  END ShouldSearchCurrentLocation;
END MPSearch.
