IMPLEMENTATION MODULE MSActions;
  (*/WIN:A*)
  
  FROM WMemory IMPORT ALLOCATE,DEALLOCATE;
  
  IMPORT WCreation,WDebugging,Windows,WMacro;
  
  IMPORT MCSpells,MMenu,MMWindow,MRandom,MResources,MSWindow;
  IMPORT DSActions,MCSActions,MSActs2,MSActs3,OSActions;
  IMPORT MCStruct,MStruct,OStruct;
  
  
  PROCEDURE AllocateCharacterInformation (VAR player : MStruct.PlayerInfoPointer);
    CONST
          InsufficientMemoryError = "Insufficient memory to create new character.";
    
    VAR
        done : BOOLEAN;
    
  BEGIN (* AllocateCharacterInformation *)
    NEW (player);
    
    IF player = NIL THEN
      WDebugging.FatalAppExit (InsufficientMemoryError);
    END; (* IF *)
    
    WITH player^ DO
      MCSActions.AllocateCharacterInformation (character,done);
      
      IF NOT (done) THEN
        WDebugging.FatalAppExit (InsufficientMemoryError);
      END; (* IF *)
      
      creatures := MResources.LoadCreatureResources ();
      
      IF creatures = NIL THEN
        WDebugging.FatalAppExit (InsufficientMemoryError);
      END; (* IF *)
      
      treasures := MResources.LoadTreasureResources ();
      
      IF treasures = NIL THEN
        WDebugging.FatalAppExit (InsufficientMemoryError);
      END; (* IF *)
      
      DSActions.AllocateDungeonInformation (dungeon);
      
      IF dungeon = NIL THEN
        WDebugging.FatalAppExit (InsufficientMemoryError);
      END; (* IF *)
      
      fileNAME := "";
      killer   := "";
    END; (* WITH *)
  END AllocateCharacterInformation;
  
  PROCEDURE ComputePlayerUseStatistic (player : MStruct.PlayerInfoPointer; stat : MCStruct.PlayerStatisticMnemonic);
  BEGIN (* ComputePlayerUseStatistic *)
    MCSActions.ModifyPlayerStatistic (player^.character^.statistics.current [stat],player^.character^.statistics.modifiers [stat],player^.character^.statistics.used [stat]);
    
    IF ((stat = MCStruct.wis) AND (player^.character^.constants.class IN MCStruct.PriestClasses)) THEN
      MCSpells.CalculatePlayerSpells (player);
      MCSpells.CalculatePlayerMana (player);
    ELSIF ((stat = MCStruct.int) AND (player^.character^.constants.class IN MCStruct.MageClasses)) THEN
      MCSpells.CalculatePlayerSpells (player);
      MCSpells.CalculatePlayerMana (player);
    ELSIF stat = MCStruct.str THEN
      INCL (player^.character^.variables.status,MCStruct.StrWgtFLAG);
      MCSActions.CalculatePlayerBonuses (player^.character);
    ELSIF stat = MCStruct.dex THEN
      MCSActions.CalculatePlayerBonuses (player^.character);
    ELSIF stat = MCStruct.con THEN
      MCSActions.CalculatePlayerHitPoints (player^.character);
    END; (* IF *)
  END ComputePlayerUseStatistic;
  
  PROCEDURE DeallocateCharacterInformation (VAR player : MStruct.PlayerInfoPointer);
    PROCEDURE DeallocateGamingWindows (VAR windows : MStruct.MoriaWindowInformation);
      VAR
          done : BOOLEAN;
      
    BEGIN (* DeallocateGamingWindows *)
      IF windows.dialog <> Windows.NULL THEN
        WCreation.DestroyWindow (windows.dialog,done);
      END; (* IF *)
      
      IF windows.message.listbox <> Windows.NULL THEN
        WCreation.DestroyWindow (windows.message.listbox,done);
      END; (* IF *)
      
      IF windows.message.frame <> Windows.NULL THEN
        WCreation.DestroyWindow (windows.message.frame,done);
      END; (* IF *)
      
      IF windows.status.handle <> Windows.NULL THEN
        WCreation.DestroyWindow (windows.status.handle,done);
      END; (* IF *)
      
      IF windows.dungeon <> Windows.NULL THEN
        WCreation.DestroyWindow (windows.dungeon,done);
      END; (* IF *)
      
      MSWindow.SetMainWindowTitle (windows.frame,NIL);
    END DeallocateGamingWindows;
    
  BEGIN (* DeallocateCharacterInformation *)
    MMenu.ResetNonPlayerMenu (player^.windows.frame);
    
    MCSActions.DeallocateCharacterInformation (player^.character);
    
    MResources.FreeCreatureResources;
    MResources.FreeTreasureResources;
    
    DSActions.DeallocateDungeonInformation (player^.dungeon);
    DeallocateGamingWindows (player^.windows);
    
    DISPOSE (player);
  END DeallocateCharacterInformation;
  
  PROCEDURE InitializeConstantPlayerInformation (player : MStruct.PlayerInfoPointer);
  BEGIN (* InitializeConstantPlayerInformation *)
    MCSActions.InitializeCharacterInformation (player^.treasures,player^.character^);
    
    OSActions.InitializeObjectNames (player^.treasures^,MRandom.RandomSeedValue ());
    DSActions.InitializeDungeonTown (player^.dungeon^);
  END InitializeConstantPlayerInformation;
  
  PROCEDURE PlayerBackpackWeightLimit (player : MStruct.PlayerInfoPointer) : CARDINAL;
  BEGIN (* PlayerBackpackWeightLimit *)
    RETURN (WMacro.MinCardinal (VAL (CARDINAL,player^.character^.statistics.used [MCStruct.str]) * 130 + player^.character^.constants.weight,3000));
  END PlayerBackpackWeightLimit;
  
  PROCEDURE RecomputeEquipmentBonuses (player : MStruct.PlayerInfoPointer; item : OStruct.InventoryItemRecord; modifier : INTEGER);
    VAR
        amount : INTEGER;
    
    PROCEDURE InventoryItemModifiesStatistics (item : OStruct.InventoryItemRecord) : BOOLEAN;
      VAR
          index : [OStruct.Strength..OStruct.Charisma];
      
    BEGIN (* InventoryItemModifiesStatistics *)
      FOR index := OStruct.Strength TO OStruct.Charisma DO
        IF index IN item.object.attributes THEN
          RETURN (TRUE);
        END; (* IF *)
      END; (* FOR *)
      
      RETURN (FALSE);
    END InventoryItemModifiesStatistics;
    
    PROCEDURE ModifiedStatistic (item : OStruct.InventoryItemRecord) : MCStruct.PlayerStatisticMnemonic;
      VAR
          index : [OStruct.Strength..OStruct.Charisma];
      
    BEGIN (* ModifiedStatistic *)
      FOR index := OStruct.Strength TO OStruct.Charisma DO
        IF index IN item.object.attributes THEN
          RETURN (VAL (MCStruct.PlayerStatisticMnemonic,ORD (index) - ORD (OStruct.Strength)));
        END; (* IF *)
      END; (* FOR *)
    END ModifiedStatistic;
    
    PROCEDURE ModifyCharacterStatistic (player : MStruct.PlayerInfoPointer; statistic : MCStruct.PlayerStatisticMnemonic; modifier : INTEGER);
    BEGIN (* ModifyCharacterStatistic *)
      player^.character^.statistics.modifiers [statistic] := player^.character^.statistics.modifiers [statistic] + modifier;
      ComputePlayerUseStatistic (player,statistic);
      
      INCL (player^.character^.variables.status,VAL (MCStruct.PlayerStatusMnemonic,ORD (statistic) + ORD (MCStruct.StrFLAG)));
    END ModifyCharacterStatistic;
    
  BEGIN (* RecomputeEquipmentBonuses *)
    amount := item.object.miscellaneous * modifier;
    
    IF InventoryItemModifiesStatistics (item) THEN
      ModifyCharacterStatistic (player,ModifiedStatistic (item),amount);
    END; (* IF *)
    
    IF OStruct.Search IN item.object.attributes THEN
      player^.character^.abilities.perception := player^.character^.abilities.perception - amount;
      player^.character^.abilities.search     := player^.character^.abilities.search + amount;
    END; (* IF *)
    
    IF OStruct.Stealth IN item.object.attributes THEN
      player^.character^.abilities.stealth := player^.character^.abilities.stealth + amount;
    END; (* IF *)
    
    IF OStruct.Speed IN item.object.attributes THEN
      MSActs3.AlterPlayerSpeed (player,-amount);
    END; (* IF *)
    
    IF OStruct.InfraVision IN item.object.attributes THEN
      player^.character^.abilities.infra := VAL (INTEGER,player^.character^.abilities.infra) + amount;
    END; (* IF *)
    
    IF modifier > 0 THEN
      IF OStruct.Blindness IN item.object.attributes THEN
        INC (player^.character^.variables.counters [MCStruct.BlindCOUNT],1000);
      END; (* IF *)
      
      IF OStruct.Timidity IN item.object.attributes THEN
        INC (player^.character^.variables.counters [MCStruct.AfraidCOUNT],50);
      END; (* IF *)
    END; (* IF *)
  END RecomputeEquipmentBonuses;
  
  PROCEDURE SuccessfulSavingsRoll (player : MStruct.PlayerInfoPointer) : BOOLEAN;
  BEGIN (* SuccessfulSavingsRoll *)
    RETURN (MRandom.IntegerRandomRangeValue (1,100) <= player^.character^.abilities.save + MCSActions.CharacterStatisticAdjustment (player^.character,MCStruct.wis) + MCSActions.ClassSaveLevelAdjustment (player^.character^.constants.class)  * player^.character^.variables.level DIV 3);
  END SuccessfulSavingsRoll;
END MSActions.
