IMPLEMENTATION MODULE MSColors;
  (*/WIN:A*)
  
  IMPORT WCreation,WDAttrib,WDebugging,WDialog,WDTool,Windows,WMacro,WPalette,WSystem;
  IMPORT SYSTEM;
  
  IMPORT MDialogs,MInitFile,MSUpdate,MSWindow;
  IMPORT MStruct;
  
  
  CONST
        NormalBackgroundEntry = "Normal_Background";
        NormalForegroundEntry = "Normal_Foreground";
        AlertBackgroundEntry  = "Alert_Background";
        AlertForegroundEntry  = "Alert_Foreground";
  
  CONST
        NormalStatusValueControl = 1922;
        AlertStatusValueControl  = 1923;
  
  TYPE
       StatusControlColorRecord = RECORD
                                    background,foreground : Windows.COLORREF;
                                    brush                 : Windows.HBRUSH;
                                  END; (* StatusControlColorRecord *)
  
  TYPE
       StatusControlColorTable        = ARRAY MStruct.StatusControlColorMnemonic OF StatusControlColorRecord;
       StatusControlColorTablePointer = POINTER TO StatusControlColorTable;
  
  VAR
      CurrentStatusControlColorTable : StatusControlColorTable;
  
  
  PROCEDURE CloseStatusColorsDialog (window : Windows.HWND; data : SYSTEM.ADDRESS);
  BEGIN (* CloseStatusColorsDialog *)
    WDialog.EndDialog (window,Windows.FALSE_BOOL);
  END CloseStatusColorsDialog;
  
  PROCEDURE DetermineStatusControlColor (player : MStruct.PlayerInfoPointer; control : Windows.HWND; context : Windows.HDC; VAR background : Windows.LRESULT);
    VAR
        previous : Windows.COLORREF;
        color    : MStruct.StatusControlColorMnemonic;
        done     : BOOLEAN;
    
    PROCEDURE DetermineColorMnemonicForControlWindow (player : MStruct.PlayerInfoPointer; control : Windows.HWND; VAR color : MStruct.StatusControlColorMnemonic; VAR done : BOOLEAN);
      VAR
          line,column : CARDINAL;
      
    BEGIN (* DetermineColorMnemonicForControlWindow *)
      done := FALSE;
      
      FOR line := 0 TO MStruct.MaxStatusDisplayLines - 1 DO
        FOR column := 0 TO MStruct.MaxStatusDisplayColumns - 1 DO
          IF player^.windows.status.table [line,column].handle = control THEN
            color := player^.windows.status.table [line,column].color;
            done  := TRUE;
          END; (* IF *)
        END; (* FOR *)
      END; (* FOR *)
    END DetermineColorMnemonicForControlWindow;
    
  BEGIN (* DetermineStatusControlColor *)
    DetermineColorMnemonicForControlWindow (player,control,color,done);
    
    IF done THEN
      WDAttrib.SetTextColor (context,CurrentStatusControlColorTable [color].foreground,previous);
      WDAttrib.SetBkColor (context,CurrentStatusControlColorTable [color].background,previous);
      
      background := CurrentStatusControlColorTable [color].brush;
    ELSE
      background := Windows.NULL;
    END; (* IF *)
  END DetermineStatusControlColor;
  
  PROCEDURE ExecuteStatusColorsCommand (player : MStruct.PlayerInfoPointer);
    VAR
        previous : Windows.WORD;
        result   : INTEGER;
        temp     : StatusControlColorTable;
    
    PROCEDURE InitializeTemporaryColorTable (original : StatusControlColorTable; VAR result : StatusControlColorTable);
      VAR
          index : MStruct.StatusControlColorMnemonic;
      
    BEGIN (* InitializeTemporaryColorTable *)
      result := original;
      
      FOR index := MIN (MStruct.StatusControlColorMnemonic) TO MAX (MStruct.StatusControlColorMnemonic) DO
        result [index].brush := WDTool.CreateSolidBrush (result [index].background);
      END; (* FOR *)
    END InitializeTemporaryColorTable;
    
  BEGIN (* ExecuteStatusColorsCommand *)
    InitializeTemporaryColorTable (CurrentStatusControlColorTable,temp);
    
    MDialogs.ExecuteMoriaColorDialogBox (MStruct.CurrentPlayerFrameWindow,"Status_Colors_Dialog",SYSTEM.ADR (temp),InitializeStatusColorsDialog,ExecuteStatusColorsDialogCommand,CloseStatusColorsDialog,StatusControlColor,result);
    
    IF result = Windows.TRUE_BOOL THEN
      ReleaseStatusColorTableBrushes (CurrentStatusControlColorTable);
      CurrentStatusControlColorTable := temp;
      
      IF player <> NIL THEN
        WCreation.SetClassWord (player^.windows.status.handle,Windows.GCW_HBRBACKGROUND,temp [MStruct.NormalValue].brush,previous);
        MSUpdate.InvalidateStatusWindows (player);
      END; (* IF *)
    ELSE
      ReleaseStatusColorTableBrushes (temp);
    END; (* IF *)
    
    MStruct.CharacterTurnExecuted := FALSE;
  END ExecuteStatusColorsCommand;
  
  PROCEDURE ExecuteStatusColorsDialogCommand (window : Windows.HWND; data : SYSTEM.ADDRESS; control : CARDINAL; command : CARDINAL);
    CONST
          NormalForegroundControl = 1911;
          NormalBackgroundControl = 2001;
          AlertForegroundControl  = 1912;
          AlertBackgroundControl  = 2002;
    
    CONST
          SaveColorsControl = 1921;
          CancelControl     = Windows.IDCANCEL;
          AcceptControl     = Windows.IDOK;
    
    PROCEDURE DetermineStatusColor (window : Windows.HWND; table : StatusControlColorTable; default : Windows.COLORREF; VAR result : Windows.COLORREF; VAR done : BOOLEAN);
      VAR
          custom : WPalette.CustomColorTable;
      
      PROCEDURE InitializeCustomColorTable (table : StatusControlColorTable; VAR result : ARRAY OF Windows.COLORREF);
        VAR
            index : CARDINAL;
        
      BEGIN (* InitializeCustomColorTable *)
        FOR index := 0 TO HIGH (result) DO
          result [index] := WMacro.RGB (255,255,255);
        END; (* FOR *)
        
        result [0] := table [MStruct.WarningValue].foreground;
        result [1] := table [MStruct.WarningValue].background;
        result [2] := table [MStruct.NormalValue].foreground;
        result [3] := table [MStruct.NormalValue].background;
      END InitializeCustomColorTable;
      
    BEGIN (* DetermineStatusColor *)
      InitializeCustomColorTable (table,custom);
      WPalette.ChooseColor (window,custom,default,result,done);
    END DetermineStatusColor;
    
    PROCEDURE ExecuteAcceptAction (window : Windows.HWND; command : CARDINAL);
    BEGIN (* ExecuteAcceptAction *)
      IF command = Windows.BN_CLICKED THEN
        WDialog.EndDialog (window,Windows.TRUE_BOOL);
      END; (* IF *)
    END ExecuteAcceptAction;
    
    PROCEDURE ExecuteBackgroundButtonAction (window : Windows.HWND; table : StatusControlColorTablePointer; color : MStruct.StatusControlColorMnemonic; command : CARDINAL);
      VAR
          value : Windows.COLORREF;
          done  : BOOLEAN;
      
    BEGIN (* ExecuteBackgroundButtonAction *)
      IF command = Windows.BN_CLICKED THEN
        DetermineStatusColor (window,table^,table^ [color].background,value,done);
        
        IF done THEN
          WDTool.DeleteObject (table^ [color].brush,done);
          
          table^ [color].background := value;
          table^ [color].brush      := WDTool.CreateSolidBrush (value);
          
          ResetColorSampleControl (window,color);
        END; (* IF *)
      END; (* IF *)
    END ExecuteBackgroundButtonAction;
    
    PROCEDURE ExecuteCancelAction (window : Windows.HWND; command : CARDINAL);
    BEGIN (* ExecuteCancelAction *)
      IF command = Windows.BN_CLICKED THEN
        WDialog.EndDialog (window,Windows.FALSE_BOOL);
      END; (* IF *)
    END ExecuteCancelAction;
    
    PROCEDURE ExecuteForegroundButtonAction (window : Windows.HWND; table : StatusControlColorTablePointer; color : MStruct.StatusControlColorMnemonic; command : CARDINAL);
      VAR
          value : Windows.COLORREF;
          done  : BOOLEAN;
      
    BEGIN (* ExecuteForegroundButtonAction *)
      IF command = Windows.BN_CLICKED THEN
        DetermineStatusColor (window,table^,table^ [color].foreground,value,done);
        
        IF done THEN
          table^ [color].foreground := value;
          ResetColorSampleControl (window,color);
        END; (* IF *)
      END; (* IF *)
    END ExecuteForegroundButtonAction;
    
    PROCEDURE ExecuteSaveColorsAction (window : Windows.HWND; table : StatusControlColorTablePointer; command : CARDINAL);
    BEGIN (* ExecuteSaveColorsAction *)
      MInitFile.SetStatusColorEntry (NormalBackgroundEntry,table^ [MStruct.NormalValue].background);
      MInitFile.SetStatusColorEntry (NormalForegroundEntry,table^ [MStruct.NormalValue].foreground);
      MInitFile.SetStatusColorEntry (AlertBackgroundEntry,table^ [MStruct.WarningValue].background);
      MInitFile.SetStatusColorEntry (AlertForegroundEntry,table^ [MStruct.WarningValue].foreground);
      
      WDialog.EndDialog (window,Windows.TRUE_BOOL);
    END ExecuteSaveColorsAction;
    
    PROCEDURE ResetColorSampleControl (window : Windows.HWND; color : MStruct.StatusControlColorMnemonic);
    BEGIN (* ResetColorSampleControl *)
      IF color = MStruct.WarningValue THEN
        WDialog.SetDlgItemText (window,AlertStatusValueControl,"Alert");
      ELSE
        WDialog.SetDlgItemText (window,NormalStatusValueControl,"Normal");
      END; (* IF *)
    END ResetColorSampleControl;
    
  BEGIN (* ExecuteStatusColorsDialogCommand *)
    CASE control OF
      NormalBackgroundControl : ExecuteBackgroundButtonAction (window,StatusControlColorTablePointer (data),MStruct.NormalValue,command);  |
      NormalForegroundControl : ExecuteForegroundButtonAction (window,StatusControlColorTablePointer (data),MStruct.NormalValue,command);  |
      AlertBackgroundControl  : ExecuteBackgroundButtonAction (window,StatusControlColorTablePointer (data),MStruct.WarningValue,command); |
      AlertForegroundControl  : ExecuteForegroundButtonAction (window,StatusControlColorTablePointer (data),MStruct.WarningValue,command); |
      
      SaveColorsControl : ExecuteSaveColorsAction (window,StatusControlColorTablePointer (data),command); |
      AcceptControl     : ExecuteAcceptAction (window,command);                                           |
      CancelControl     : ExecuteCancelAction (window,command);                                           |
    ELSE
      (* no action *)
    END; (* CASE *)
  END ExecuteStatusColorsDialogCommand;
  
  PROCEDURE InitializeStatusColorsDialog (window : Windows.HWND; data : SYSTEM.ADDRESS);
  BEGIN (* InitializeStatusColorsDialog *)
    (* no action *)
  END InitializeStatusColorsDialog;
  
  PROCEDURE InitializeStatusControlColorTable (VAR table : StatusControlColorTable);
    PROCEDURE LoadBackgroundStatusColor (entry : ARRAY OF CHAR; default : Windows.COLORREF; VAR background : Windows.COLORREF; VAR brush : Windows.HBRUSH);
    BEGIN (* LoadBackgroundStatusColor *)
      MInitFile.DetermineStatusColorEntry (entry,default,background);
      brush := WDTool.CreateSolidBrush (background);
    END LoadBackgroundStatusColor;
    
  BEGIN (* InitializeStatusControlColorTable *)
    MInitFile.DetermineStatusColorEntry (AlertForegroundEntry,WSystem.GetSysColor (Windows.COLOR_HIGHLIGHTTEXT),table [MStruct.WarningValue].foreground);
    MInitFile.DetermineStatusColorEntry (NormalForegroundEntry,WSystem.GetSysColor (Windows.COLOR_WINDOWTEXT),table [MStruct.NormalValue].foreground);
    
    LoadBackgroundStatusColor (AlertBackgroundEntry,WSystem.GetSysColor (Windows.COLOR_HIGHLIGHT),table [MStruct.WarningValue].background,table [MStruct.WarningValue].brush);
    LoadBackgroundStatusColor (NormalBackgroundEntry,WSystem.GetSysColor (Windows.COLOR_WINDOW),table [MStruct.NormalValue].background,table [MStruct.NormalValue].brush);
  END InitializeStatusControlColorTable;
  
  PROCEDURE ReleaseBackgroundBrushes;
  BEGIN (* ReleaseBackgroundBrushes *)
    ReleaseStatusColorTableBrushes (CurrentStatusControlColorTable);
  END ReleaseBackgroundBrushes;
  
  PROCEDURE ReleaseStatusColorTableBrushes (VAR table : StatusControlColorTable);
    VAR
        index : MStruct.StatusControlColorMnemonic;
        done  : BOOLEAN;
    
  BEGIN (* ReleaseStatusColorTableBrushes *)
    FOR index := MIN (MStruct.StatusControlColorMnemonic) TO MAX (MStruct.StatusControlColorMnemonic) DO
      WDTool.DeleteObject (table [index].brush,done);
    END; (* FOR *)
  END ReleaseStatusColorTableBrushes;
  
  PROCEDURE StatusControlColor (control : Windows.HWND; class : INTEGER; context : Windows.HDC; data : SYSTEM.ADDRESS) : Windows.HBRUSH;
    VAR
        previous : Windows.COLORREF;
        table    : StatusControlColorTablePointer;
    
  BEGIN (* StatusControlColor *)
    table := StatusControlColorTablePointer (data);
    
    IF WDialog.GetDlgCtrlID (control) = NormalStatusValueControl THEN
      WDAttrib.SetTextColor (context,table^ [MStruct.NormalValue].foreground,previous);
      WDAttrib.SetBkColor (context,table^ [MStruct.NormalValue].background,previous);
      
      RETURN (table^ [MStruct.NormalValue].brush);
    ELSIF WDialog.GetDlgCtrlID (control) = AlertStatusValueControl THEN
      WDAttrib.SetTextColor (context,table^ [MStruct.WarningValue].foreground,previous);
      WDAttrib.SetBkColor (context,table^ [MStruct.WarningValue].background,previous);
      
      RETURN (table^ [MStruct.WarningValue].brush);
    ELSE
      RETURN (Windows.NULL);
    END; (* IF *)
  END StatusControlColor;

BEGIN (* MSColors *)
  InitializeStatusControlColorTable (CurrentStatusControlColorTable);
END MSColors.
