IMPLEMENTATION MODULE MScores;
  (*/WIN:A*)
  
  FROM WMemory IMPORT ALLOCATE,DEALLOCATE;
  
  IMPORT MCStruct,MInitFile,MStores,MStruct,OStruct;
  IMPORT WError,Windows,WinFiles;
  IMPORT FileSystem,Strings;
  
  
  PROCEDURE LoadMoriaScoreTable (window : Windows.HWND; VAR name : ARRAY OF CHAR; VAR result : MoriaScoreTable);
    VAR
        done : BOOLEAN;
    
    PROCEDURE ClearMoriaScoreTable (VAR table : ARRAY OF MoriaScoreEntry);
      VAR
          index : CARDINAL;
      
    BEGIN (* ClearMoriaScoreTable *)
      FOR index := 0 TO HIGH (table) DO
        table [index].killer [0] := 0C;
      END; (* FOR *)
    END ClearMoriaScoreTable;
    
    PROCEDURE ReadMoriaScoreTable (name : ARRAY OF CHAR; VAR result : MoriaScoreTable; VAR done : BOOLEAN);
      VAR
          f : FileSystem.File;
      
    BEGIN (* ReadMoriaScoreTable *)
      WinFiles.OpenFile (f,name);
      
      IF f.res = FileSystem.done THEN
        WinFiles.ReadDataStructure (f,result);
        done := (f.res = FileSystem.done);
        
        WinFiles.CloseFile (f);
      ELSE
        done := FALSE;
      END; (* IF *)
    END ReadMoriaScoreTable;
    
  BEGIN (* LoadMoriaScoreTable *)
    MInitFile.DetermineMoriaScoreTableFileName (name);
    ReadMoriaScoreTable (name,result,done);
      
    IF NOT (done) THEN
      ClearMoriaScoreTable (result);
    END; (* IF *)
  END LoadMoriaScoreTable;
  
  PROCEDURE UpdateHighScoreInformation (player : MStruct.PlayerInfoPointer);
    VAR
        table : MoriaScoreTablePointer;
        name  : ARRAY [0..127] OF CHAR;
        done  : BOOLEAN;
    
    PROCEDURE AddPlayerScoreToTable (player : MStruct.PlayerInfoPointer; VAR table : ARRAY OF MoriaScoreEntry);
      VAR
          entry : MoriaScoreEntry;
          index : CARDINAL;
      
      PROCEDURE InsertMoriaScoreEntry (entry : MoriaScoreEntry; VAR table : ARRAY OF MoriaScoreEntry; position : CARDINAL);
        VAR
            index : CARDINAL;
        
      BEGIN (* InsertMoriaScoreEntry *)
        FOR index := HIGH (table) TO position + 1 BY -1 DO
          table [index] := table [index - 1];
        END; (* FOR *)
        
        table [position] := entry;
      END InsertMoriaScoreEntry;
      
      PROCEDURE ReturnPlayerScoreEntry (player : MStruct.PlayerInfoPointer; VAR result : MoriaScoreEntry);
        PROCEDURE TotalCharacterPoints (player : MStruct.PlayerInfoPointer) : LONGINT;
          VAR
              value : LONGINT;
          
          PROCEDURE InventoryScoreValue (treasures : OStruct.TreasureInfoPointer; inventory : MCStruct.PlayerInventoryInformation) : LONGINT;
            PROCEDURE InventoryArrayValue (treasures : OStruct.TreasureInfoPointer; inventory : ARRAY OF OStruct.InventoryItemRecord) : LONGINT;
              VAR
                  index : CARDINAL;
                  total : LONGINT;
              
            BEGIN (* InventoryArrayValue *)
              total := 0;
              
              FOR index := 0 TO HIGH (inventory) DO
                IF inventory [index].object.class <> OStruct.Nothing THEN
                  total := total + MStores.BasicItemValue (treasures,inventory [index]);
                END; (* IF *)
              END; (* FOR *)
              
              RETURN (total);
            END InventoryArrayValue;
            
          BEGIN (* InventoryScoreValue *)
            RETURN (InventoryArrayValue (treasures,inventory.backpack) + InventoryArrayValue (treasures,inventory.equipment));
          END InventoryScoreValue;
          
        BEGIN (* TotalCharacterPoints *)
          value := LONGTRUNC (player^.character^.variables.experCUR);
          value := value + VAL (LONGINT,100 * player^.character^.variables.depthMAX);
          value := value + VAL (LONGINT,player^.character^.variables.currentGOLD DIV 100);
          value := value + VAL (LONGINT,50 * player^.character^.variables.depthCUR);
          value := value + InventoryScoreValue (player^.treasures,player^.character^.inventory);
          
          RETURN (value);
        END TotalCharacterPoints;
        
      BEGIN (* ReturnPlayerScoreEntry *)
        Strings.Assign (player^.character^.constants.name,result.name);
        Strings.Assign (player^.killer,result.killer);
        
        result.points := TotalCharacterPoints (player);
        result.class  := player^.character^.constants.class;
        result.level  := player^.character^.variables.level;
        result.race   := player^.character^.constants.race;
        result.sex    := player^.character^.constants.sex;
      END ReturnPlayerScoreEntry;
      
    BEGIN (* AddPlayerScoreToTable *)
      ReturnPlayerScoreEntry (player,entry);
      
      index := 0;
      
      WHILE ((index <= HIGH (table)) AND (Strings.Length (table [index].killer) > 0) AND (entry.points <= table [index].points)) DO
        INC (index);
      END; (* WHILE *)
      
      IF index <= HIGH (table) THEN
        InsertMoriaScoreEntry (entry,table,index);
      END; (* IF *)
    END AddPlayerScoreToTable;
    
    PROCEDURE SaveMoriaScoreTable (window : Windows.HWND; name : ARRAY OF CHAR; VAR table : MoriaScoreTable);
      VAR
          f : FileSystem.File;
      
    BEGIN (* SaveMoriaScoreTable *)
      WinFiles.CreateFileWithBackup (f,name);
      
      IF f.res = FileSystem.done THEN
        WinFiles.WriteDataStructure (f,table);
        done := (f.res = FileSystem.done);
        
        WinFiles.CloseFile (f);
        
        done := (done AND (f.res = FileSystem.done));
      ELSE
        done := FALSE;
      END; (* IF *)
      
      IF NOT (done) THEN
        WError.DisplayInformationBox (window,MStruct.MainWindowTitle,"Unable to update high score file.");
      END; (* IF *)
    END SaveMoriaScoreTable;
    
  BEGIN (* UpdateHighScoreInformation *)
    NEW (table);
    
    LoadMoriaScoreTable (player^.windows.frame,name,table^);
    AddPlayerScoreToTable (player,table^);
      
    SaveMoriaScoreTable (player^.windows.frame,name,table^);
    
    DISPOSE (table);
  END UpdateHighScoreInformation;
END MScores.
