IMPLEMENTATION MODULE MScrl4;
  (*/WIN:A*)
  
  IMPORT MCSActions,MSActions,OSActions;
  IMPORT MCStruct,MStruct,OStruct;
  IMPORT MMWindow,MRandom;
  
  
  PROCEDURE DetermineArmorToEnchant (VAR inventory : MCStruct.PlayerInventoryInformation; VAR position : MCStruct.PlayerEquipmentMnemonic; VAR done : BOOLEAN);
    TYPE
         PlayerEquipmentSet = SET OF MCStruct.PlayerEquipmentMnemonic;
    
    CONST
          PlayerArmorPositions = PlayerEquipmentSet {MCStruct.Body,MCStruct.Arm,MCStruct.Outer,MCStruct.Head,MCStruct.Hands,MCStruct.Feet};
    
    PROCEDURE AnyCursedArmor (VAR inventory : MCStruct.PlayerInventoryInformation) : BOOLEAN;
      VAR
          index : MCStruct.PlayerEquipmentMnemonic;
      
    BEGIN (* AnyCursedArmor *)
      FOR index := MIN (MCStruct.PlayerEquipmentMnemonic) TO MAX (MCStruct.PlayerEquipmentMnemonic) DO
        IF ((index IN PlayerArmorPositions) AND (OStruct.Cursed IN inventory.equipment [position].object.attributes)) THEN
          RETURN (TRUE);
        END; (* IF *)
      END; (* FOR *)
      
      RETURN (FALSE);
    END AnyCursedArmor;
    
    PROCEDURE CursedArmorPosition (VAR inventory : MCStruct.PlayerInventoryInformation) : MCStruct.PlayerEquipmentMnemonic;
    BEGIN (* CursedArmorPosition *)
      IF OStruct.Cursed IN inventory.equipment [MCStruct.Body].object.attributes THEN
        RETURN (MCStruct.Body);
      ELSIF OStruct.Cursed IN inventory.equipment [MCStruct.Arm].object.attributes THEN
        RETURN (MCStruct.Arm);
      ELSIF OStruct.Cursed IN inventory.equipment [MCStruct.Outer].object.attributes THEN
        RETURN (MCStruct.Outer);
      ELSIF OStruct.Cursed IN inventory.equipment [MCStruct.Head].object.attributes THEN
        RETURN (MCStruct.Head);
      ELSIF OStruct.Cursed IN inventory.equipment [MCStruct.Hands].object.attributes THEN
        RETURN (MCStruct.Hands);
      ELSIF OStruct.Cursed IN inventory.equipment [MCStruct.Feet].object.attributes THEN
        RETURN (MCStruct.Feet);
      END; (* IF *)
    END CursedArmorPosition;
    
    PROCEDURE DetermineRandomArmorPosition (VAR inventory : MCStruct.PlayerInventoryInformation; VAR position : MCStruct.PlayerEquipmentMnemonic; VAR done : BOOLEAN);
      VAR
          number : CARDINAL;
          table  : ARRAY [0..15] OF MCStruct.PlayerEquipmentMnemonic;
      
      PROCEDURE ReturnArmorEquipmentTable (VAR inventory : MCStruct.PlayerInventoryInformation; VAR table : ARRAY OF MCStruct.PlayerEquipmentMnemonic; VAR number : CARDINAL);
        VAR
            index : MCStruct.PlayerEquipmentMnemonic;
        
      BEGIN (* ReturnArmorEquipmentTable *)
        number := 0;
        
        FOR index := MIN (MCStruct.PlayerEquipmentMnemonic) TO MAX (MCStruct.PlayerEquipmentMnemonic) DO
          IF ((index IN PlayerArmorPositions) AND (inventory.equipment [index].object.class <> OStruct.Nothing)) THEN
            table [number] := index;
            number         := number + 1;
          END; (* IF *)
        END; (* FOR *)
      END ReturnArmorEquipmentTable;
      
    BEGIN (* DetermineRandomArmorPosition *)
      ReturnArmorEquipmentTable (inventory,table,number);
      done := (number > 0);
      
      IF done THEN
        position := table [MRandom.CardinalRandomRangeValue (0,number - 1)];
      END; (* IF *)
    END DetermineRandomArmorPosition;
    
  BEGIN (* DetermineArmorToEnchant *)
    IF AnyCursedArmor (inventory) THEN
      position := CursedArmorPosition (inventory);
      done     := TRUE;
    ELSE
      DetermineRandomArmorPosition (inventory,position,done);
    END; (* IF *)
  END DetermineArmorToEnchant;
  
  PROCEDURE EnchantItemModifier (VAR modifier : INTEGER; limit : INTEGER; VAR done : BOOLEAN);
    PROCEDURE EnchantmentChance (modifier : CARDINAL) : CARDINAL;
    BEGIN (* EnchantmentChance *)
      IF MRandom.CardinalRandomRangeValue (1,100) = 1 THEN
        RETURN (MRandom.CardinalRandomRangeValue (1,modifier) - 1);
      ELSE
        RETURN (modifier);
      END; (* IF *)
    END EnchantmentChance;
    
  BEGIN (* EnchantItemModifier *)
    done := ((limit > 0) AND (modifier >= 0) AND (MRandom.CardinalRandomRangeValue (1,limit) > EnchantmentChance (modifier)));
    
    IF done THEN
      INC (modifier);
    END; (* IF *)
  END EnchantItemModifier;
  
  PROCEDURE EnchantWeaponToDam (VAR item : OStruct.InventoryItemRecord; number : CARDINAL; VAR enchanted : BOOLEAN);
    VAR
        count : CARDINAL;
        limit : CARDINAL;
        done  : BOOLEAN;
    
  BEGIN (* EnchantWeaponToDam *)
    IF item.object.class IN OStruct.ObjectClassSet {OStruct.HaftedWeapon..OStruct.DiggingTool} THEN
      limit := item.object.damage.max;
    ELSE
      limit := 10;
    END; (* IF *)
    
    enchanted := FALSE;
    
    FOR count := 1 TO number DO
      EnchantItemModifier (item.object.damageMOD,limit,done);
      enchanted := (enchanted OR done);
    END; (* FOR *)
  END EnchantWeaponToDam;
  
  PROCEDURE EnchantWeaponToHit (VAR item : OStruct.InventoryItemRecord; number : CARDINAL; VAR enchanted : BOOLEAN);
    VAR
        count : CARDINAL;
        done  : BOOLEAN;
    
  BEGIN (* EnchantWeaponToHit *)
    enchanted := FALSE;
    
    FOR count := 1 TO number DO
      EnchantItemModifier (item.object.strikeMOD,10,done);
      enchanted := (enchanted OR done);
    END; (* FOR *)
  END EnchantWeaponToHit;
  
  PROCEDURE ExecuteCurseArmorScroll (player : MStruct.PlayerInfoPointer; VAR identified : BOOLEAN; VAR executed : BOOLEAN);
    VAR
        position : MCStruct.PlayerEquipmentMnemonic;
    
    PROCEDURE CurseArmorItem (player : MStruct.PlayerInfoPointer; VAR item : OStruct.InventoryItemRecord);
    BEGIN (* CurseArmorItem *)
      OSActions.DisplayObjectMessage (player^.windows.frame,player,"Your %OBJECT glows black, fades.","%OBJECT",item,FALSE,TRUE);
      
      INCL (item.object.attributes,OStruct.Cursed);
      
      item.object.armorMOD := -MRandom.IntegerRandomRangeValue (1,5) - MRandom.IntegerRandomRangeValue (1,5);
      item.special         := OStruct.NameNull;
      
      MCSActions.CalculatePlayerBonuses (player^.character);
    END CurseArmorItem;
    
    PROCEDURE DetermineArmorToCurse (VAR inventory : MCStruct.PlayerInventoryInformation; VAR position : MCStruct.PlayerEquipmentMnemonic; VAR done : BOOLEAN);
    BEGIN (* DetermineArmorToCurse *)
      done := TRUE;
      
      IF ((inventory.equipment [MCStruct.Body].object.class <> OStruct.Nothing) AND (MRandom.CardinalRandomRangeValue (1,4) = 1)) THEN
        position := MCStruct.Body;
      ELSIF ((inventory.equipment [MCStruct.Arm].object.class <> OStruct.Nothing) AND (MRandom.CardinalRandomRangeValue (1,3) = 1)) THEN
        position := MCStruct.Arm;
      ELSIF ((inventory.equipment [MCStruct.Outer].object.class <> OStruct.Nothing) AND (MRandom.CardinalRandomRangeValue (1,3) = 1)) THEN
        position := MCStruct.Outer;
      ELSIF ((inventory.equipment [MCStruct.Head].object.class <> OStruct.Nothing) AND (MRandom.CardinalRandomRangeValue (1,3) = 1)) THEN
        position := MCStruct.Head;
      ELSIF ((inventory.equipment [MCStruct.Hands].object.class <> OStruct.Nothing) AND (MRandom.CardinalRandomRangeValue (1,3) = 1)) THEN
        position := MCStruct.Hands;
      ELSIF ((inventory.equipment [MCStruct.Feet].object.class <> OStruct.Nothing) AND (MRandom.CardinalRandomRangeValue (1,3) = 1)) THEN
        position := MCStruct.Feet;
      ELSIF inventory.equipment [MCStruct.Body].object.class <> OStruct.Nothing THEN
        position := MCStruct.Body;
      ELSIF inventory.equipment [MCStruct.Arm].object.class <> OStruct.Nothing THEN
        position := MCStruct.Arm;
      ELSIF inventory.equipment [MCStruct.Outer].object.class <> OStruct.Nothing THEN
        position := MCStruct.Outer;
      ELSIF inventory.equipment [MCStruct.Head].object.class <> OStruct.Nothing THEN
        position := MCStruct.Head;
      ELSIF inventory.equipment [MCStruct.Hands].object.class <> OStruct.Nothing THEN
        position := MCStruct.Hands;
      ELSIF inventory.equipment [MCStruct.Feet].object.class <> OStruct.Nothing THEN
        position := MCStruct.Feet;
      ELSE
        done := FALSE;
      END; (* IF *)
    END DetermineArmorToCurse;
    
  BEGIN (* ExecuteCurseArmorScroll *)
    DetermineArmorToCurse (player^.character^.inventory,position,identified);
    
    IF identified THEN
      CurseArmorItem (player,player^.character^.inventory.equipment [position]);
    END; (* IF *)
    
    executed := TRUE;
  END ExecuteCurseArmorScroll;
  
  PROCEDURE ExecuteCurseWeaponScroll (player : MStruct.PlayerInfoPointer; VAR identified : BOOLEAN; VAR executed : BOOLEAN);
    PROCEDURE CurseWeapon (player : MStruct.PlayerInfoPointer; VAR item : OStruct.InventoryItemRecord);
    BEGIN (* CurseWeapon *)
      OSActions.DisplayObjectMessage (player^.windows.frame,player,"Your %OBJECT glows black, fades.","%OBJECT",item,FALSE,TRUE);
      
      item.object.damageMOD := -MRandom.IntegerRandomRangeValue (1,5) - MRandom.IntegerRandomRangeValue (1,5);
      item.object.strikeMOD := -MRandom.IntegerRandomRangeValue (1,5) - MRandom.IntegerRandomRangeValue (1,5);
      item.special          := OStruct.NameNull;
      
      INCL (item.object.attributes,OStruct.Cursed);
      
      MSActions.RecomputeEquipmentBonuses (player,item,MSActions.RemoveEquipmentBonuses);
      MCSActions.CalculatePlayerBonuses (player^.character);
    END CurseWeapon;
    
  BEGIN (* ExecuteCurseWeaponScroll *)
    identified := (player^.character^.inventory.equipment [MCStruct.Weapon1].object.class <> OStruct.Nothing);
    executed   := TRUE;
    
    IF identified THEN
      CurseWeapon (player,player^.character^.inventory.equipment [MCStruct.Weapon1]);
    END; (* IF *)
  END ExecuteCurseWeaponScroll;
  
  PROCEDURE ExecuteEnchantArmorScroll (player : MStruct.PlayerInfoPointer; VAR identified : BOOLEAN; VAR executed : BOOLEAN);
    VAR
        position : MCStruct.PlayerEquipmentMnemonic;
    
    PROCEDURE EnchantArmor (player : MStruct.PlayerInfoPointer; VAR item : OStruct.InventoryItemRecord);
      VAR
          done : BOOLEAN;
      
    BEGIN (* EnchantArmor *)
      OSActions.DisplayObjectMessage (player^.windows.frame,player,"Your %OBJECT glows faintly!","%OBJECT",item,FALSE,TRUE);
      EnchantItemModifier (item.object.strikeMOD,10,done);
      
      IF done THEN
        MCSActions.CalculatePlayerBonuses (player^.character);
        EXCL (item.object.attributes,OStruct.Cursed);
      ELSE
        MMWindow.DisplayPopupMessage (player^.windows.frame,player,"The enchantment fails.");
      END; (* IF *)
    END EnchantArmor;
    
  BEGIN (* ExecuteEnchantArmorScroll *)
    DetermineArmorToEnchant (player^.character^.inventory,position,identified);
    
    IF identified THEN
      EnchantArmor (player,player^.character^.inventory.equipment [position]);
    END; (* IF *)
    
    executed := TRUE;
  END ExecuteEnchantArmorScroll;
  
  PROCEDURE ExecuteEnchantArmor2Scroll (player : MStruct.PlayerInfoPointer; VAR identified : BOOLEAN; VAR executed : BOOLEAN);
    VAR
        position : MCStruct.PlayerEquipmentMnemonic;
    
    PROCEDURE EnchantArmor2 (player : MStruct.PlayerInfoPointer; VAR item : OStruct.InventoryItemRecord);
      VAR
          done,enchanted : BOOLEAN;
          count          : CARDINAL;
      
    BEGIN (* EnchantArmor2 *)
      OSActions.DisplayObjectMessage (player^.windows.frame,player,"Your %OBJECT glows brightly!","%OBJECT",item,FALSE,TRUE);
      
      enchanted := FALSE;
      
      FOR count := 1 TO MRandom.CardinalRandomRangeValue (2,3) DO
        EnchantItemModifier (item.object.armorMOD,10,done);
        enchanted := (enchanted OR done);
      END; (* FOR *)
      
      IF enchanted THEN
        MCSActions.CalculatePlayerBonuses (player^.character);
        EXCL (item.object.attributes,OStruct.Cursed);
      ELSE
        MMWindow.DisplayPopupMessage (player^.windows.frame,player,"The enchantment fails.");
      END; (* IF *)
    END EnchantArmor2;
    
  BEGIN (* ExecuteEnchantArmor2Scroll *)
    DetermineArmorToEnchant (player^.character^.inventory,position,identified);
    
    IF identified THEN
      EnchantArmor2 (player,player^.character^.inventory.equipment [position]);
    END; (* IF *)
    
    executed := TRUE;
  END ExecuteEnchantArmor2Scroll;
  
  PROCEDURE ExecuteEnchantWeaponScroll (player : MStruct.PlayerInfoPointer; VAR identified : BOOLEAN; VAR executed : BOOLEAN);
    PROCEDURE EnchantWeapon (player : MStruct.PlayerInfoPointer; VAR item : OStruct.InventoryItemRecord);
      VAR
          enchanted1,enchanted2 : BOOLEAN;
      
    BEGIN (* EnchantWeapon *)
      OSActions.DisplayObjectMessage (player^.windows.frame,player,"Your %OBJECT glows brightly!","%OBJECT",item,FALSE,TRUE);
      
      EnchantWeaponToDam (item,MRandom.CardinalRandomRangeValue (1,2),enchanted1);
      EnchantWeaponToHit (item,MRandom.CardinalRandomRangeValue (1,2),enchanted2);
      
      IF (enchanted1 OR enchanted2) THEN
        MCSActions.CalculatePlayerBonuses (player^.character);
        EXCL (item.object.attributes,OStruct.Cursed);
      ELSE
        MMWindow.DisplayPopupMessage (player^.windows.frame,player,"The enchantment fails.");
      END; (* IF *)
    END EnchantWeapon;
    
  BEGIN (* ExecuteEnchantWeaponScroll *)
    identified := (player^.character^.inventory.equipment [MCStruct.Weapon1].object.class <> OStruct.Nothing);
    executed   := TRUE;
    
    IF identified THEN
      EnchantWeapon (player,player^.character^.inventory.equipment [MCStruct.Weapon1]);
    END; (* IF *)
  END ExecuteEnchantWeaponScroll;
  
  PROCEDURE ExecuteEnchantWeaponToDamScroll (player : MStruct.PlayerInfoPointer; VAR identified : BOOLEAN; VAR executed : BOOLEAN);
    VAR
        done : BOOLEAN;
    
  BEGIN (* ExecuteEnchantWeaponToDamScroll *)
    identified := (player^.character^.inventory.equipment [MCStruct.Weapon1].object.class <> OStruct.Nothing);
    executed   := TRUE;
    
    IF identified THEN
      OSActions.DisplayObjectMessage (player^.windows.frame,player,"Your %WEAPON glows faintly!","%WEAPON",player^.character^.inventory.equipment [MCStruct.Weapon1],FALSE,TRUE);
      EnchantWeaponToDam (player^.character^.inventory.equipment [MCStruct.Weapon1],1,done);
      
      IF done THEN
        EXCL (player^.character^.inventory.equipment [MCStruct.Weapon1].object.attributes,OStruct.Cursed);
        MCSActions.CalculatePlayerBonuses (player^.character);
      ELSE
        MMWindow.DisplayPopupMessage (player^.windows.frame,player,"The enchantment fails.");
      END; (* IF *)
    END; (* IF *)
  END ExecuteEnchantWeaponToDamScroll;
  
  PROCEDURE ExecuteEnchantWeaponToHitScroll (player : MStruct.PlayerInfoPointer; VAR identified : BOOLEAN; VAR executed : BOOLEAN);
    VAR
        done : BOOLEAN;
    
  BEGIN (* ExecuteEnchantWeaponToHitScroll *)
    identified := (player^.character^.inventory.equipment [MCStruct.Weapon1].object.class <> OStruct.Nothing);
    executed   := TRUE;
    
    IF identified THEN
      OSActions.DisplayObjectMessage (player^.windows.frame,player,"Your %WEAPON glows faintly!","%WEAPON",player^.character^.inventory.equipment [MCStruct.Weapon1],FALSE,TRUE);
      EnchantWeaponToHit (player^.character^.inventory.equipment [MCStruct.Weapon1],1,done);
      
      IF done THEN
        EXCL (player^.character^.inventory.equipment [MCStruct.Weapon1].object.attributes,OStruct.Cursed);
        MCSActions.CalculatePlayerBonuses (player^.character);
      ELSE
        MMWindow.DisplayPopupMessage (player^.windows.frame,player,"The enchantment fails.");
      END; (* IF *)
    END; (* IF *)
  END ExecuteEnchantWeaponToHitScroll;
  
  PROCEDURE ExecuteRemoveCurseScroll (player : MStruct.PlayerInfoPointer; VAR identified : BOOLEAN; VAR executed : BOOLEAN);
  BEGIN (* ExecuteRemoveCurseScroll *)
    ExecuteRemoveCurseScrollAction (player,identified);
    
    IF identified THEN
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You feel as if someone is watching over you.");
    END; (* IF *)
    
    executed := TRUE;
  END ExecuteRemoveCurseScroll;
  
  PROCEDURE ExecuteRemoveCurseScrollAction (player : MStruct.PlayerInfoPointer; VAR done : BOOLEAN);
    VAR
        index : MCStruct.PlayerEquipmentMnemonic;
    
  BEGIN (* ExecuteRemoveCurseScrollAction *)
    done := FALSE;
    
    FOR index := MIN (MCStruct.PlayerEquipmentMnemonic) TO MAX (MCStruct.PlayerEquipmentMnemonic) DO
      IF OStruct.Cursed IN player^.character^.inventory.equipment [index].object.attributes THEN
        EXCL (player^.character^.inventory.equipment [index].object.attributes,OStruct.Cursed);
        MCSActions.CalculatePlayerBonuses (player^.character);
        
        done := TRUE;
      END; (* IF *)
    END; (* FOR *)
  END ExecuteRemoveCurseScrollAction;
END MScrl4.
