IMPLEMENTATION MODULE MSScrolling;
  (*/WIN:A*)
  
  IMPORT WDisplay,Windows,WMacro,WScrolling;
  
  IMPORT MFonts,MSWindow;
  IMPORT MStruct;
  
  
  
  PROCEDURE ExecuteHScrollAction (player : MStruct.PlayerInfoPointer; command : CARDINAL; thumb : CARDINAL);
    VAR
        position : INTEGER;
        min,max  : INTEGER;
        width    : INTEGER;
    
    PROCEDURE AverageCharacterWidth (window : Windows.HWND) : INTEGER;
      VAR
          height,width : INTEGER;
      
    BEGIN (* AverageCharacterWidth *)
      MFonts.DetermineWindowCharacterMetrics (window,MSWindow.StatusWindowFont,height,width);
      RETURN (width);
    END AverageCharacterWidth;
    
    PROCEDURE CurrentPageWidth (window : Windows.HWND) : INTEGER;
      VAR
          client : Windows.RECT;
      
    BEGIN (* CurrentPageWidth *)
      WDisplay.GetClientRect (window,client);
      RETURN (client.right);
    END CurrentPageWidth;
    
    PROCEDURE ScrollLineLeft (VAR info : MStruct.StatusWindowInformation; min,max : INTEGER; width : INTEGER; VAR position : INTEGER);
    BEGIN (* ScrollLineLeft *)
      IF position > min THEN
        width := WMacro.MinInteger (width,position - min);
        
        WScrolling.ScrollEntireWindow (info.handle,+width,0);
        DEC (position,width);
      END; (* IF *)
    END ScrollLineLeft;
    
    PROCEDURE ScrollLineRight (VAR info : MStruct.StatusWindowInformation; min,max : INTEGER; width : INTEGER; VAR position : INTEGER);
    BEGIN (* ScrollLineRight *)
      IF position < max THEN
        width := WMacro.MinInteger (width,max - position);
        
        WScrolling.ScrollEntireWindow (info.handle,-width,0);
        INC (position,width);
      END; (* IF *)
    END ScrollLineRight;
    
    PROCEDURE ScrollPageLeft (VAR info : MStruct.StatusWindowInformation; min,max : INTEGER; VAR position : INTEGER);
      VAR
          distance : INTEGER;
      
    BEGIN (* ScrollPageLeft *)
      IF position > min THEN
        distance := WMacro.MinInteger (CurrentPageWidth (info.handle),position - min);
        
        WScrolling.ScrollEntireWindow (info.handle,+distance,0);
        
        position := position - distance;
      END; (* IF *)
    END ScrollPageLeft;
    
    PROCEDURE ScrollPageRight (VAR info : MStruct.StatusWindowInformation; min,max : INTEGER; VAR position : INTEGER);
      VAR
          distance : INTEGER;
      
    BEGIN (* ScrollPageRight *)
      IF position < max THEN
        distance := WMacro.MinInteger (CurrentPageWidth (info.handle),max - position);
        
        WScrolling.ScrollEntireWindow (info.handle,-distance,0);
        
        position := position + distance;
      END; (* IF *)
    END ScrollPageRight;
    
    PROCEDURE SetHorizontalThumbPosition (VAR info : MStruct.StatusWindowInformation; thumb : INTEGER; VAR position : INTEGER);
    BEGIN (* SetHorizontalThumbPosition *)
      WScrolling.ScrollEntireWindow (info.handle,position - thumb,0);
      position := thumb;
    END SetHorizontalThumbPosition;
    
  BEGIN (* ExecuteHScrollAction *)
    WScrolling.GetScrollRange (player^.windows.status.handle,Windows.SB_HORZ,min,max);
    
    position := WScrolling.GetScrollPos (player^.windows.status.handle,Windows.SB_HORZ);
    width    := AverageCharacterWidth (player^.windows.status.handle);
    
    CASE command OF
      Windows.SB_LINERIGHT : ScrollLineRight (player^.windows.status,min,max,width,position); |
      Windows.SB_LINELEFT  : ScrollLineLeft (player^.windows.status,min,max,width,position);  |
      
      Windows.SB_PAGERIGHT : ScrollPageRight (player^.windows.status,min,max,position); |
      Windows.SB_PAGELEFT  : ScrollPageLeft (player^.windows.status,min,max,position);  |
      
      Windows.SB_THUMBPOSITION : SetHorizontalThumbPosition (player^.windows.status,thumb,position);  |
    ELSE
      (* no action *)
    END; (* CASE *)
    
    WScrolling.SetScrollPos (player^.windows.status.handle,Windows.SB_HORZ,position,TRUE,position);
  END ExecuteHScrollAction;
  
  PROCEDURE ExecuteVScrollAction (player : MStruct.PlayerInfoPointer; command : CARDINAL; thumb : CARDINAL);
    VAR
        position : INTEGER;
        min,max  : INTEGER;
        height   : INTEGER;
    
    PROCEDURE AverageCharacterHeight (window : Windows.HWND) : INTEGER;
      VAR
          height,width : INTEGER;
      
    BEGIN (* AverageCharacterHeight *)
      MFonts.DetermineWindowCharacterMetrics (window,MSWindow.StatusWindowFont,height,width);
      RETURN (height);
    END AverageCharacterHeight;
    
    PROCEDURE CurrentPageHeight (window : Windows.HWND) : INTEGER;
      VAR
          client : Windows.RECT;
      
    BEGIN (* CurrentPageHeight *)
      WDisplay.GetClientRect (window,client);
      RETURN (client.bottom);
    END CurrentPageHeight;
    
    PROCEDURE ScrollLineDown (VAR info : MStruct.StatusWindowInformation; min,max : INTEGER; height : INTEGER; VAR position : INTEGER);
    BEGIN (* ScrollLineDown *)
      IF position < max THEN
        height := WMacro.MinInteger (height,max - position);
        
        WScrolling.ScrollEntireWindow (info.handle,0,-height);
        INC (position,height);
      END; (* IF *)
    END ScrollLineDown;
    
    PROCEDURE ScrollLineUp (VAR info : MStruct.StatusWindowInformation; min,max : INTEGER; height : INTEGER; VAR position : INTEGER);
    BEGIN (* ScrollLineUp *)
      IF position > min THEN
        height := WMacro.MinInteger (height,position - min);
        
        WScrolling.ScrollEntireWindow (info.handle,0,+height);
        DEC (position,height);
      END; (* IF *)
    END ScrollLineUp;
    
    PROCEDURE ScrollPageDown (VAR info : MStruct.StatusWindowInformation; min,max : INTEGER; VAR position : INTEGER);
      VAR
          distance : INTEGER;
      
    BEGIN (* ScrollPageDown *)
      IF position < max THEN
        distance := WMacro.MinInteger (CurrentPageHeight (info.handle),max - position);
        
        WScrolling.ScrollEntireWindow (info.handle,0,-distance);
        
        position := position + distance;
      END; (* IF *)
    END ScrollPageDown;
    
    PROCEDURE ScrollPageUp (VAR info : MStruct.StatusWindowInformation; min,max : INTEGER; VAR position : INTEGER);
      VAR
          distance : INTEGER;
      
    BEGIN (* ScrollPageUp *)
      IF position > min THEN
        distance := WMacro.MinInteger (CurrentPageHeight (info.handle),position - min);
        
        WScrolling.ScrollEntireWindow (info.handle,0,+distance);
        
        position := position - distance;
      END; (* IF *)
    END ScrollPageUp;
    
    PROCEDURE SetVerticalThumbPosition (VAR info : MStruct.StatusWindowInformation; thumb : INTEGER; VAR position : INTEGER);
    BEGIN (* SetVerticalThumbPosition *)
      WScrolling.ScrollEntireWindow (info.handle,0,position - thumb);
      position := thumb;
    END SetVerticalThumbPosition;
    
  BEGIN (* ExecuteVScrollAction *)
    WScrolling.GetScrollRange (player^.windows.status.handle,Windows.SB_VERT,min,max);
    
    position := WScrolling.GetScrollPos (player^.windows.status.handle,Windows.SB_VERT);
    height   := AverageCharacterHeight (player^.windows.status.handle);
    
    CASE command OF
      Windows.SB_LINEDOWN : ScrollLineDown (player^.windows.status,min,max,height,position); |
      Windows.SB_LINEUP   : ScrollLineUp (player^.windows.status,min,max,height,position);   |
      
      Windows.SB_PAGEDOWN : ScrollPageDown (player^.windows.status,min,max,position); |
      Windows.SB_PAGEUP   : ScrollPageUp (player^.windows.status,min,max,position);   |
      
      Windows.SB_THUMBPOSITION : SetVerticalThumbPosition (player^.windows.status,thumb,position);  |
    ELSE
      (* no action *)
    END; (* CASE *)
    
    WScrolling.SetScrollPos (player^.windows.status.handle,Windows.SB_VERT,position,TRUE,position);
  END ExecuteVScrollAction;
  
  PROCEDURE InitializeStatusScrollBars (VAR info : MStruct.StatusWindowInformation);
    VAR
        height,width : INTEGER;
    
  BEGIN (* InitializeStatusScrollBars *)
    MFonts.DetermineWindowCharacterMetrics (info.handle,MSWindow.StatusWindowFont,height,width);
    
    WScrolling.SetScrollRange (info.handle,Windows.SB_HORZ,0,MSWindow.ColumnsPerStatusControl * MStruct.MaxStatusDisplayColumns * width - 1,TRUE);
    WScrolling.SetScrollRange (info.handle,Windows.SB_VERT,0,MStruct.MaxStatusDisplayLines * height - 1,TRUE);
  END InitializeStatusScrollBars;
END MSScrolling.
