IMPLEMENTATION MODULE MSWindow;
  (*/WIN:A*)
  
  IMPORT WCreation,WDisplay,Windows,WinStart,WMacro,WMessage,WString;
  
  IMPORT MSColors,MSScrolling;
  IMPORT MStruct;
  IMPORT MFonts;
  
  
  PROCEDURE CreateStatusWindow (parent : Windows.HWND; VAR result : MStruct.StatusWindowInformation);
    PROCEDURE CreateStatusControls (VAR info : MStruct.StatusWindowInformation; font : Windows.HFONT);
      VAR
          width,height : INTEGER;
          column       : INTEGER;
          line         : INTEGER;
      
      PROCEDURE CreateStatusControl (parent : Windows.HWND; x,y : INTEGER; width,height : INTEGER; font : Windows.HFONT; VAR result : Windows.HWND);
        VAR
            shown : BOOLEAN;
            dummy : LONGINT;
        
      BEGIN (* CreateStatusControl *)
        WCreation.CreateWindow ("STATIC","",Windows.SS_LEFTNOWORDWRAP + Windows.WS_CHILD,x,y,width,height,parent,Windows.NULL,WinStart.hInstance,Windows.LP_NULL,result);
        WDisplay.ShowWindow (result,Windows.SW_SHOWNORMAL,shown);
        
        WMessage.SendMessage (result,Windows.WM_SETFONT,font,Windows.FALSE_BOOL,dummy);
      END CreateStatusControl;
      
      PROCEDURE DetermineStatusControlDimensions (parent : Windows.HWND; VAR width,height : INTEGER);
        VAR
            fontHEIGHT,fontWIDTH : INTEGER;
        
      BEGIN (* DetermineStatusControlDimensions *)
        MFonts.DetermineWindowCharacterMetrics (parent,StatusWindowFont,fontHEIGHT,fontWIDTH);
        
        height := fontHEIGHT;
        width  := fontWIDTH * ColumnsPerStatusControl;
      END DetermineStatusControlDimensions;
      
    BEGIN (* CreateStatusControls *)
      DetermineStatusControlDimensions (info.handle,width,height);
      
      FOR line := 0 TO MStruct.MaxStatusDisplayLines - 1 DO
        FOR column := 0 TO MStruct.MaxStatusDisplayColumns - 1 DO
          CreateStatusControl (info.handle,column * width,line * height,width,height,font,info.table [line,column].handle);
        END; (* FOR *)
      END; (* FOR *)
    END CreateStatusControls;
    
    PROCEDURE CreateStatusFrameWindow (parent : Windows.HWND; VAR result : Windows.HWND);
      CONST
            StatusWindowStyle = Windows.WS_CAPTION + Windows.WS_CHILD + Windows.WS_CLIPSIBLINGS + Windows.WS_CLIPCHILDREN + Windows.WS_HSCROLL + Windows.WS_VSCROLL;
      
      VAR
          y,height : INTEGER;
          x,width  : INTEGER;
          shown    : BOOLEAN;
      
    BEGIN (* CreateStatusFrameWindow *)
      DetermineStatusWindowPosition (parent,x,y,width,height);
      
      WCreation.CreateWindow (MStruct.StatusWindowClassName,"Status",StatusWindowStyle,x,y,width,height,parent,Windows.NULL,WinStart.hInstance,Windows.LP_NULL,result);
      WDisplay.ShowWindow (result,Windows.SW_SHOWNORMAL,shown);
    END CreateStatusFrameWindow;
    
  BEGIN (* CreateStatusWindow *)
    CreateStatusFrameWindow (parent,result.handle);
    
    CreateStatusControls (result,MFonts.StockFontHandle (StatusWindowFont));
    MSScrolling.InitializeStatusScrollBars (result);
  END CreateStatusWindow;
  
  PROCEDURE DetermineStatusWindowPosition (window : Windows.HWND; VAR x,y : INTEGER; VAR width,height : INTEGER);
    VAR
        client : Windows.RECT;
    
  BEGIN (* DetermineStatusWindowPosition *)
    WDisplay.GetClientRect (window,client);
    
    height := client.bottom;
    width  := client.right DIV 4;
    x      := 0;
    y      := 0;
  END DetermineStatusWindowPosition;
  
  PROCEDURE ResizeStatusWindow (player : MStruct.PlayerInfoPointer);
    VAR
        y,height : INTEGER;
        x,width  : INTEGER;
        done     : BOOLEAN;
    
  BEGIN (* ResizeStatusWindow *)
    IF player <> NIL THEN
      DetermineStatusWindowPosition (player^.windows.frame,x,y,width,height);
      WDisplay.MoveWindow (player^.windows.status.handle,x,y,width,height,TRUE,done);
    END; (* IF *)
  END ResizeStatusWindow;
  
  PROCEDURE SetMainWindowTitle (window : Windows.HWND; player : MStruct.PlayerInfoPointer);
    VAR
        title : ARRAY [0..127] OF CHAR;
    
  BEGIN (* SetMainWindowTitle *)
    IF player <> NIL THEN
      WString.ReplaceStringParameter ("Moria - %PLAYER","%PLAYER",player^.character^.constants.name,title);
    ELSE
      title := MStruct.MainWindowTitle;
    END; (* IF *)
    
    WDisplay.SetWindowText (window,title);
  END SetMainWindowTitle;
  
  PROCEDURE StatusWindowTranslator ['STATUSWINDOWTRANSLATOR'] (window : Windows.HWND; message : CARDINAL; param1 : Windows.WPARAM; param2 : Windows.LPARAM) : Windows.LRESULT [WINDOWS,LONG];
    VAR
        result : Windows.LRESULT;
    
  BEGIN (* StatusWindowTranslator *)
    result := Windows.FALSE_BOOL;
    
    CASE message OF
      Windows.WM_CTLCOLOR : MSColors.DetermineStatusControlColor (MStruct.CurrentPlayerInformation,WMacro.LOWORD (param2),param1,result); |
      Windows.WM_HSCROLL  : MSScrolling.ExecuteHScrollAction (MStruct.CurrentPlayerInformation,param1,WMacro.LOWORD (param2));            |
      Windows.WM_VSCROLL  : MSScrolling.ExecuteVScrollAction (MStruct.CurrentPlayerInformation,param1,WMacro.LOWORD (param2));            |
    ELSE
      result := WCreation.DefWindowProc (window,message,param1,param2);
    END; (* CASE *)
    
    RETURN (result);
  END StatusWindowTranslator;
END MSWindow.
