IMPLEMENTATION MODULE MTunnel;
  (*/WIN:A*)
  
  IMPORT MDGAlloc,MDPainting,MDWindow,MMWindow,MPAttacks,MPMovement,MPSearch,MRandom;
  IMPORT DStruct,MCStruct,MStruct,OStruct;
  IMPORT WDebugging,WMacro,WString;
  IMPORT DSActions;
  
  
  PROCEDURE ExecuteTunnelAction (player : MStruct.PlayerInfoPointer; direction : CARDINAL);
    VAR
        column : DStruct.DungeonColumnRange;
        line   : DStruct.DungeonLineRange;
    
    PROCEDURE AttemptToTunnelAtLocation (player : MStruct.PlayerInfoPointer; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange; VAR location : DStruct.PhysicalDungeonUnit);
      PROCEDURE TunnelAtLocation (player : MStruct.PlayerInfoPointer; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange; VAR location : DStruct.PhysicalDungeonUnit);
        PROCEDURE ExecuteTunnelingAttack (player : MStruct.PlayerInfoPointer; creature : DStruct.DungeonCreatureRecord);
          VAR
              message : ARRAY [0..127] OF CHAR;
          
        BEGIN (* ExecuteTunnelingAttack *)
          IF creature.visible THEN
            WString.ReplaceStringParameter ("The %CREATURE is in your way!","%CREATURE",player^.creatures^.table [creature.cursor].description,message);
          ELSE
            message := "Something is in your way!";
          END; (* IF *)
          
          MMWindow.DisplayPopupMessage (player^.windows.frame,player,message);
          MPAttacks.AttackCreatureAtLocation (player,creature.line,creature.column);
        END ExecuteTunnelingAttack;
        
        PROCEDURE PlayerTunnelingAbility (player : MStruct.PlayerInfoPointer; weapon : OStruct.InventoryItemRecord) : CARDINAL;
          VAR
              ability : INTEGER;
          
        BEGIN (* PlayerTunnelingAbility *)
          ability := player^.character^.statistics.used [MCStruct.str];
          
          IF OStruct.Tunneling IN weapon.object.attributes THEN
            INC (ability,25 + 50 * weapon.object.miscellaneous);
          ELSE
            ability := (ability + VAL (INTEGER,weapon.object.damage.max) + weapon.object.strikeMOD + weapon.object.damageMOD) DIV 2;
          END; (* IF *)
          
          IF MCStruct.WeaponHeavyFLAG IN player^.character^.variables.status THEN
            ability := WMacro.MaxInteger (ability + 15 * player^.character^.statistics.used [MCStruct.str] - weapon.object.weight,0);
          END; (* IF *)
          
          RETURN (ability);
        END PlayerTunnelingAbility;
        
        PROCEDURE TunnelIntoArtifact (player : MStruct.PlayerInfoPointer; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange; artifact : OStruct.InventoryItemRecord);
          PROCEDURE TunnelIntoRubble (player : MStruct.PlayerInfoPointer; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange);
          BEGIN (* TunnelIntoRubble *)
            IF PlayerTunnelingAbility (player,player^.character^.inventory.equipment [MCStruct.Weapon1]) > MRandom.CardinalRandomRangeValue (1,180) THEN
              MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You have removed the rubble.");
              
              DSActions.DeleteArtifactInformation (player^.dungeon^,line,column);
              
              IF MRandom.CardinalRandomRangeValue (1,10) = 1 THEN
                IF DSActions.LocationHasLight (player^.dungeon^,line,column) THEN
                  MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You have found something!");
                END; (* IF *)
                
                MDGAlloc.PlaceRandomObject (player,player^.dungeon^,line,column,player^.character^.variables.depthCUR,FALSE);
              END; (* IF *)
              
              MDPainting.InvalidateDungeonLocation (player,line,column,FALSE,TRUE);
            ELSE
              MMWindow.AddMessageToList (player,"You dig in the rubble.");
            END; (* IF *)
          END TunnelIntoRubble;
          
          PROCEDURE TunnelIntoSecretDoor (player : MStruct.PlayerInfoPointer);
          BEGIN (* TunnelIntoSecretDoor *)
            MMWindow.AddMessageToList (player,"You tunnel into the granite wall.");
            MPSearch.SearchCurrentLocation (player);
          END TunnelIntoSecretDoor;
          
        BEGIN (* TunnelIntoArtifact *)
          IF artifact.object.class = OStruct.SecretDoor THEN
            TunnelIntoSecretDoor (player);
          ELSIF artifact.object.class = OStruct.Rubble THEN
            TunnelIntoRubble (player,line,column);
          ELSE
            WDebugging.FatalAppExit ("Invalid attempt to tunnel into artifact.");
          END; (* IF *)
        END TunnelIntoArtifact;
        
        PROCEDURE TunnelIntoWall (player : MStruct.PlayerInfoPointer; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange; VAR location : DStruct.PhysicalDungeonUnit);
          PROCEDURE WallTunnelingResistance (elements : DStruct.DungeonElementSet) : CARDINAL;
          BEGIN (* WallTunnelingResistance *)
            IF DStruct.GraniteWall IN elements THEN
              RETURN (MRandom.CardinalRandomRangeValue (1,1200) + 80);
            ELSIF DStruct.QuartzWall IN elements THEN
              RETURN (MRandom.CardinalRandomRangeValue (1,400) + 10);
            ELSIF DStruct.MagmaWall IN elements THEN
              RETURN (MRandom.CardinalRandomRangeValue (1,600) + 10);
            ELSE
              RETURN (MAX (CARDINAL));
            END; (* IF *)
          END WallTunnelingResistance;
          
        BEGIN (* TunnelIntoWall *)
          IF PlayerTunnelingAbility (player,player^.character^.inventory.equipment [MCStruct.Weapon1]) > WallTunnelingResistance (location.elements) THEN
            MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You have finished the tunnel.");
            MeltWallAtLocation (player,line,column,location,TRUE);
          ELSIF DStruct.BoundaryWall IN location.elements THEN
            MMWindow.DisplayPopupMessage (player^.windows.frame,player,"This seems to be permanent rock.");
          ELSIF DStruct.GraniteWall IN location.elements THEN
            MMWindow.AddMessageToList (player,"You tunnel into the granite wall.");
          ELSIF DStruct.QuartzWall IN location.elements THEN
            MMWindow.AddMessageToList (player,"You tunnel into the quartz vein.");
          ELSIF DStruct.MagmaWall IN location.elements THEN
            MMWindow.AddMessageToList (player,"You tunnel into the magma intrusion.");
          ELSE
            WDebugging.FatalAppExit ("Unrecognized wall type in MTunnel.TunnelIntoWall.");
          END; (* IF *)
        END TunnelIntoWall;
        
      BEGIN (* TunnelAtLocation *)
        IF location.creature <> DStruct.InvalidCreatureCursor THEN
          ExecuteTunnelingAttack (player,player^.dungeon^.creatures [location.creature]);
        ELSIF player^.character^.inventory.equipment [MCStruct.Weapon1].object.class = OStruct.Nothing THEN
          MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You dig with your hands, making no progress.");
        ELSIF location.elements * DStruct.DungeonWallElements <> DStruct.EmptyDungeonElementSet THEN
          TunnelIntoWall (player,line,column,location);
        ELSE
          TunnelIntoArtifact (player,line,column,player^.dungeon^.artifacts [location.artifact]);
        END; (* IF *)
      END TunnelAtLocation;
      
    BEGIN (* AttemptToTunnelAtLocation *)
      IF ((location.elements * DStruct.DungeonWallElements <> DStruct.EmptyDungeonElementSet) OR ((location.artifact <> DStruct.InvalidArtifactCursor) AND (player^.dungeon^.artifacts [location.artifact].object.class IN OStruct.ObjectClassSet {OStruct.Rubble,OStruct.SecretDoor}))) THEN
        TunnelAtLocation (player,line,column,location);
        MStruct.CharacterTurnExecuted := TRUE;
      ELSIF location.artifact = DStruct.InvalidArtifactCursor THEN
        MMWindow.DisplayPopupMessage (player^.windows.frame,player,"Tunnel through what?  Empty air?!?");
        MStruct.CharacterTurnExecuted := FALSE;
      ELSE
        MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You can't tunnel through that.");
        MStruct.CharacterTurnExecuted := FALSE;
      END; (* IF *)
    END AttemptToTunnelAtLocation;
    
  BEGIN (* ExecuteTunnelAction *)
    IF player^.character^.variables.counters [MCStruct.ConfusedCOUNT] > 0 THEN
      MPMovement.DetermineConfusedDirection (player,"",direction);
    END; (* IF *)
    
    DSActions.DetermineDungeonLocation (player^.character^.line,player^.character^.column,direction,line,column);
    AttemptToTunnelAtLocation (player,line,column,player^.dungeon^.physical [line]^ [column]);
  END ExecuteTunnelAction;

  PROCEDURE MeltWallAtLocation (player : MStruct.PlayerInfoPointer; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange; VAR location : DStruct.PhysicalDungeonUnit; messages : BOOLEAN);
    CONST
          DungeonFValElements = DStruct.DungeonElementSet {DStruct.NullWall..DStruct.BoundaryWall};
    
    VAR
        done : BOOLEAN;
    
    PROCEDURE DetermineRoomFloorElements (VAR dungeon : DStruct.DungeonInformation; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange; VAR elements : DStruct.DungeonElementSet; VAR done : BOOLEAN);
      VAR
          temp : DStruct.DungeonElementSet;
      
      PROCEDURE AdjacentRoomElements (VAR dungeon : DStruct.DungeonInformation; line : DStruct.DungeonLineRange; column : DStruct.DungeonColumnRange) : DStruct.DungeonElementSet;
        VAR
            testCOLUMN : DStruct.DungeonColumnRange;
            testLINE   : DStruct.DungeonLineRange;
        
      BEGIN (* AdjacentRoomElements *)
        FOR testLINE := line - 1 TO line + 1 DO
          FOR testCOLUMN := column - 1 TO column + 1 DO
            IF dungeon.physical [testLINE]^ [testCOLUMN].elements * DStruct.DungeonElementSet {DStruct.DarkFloor,DStruct.LightFloor} <> DStruct.EmptyDungeonElementSet THEN
              RETURN (dungeon.physical [testLINE]^ [testCOLUMN].elements);
            END; (* IF *)
          END; (* FOR *)
        END; (* FOR *)
        
        RETURN (DStruct.EmptyDungeonElementSet);
      END AdjacentRoomElements;
      
    BEGIN (* DetermineRoomFloorElements *)
      temp := AdjacentRoomElements (dungeon,line,column);
      done := (temp <> DStruct.EmptyDungeonElementSet);
      
      IF done THEN
        elements := elements + (temp * (DungeonFValElements + DStruct.DungeonElementSet {DStruct.plFLAG}));
      END; (* IF *)
    END DetermineRoomFloorElements;
    
  BEGIN (* MeltWallAtLocation *)
    location.elements := location.elements - (DungeonFValElements + DStruct.DungeonElementSet {DStruct.plFLAG});
    
    IF DStruct.lrFLAG IN location.elements THEN
      DetermineRoomFloorElements (player^.dungeon^,line,column,location.elements,done);
    ELSE
      done := FALSE;
    END; (* IF *)
    
    IF NOT (done) THEN
      location.elements := location.elements + DStruct.DungeonElementSet {DStruct.CorridorFloor};
    END; (* IF *)
    
    IF (messages AND (location.artifact <> DStruct.InvalidArtifactCursor)) THEN
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,"You have found something!");
    END; (* IF *)
    
    MDPainting.InvalidateDungeonLocation (player,line,column,FALSE,TRUE);
  END MeltWallAtLocation;
END MTunnel.
