IMPLEMENTATION MODULE OMagic;
  (*/WIN:A*)
  
  IMPORT MRandom,OSActions,OStruct;
  IMPORT WMacro;
  
  
  CONST
        MissileCounterLimit = MAX (INTEGER);
  
  VAR
      CurrentMissileCounter : INTEGER;
  
  
  PROCEDURE DetermineObjectMagic (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL);
    CONST
          MaximumBaseMagicChance = 70;
          BaseMagicChance        = 15;
    
    CONST
          SpecialMagicChanceDivisor = 6;
          CursedMagicChanceDivisor  = 13;
    
    VAR
        chance,cursed,special : CARDINAL;
    
    PROCEDURE DetermineAmuletMagic (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL; chance,special,cursed : CARDINAL);
      PROCEDURE DetermineAmuletMagiModifier (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL);
      BEGIN (* DetermineAmuletMagiModifier *)
        item.object.miscellaneous := 5 * MagicBonus (depth,1,25);
        INC (item.object.value,20 * item.object.miscellaneous);
      END DetermineAmuletMagiModifier;
      
      PROCEDURE DetermineAmuletSearchingModifier (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL; cursed : CARDINAL);
      BEGIN (* DetermineAmuletSearchingModifier *)
        item.object.miscellaneous := 5 * MagicBonus (depth,1,25);
        
        IF ObjectIsMagic (cursed) THEN
          item.object.miscellaneous := -item.object.miscellaneous;
          item.object.value         := -item.object.value;
          
          INCL (item.object.attributes,OStruct.Cursed);
        ELSE
          INC (item.object.value,50 * item.object.miscellaneous);
        END; (* IF *)
      END DetermineAmuletSearchingModifier;
      
      PROCEDURE DetermineAmuletStatisticModifier (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL; cursed : CARDINAL);
      BEGIN (* DetermineAmuletStatisticModifier *)
        IF ObjectIsMagic (cursed) THEN
          item.object.miscellaneous := -MagicBonus (depth,1,20);
          item.object.value         := -item.object.value;
          
          INCL (item.object.attributes,OStruct.Cursed);
        ELSE
          item.object.miscellaneous := MagicBonus (depth,1,10);
          INC (item.object.value,100 * item.object.miscellaneous);
        END; (* IF *)
      END DetermineAmuletStatisticModifier;
      
    BEGIN (* DetermineAmuletMagic *)
      IF item.object.subclass < 2 THEN
        DetermineAmuletStatisticModifier (item,depth,cursed);
      ELSIF item.object.subclass = 2 THEN
        DetermineAmuletSearchingModifier (item,depth,cursed);
      ELSIF item.object.subclass = 8 THEN
        DetermineAmuletMagiModifier (item,depth);
      END; (* IF *)
    END DetermineAmuletMagic;
    
    PROCEDURE DetermineChestMagic (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL; chance,special,cursed : CARDINAL);
      PROCEDURE SetChestAttributes (VAR item : OStruct.InventoryItemRecord; name : OStruct.SpecialNameMnemonic; attributes : OStruct.ObjectAttributeFlags);
      BEGIN (* SetChestAttributes *)
        OSActions.AddToObjectAttributeFlags (item.object.attributes,attributes);
        item.special := name;
      END SetChestAttributes;
      
    BEGIN (* DetermineChestMagic *)
      CASE MRandom.CardinalRandomRangeValue (1,depth + 4) OF
        1        : SetChestAttributes (item,OStruct.NameEmpty,OStruct.ObjectAttributeFlags {});                                                     |
        2        : SetChestAttributes (item,OStruct.NameLocked,OStruct.ObjectAttributeFlags {OStruct.ChestLocked});                                 |
        3,4      : SetChestAttributes (item,OStruct.NamePoisonNeedle,OStruct.ObjectAttributeFlags {OStruct.ChestTrapped,OStruct.ChestLoseStrength,OStruct.ChestLocked}); |
        5,6      : SetChestAttributes (item,OStruct.NamePoisonNeedle,OStruct.ObjectAttributeFlags {OStruct.ChestTrapped,OStruct.ChestPoison,OStruct.ChestLocked});       |
        7,8,9    : SetChestAttributes (item,OStruct.NameGasTrap,OStruct.ObjectAttributeFlags {OStruct.ChestTrapped,OStruct.ChestParalyse,OStruct.ChestLocked});      |
        10,11    : SetChestAttributes (item,OStruct.NameExplosionDevice,OStruct.ObjectAttributeFlags {OStruct.ChestTrapped,OStruct.ChestExplode,OStruct.ChestLocked}); |
        12,13,14 : SetChestAttributes (item,OStruct.NameSummoningRunes,OStruct.ObjectAttributeFlags {OStruct.ChestTrapped,OStruct.ChestSummon,OStruct.ChestLocked});     |
        15,16,17 : SetChestAttributes (item,OStruct.NameMultipleTraps,OStruct.ObjectAttributeFlags {OStruct.ChestTrapped,OStruct.ChestPoison,OStruct.ChestParalyse,OStruct.ChestLoseStrength,OStruct.ChestLocked}); |
      ELSE
        SetChestAttributes (item,OStruct.NameMultipleTraps,OStruct.ObjectAttributeFlags {OStruct.ChestTrapped,OStruct.ChestSummon,OStruct.ChestExplode,OStruct.ChestLocked});
      END; (* CASE *)
    END DetermineChestMagic;
    
    PROCEDURE DetermineDiggingToolMagic (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL; chance,special,cursed : CARDINAL);
    BEGIN (* DetermineDiggingToolMagic *)
      INCL (item.identification,OStruct.IdentShowHitDam);
      
      IF ObjectIsMagic (chance) THEN
        IF MRandom.CardinalRandomRangeValue (1,3) < 3 THEN
          INC (item.object.miscellaneous,MagicBonus (depth,0,25));
        ELSE
          INCL (item.object.attributes,OStruct.Cursed);
          
          item.object.miscellaneous := -MagicBonus (depth,1,30);
          item.object.value         := 0;
        END; (* IF *)
      END; (* IF *)
    END DetermineDiggingToolMagic;
    
    PROCEDURE DetermineFootArmorMagic (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL; chance,special,cursed : CARDINAL);
      PROCEDURE DetermineFootArmorCurse (VAR item : OStruct.InventoryItemRecord);
        PROCEDURE AddFootArmorGreatMass (VAR item : OStruct.InventoryItemRecord);
        BEGIN (* AddFootArmorGreatMass *)
          item.object.weight := 5 * item.object.weight;
          item.special       := OStruct.NameGreatMass;
        END AddFootArmorGreatMass;
        
        PROCEDURE AddFootArmorNoise (VAR item : OStruct.InventoryItemRecord);
        BEGIN (* AddFootArmorNoise *)
          INCL (item.object.attributes,OStruct.Aggravation);
          item.special := OStruct.NameNoise;
        END AddFootArmorNoise;
        
        PROCEDURE AddFootArmorSlowness (VAR item : OStruct.InventoryItemRecord);
        BEGIN (* AddFootArmorSlowness *)
          INCL (item.identification,OStruct.IdentShowP1);
          INCL (item.object.attributes,OStruct.Speed);
          
          item.object.miscellaneous := -1;
          item.special              := OStruct.NameSlowness;
        END AddFootArmorSlowness;
        
      BEGIN (* DetermineFootArmorCurse *)
        CASE MRandom.CardinalRandomRangeValue (1,3) OF
          1 : AddFootArmorSlowness (item);  |
          2 : AddFootArmorNoise (item);     |
          3 : AddFootArmorGreatMass (item); |
        END; (* CASE *)
      END DetermineFootArmorCurse;
      
      PROCEDURE DetermineFootArmorSpecial (VAR item : OStruct.InventoryItemRecord);
        PROCEDURE AddFootArmorFeatherFalling (VAR item : OStruct.InventoryItemRecord);
        BEGIN (* AddFootArmorFeatherFalling *)
          INCL (item.object.attributes,OStruct.FeatherFalling);
          INC (item.object.value,250);
          
          item.special := OStruct.NameSlowDescent;
        END AddFootArmorFeatherFalling;
        
        PROCEDURE AddFootArmorSpeed (VAR item : OStruct.InventoryItemRecord);
        BEGIN (* AddFootArmorSpeed *)
          INCL (item.identification,OStruct.IdentShowP1);
          INCL (item.object.attributes,OStruct.Speed);
          
          item.object.miscellaneous := 1;
          item.special              := OStruct.NameSpeed;
          
          INC (item.object.value,5000);
        END AddFootArmorSpeed;
        
        PROCEDURE AddFootArmorStealth (VAR item : OStruct.InventoryItemRecord);
        BEGIN (* AddFootArmorStealth *)
          INCL (item.identification,OStruct.IdentShowP1);
          INCL (item.object.attributes,OStruct.Stealth);
          
          item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,3);
          item.special              := OStruct.NameStealth;
          
          INC (item.object.value,500);
        END AddFootArmorStealth;
        
      BEGIN (* DetermineFootArmorSpecial *)
        CASE MRandom.CardinalRandomRangeValue (1,12) OF
          6..12 : AddFootArmorFeatherFalling (item); |
          2..5  : AddFootArmorStealth (item);        |
          1     : AddFootArmorSpeed (item);          |
        END; (* CASE *)
      END DetermineFootArmorSpecial;
      
    BEGIN (* DetermineFootArmorMagic *)
      IF ObjectIsMagic (chance) THEN
        INC (item.object.armorMOD,MagicBonus (depth,1,20));
        
        IF ObjectIsMagic (special) THEN
          DetermineFootArmorSpecial (item);
        END; (* IF *)
      ELSIF ObjectIsMagic (cursed) THEN
        DEC (item.object.armorMOD,MagicBonus (depth,2,45));
        item.object.value := 0;
        
        DetermineFootArmorCurse (item);
        
        INCL (item.object.attributes,OStruct.Cursed);
      END; (* IF *)
    END DetermineFootArmorMagic;
    
    PROCEDURE DetermineGeneralArmorMagic (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL; chance,special,cursed : CARDINAL);
      PROCEDURE DetermineGeneralArmorCurse (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL; special : CARDINAL);
      BEGIN (* DetermineGeneralArmorCurse *)
        DEC (item.object.armorMOD,MagicBonus (depth,1,40));
        INCL (item.object.attributes,OStruct.Cursed);
        
        item.object.value := 0;
      END DetermineGeneralArmorCurse;
      
      PROCEDURE DetermineGeneralArmorEnchantment (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL; special : CARDINAL);
        PROCEDURE DetermineGeneralArmorSpecial (VAR item : OStruct.InventoryItemRecord);
          PROCEDURE AddGeneralArmorResistAcid (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddGeneralArmorResistAcid *)
            INCL (item.object.attributes,OStruct.ResistAcid);
            INC (item.object.value,1000);
            
            item.special := OStruct.NameResistAcid;
          END AddGeneralArmorResistAcid;
          
          PROCEDURE AddGeneralArmorResistance (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddGeneralArmorResistance *)
            OSActions.AddToObjectAttributeFlags (item.object.attributes,OStruct.ObjectAttributeFlags {OStruct.ResistLightning,OStruct.ResistCold,OStruct.ResistAcid,OStruct.ResistFire});
            
            item.special := OStruct.NameResistance;
            
            INC (item.object.armorMOD,5);
            INC (item.object.value,2500);
          END AddGeneralArmorResistance;
          
          PROCEDURE AddGeneralArmorResistCold (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddGeneralArmorResistCold *)
            INCL (item.object.attributes,OStruct.ResistCold);
            INC (item.object.value,600);
            
            item.special := OStruct.NameResistCold;
          END AddGeneralArmorResistCold;
          
          PROCEDURE AddGeneralAmrorResistFire (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddGeneralAmrorResistFire *)
            INCL (item.object.attributes,OStruct.ResistFire);
            INC (item.object.value,600);
            
            item.special := OStruct.NameResistFire;
          END AddGeneralAmrorResistFire;
          
          PROCEDURE AddGeneralArmorResistLightning (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddGeneralArmorResistLightning *)
            INCL (item.object.attributes,OStruct.ResistLightning);
            INC (item.object.value,500);
            
            item.special := OStruct.NameResistLightning;
          END AddGeneralArmorResistLightning;
          
        BEGIN (* DetermineGeneralArmorSpecial *)
          CASE MRandom.CardinalRandomRangeValue (1,9) OF
            1     : AddGeneralArmorResistance (item);      |
            2     : AddGeneralArmorResistAcid (item);      |
            3,4   : AddGeneralAmrorResistFire (item);      |
            5,6   : AddGeneralArmorResistCold (item);      |
            7,8,9 : AddGeneralArmorResistLightning (item); |
          END; (* CASE *)
        END DetermineGeneralArmorSpecial;
        
      BEGIN (* DetermineGeneralArmorEnchantment *)
        INC (item.object.armorMOD,MagicBonus (depth,1,30));
        
        IF ObjectIsMagic (special) THEN
          DetermineGeneralArmorSpecial (item);
        END; (* IF *)
      END DetermineGeneralArmorEnchantment;
      
    BEGIN (* DetermineGeneralArmorMagic *)
      IF ObjectIsMagic (chance) THEN
        DetermineGeneralArmorEnchantment (item,depth,special);
      ELSIF ObjectIsMagic (cursed) THEN
        DetermineGeneralArmorCurse (item,depth,special);
      END; (* IF *)
    END DetermineGeneralArmorMagic;
    
    PROCEDURE DetermineGeneralWeaponMagic (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL; chance,special,cursed : CARDINAL);
      PROCEDURE DetermineGeneralWeaponCurse (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL);
      BEGIN (* DetermineGeneralWeaponCurse *)
        DEC (item.object.damageMOD,MagicBonus (item.object.damage.max * depth DIV 10,1,11 * item.object.damage.max DIV 2));
        DEC (item.object.strikeMOD,MagicBonus (depth,1,55));
        
        item.object.value := 0;
        
        INCL (item.object.attributes,OStruct.Cursed);
      END DetermineGeneralWeaponCurse;
      
      PROCEDURE DetermineGeneralWeaponEnchantment (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL; special : CARDINAL);
        PROCEDURE DetermineGeneralWeaponSpecial (VAR item : OStruct.InventoryItemRecord);
          PROCEDURE AddGeneralWeaponDefender (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddGeneralWeaponDefender *)
            OSActions.AddToObjectAttributeFlags (item.object.attributes,OStruct.ObjectAttributeFlags {OStruct.FeatherFalling,OStruct.ResistLightning,OStruct.SeeInvisible,OStruct.FreeAction,OStruct.ResistCold,OStruct.ResistAcid,OStruct.ResistFire,OStruct.Regeneration,OStruct.Stealth});
            
            item.special := OStruct.NameDefender;
            item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,3);
            
            INC (item.object.armorMOD,MRandom.CardinalRandomRangeValue (6,10));
            INC (item.object.value,7500 + 500 * item.object.miscellaneous);
            INC (item.object.strikeMOD,3);
            INC (item.object.damageMOD,3);
          END AddGeneralWeaponDefender;
          
          PROCEDURE AddGeneralWeaponFlameTongue (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddGeneralWeaponFlameTongue *)
            INCL (item.object.attributes,OStruct.FlameTongue);
            
            INC (item.object.strikeMOD);
            INC (item.object.damageMOD,3);
            INC (item.object.value,2000);
            
            item.special := OStruct.NameFlameTongue;
          END AddGeneralWeaponFlameTongue;
          
          PROCEDURE AddGeneralWeaponFrostBrand (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddGeneralWeaponFrostBrand *)
            INCL (item.object.attributes,OStruct.FrostBrand);
            
            INC (item.object.damageMOD);
            INC (item.object.strikeMOD);
            INC (item.object.value,1200);
            
            item.special := OStruct.NameFrostBrand;
          END AddGeneralWeaponFrostBrand;
          
          PROCEDURE AddGeneralWeaponHolyAvenger (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddGeneralWeaponHolyAvenger *)
            OSActions.AddToObjectAttributeFlags (item.object.attributes,OStruct.ObjectAttributeFlags {OStruct.SeeInvisible,OStruct.SustainStat,OStruct.SlayUndead,OStruct.SlayEvil,OStruct.Strength});
            
            item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,4);
            item.special              := OStruct.NameHolyAvenger;
            
            INC (item.object.armorMOD,MRandom.CardinalRandomRangeValue (1,4));
            INC (item.object.value,10000 + 500 * item.object.miscellaneous);
            INC (item.object.damageMOD,5);
            INC (item.object.strikeMOD,5);
          END AddGeneralWeaponHolyAvenger;
          
          PROCEDURE AddGeneralWeaponSlayAnimal (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddGeneralWeaponSlayAnimal *)
            INCL (item.object.attributes,OStruct.SlayAnimal);
            
            INC (item.object.damageMOD,3);
            INC (item.object.strikeMOD,3);
            INC (item.object.value,5000);
            
            item.special := OStruct.NameSlayAnimal;
          END AddGeneralWeaponSlayAnimal;
          
          PROCEDURE AddGeneralWeaponSlayDragon (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddGeneralWeaponSlayDragon *)
            INCL (item.object.attributes,OStruct.SlayDragon);
            
            INC (item.object.strikeMOD,3);
            INC (item.object.damageMOD,3);
            INC (item.object.value,4000);
            
            item.special := OStruct.NameSlayDragon;
          END AddGeneralWeaponSlayDragon;
          
          PROCEDURE AddGeneralWeaponSlayEvil (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddGeneralWeaponSlayEvil *)
            INCL (item.object.attributes,OStruct.SlayEvil);
            
            INC (item.object.damageMOD,3);
            INC (item.object.strikeMOD,3);
            INC (item.object.value,4000);
            
            item.special := OStruct.NameSlayEvil;
          END AddGeneralWeaponSlayEvil;
          
          PROCEDURE AddGeneralWeaponSlayUndead (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddGeneralWeaponSlayUndead *)
            OSActions.AddToObjectAttributeFlags (item.object.attributes,OStruct.ObjectAttributeFlags {OStruct.SeeInvisible,OStruct.SlayUndead});
            
            INC (item.object.strikeMOD,2);
            INC (item.object.damageMOD,2);
            INC (item.object.value,3000);
            
            item.special := OStruct.NameSlayUndead;
          END AddGeneralWeaponSlayUndead;
          
        BEGIN (* DetermineGeneralWeaponSpecial *)
          CASE MRandom.CardinalRandomRangeValue (1,16) OF
            1        : AddGeneralWeaponHolyAvenger (item); |
            2        : AddGeneralWeaponDefender (item);    |
            3,4      : AddGeneralWeaponSlayAnimal (item);  |
            5,6      : AddGeneralWeaponSlayDragon (item);  |
            7,8      : AddGeneralWeaponSlayEvil (item);    |
            9,10     : AddGeneralWeaponSlayUndead (item);  |
            11,12,13 : AddGeneralWeaponFlameTongue (item); |
            14,15,16 : AddGeneralWeaponFrostBrand (item);  |
          END; (* CASE *)
        END DetermineGeneralWeaponSpecial;
        
      BEGIN (* DetermineGeneralWeaponEnchantment *)
        INC (item.object.damageMOD,MagicBonus (item.object.damage.max * depth DIV 10,0,4 * item.object.damage.max));
        INC (item.object.strikeMOD,MagicBonus (depth,0,40));
        
        IF ObjectIsMagic (3 * special DIV 2) THEN
          DetermineGeneralWeaponSpecial (item);
        END; (* IF *)
      END DetermineGeneralWeaponEnchantment;
      
    BEGIN (* DetermineGeneralWeaponMagic *)
      INCL (item.identification,OStruct.IdentShowHitDam);
      
      IF ObjectIsMagic (chance) THEN
        DetermineGeneralWeaponEnchantment (item,depth,special);
      ELSIF ObjectIsMagic (cursed) THEN
        DetermineGeneralWeaponCurse (item,depth);
      END; (* IF *)
    END DetermineGeneralWeaponMagic;
    
    PROCEDURE DetermineLightMagic (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL; chance,special,cursed : CARDINAL);
    BEGIN (* DetermineLightMagic *)
      IF ODD (item.object.subclass) THEN
        item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,item.object.miscellaneous);
        item.object.subclass      := item.object.subclass - 1;
      END; (* IF *)
    END DetermineLightMagic;
    
    PROCEDURE DetermineHandArmorMagic (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL; chance,special,cursed : CARDINAL);
      PROCEDURE DetermineHandArmorCurse (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL; special : CARDINAL);
        PROCEDURE DetermineHandArmorSpecialCurse (VAR item : OStruct.InventoryItemRecord);
          PROCEDURE AddHandArmorClumsiness (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddHandArmorClumsiness *)
            INCL (item.object.attributes,OStruct.Dexterity);
            item.special := OStruct.NameClumsiness;
          END AddHandArmorClumsiness;
          
          PROCEDURE AddHandArmorWeakness (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddHandArmorWeakness *)
            INCL (item.object.attributes,OStruct.Strength);
            item.special := OStruct.NameWeakness;
          END AddHandArmorWeakness;
          
        BEGIN (* DetermineHandArmorSpecialCurse *)
          INCL (item.identification,OStruct.IdentShowP1);
          
          CASE MRandom.CardinalRandomRangeValue (1,2) OF
            1 : AddHandArmorClumsiness (item); |
            2 : AddHandArmorWeakness (item);   |
          END; (* CASE *)
          
          item.object.miscellaneous := -MagicBonus (depth,1,10);
        END DetermineHandArmorSpecialCurse;
        
      BEGIN (* DetermineHandArmorCurse *)
        DEC (item.object.armorMOD,MagicBonus (depth,1,40));
        item.object.value := 0;
        
        IF ObjectIsMagic (special) THEN
          DetermineHandArmorSpecialCurse (item);
        END; (* IF *)
        
        INCL (item.object.attributes,OStruct.Cursed);
      END DetermineHandArmorCurse;
      
      PROCEDURE DetermineHandArmorEnchantment (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL; special : CARDINAL);
        PROCEDURE DetermineHandArmorSpecial (VAR item : OStruct.InventoryItemRecord);
          PROCEDURE AddHandArmorFreeAction (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddHandArmorFreeAction *)
            INCL (item.object.attributes,OStruct.FreeAction);
            INC (item.object.value,1000);
            
            item.special := OStruct.NameFreeAction;
          END AddHandArmorFreeAction;
          
          PROCEDURE AddHandArmorSlaying (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddHandArmorSlaying *)
            INCL (item.identification,OStruct.IdentShowHitDam);
            
            INC (item.object.strikeMOD,MRandom.CardinalRandomRangeValue (2,4));
            INC (item.object.damageMOD,MRandom.CardinalRandomRangeValue (2,4));
            INC (item.object.value,250 * (item.object.damageMOD + item.object.strikeMOD));
            
            item.special := OStruct.NameSlaying;
          END AddHandArmorSlaying;
          
        BEGIN (* DetermineHandArmorSpecial *)
          CASE MRandom.CardinalRandomRangeValue (1,2) OF
            1 : AddHandArmorFreeAction (item); |
            2 : AddHandArmorSlaying (item);    |
          END; (* CASE *)
        END DetermineHandArmorSpecial;
        
      BEGIN (* DetermineHandArmorEnchantment *)
        INC (item.object.armorMOD,MagicBonus (depth,1,20));
        
        IF ObjectIsMagic (special) THEN
          DetermineHandArmorSpecial (item);
        END; (* IF *)
      END DetermineHandArmorEnchantment;
      
    BEGIN (* DetermineHandArmorMagic *)
      IF ObjectIsMagic (chance) THEN
        DetermineHandArmorEnchantment (item,depth,special);
      ELSIF ObjectIsMagic (cursed) THEN
        DetermineHandArmorCurse (item,depth,special);
      END; (* IF *)
    END DetermineHandArmorMagic;
    
    PROCEDURE DetermineHeadArmorMagic (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL; chance,special,cursed : CARDINAL);
      PROCEDURE DetermineHeadArmorCurse (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL; special : CARDINAL);
        PROCEDURE DetermineHeadArmorSpecialCurse (VAR item : OStruct.InventoryItemRecord);
          PROCEDURE AddHeadArmorBlindness (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddHeadArmorBlindness *)
            INCL (item.object.attributes,OStruct.Blindness);
            item.special := OStruct.NameBlindness;
          END AddHeadArmorBlindness;
          
          PROCEDURE AddHeadArmorDullness (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddHeadArmorDullness *)
            INCL (item.identification,OStruct.IdentShowP1);
            INCL (item.object.attributes,OStruct.Wisdom);
            
            item.object.miscellaneous := -MRandom.IntegerRandomRangeValue (1,5);
            item.special              := OStruct.NameDullness;
          END AddHeadArmorDullness;
          
          PROCEDURE AddHeadArmorStipidity (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddHeadArmorStipidity *)
            INCL (item.object.attributes,OStruct.Intelligence);
            INCL (item.identification,OStruct.IdentShowP1);
            
            item.object.miscellaneous := -MRandom.IntegerRandomRangeValue (1,5);
            item.special              := OStruct.NameStupidity;
          END AddHeadArmorStipidity;
          
          PROCEDURE AddHeadArmorTeleportation (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddHeadArmorTeleportation *)
            INCL (item.object.attributes,OStruct.Teleportation);
            item.special := OStruct.NameTeleportation;
          END AddHeadArmorTeleportation;
          
          PROCEDURE AddHeadArmorTimidness (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddHeadArmorTimidness *)
            INCL (item.object.attributes,OStruct.Timidity);
            item.special := OStruct.NameTimidity;
          END AddHeadArmorTimidness;
          
          PROCEDURE AddHeadArmorUgliness (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddHeadArmorUgliness *)
            INCL (item.identification,OStruct.IdentShowP1);
            INCL (item.object.attributes,OStruct.Charisma);
            
            item.object.miscellaneous := -MRandom.IntegerRandomRangeValue (1,5);
            item.special              := OStruct.NameUgliness;
          END AddHeadArmorUgliness;
          
          PROCEDURE AddHeadArmorWeakness (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddHeadArmorWeakness *)
            INCL (item.object.attributes,OStruct.Strength);
            INCL (item.identification,OStruct.IdentShowP1);
            
            item.object.miscellaneous := -MRandom.IntegerRandomRangeValue (1,5);
            item.special              := OStruct.NameWeakness;
          END AddHeadArmorWeakness;
          
        BEGIN (* DetermineHeadArmorSpecialCurse *)
          CASE MRandom.CardinalRandomRangeValue (1,7) OF
            1 : AddHeadArmorStipidity (item);     |
            2 : AddHeadArmorDullness (item);      |
            3 : AddHeadArmorBlindness (item);     |
            4 : AddHeadArmorTimidness (item);     |
            5 : AddHeadArmorWeakness (item);      |
            6 : AddHeadArmorTeleportation (item); |
            7 : AddHeadArmorUgliness (item);      |
          END; (* CASE *)
        END DetermineHeadArmorSpecialCurse;
        
      BEGIN (* DetermineHeadArmorCurse *)
        INCL (item.object.attributes,OStruct.Cursed);
        DEC (item.object.armorMOD,MagicBonus (depth,1,45));
        
        IF ObjectIsMagic (special) THEN
          DetermineHeadArmorSpecialCurse (item);
        END; (* IF *)
        
        item.object.value := 0;
      END DetermineHeadArmorCurse;
      
      PROCEDURE DetermineHeadArmorEnchantment (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL; special : CARDINAL);
        PROCEDURE DetermineCrownSpecial (VAR item : OStruct.InventoryItemRecord);
          PROCEDURE AddCrownBeauty (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddCrownBeauty *)
            INCL (item.object.attributes,OStruct.Charisma);
            INCL (item.identification,OStruct.IdentShowP1);
            INC (item.object.value,750);
            
            item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,3);
            item.special              := OStruct.NameBeauty;
          END AddCrownBeauty;
          
          PROCEDURE AddCrownLordliness (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddCrownLordliness *)
            OSActions.AddToObjectAttributeFlags (item.object.attributes,OStruct.ObjectAttributeFlags {OStruct.Charisma,OStruct.Wisdom});
            
            item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,3);
            item.special              := OStruct.NameLordliness;
            
            INCL (item.identification,OStruct.IdentShowP1);
            INC (item.object.value,1000 + 500 * item.object.miscellaneous);
          END AddCrownLordliness;
          
          PROCEDURE AddCrownMagi (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddCrownMagi *)
            OSActions.AddToObjectAttributeFlags (item.object.attributes,OStruct.ObjectAttributeFlags {OStruct.ResistLightning,
            OStruct.ResistAcid,OStruct.ResistCold,OStruct.ResistFire,OStruct.Intelligence});
            
            item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,3);
            item.special              := OStruct.NameMagi;
            
            INCL (item.identification,OStruct.IdentShowP1);
            INC (item.object.value,3000 + 500 * item.object.miscellaneous);
          END AddCrownMagi;
          
          PROCEDURE AddCrownMight (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddCrownMight *)
            OSActions.AddToObjectAttributeFlags (item.object.attributes,OStruct.ObjectAttributeFlags {OStruct.FreeAction,OStruct.Constitution,OStruct.Dexterity,OStruct.Strength});
            
            item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,3);
            item.special              := OStruct.NameMight;
            
            INCL (item.identification,OStruct.IdentShowP1);
            INC (item.object.value,1000 + 500 * item.object.miscellaneous);
          END AddCrownMight;
          
          PROCEDURE AddCrownRegeneration (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddCrownRegeneration *)
            INCL (item.object.attributes,OStruct.Regeneration);
            INC (item.object.value,1500);
            
            item.special := OStruct.NameRegeneration;
          END AddCrownRegeneration;
          
          PROCEDURE AddCrownSeeing (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddCrownSeeing *)
            OSActions.AddToObjectAttributeFlags (item.object.attributes,OStruct.ObjectAttributeFlags {OStruct.SeeInvisible,OStruct.Search});
            
            item.object.miscellaneous := 5 * MRandom.CardinalRandomRangeValue (2,5);
            item.special              := OStruct.NameSeeing;
            
            INCL (item.identification,OStruct.IdentShowP1);
            INC (item.object.value,1000 + 100 * item.object.miscellaneous);
          END AddCrownSeeing;
          
        BEGIN (* DetermineCrownSpecial *)
          CASE MRandom.CardinalRandomRangeValue (1,6) OF
            1 : AddCrownMight (item);        |
            2 : AddCrownLordliness (item);   |
            3 : AddCrownMagi (item);         |
            4 : AddCrownBeauty (item);       |
            5 : AddCrownSeeing (item);       |
            6 : AddCrownRegeneration (item); |
          END; (* CASE *)
        END DetermineCrownSpecial;
        
        PROCEDURE DetermineHelmSpecial (VAR item : OStruct.InventoryItemRecord);
          PROCEDURE AddHelmInfraVision (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddHelmInfraVision *)
            INCL (item.object.attributes,OStruct.InfraVision);
            
            item.object.miscellaneous := MRandom.CardinalRandomRangeValue (2,5);
            item.special              := OStruct.NameInfraVision;
            
            INC (item.object.value,250 * item.object.miscellaneous);
          END AddHelmInfraVision;
          
          PROCEDURE AddHelmIntelligence (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddHelmIntelligence *)
            INCL (item.object.attributes,OStruct.Intelligence);
            
            item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,2);
            item.special              := OStruct.NameIntelligence;
            
            INC (item.object.value,500 * item.object.miscellaneous);
          END AddHelmIntelligence;
          
          PROCEDURE AddHelmWisdom (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddHelmWisdom *)
            INCL (item.object.attributes,OStruct.Wisdom);
            
            item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,2);
            item.special              := OStruct.NameWisdom;
            
            INC (item.object.value,500 * item.object.miscellaneous);
          END AddHelmWisdom;
          
        BEGIN (* DetermineHelmSpecial *)
          INCL (item.identification,OStruct.IdentShowP1);
          
          CASE MRandom.CardinalRandomRangeValue (1,3) OF
            1 : AddHelmIntelligence (item); |
            2 : AddHelmInfraVision (item);  |
            3 : AddHelmWisdom (item);       |
          END; (* CASE *)
        END DetermineHelmSpecial;
        
      BEGIN (* DetermineHeadArmorEnchantment *)
        INC (item.object.armorMOD,MagicBonus (depth,1,20));
        
        IF ObjectIsMagic (special) THEN
          IF item.object.subclass < 6 THEN
            DetermineHelmSpecial (item);
          ELSE
            DetermineCrownSpecial (item);
          END; (* IF *)
        END; (* IF *)
      END DetermineHeadArmorEnchantment;
      
    BEGIN (* DetermineHeadArmorMagic *)
      IF ((item.object.subclass >= 6) AND (item.object.subclass <= 8)) THEN
        special := 2 * special;
        chance  := chance + VAL (CARDINAL,item.object.value DIV 100);
      END; (* IF *)
      
      IF ObjectIsMagic (chance) THEN
        DetermineHeadArmorEnchantment (item,depth,special);
      ELSIF ObjectIsMagic (cursed) THEN
        DetermineHeadArmorCurse (item,depth,special);
      END; (* IF *)
    END DetermineHeadArmorMagic;
    
    PROCEDURE DetermineMissileMagic (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL; chance,special,cursed : CARDINAL);
      PROCEDURE DetermineMissileCurse (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL);
      BEGIN (* DetermineMissileCurse *)
        DEC (item.object.strikeMOD,MagicBonus (depth,5,55));
        DEC (item.object.damageMOD,MagicBonus (depth,5,55));
        
        INCL (item.object.attributes,OStruct.Cursed);
        
        item.object.value := 0;
      END DetermineMissileCurse;
      
      PROCEDURE DetermineMissileEnchantment (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL; special : CARDINAL);
        PROCEDURE DetermineMissileSpecial (VAR item : OStruct.InventoryItemRecord);
          PROCEDURE AddMissileSlayEvil (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddMissileSlayEvil *)
            INCL (item.object.attributes,OStruct.SlayEvil);
            
            INC (item.object.damageMOD,3);
            INC (item.object.strikeMOD,3);
            INC (item.object.value,25);
            
            item.special := OStruct.NameSlayEvil;
          END AddMissileSlayEvil;
          
          PROCEDURE AddMissileFlameTongue (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddMissileFlameTongue *)
            INCL (item.object.attributes,OStruct.FlameTongue);
            
            INC (item.object.strikeMOD,2);
            INC (item.object.damageMOD,4);
            INC (item.object.value,25);
            
            item.special := OStruct.NameFire;
          END AddMissileFlameTongue;
          
          PROCEDURE AddMissileSlayAnimal (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddMissileSlayAnimal *)
            INCL (item.object.attributes,OStruct.SlayAnimal);
            
            INC (item.object.damageMOD,2);
            INC (item.object.strikeMOD,2);
            INC (item.object.value,30);
            
            item.special := OStruct.NameSlayAnimal;
          END AddMissileSlayAnimal;
          
          PROCEDURE AddMissileSlayDragon (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddMissileSlayDragon *)
            INCL (item.object.attributes,OStruct.SlayDragon);
            
            INC (item.object.damageMOD,3);
            INC (item.object.strikeMOD,3);
            INC (item.object.value,35);
            
            item.special := OStruct.NameDragonSlaying;
          END AddMissileSlayDragon;
          
          PROCEDURE AddMissileSlaying (VAR item : OStruct.InventoryItemRecord);
          BEGIN (* AddMissileSlaying *)
            INC (item.object.strikeMOD,5);
            INC (item.object.damageMOD,5);
            INC (item.object.value,20);
            
            item.special := OStruct.NameSlaying;
          END AddMissileSlaying;
          
        BEGIN (* DetermineMissileSpecial *)
          CASE MRandom.CardinalRandomRangeValue (1,10) OF
            1,2,3 : AddMissileSlaying (item);     |
            4,5   : AddMissileFlameTongue (item); |
            6,7   : AddMissileSlayEvil (item);    |
            8,9   : AddMissileSlayAnimal (item);  |
            10    : AddMissileSlayDragon (item);  |
          END; (* CASE *)
        END DetermineMissileSpecial;
        
      BEGIN (* DetermineMissileEnchantment *)
        INC (item.object.strikeMOD,MagicBonus (depth,1,35));
        INC (item.object.damageMOD,MagicBonus (depth,1,35));
        
        IF ObjectIsMagic (3 * special DIV 2) THEN
          DetermineMissileSpecial (item);
        END; (* IF *)
      END DetermineMissileEnchantment;
      
      PROCEDURE DetermineMissileQuantity (VAR item : OStruct.InventoryItemRecord);
      BEGIN (* DetermineMissileQuantity *)
        item.object.miscellaneous := CurrentMissileCounter;
        item.object.quantity      := MRandom.CardinalRandomRangeValue (7,42);
        
        CurrentMissileCounter := (CurrentMissileCounter + 1) MOD MissileCounterLimit;
      END DetermineMissileQuantity;
      
    BEGIN (* DetermineMissileMagic *)
      DetermineMissileQuantity (item);
      
      IF item.object.class <> OStruct.Spike THEN
        INCL (item.identification,OStruct.IdentShowHitDam);
        
        IF ObjectIsMagic (chance) THEN
          DetermineMissileEnchantment (item,depth,special);
        ELSIF ObjectIsMagic (cursed) THEN
          DetermineMissileCurse (item,depth);
        END; (* IF *)
      END; (* IF *)
    END DetermineMissileMagic;
    
    PROCEDURE DetermineMissileWeaponMagic (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL; chance,special,cursed : CARDINAL);
    BEGIN (* DetermineMissileWeaponMagic *)
      INCL (item.identification,OStruct.IdentShowHitDam);
      
      IF ObjectIsMagic (chance) THEN
        INC (item.object.strikeMOD,MagicBonus (depth,1,30));
        INC (item.object.damageMOD,MagicBonus (depth,1,20));
      ELSIF ObjectIsMagic (cursed) THEN
        INCL (item.object.attributes,OStruct.Cursed);
        
        DEC (item.object.strikeMOD,MagicBonus (depth,1,50));
        DEC (item.object.damageMOD,MagicBonus (depth,1,30));
        
        item.object.value := 0;
      END; (* IF *)
    END DetermineMissileWeaponMagic;
    
    PROCEDURE DetermineOuterArmorMagic (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL; chance,special,cursed : CARDINAL);
      PROCEDURE DetermineOuterArmorCurse (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL);
        PROCEDURE AddOuterArmorAggravation (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL);
        BEGIN (* AddOuterArmorAggravation *)
          INCL (item.identification,OStruct.IdentShowHitDam);
          INCL (item.object.attributes,OStruct.Aggravation);
          
          item.object.damageMOD := -MagicBonus (depth,1,10);
          item.object.strikeMOD := -MagicBonus (depth,1,10);
          item.object.armorMOD  := -MagicBonus (depth,1,10);
          item.special          := OStruct.NameIrritation;
        END AddOuterArmorAggravation;
        
        PROCEDURE AddOuterArmorEnveloping (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL);
        BEGIN (* AddOuterArmorEnveloping *)
          INCL (item.identification,OStruct.IdentShowHitDam);
          
          item.object.damageMOD := -MagicBonus (depth + 10,2,40);
          item.object.strikeMOD := -MagicBonus (depth + 10,2,40);
          item.object.armorMOD  := -MagicBonus (depth,1,10);
          item.special          := OStruct.NameEnveloping;
        END AddOuterArmorEnveloping;
        
        PROCEDURE AddOuterArmorVulnerability (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL);
        BEGIN (* AddOuterArmorVulnerability *)
          item.object.armorMOD := -MagicBonus (depth + 50,10,100);
          item.special         := OStruct.NameVulnerability;
        END AddOuterArmorVulnerability;
        
      BEGIN (* DetermineOuterArmorCurse *)
        item.object.value := 0;
        
        CASE MRandom.CardinalRandomRangeValue (1,3) OF
          1 : AddOuterArmorAggravation (item,depth);   |
          2 : AddOuterArmorVulnerability (item,depth); |
          3 : AddOuterArmorEnveloping (item,depth);    |
        END; (* CASE *)
        
        INCL (item.object.attributes,OStruct.Cursed);
      END DetermineOuterArmorCurse;
      
      PROCEDURE DetermineOuterArmorEnchantment (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL; special : CARDINAL);
        PROCEDURE AddOuterArmorProtection (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL);
        BEGIN (* AddOuterArmorProtection *)
          INC (item.object.armorMOD,MagicBonus (depth,2,40));
          INC (item.object.value,250);
          
          item.special := OStruct.NameProtection;
        END AddOuterArmorProtection;
        
        PROCEDURE AddOuterArmorStealth (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL);
        BEGIN (* AddOuterArmorStealth *)
          INC (item.object.armorMOD,MagicBonus (depth,1,20));
          INC (item.object.value,500);
          
          item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,3);
          item.special              := OStruct.NameStealth;
          
          INCL (item.identification,OStruct.IdentShowP1);
          INCL (item.object.attributes,OStruct.Stealth);
        END AddOuterArmorStealth;
        
      BEGIN (* DetermineOuterArmorEnchantment *)
        IF NOT (ObjectIsMagic (special)) THEN
          INC (item.object.armorMOD,MagicBonus (depth,1,20));
        ELSIF MRandom.CardinalRandomRangeValue (1,2) = 1 THEN
          AddOuterArmorProtection (item,depth);
        ELSE
          AddOuterArmorStealth (item,depth);
        END; (* IF *)
      END DetermineOuterArmorEnchantment;
      
    BEGIN (* DetermineOuterArmorMagic *)
      IF ObjectIsMagic (chance) THEN
        DetermineOuterArmorEnchantment (item,depth,special);
      ELSIF ObjectIsMagic (cursed) THEN
        DetermineOuterArmorCurse (item,depth);
      END; (* IF *)
    END DetermineOuterArmorMagic;
    
    PROCEDURE DeterminePotionMagic (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL; chance,special,cursed : CARDINAL);
    BEGIN (* DeterminePotionMagic *)
      IF ((item.object.class = OStruct.Potion1) AND (item.object.subclass = 76)) THEN
        item.object.depth := 0;
      END; (* IF *)
    END DeterminePotionMagic;
    
    PROCEDURE DetermineRingMagic (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL; chance,special,cursed : CARDINAL);
      PROCEDURE DetermineRingArmorModifier (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL; cursed : CARDINAL);
      BEGIN (* DetermineRingArmorModifier *)
        INC (item.object.armorMOD,MagicBonus (depth,1,20));
        INC (item.object.value,100 * item.object.armorMOD);
        
        IF ObjectIsMagic (cursed) THEN
          item.object.armorMOD := -item.object.armorMOD;
          item.object.value    := -item.object.value;
          
          INCL (item.object.attributes,OStruct.Cursed);
        END; (* IF *)
      END DetermineRingArmorModifier;
      
      PROCEDURE DetermineRingDamageModifier (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL; cursed : CARDINAL);
      BEGIN (* DetermineRingDamageModifier *)
        INC (item.object.damageMOD,MagicBonus (depth,1,20));
        INC (item.object.value,100 * item.object.damageMOD);
        
        IF ObjectIsMagic (cursed) THEN
          item.object.damageMOD := -item.object.damageMOD;
          item.object.value     := -item.object.value;
          
          INCL (item.object.attributes,OStruct.Cursed);
        END; (* IF *)
      END DetermineRingDamageModifier;
      
      PROCEDURE DetermineRingSearchModifier (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL; cursed : CARDINAL);
      BEGIN (* DetermineRingSearchModifier *)
        item.object.miscellaneous := 5 * MagicBonus (depth,1,20);
        INC (item.object.value,50 * item.object.miscellaneous);
        
        IF ObjectIsMagic (cursed) THEN
          item.object.miscellaneous := -item.object.miscellaneous;
          item.object.value         := -item.object.value;
          
          INCL (item.object.attributes,OStruct.Cursed);
        END; (* IF *)
      END DetermineRingSearchModifier;
      
      PROCEDURE DetermineRingSlayingModifiers (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL; cursed : CARDINAL);
      BEGIN (* DetermineRingSlayingModifiers *)
        INCL (item.identification,OStruct.IdentShowHitDam);
        
        INC (item.object.damageMOD,MagicBonus (depth,1,25));
        INC (item.object.strikeMOD,MagicBonus (depth,1,25));
        INC (item.object.value,100 * (item.object.damageMOD + item.object.strikeMOD));
        
        IF ObjectIsMagic (cursed) THEN
          item.object.damageMOD := -item.object.damageMOD;
          item.object.strikeMOD := -item.object.strikeMOD;
          item.object.value     := -item.object.value;
          
          INCL (item.object.attributes,OStruct.Cursed);
        END; (* IF *)
      END DetermineRingSlayingModifiers;
      
      PROCEDURE DetermineRingSpeedModifier (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL; cursed : CARDINAL);
      BEGIN (* DetermineRingSpeedModifier *)
        IF ObjectIsMagic (cursed) THEN
          item.object.miscellaneous := -MRandom.IntegerRandomRangeValue (1,3);
          item.object.value         := -item.object.value;
          
          INCL (item.object.attributes,OStruct.Cursed);
        ELSE
          item.object.miscellaneous := 1;
        END; (* IF *)
      END DetermineRingSpeedModifier;
      
      PROCEDURE DetermineRingStatisticModifier (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL; cursed : CARDINAL);
      BEGIN (* DetermineRingStatisticModifier *)
        IF ObjectIsMagic (cursed) THEN
          item.object.miscellaneous := -MagicBonus (depth,1,20);
          item.object.value         := -item.object.value;
          
          INCL (item.object.attributes,OStruct.Cursed);
        ELSE
          item.object.miscellaneous := MagicBonus (depth,1,10);
          INC (item.object.value,100 * item.object.miscellaneous);
        END; (* IF *)
      END DetermineRingStatisticModifier;
      
      PROCEDURE DetermineRingStrikeModifier (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL; cursed : CARDINAL);
      BEGIN (* DetermineRingStrikeModifier *)
        INC (item.object.strikeMOD,MagicBonus (depth,1,20));
        INC (item.object.value,100 * item.object.strikeMOD);
        
        IF ObjectIsMagic (cursed) THEN
          item.object.strikeMOD := -item.object.strikeMOD;
          item.object.value     := -item.object.value;
          
          INCL (item.object.attributes,OStruct.Cursed);
        END; (* IF *)
      END DetermineRingStrikeModifier;
      
    BEGIN (* DetermineRingMagic *)
      CASE item.object.subclass OF
        0,1,2,3 : DetermineRingStatisticModifier (item,depth,cursed); |
        4       : DetermineRingSpeedModifier (item,depth,cursed);     |
        5       : DetermineRingSearchModifier (item,depth,cursed);    |
        19      : DetermineRingDamageModifier (item,depth,cursed);    |
        20      : DetermineRingStrikeModifier (item,depth,cursed);    |
        21      : DetermineRingArmorModifier (item,depth,cursed);     |
        30      : DetermineRingSlayingModifiers (item,depth,cursed);  |
        
        24..29  : INCL (item.identification,OStruct.IdentHideP1); |
      ELSE
        (* no action *)
      END; (* CASE *)
    END DetermineRingMagic;
    
    PROCEDURE DetermineScrollMagic (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL; chance,special,cursed : CARDINAL);
    BEGIN (* DetermineScrollMagic *)
      IF item.object.class = OStruct.Scroll1 THEN
        IF item.object.subclass = 67 THEN
          item.object.depth := 1;
        ELSIF item.object.subclass = 69 THEN
          item.object.depth := 0;
        ELSIF item.object.subclass = 80 THEN
          item.object.depth := 5;
        ELSIF item.object.subclass = 81 THEN
          item.object.depth := 5;
        END; (* IF *)
      END; (* IF *)
    END DetermineScrollMagic;
    
    PROCEDURE DetermineStaffMagic (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL; chance,special,cursed : CARDINAL);
    BEGIN (* DetermineStaffMagic *)
      CASE item.object.subclass OF
         0 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,20) + 12; |
         1 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,8) + 6;   |
         2 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,5) + 6;   |
         3 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,20) + 12; |
         4 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,15) + 6;  |
         5 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,4) + 5;   |
         6 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,5) + 3;   |
         7 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,3) + 1; 
             item.object.depth         := 10;                                           |
         8 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,3) + 1;   |
         9 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,5) + 6;   |
        10 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,10) + 12; |
        11 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,5) + 6;   |
        12 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,5) + 6;   |
        13 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,5) + 6;   |
        14 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,10) + 12; |
        15 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,3) + 4;   |
        16 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,5) + 6;   |
        17 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,5) + 6;   |
        18 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,3) + 4;   |
        19 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,10) + 12; |
        20 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,3) + 4;   |
        21 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,3) + 4;   |
        22 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,10) + 6;  
             item.object.depth         := 5;                                            |
      END; (* CASE *)
    END DetermineStaffMagic;
    
    PROCEDURE DetermineWandMagic (VAR item : OStruct.InventoryItemRecord; depth : CARDINAL; chance,special,cursed : CARDINAL);
    BEGIN (* DetermineWandMagic *)
      CASE item.object.subclass OF
         0 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,10) + 6; |
         1 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,8) + 6; |
         2 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,5) + 6; |
         3 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,8) + 6; |
         4 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,4) + 3; |
         5 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,8) + 6; |
         6 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,20) + 12; |
         7 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,20) + 12; |
         8 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,10) + 6; |
         9 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,12) + 6; |
        10 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,10) + 12; |
        11 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,3) + 3; |
        12 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,8) + 6; |
        13 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,10) + 6; |
        14 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,5) + 3; |
        15 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,5) + 3; |
        16 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,5) + 6; |
        17 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,5) + 4; |
        18 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,8) + 4; |
        19 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,6) + 2; |
        20 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,4) + 2; |
        21 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,8) + 6; |
        22 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,5) + 2; |
        23 : item.object.miscellaneous := MRandom.CardinalRandomRangeValue (1,12) + 12; |
      END; (* CASE *)
    END DetermineWandMagic;
    
    PROCEDURE MagicBonus (depth : CARDINAL; base : INTEGER; deviationMAX : CARDINAL) : INTEGER;
    BEGIN (* MagicBonus *)
      RETURN (base + ABS (MRandom.RandomNormalValue (0,WMacro.MinCardinal (125 * depth DIV 100 + 7,deviationMAX))) DIV 10);
    END MagicBonus;
    
    PROCEDURE ObjectIsMagic (chance : CARDINAL) : BOOLEAN;
    BEGIN (* ObjectIsMagic *)
      RETURN (MRandom.CardinalRandomRangeValue (1,100) <= chance);
    END ObjectIsMagic;
    
  BEGIN (* DetermineObjectMagic *)
    chance  := WMacro.MinCardinal (BaseMagicChance + depth,MaximumBaseMagicChance);
    special := chance DIV SpecialMagicChanceDivisor;
    cursed  := 10 * chance DIV CursedMagicChanceDivisor;
    
    CASE item.object.class OF
      OStruct.HardBodyArmor : DetermineGeneralArmorMagic (item,depth,chance,special,cursed); |
      OStruct.MissileWeapon : DetermineMissileWeaponMagic (item,depth,chance,special,cursed); |
      OStruct.SoftBodyArmor : DetermineGeneralArmorMagic (item,depth,chance,special,cursed); |
      OStruct.HaftedWeapon  : DetermineGeneralWeaponMagic (item,depth,chance,special,cursed); |
      OStruct.BladeWeapon   : DetermineGeneralWeaponMagic (item,depth,chance,special,cursed); |
      OStruct.DiggingTool   : DetermineDiggingToolMagic (item,depth,chance,special,cursed); |
      OStruct.PoleWeapon    : DetermineGeneralWeaponMagic (item,depth,chance,special,cursed); |
      OStruct.SlingAmmo     : DetermineMissileMagic (item,depth,chance,special,cursed);       |
      OStruct.Potion1       : DeterminePotionMagic (item,depth,chance,special,cursed); |
      OStruct.Potion2       : DeterminePotionMagic (item,depth,chance,special,cursed); |
      OStruct.Scroll1       : DetermineScrollMagic (item,depth,chance,special,cursed); |
      OStruct.Scroll2       : DetermineScrollMagic (item,depth,chance,special,cursed); |
      OStruct.Amulet        : DetermineAmuletMagic (item,depth,chance,special,cursed); |
      OStruct.Gloves        : DetermineHandArmorMagic (item,depth,chance,special,cursed); |
      OStruct.Shield        : DetermineGeneralArmorMagic (item,depth,chance,special,cursed); |
      OStruct.Arrow         : DetermineMissileMagic (item,depth,chance,special,cursed);       |
      OStruct.Boots         : DetermineFootArmorMagic (item,depth,chance,special,cursed); |
      OStruct.Chest         : DetermineChestMagic (item,depth,chance,special,cursed);         |
      OStruct.Cloak         : DetermineOuterArmorMagic (item,depth,chance,special,cursed); |
      OStruct.Light         : DetermineLightMagic (item,depth,chance,special,cursed);         |
      OStruct.Spike         : DetermineMissileMagic (item,depth,chance,special,cursed);       |
      OStruct.Staff         : DetermineStaffMagic (item,depth,chance,special,cursed); |
      OStruct.Bolt          : DetermineMissileMagic (item,depth,chance,special,cursed);       |
      OStruct.Helm          : DetermineHeadArmorMagic (item,depth,chance,special,cursed); |
      OStruct.Ring          : DetermineRingMagic (item,depth,chance,special,cursed); |
      OStruct.Wand          : DetermineWandMagic (item,depth,chance,special,cursed); |
    ELSE
      (* no action *)
    END; (* CASE *)
  END DetermineObjectMagic;

BEGIN (* OMagic *)
  CurrentMissileCounter := 0;
END OMagic.
