IMPLEMENTATION MODULE OSActions;
  (*/WIN:A*)
  
  IMPORT MMWindow,MRandom,ODescriptions;
  IMPORT Windows,WString;
  IMPORT MStruct,OStruct;
  IMPORT Strings;
  
  
  PROCEDURE AddToObjectAttributeFlags (VAR attributes : OStruct.ObjectAttributeFlags; flags : OStruct.ObjectAttributeFlags);
    VAR
        index : OStruct.ObjectAttributeMnemonic;
    
  BEGIN (* AddToObjectAttributeFlags *)
    FOR index := MIN (OStruct.ObjectAttributeMnemonic) TO MAX (OStruct.ObjectAttributeMnemonic) DO
      IF index IN flags THEN
        INCL (attributes,index);
      END; (* IF *)
    END; (* FOR *)
  END AddToObjectAttributeFlags;
  
  PROCEDURE ClearInventoryItemArray (treasures : OStruct.TreasureInfoPointer; VAR inventory : ARRAY OF OStruct.InventoryItemRecord);
    VAR
        index : CARDINAL;
    
  BEGIN (* ClearInventoryItemArray *)
    ReturnInventoryItemRecord (treasures,OStruct.EmptyObjectCursor,inventory [0]);
    
    FOR index := 1 TO HIGH (inventory) DO
      inventory [index] := inventory [0];
    END; (* FOR *)
  END ClearInventoryItemArray;
  
  PROCEDURE ClearInventoryItemKnown1 (treasures : OStruct.TreasureInfoPointer; item : OStruct.InventoryItemRecord; tried : BOOLEAN);
    VAR
        index : INTEGER;
    
  BEGIN (* ClearInventoryItemKnown1 *)
    index := GlobalIdentificationIndex (item.object);
    
    IF index <> OStruct.InvalidGlobalIdentIndex THEN
      EXCL (treasures^.identification [index],OStruct.GlobalIdentKnown1);
      
      IF tried THEN
        INCL (treasures^.identification [index],OStruct.GlobalIdentTried);
      ELSE
        EXCL (treasures^.identification [index],OStruct.GlobalIdentTried);
      END; (* IF *)
    END; (* IF *)
  END ClearInventoryItemKnown1;
  
  PROCEDURE DamagedByAcid (item : OStruct.InventoryItemRecord) : BOOLEAN;
  BEGIN (* DamagedByAcid *)
    IF item.object.class IN OStruct.ObjectClassSet {OStruct.Bolt,OStruct.Arrow,OStruct.MissileWeapon,OStruct.HaftedWeapon,OStruct.PoleWeapon,OStruct.Boots,OStruct.Gloves,OStruct.Cloak,OStruct.SoftBodyArmor} THEN
      RETURN (NOT (OStruct.ResistAcid IN item.object.attributes));
    ELSE
      RETURN (item.object.class IN OStruct.ObjectClassSet {OStruct.Miscellaneous,OStruct.Chest});
    END; (* IF *)
  END DamagedByAcid;
  
  PROCEDURE DamagedByCold (item : OStruct.InventoryItemRecord) : BOOLEAN;
  BEGIN (* DamagedByCold *)
    RETURN (item.object.class IN OStruct.ObjectClassSet {OStruct.Potion1,OStruct.Potion2,OStruct.Flask});
  END DamagedByCold;
  
  PROCEDURE DamagedByCorrosion (item : OStruct.InventoryItemRecord) : BOOLEAN;
  BEGIN (* DamagedByCorrosion *)
    RETURN (item.object.class IN OStruct.ObjectClassSet {OStruct.BladeWeapon,OStruct.Helm,OStruct.Shield,OStruct.HardBodyArmor,OStruct.Wand});
  END DamagedByCorrosion;
  
  PROCEDURE DamagedByFlame (item : OStruct.InventoryItemRecord) : BOOLEAN;
  BEGIN (* DamagedByFlame *)
    IF item.object.class IN OStruct.ObjectClassSet {OStruct.Arrow,OStruct.MissileWeapon,OStruct.HaftedWeapon,OStruct.PoleWeapon,OStruct.Boots,OStruct.Gloves,OStruct.Cloak,OStruct.SoftBodyArmor} THEN
      RETURN (NOT (OStruct.ResistFire IN item.object.attributes));
    ELSE
      RETURN (item.object.class IN OStruct.ObjectClassSet {OStruct.Staff,OStruct.Scroll1,OStruct.Scroll2});
    END; (* IF *)
  END DamagedByFlame;
  
  PROCEDURE DamagedByLightning (item : OStruct.InventoryItemRecord) : BOOLEAN;
  BEGIN (* DamagedByLightning *)
    RETURN (item.object.class IN OStruct.ObjectClassSet {OStruct.Ring,OStruct.Wand,OStruct.Spike});
  END DamagedByLightning;
  
  PROCEDURE DecrementItemCharges (player : MStruct.PlayerInfoPointer; VAR item : OStruct.InventoryItemRecord);
    VAR
        message : ARRAY [0..127] OF CHAR;
    
  BEGIN (* DecrementItemCharges *)
    DEC (item.object.miscellaneous);
    
    IF InventoryItemIsKnown2 (item) THEN
      WString.ReplaceCardinalParameter ("You have %d charges remaining.","%d",item.object.miscellaneous,message);
      MMWindow.DisplayPopupMessage (player^.windows.frame,player,message);
    END; (* IF *)
  END DecrementItemCharges;
  
  PROCEDURE DescribeRemainingObjectQuantity (player : MStruct.PlayerInfoPointer; item : OStruct.InventoryItemRecord);
  BEGIN (* DescribeRemainingObjectQuantity *)
    DEC (item.object.quantity);
    
    DisplayObjectMessage (player^.windows.frame,player,"You have %OBJECT","%OBJECT",item,TRUE,TRUE);
  END DescribeRemainingObjectQuantity;
  
  PROCEDURE DisplayObjectMessage (window : Windows.HWND; player : MStruct.PlayerInfoPointer; text : ARRAY OF CHAR; formal : ARRAY OF CHAR; item : OStruct.InventoryItemRecord; prefix : BOOLEAN; popup : BOOLEAN);
    VAR
        message : ARRAY [0..127] OF CHAR;
    
  BEGIN (* DisplayObjectMessage *)
    ODescriptions.ReplaceObjectDescription (text,formal,player,item,prefix,message);
    
    IF popup THEN
      MMWindow.DisplayPopupMessage (window,player,message);
    ELSE
      MMWindow.AddMessageToList (player,message);
    END; (* IF *)
  END DisplayObjectMessage;
  
  PROCEDURE FinalObjectWithClass (VAR inventory : ARRAY OF OStruct.InventoryItemRecord; class : OStruct.ObjectClassMnemonic) : CARDINAL;
    VAR
        index : CARDINAL;
    
  BEGIN (* FinalObjectWithClass *)
    FOR index := HIGH (inventory) TO 0 BY -1 DO
      IF inventory [index].object.class = class THEN
        RETURN (index);
      END; (* IF *)
    END; (* FOR *)
    
    RETURN (MAX (CARDINAL));
  END FinalObjectWithClass;
  
  PROCEDURE FirstObjectWithClass (VAR inventory : ARRAY OF OStruct.InventoryItemRecord; class : OStruct.ObjectClassMnemonic) : CARDINAL;
    VAR
        index : CARDINAL;
    
  BEGIN (* FirstObjectWithClass *)
    FOR index := 0 TO HIGH (inventory) DO
      IF inventory [index].object.class = class THEN
        RETURN (index);
      END; (* IF *)
    END; (* FOR *)
    
    RETURN (MAX (CARDINAL));
  END FirstObjectWithClass;
  
  PROCEDURE GlobalIdentificationIndex (object : OStruct.ObjectInfoRecord) : INTEGER;
    CONST
          InvalidObjectClassCursor = -1;
    
    VAR
        class : INTEGER;
    
    PROCEDURE ObjectClassValue (class : OStruct.ObjectClassMnemonic) : INTEGER;
      PROCEDURE FoodObjectSubclassValue (subclass : CARDINAL) : INTEGER;
      BEGIN (* FoodObjectSubclassValue *)
        IF subclass MOD OStruct.ObjectMinimumSingleStackSubclass  < OStruct.NumberMushroomDescriptions THEN
          RETURN (6);
        ELSE
          RETURN (InvalidObjectClassCursor);
        END; (* IF *)
      END FoodObjectSubclassValue;
      
    BEGIN (* ObjectClassValue *)
      CASE object.class OF
        OStruct.Potion1,OStruct.Potion2 : RETURN (5); |
        OStruct.Scroll1,OStruct.Scroll2 : RETURN (4); |
        OStruct.Amulet                  : RETURN (0); |
        OStruct.Staff                   : RETURN (2); |
        OStruct.Ring                    : RETURN (1); |
        OStruct.Wand                    : RETURN (3); |
        
        OStruct.Food : RETURN (FoodObjectSubclassValue (object.subclass)); |
      ELSE
        RETURN (InvalidObjectClassCursor);
      END; (* CASE *)
    END ObjectClassValue;
    
  BEGIN (* GlobalIdentificationIndex *)
    class := ObjectClassValue (object.class);
    
    IF class <> InvalidObjectClassCursor THEN
      RETURN (class * 64 + VAL (INTEGER,object.subclass MOD (OStruct.ObjectMinimumSingleStackSubclass - 1)));
    ELSE
      RETURN (OStruct.InvalidGlobalIdentIndex);
    END; (* IF *)
  END GlobalIdentificationIndex;
  
  PROCEDURE IdentifyInventoryItemAsKnown1 (treasures : OStruct.TreasureInfoPointer; VAR item : OStruct.InventoryItemRecord);
    VAR
        index : CARDINAL;
    
  BEGIN (* IdentifyInventoryItemAsKnown1 *)
    index := GlobalIdentificationIndex (item.object);
    
    IF index <> OStruct.InvalidGlobalIdentIndex THEN
      INCL (treasures^.identification [index],OStruct.GlobalIdentKnown1);
      EXCL (treasures^.identification [index],OStruct.GlobalIdentTried);
    END; (* IF *)
  END IdentifyInventoryItemAsKnown1;
  
  PROCEDURE IdentifyObjectAsKnown2 (treasures : OStruct.TreasureInfoPointer; VAR item : OStruct.InventoryItemRecord);
    PROCEDURE UnsampleObjectInformation (treasures : OStruct.TreasureInfoPointer; VAR item : OStruct.InventoryItemRecord);
      VAR
          index : INTEGER;
      
    BEGIN (* UnsampleObjectInformation *)
      EXCL (item.identification,OStruct.IdentEmpty);
      EXCL (item.identification,OStruct.IdentMagic);
      
      index := GlobalIdentificationIndex (item.object);
      
      IF index <> OStruct.InvalidGlobalIdentIndex THEN
        EXCL (treasures^.identification [index],OStruct.GlobalIdentTried);
      END; (* IF *)
    END UnsampleObjectInformation;
    
  BEGIN (* IdentifyObjectAsKnown2 *)
    UnsampleObjectInformation (treasures,item);
    INCL (item.identification,OStruct.IdentKnown2);
  END IdentifyObjectAsKnown2;
  
  PROCEDURE IdentifyObjectAsStoreBought (treasures : OStruct.TreasureInfoPointer; VAR item : OStruct.InventoryItemRecord);
  BEGIN (* IdentifyObjectAsStoreBought *)
    INCL (item.identification,OStruct.IdentStoreBought);
    IdentifyObjectAsKnown2 (treasures,item);
  END IdentifyObjectAsStoreBought;
  
  PROCEDURE InitializeObjectNames (VAR treasures : OStruct.TreasureInformation; seed : CARDINAL);
    PROCEDURE InitializeScrollTitleList (VAR syllables : ARRAY OF OStruct.TitleSyllableString; VAR list : ARRAY OF OStruct.ObjectDescriptionString);
      VAR
          index : CARDINAL;
      
      PROCEDURE InitializeScrollTitle (VAR syllables : ARRAY OF OStruct.TitleSyllableString; VAR result : OStruct.ObjectDescriptionString);
        VAR
            word,words : CARDINAL;
            syllable   : CARDINAL;
        
      BEGIN (* InitializeScrollTitle *)
        result [0] := 0C;
        words      := MRandom.CardinalRandomRangeValue (2,3);
        
        FOR word := 1 TO words DO
          FOR syllable := 1 TO MRandom.CardinalRandomRangeValue (1,2) DO
            Strings.Insert (syllables [MRandom.CardinalRandomRangeValue (0,HIGH (syllables))],result,HIGH (result));
          END; (* FOR *)
          
          IF word < words THEN
            Strings.Insert (' ',result,HIGH (result));
          END; (* IF *)
        END; (* FOR *)
      END InitializeScrollTitle;
      
    BEGIN (* InitializeScrollTitleList *)
      FOR index := 0 TO HIGH (list) DO
        InitializeScrollTitle (syllables,list [index]);
      END; (* FOR *)
    END InitializeScrollTitleList;
    
    PROCEDURE RandomizeObjectDescriptionList (VAR list : ARRAY OF OStruct.ObjectDescriptionString; start : CARDINAL);
      VAR
          index : CARDINAL;
      
      PROCEDURE SwapObjectDescriptionStrings (VAR value1,value2 : OStruct.ObjectDescriptionString);
        VAR
            temp : OStruct.ObjectDescriptionString;
        
      BEGIN (* SwapObjectDescriptionStrings *)
        temp   := value1;
        value1 := value2;
        value2 := temp;
      END SwapObjectDescriptionStrings;
      
    BEGIN (* RandomizeObjectDescriptionList *)
      FOR index := start TO HIGH (list) DO
        SwapObjectDescriptionStrings (list [index],list [MRandom.CardinalRandomRangeValue (start,HIGH (list))]);
      END; (* FOR *)
    END RandomizeObjectDescriptionList;
    
  BEGIN (* InitializeObjectNames *)
    treasures.seed := seed;
    
    WITH treasures DO
      MRandom.SetRandomSeed (seed);
      
      RandomizeObjectDescriptionList (mushrooms,0);
      RandomizeObjectDescriptionList (amulets,0);
      RandomizeObjectDescriptionList (potions,3);
      RandomizeObjectDescriptionList (staves,0);
      RandomizeObjectDescriptionList (rings,0);
      RandomizeObjectDescriptionList (wands,0);
      
      InitializeScrollTitleList (syllables,titles);
      
      MRandom.Randomize;
    END; (* WITH *)
  END InitializeObjectNames;
  
  PROCEDURE InventoryItemHasBeenTried (treasures : OStruct.TreasureInfoPointer; item : OStruct.InventoryItemRecord) : BOOLEAN;
  BEGIN (* InventoryItemHasBeenTried *)
    IF NOT (ObjectIsAlwaysKnown1 (item.object) OR ObjectIsStoreBought (item)) THEN
      RETURN (OStruct.GlobalIdentTried IN treasures^.identification [GlobalIdentificationIndex (item.object)]);
    ELSE
      RETURN (FALSE);
    END; (* IF *)
  END InventoryItemHasBeenTried;
  
  PROCEDURE InventoryItemIsKnown1 (treasures : OStruct.TreasureInfoPointer; item : OStruct.InventoryItemRecord) : BOOLEAN;
  BEGIN (* InventoryItemIsKnown1 *)
    IF NOT (ObjectIsAlwaysKnown1 (item.object) OR ObjectIsStoreBought (item)) THEN
      RETURN (OStruct.GlobalIdentKnown1 IN treasures^.identification [GlobalIdentificationIndex (item.object)]);
    ELSE
      RETURN (TRUE);
    END; (* IF *)
  END InventoryItemIsKnown1;
  
  PROCEDURE InventoryItemIsKnown2 (item : OStruct.InventoryItemRecord) : BOOLEAN;
  BEGIN (* InventoryItemIsKnown2 *)
    RETURN (OStruct.IdentKnown2 IN item.identification);
  END InventoryItemIsKnown2;
  
  PROCEDURE ObjectIsAlwaysKnown1 (object : OStruct.ObjectInfoRecord) : BOOLEAN;
  BEGIN (* ObjectIsAlwaysKnown1 *)
    RETURN (GlobalIdentificationIndex (object) = OStruct.InvalidGlobalIdentIndex);
  END ObjectIsAlwaysKnown1;
  
  PROCEDURE ObjectIsStoreBought (item : OStruct.InventoryItemRecord) : BOOLEAN;
  BEGIN (* ObjectIsStoreBought *)
    RETURN (OStruct.IdentStoreBought IN item.identification);
  END ObjectIsStoreBought;
  
  PROCEDURE ReturnInventoryItemRecord (treasures : OStruct.TreasureInfoPointer; cursor : CARDINAL; VAR result : OStruct.InventoryItemRecord);
  BEGIN (* ReturnInventoryItemRecord *)
    result.inscription [0] := 0C;
    result.identification  := OStruct.ObjectIdentificationFlags {};
    result.special         := OStruct.NameNull;
    result.object          := treasures^.table [cursor];
    result.cursor          := cursor;
  END ReturnInventoryItemRecord;

  PROCEDURE SampleInventoryItem (treasure : OStruct.TreasureInfoPointer; item : OStruct.InventoryItemRecord);
    VAR
        position : INTEGER;
    
  BEGIN (* SampleInventoryItem *)
    position := GlobalIdentificationIndex (item.object);
    
    IF position <> OStruct.InvalidGlobalIdentIndex THEN
      INCL (treasure^.identification [position],OStruct.GlobalIdentTried);
    END; (* IF *)
  END SampleInventoryItem;
END OSActions.
